/*
 * hfcl_conf.c
 * Copyright (C) 2007, 2012, Hitachi, Ltd.
 * Author(s): Yoshihiro Toyohara <yoshihiro.toyohara.qs@hitachi.com>
 *
 */
/*
 * $Id: hfcl_conf.c,v 1.5.2.12.2.3.6.4 2012/07/04 10:17:50 toyo Exp $
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/pci.h>

#ifdef _HFC_FCGW														/* FC-GW */
#include <scsi/scsi_device.h>											/* FC-GW */
#include <scsi/scsi_host.h>												/* FC-GW */
#include <scsi/scsi_transport_fc.h>										/* FC-GW */
#endif																	/* FC-GW */

#include "hfcldd.h"
#include "hfcl_detect.h"
#include "hfcl_modulever.h"

#ifndef _HFC_NO_RASLOG
#include "hraslog.h"
#endif

extern struct manage_info hfc_manage_info;

#ifndef _HFC_NO_RASLOG
extern struct hraslogopt_st hraslogopt;
extern	uint	raslog_install;
#endif

static struct pci_device_id hfcldd_pci_tbl[] = {
	{HFC_PCI_VENDOR_ID, HFC_PCI_DEVICE_ID_3009,
	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{HFC_PCI_VENDOR_ID, HFC_PCI_DEVICE_ID_300A,
	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{HFC_PCI_VENDOR_ID, HFC_PCI_DEVICE_ID_300B,
	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{HFC_PCI_VENDOR_ID, HFC_PCI_DEVICE_ID_300C,
	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{HFC_PCI_VENDOR_ID, HFC_PCI_DEVICE_ID_300D,
	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{0,}
};

MODULE_DEVICE_TABLE(pci, hfcldd_pci_tbl);


static struct pci_driver hfcldd_pci_driver = {
    .name           = "hfcldd",
    .id_table       = hfcldd_pci_tbl,
    .probe          = hfc_probe_one,
    .remove         = __devexit_p(hfc_remove_one),
};

#ifdef HFC_SYSFS_FCGW																	/* FC-GW */
extern struct scsi_transport_template *hfc_fc_attach_transport;			/* FC-GW */
extern struct fc_function_template hfcldd_fc_function_template;			/* FC-GW */
#endif /* HFC_SYSFS_FCGW */					/* FC-GW */

/*
 * Function:    hfcldd_init
 *
 * Purpose:     
 *
 * Arguments:   
 *
 * Returns:     
 *
 * Lock status: 
 *
 * Notes:       
 */
static int __init
hfcldd_init(void)
{
#ifdef HFC_SYSFS_FCGW																				/* FC-GW */
    hfc_fc_attach_transport = fc_attach_transport(&hfcldd_fc_function_template);	/* FC-GW */
#endif /* HFC_SYSFS_FCGW */			/* FC-GW */
    return pci_module_init(&hfcldd_pci_driver);
}

/*
 * Function:    hfcldd_exit
 *
 * Purpose:     
 *
 * Arguments:   
 *
 * Returns:     
 *
 * Lock status: 
 *
 * Notes:       
 */
static void __exit
hfcldd_exit(void)
{
#ifndef _HFC_NO_RASLOG
	if( !raslog_install ){
		hraslogopt.func_code = CLOSE_RASLOG;
		raslog_install = _hraslogserv( &hraslogopt );
		if( raslog_install == 1 ){
			HFC_WRNPRT("hfcldd : HFC_ERR9 FC Adapter Driver error (ErrNo:0xC7)  Failed to close hraslog \n");
		}
		else if( raslog_install == 2 ){
			HFC_WRNPRT("hfcldd : HFC_ERR9 FC Adapter Driver error (ErrNo:0xC8) \n");
		}
		else{
			HFC_INFPRT("hfcldd : Closed hraslog\n");
		}
	}
#endif

	if(hfc_manage_info.hfcldd_mp_mod) {
		module_put(hfc_manage_info.mp_manage_info->hfcldd_mp);
	}
    pci_unregister_driver(&hfcldd_pci_driver);
#ifdef HFC_SYSFS_FCGW																	/* FC-GW */
    fc_release_transport(hfc_fc_attach_transport);						/* FC-GW */
#endif /* HFC_SYSFS_FCGW */					/* FC-GW */
}

module_init(hfcldd_init);
module_exit(hfcldd_exit);

MODULE_LICENSE("GPL");

MODULE_DESCRIPTION("Hitachi Fibre Channel Host Bus Adapter driver module ");
MODULE_AUTHOR("Hitachi, Ltd.");
MODULE_VERSION(HFC_MODULEVER);

