/*
 * hfcl_detect.c
 * Copyright (C) 2007, 2012, Hitachi, Ltd.
 * Author(s): Yoshihiro Toyohara <yoshihiro.toyohara.qs@hitachi.com>
 */

char det_rcsid[] = "$Id: hfcl_detect.c,v 1.115.2.42.2.39.2.4.2.23 2012/07/04 10:17:50 toyo Exp $";

#ifdef _HFC_FCGW														/* FC-GW */
#include <scsi/scsi_device.h>											/* FC-GW */
#include <scsi/scsi_host.h>												/* FC-GW */
#include <scsi/scsi_transport_fc.h>										/* FC-GW */
#endif																	/* FC-GW */

#include "hfcldd.h"
#include "hfcl_tbol.h"
#include "hfcl_strategy.h"
#include "hfcl_stra_trace.h"
#include "hfcl_timer_recovery.h"
#include "hfcl_ioctl.h"
#include "hfcl_top.h"
#include "hfcl_detect.h"
#include "hfcl_handler.h"
#include "hfcl_mlpf.h"
#include "hfcl_version.h"
#include "hfcmpcfg.h"
#include "hfcddwwn.h"
#include "hfcldd_conf.h"
#include "hfcl_hand_timer_trace.h"

#ifndef _HFC_NO_RASLOG
#include "hraslog.h"
#endif

void structdump( int loc, uchar *p, int size );
#ifndef _HFC_DEBUG	/* FCLNX-0510	*/
	#define	STRUCTDUMP( LOC, PTR, SIZE ) { while(0) {}; }
#else
	#define STRUCTDUMP( LOC, PTR, SIZE ) { structdump( LOC, PTR, SIZE ); }
#endif			/* FCLNX-0510	*/

#if defined(__x86_64) && !defined(_HFC_FCGW) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
int x86_64_32_reg_increasing(void);
int x86_64_32_unreg_increasing(void);
#endif /*__x86_64 && !_HFC_FCGW*/


/* PCI memory space address map */
const struct pkg_map hfc_pkg_map[2] = {
	{/* FPP */
		{ 
		  /* IOSPACE MAP */	/* no : name		 			   size */
			{	0x0000,			/* 0  :  HFC_IOSPACE_ZERO			4B	*/
				0x0001, 		/* 1  :  HFC_IOSPACE_LSICODE		1B	*/
				0x0003, 		/* 2  :  HFC_IOSPACE_LSIREV			1B	*/
				0x0004, 		/* 3  :  HFC_IOSPACE_OFS4			4B	*/
				0x0005, 		/* 4  :  HFC_IOSPACE_PKCODE			1B	*/
				0x0007, 		/* 5  :  HFC_IOSPACE_PKREV			1B	*/
				0x0008, 		/* 6  :  HFC_IOSPACE_CHNO			1B	*/
				0x0010, 		/* 7  :  HFC_IOSPACE_STATUS0		4B	*/
				0x0014, 		/* 8  :  HFC_IOSPACE_STATUS1		4B	*/
				0x0018, 		/* 9  :  HFC_IOSPACE_ERRDETAIL0		4B	*/
				0x001C, 		/* 10 :  HFC_IOSPACE_ERRDETAIL1		4B	*/
				0x0020, 		/* 11 :  HFC_IOSPACE_INTA			4B	*/
				0x0024, 		/* 12 :  HFC_IOSPACE_MPINTAC		4B	*/
				0x0028, 		/* 13 :  HFC_IOSPACE_INTA_MSK		4B	*/
				0x002c, 		/* 14 :  HFC_IOSPACE_INTA_RST		4B	*/
				0x0040, 		/* 15 :  HFC_IOSPACE_CMDRES			1B	*/
				0x0041, 		/* 16 :  HFC_IOSPACE_CMDCTL			1B	*/
				0x0042, 		/* 17 :  HFC_IOSPACE_CMDBOOT		1B	*/
				0x0043, 		/* 18 :  HFC_IOSPACE_CMDLED			1B	*/
				0x0044, 		/* 19 :  HFC_IOSPACE_CMDFMEM		1B	*/
				0x0045, 		/* 20 :  HFC_IOSPACE_CMDFCIF		1B	*/
				0x0047, 		/* 21 :  HFC_IOSPACE_CMDMODE		1B	*/
				0x00F0, 		/* 22 :  HFC_IOSPACE_TRCA0			2B	*/
				0x00F2, 		/* 23 :  HFC_IOSPACE_TRCA1			2B	*/
				0x02F8, 		/* 24 :  HFC_IOSPACE_RAMMSK			1B	*/
				0x02FC, 		/* 25 :  HFC_IOSPACE_RAMADR			4B	*/
				0x0200, 		/* 26 :  HFC_IOSPACE_HTYP			1B	*/
				0x0201, 		/* 27 :  HFC_IOSPACE_OTYP			1B	*/
				0x0202, 		/* 28 :  HFC_IOSPACE_HWINF			1B	*/
				0x0203, 		/* 29 :  HFC_IOSPACE_LAPC			1B	*/
				0x0000, 		/* 30 :  HFC_IOSPACE_CMDSCAN		1B  */
				0x0000, 		/* 31 :  */
				0x0000, 		/* 32 :  */
				0x0000, 		/* 33 :  */
				0x0000, 		/* 34 :  */
				0x0000, 		/* 35 :  */
				0x0000, 		/* 36 :  */
				0x0000, 		/* 37 :  */
				0x0000, 		/* 38 :  */
				0x0000, 		/* 39 :  */
				0x0400, 		/* 40 :  HFC_IOSPACE_FRAMEA			4B	*/
				0x0500, 		/* 41 :  HFC_IOSPACE_INDAREA		4B	*/
				0x0600, 		/* 42 :  HFC_IOSPACE_INDAREA_P		4B	*/
				0x0800, 		/* 43 :  HFC_IOSPACE_SCANAREA		4B	*/
				0x0000, 		/* 44 :  */
				0x0000, 		/* 45 :  */
				0x0300, 		/* 46 :  HFC_IOSPACE_CA_POSTRESULT	4B	*/
				0x0304, 		/* 47 :  HFC_IOSPACE_CA_MPCHK_CODE	4B	*/
				0x030c, 		/* 48 :  HFC_IOSPACE_CA_FLAG		1B	*/
				0x0310, 		/* 49 :  HFC_IOSPACE_CA_INIT_ADDR0	4B	*/
				0x0314, 		/* 50 :  HFC_IOSPACE_CA_INIT_ADDR1	4B	*/
				0x0318, 		/* 51 :  HFC_IOSPACE_CA_RSTINFO		1B	*/
				0x0319, 		/* 52 :  HFC_IOSPACE_CA_ALPA		1B	*/
				0x031d, 		/* 53 :  HFC_IOSPACE_CA_PORTNO		1B  */
				0x031c, 		/* 54 :  HFC_IOSPACE_CA_POST		1B	*/
				0x0350,			/* 55 :  HFC_IOSPACE_CA_WWPN0		4B  */
				0x0354,			/* 56 :  HFC_IOSPACE_CA_WWPN1		4B  */
				0x0358,			/* 57 :  HFC_IOSPACE_CA_WWNN0		4B  */
				0x035c,			/* 58 :  HFC_IOSPACE_CA_WWNN1		4B  */
				0x03c0, 		/* 59 :  HFC_IOSPACE_DRV_USED		4B	*/
				0x03c4, 		/* 60 :  HFC_IOSPACE_DRV_USED1		4B	*/
				0x03c8, 		/* 61 :  HFC_IOSPACE_DRV_USED2		4B	*/
				0x03cc, 		/* 62 :  HFC_IOSPACE_DRV_USED3		4B	*/
				0x031a,			/* 63 :  HFC_IOSPACE_CA_LNKSPD		1B  */
				0x031b,			/* 64 :  HFC_IOSPACE_CA_CNTTYP		1B  */
				0x0000, 		/* 65 :  */
				0x0000, 		/* 66 :  */
			}
		}
	},

	/* FIVE */
	{
		{
			/* IOSPACE MAP */	/* no : name		 			   size */
			{	0x0000,			/* 0  :  HFC_IOSPACE_ZERO			4B	*/
				0x0001, 		/* 1  :  HFC_IOSPACE_LSICODE		1B	*/
				0x0003, 		/* 2  :  HFC_IOSPACE_LSIREV			1B	*/
				0x0004, 		/* 3  :  HFC_IOSPACE_OFS4			4B	*/
				0x0005, 		/* 4  :  HFC_IOSPACE_PKCODE			1B	*/
				0x0007, 		/* 5  :  HFC_IOSPACE_PKREV			1B	*/
				0x0008, 		/* 6  :  HFC_IOSPACE_CHNO			1B	*/
				0x0010, 		/* 7  :  HFC_IOSPACE_STATUS0		4B	*/
				0x0014, 		/* 8  :  HFC_IOSPACE_STATUS1		4B	*/
				0x0018, 		/* 9  :  HFC_IOSPACE_ERRDETAIL0		4B	*/
				0x0000, 		/* 10 :  It is not possible to use it	*/
				0x00a0, 		/* 11 :  HFC_IOSPACE_INTA			4B	*/
				0x00a4, 		/* 12 :  HFC_IOSPACE_MPINTAC		4B	*/
				0x00a8, 		/* 13 :  HFC_IOSPACE_INTA_MSK		4B	*/
				0x00ac, 		/* 14 :  HFC_IOSPACE_INTA_RST		4B	*/
				0x0030, 		/* 15 :  HFC_IOSPACE_CMDRES			1B	*/
				0x0031, 		/* 16 :  HFC_IOSPACE_CMDCTL			1B	*/
				0x0032, 		/* 17 :  HFC_IOSPACE_CMDBOOT		1B	*/
				0x0038, 		/* 18 :  HFC_IOSPACE_CMDLED		(*)	4B	*/
				0x0034, 		/* 19 :  HFC_IOSPACE_CMDFMEM		1B	*/
				0x003C, 		/* 20 :  HFC_IOSPACE_CMDFCIF	(*)	1B	*/
				0x0037, 		/* 21 :  HFC_IOSPACE_CMDMODE		1B	*/
				0x00f0, 		/* 22 :  HFC_IOSPACE_TRCA0			2B	*/
				0x00f2, 		/* 23 :  HFC_IOSPACE_TRCA1			2B	*/
				0x02f8, 		/* 24 :  HFC_IOSPACE_RAMMSK			1B	*/
				0x02fC, 		/* 25 :  HFC_IOSPACE_RAMADR			4B	*/
				0x001c, 		/* 26 :  HFC_IOSPACE_HTYP			1B	*/
				0x001d, 		/* 27 :  HFC_IOSPACE_OTYP			1B	*/
				0x001e, 		/* 28 :  HFC_IOSPACE_HWINF			1B	*/
				0x001f, 		/* 29 :  HFC_IOSPACE_LAPC			1B	*/
				0x0036,			/* 30 :  HFC_IOSPACE_CMDSCAN		1B  */
				0x02fb, 		/* 31 :  HFC_IOSPACE_IDFLGEN		1B	*/
				0x0000, 		/* 32 :  */
				0x0000, 		/* 33 :  */
				0x0000, 		/* 34 :  */
				0x0000, 		/* 35 :  */
				0x0000, 		/* 36 :  */
				0x0000, 		/* 37 :  */
				0x0000, 		/* 38 :  */
				0x0000, 		/* 39 :  */
				0x0500, 		/* 40 :  HFC_IOSPACE_FRAMEA			4B	*/
				0x0600, 		/* 41 :  HFC_IOSPACE_INDAREA		4B	*/
				0x0700, 		/* 42 :  HFC_IOSPACE_INDAREA_P		4B	*/
				0x1000, 		/* 43 :  HFC_IOSPACE_SCANAREA		4B	*/
				0x0000, 		/* 44 :  */
				0x0000, 		/* 45 :  */
				0x0300, 		/* 46 :  HFC_IOSPACE_CA_POSTRESULT	4B	*/
				0x0304, 		/* 47 :  HFC_IOSPACE_CA_MPCHK_CODE	4B	*/
				0x030c, 		/* 48 :  HFC_IOSPACE_CA_FLAG		1B	*/
				0x0310, 		/* 49 :  HFC_IOSPACE_CA_INIT_ADDR0	4B	*/
				0x0314, 		/* 50 :  HFC_IOSPACE_CA_INIT_ADDR1	4B	*/
				0x0318, 		/* 51 :  HFC_IOSPACE_CA_RSTINFO		1B	*/
				0x0319, 		/* 52 :  HFC_IOSPACE_CA_ALPA		1B	*/
				0x031d, 		/* 53 :  HFC_IOSPACE_CA_PORTNO		1B	*/
				0x031c, 		/* 54 :  HFC_IOSPACE_CA_POST		1B	*/
				0x0350, 		/* 55 :  HFC_IOSPACE_CA_WWPN0		4B	*/
				0x0354, 		/* 56 :  HFC_IOSPACE_CA_WWPN1		4B	*/
				0x0358, 		/* 57 :  HFC_IOSPACE_CA_WWNN0		4B	*/
				0x035c, 		/* 58 :  HFC_IOSPACE_CA_WWNN1		4B	*/
				0x0360, 		/* 59 :  HFC_IOSPACE_DRV_USED		4B	*/
				0x0364, 		/* 60 :  HFC_IOSPACE_DRV_USED1		4B	*/
				0x0368, 		/* 61 :  HFC_IOSPACE_DRV_USED2		4B	*/
				0x036c, 		/* 62 :  HFC_IOSPACE_DRV_USED3		4B	*/
				0x031a, 		/* 63 :  HFC_IOSPACE_CA_LNKSPD		1B  */
				0x031b, 		/* 64 :  HFC_IOSPACE_CA_CNTTYP		1B  */
				0x0370, 		/* 65 :  HFC_IOSPACE_FW_SUPPORT		1B	*/
				0x0000  		/* 66 :  */
			}
		}
	}
};


/* AL-PA# */
const uchar posmap_lisa[128] = {
	0x00, 0x00, 0x01, 0x02, 0x04, 0x08, 0x0f, 0x10,
	0x17, 0x18, 0x1b, 0x1d, 0x1e, 0x1f, 0x23, 0x25,
	0x26, 0x27, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e,
	0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x39, 0x3a,
	0x3c, 0x43, 0x45, 0x46, 0x47, 0x49, 0x4a, 0x4b,
	0x4c, 0x4d, 0x4e, 0x51, 0x52, 0x53, 0x54, 0x55,
	0x56, 0x59, 0x5a, 0x5c, 0x63, 0x65, 0x66, 0x67,
	0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x71, 0x72,
	0x73, 0x74, 0x75, 0x76, 0x79, 0x7a, 0x7c, 0x80,
	0x81, 0x82, 0x84, 0x88, 0x8f, 0x90, 0x97, 0x98,
	0x9b, 0x9d, 0x9e, 0x9f, 0xa3, 0xa5, 0xa6, 0xa7,
	0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xb1, 0xb2,
	0xb3, 0xb4, 0xb5, 0xb6, 0xb9, 0xba, 0xbc, 0xc3,
	0xc5, 0xc6, 0xc7, 0xc9, 0xca, 0xcb, 0xcc, 0xcd,
	0xce, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd9,
	0xda, 0xdc, 0xe0, 0xe1, 0xe2, 0xe4, 0xe8, 0xef
};


extern const char package_ver[16]; 		/* package version               */

#ifdef HFC_SYSFS_FCGW															/* FC-GW */
struct scsi_transport_template *hfc_fc_attach_transport=NULL;	/* FC-GW */
#endif /* HFC_SYSFS_FCGW */				/* FC-GW */

static struct file_operations hfc_fops = {
	owner: THIS_MODULE,
#ifdef HAVE_UNLOCKED_IOCTL
	unlocked_ioctl:hfc_ioctl64,
	compat_ioctl:hfc_ioctl32,
#else
	ioctl:hfc_ioctl,
#endif
	open:hfc_open,
	release:hfc_close
};


const struct cr_PartsNumber cr_pn[] = {				/* FCLNX-0329 */
	{ 0x85, 1, 0, "3", "3", "-C", "HFC0401",   },	/* FIVE 2G 1Port */
	{ 0x85, 2, 0, "3", "2", "-B", "HFC0402",   },	/* FIVE 2G 2Port */
	{ 0x85, 4, 0, "3", "1", "-A", "HFC0404",   },	/* FIVE 2G 4Port */
	{ 0x82, 1, 0, "3", "2", "-B", "HFC0401",   },	/* FIVE 4G 1Port */
	{ 0x82, 2, 0, "3", "1", "-A", "HFC0402",   },	/* FIVE 4G 2Port */
	{ 0x86, 1, 0, "3", "2", "-B", "HFC0401",   },	/* FIVE 4G 1Port */
	{ 0x86, 2, 0, "3", "1", "-A", "HFC0402",   },	/* FIVE 4G 2Port */
	{ 0x87, 1, 0, "3", "1", "-A", "HFC1001",   },	/* FIVE 10G */
	{ 0x88, 1, 0, "3", "2", "-B", "HFC0401-C", },	/* Type3 Combo 1Port */
	{ 0x88, 2, 0, "3", "1", "-A", "HFC0402-C", },	/* Type3 Combo 2Port */
	{ 0x89, 1, 0, "3", "1", "-A", "HFC0401-C", },	/* FCSWM 1Port */
	{ 0x8a, 1, 0, "3", "2", "-B", "HFC0401-M", },	/* MEZANINE 1Port */
	{ 0x8a, 2, 0, "3", "1", "-A", "HFC0402-M", },	/* MEZANINE 2Port */
	{ 0x8b, 1, 0, "3", "2", "-B", "HFC0401-M", },	/* Enhanced MEZANINE 1Port *//* FCLNX-GPL-054 */
	{ 0x8b, 2, 0, "3", "1", "-A", "HFC0402-M", },	/* Enhanced MEZANINE 2Port *//* FCLNX-GPL-054 */
	{ 0x8f, 2, 0, "3", "1", "-A", "HFC0402-N", },	/* FC-GW 2Port */	/* FC-GW */
	{ 0x8f, 1, 0, "3", "2", "-B", "HFC0401-N", },	/* FC-GW 1Port */	/* FC-GW */
	{ 0x84, 1, 0, "3", "2", "-B", "HFC0401-E", },   /* FIVE 4G PCIe 1 port */    /* FCLNX-GPL-054 */
	{ 0x84, 2, 0, "3", "1", "-A", "HFC0402-E", },   /* FIVE 4G PCIe 2 port */    /* FCLNX-GPL-054 */
	{ 0,    0, 0, "",  "",  "",   "UNKNOWN"    }	/* END */
};


/* driver information */
struct manage_info hfc_manage_info;			/* driver information */
int instance = 0;
int hfc_major = 0;
int hba_num = 0;


int adapter_bindings = 0;
struct narrow_dev hfc_narrow_dev;												/* FCLNX-0392 */

static char *hfclddconf = NULL;

#if _HFC_ERROR_INJ
int hfc_debug_ioerr_code = 0;
#endif

#ifndef _HFC_NO_RASLOG
struct hraslogopt_st hraslogopt;
#endif

uint	raslog_install = 2;

#include "hfcl_moduleparam.h"

void print_module_parameter(void);
void hfc_read_hfcbios(struct adap_info *ap);									/* hotplug */

#define MIN_IOBASE_LEN          0x100


int hfc_pci_conf(struct adap_info *ap)
{
	struct pci_dev *pdev	= NULL;
	unsigned long base0, len0, flag0, base4;
	ushort pci_reg;
	uchar  basic = 0;
	
	HFC_ENTRY("hfc_pci_conf");
																	/* @MLPF */


	pdev = ap->pci_cfginf;
	
	base4 = pci_resource_start(pdev, 4);
	if(base4 == 0)
		basic = 1;
																	/* @MLPF */
	if ( basic == 1 || HFC_MMODE_CHECK_SHADOW(ap) )
	{
		/* Initialize PCI config register */ 
		pci_reg = (ushort) hfc_read_cnfg (ap, HFC_HOST_STAT_CMD, 0x2);
		hfc_write_cnfg(ap, HFC_HOST_STAT_CMD, 0x2, (uint64_t)(pci_reg|HFC_HSC_MSS|HFC_HSC_MB));
		
		if ( (ushort) hfc_read_cnfg (ap, HFC_HOST_VENDER_ID, 0x2) == 0x1054 )	/* FIVE FCWIN-0217 */
		{
			ushort device_id = (ushort) hfc_read_cnfg (ap, HFC_HOST_DEVICE_ID, 0x2);
		
			if ( (device_id == 0x300A)
			  || (device_id == 0x300B)
			  || (device_id == 0x300C)
			  || (device_id == 0x300D) )
			{
				uchar lsi_rev = (uchar)hfc_read_cnfg(ap, 0x4a, 0x01);
				lsi_rev &= 0x07;
				if ( lsi_rev == 0x02 )
				{
					ushort      wkc;
				
					// Set registers for FIVE Pass2				/* @@ */
					
					// It is set by Max Outstanding Split Transaction one
					wkc = (ushort)hfc_read_cnfg(ap, 0x6a, 0x02);
					wkc &= 0xff8f;
					hfc_write_cnfg(ap, 0x6a, 0x02, wkc);
				
					// Clear Base Address Register (Upper 32-bit) with 0
					hfc_write_cnfg(ap, 0x14, 0x04, 0x00000000);
					
					// Conventional PCI (not PCI-X)? 
					wkc = (ushort)hfc_read_cnfg(ap, 0x94, 0x02);
					wkc &= 0x000f;
					if ( wkc == 0x0000 )
					{
						// Set Disable REQ64# Assertion
						wkc = (ushort)hfc_read_cnfg(ap, 0x90, 0x02);
						wkc |= 0x0001;
						hfc_write_cnfg(ap, 0x90, 0x02, wkc);
					}
				}
			}
		}
	}
																	/* @MLPF */


	HFC_DBGPRT( "  hfcldd : hfc_pci_conf - allocate pci_resource\n"); 
	
	base0 = pci_resource_start(pdev, 0);
	len0 = pci_resource_len(pdev, 0);
	flag0 = pci_resource_flags(pdev, 0);
	HFC_DBGPRT("PCI resource 0 base0:%x, len0:%x, flag0:%x \n", (uint)base0, (uint)len0, (uint)flag0);

	if (!(flag0 & IORESOURCE_MEM)) {
		HFC_DBGPRT("  scsi(%ld): region #0 not a PIO resource, aborting\n",
				ap->host_no);

		if( ap->raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xCA, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xCA, NULL, 0) ;

		goto resource_error_exit;
	}

	if (len0 < MIN_IOBASE_LEN) {
		HFC_DBGPRT("  scsi(%d): Invalid PCI I/O region size, aborting\n",
				(uint)ap->host_no);

		if( ap->raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xCB, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xCB, NULL, 0) ;

		goto resource_error_exit;
	}

	if (pci_request_regions(pdev, "hfcldd")) {
		HFC_DBGPRT("  scsi(%d): Failed to reserve PCI_MEMORY_SPACE\n",
				(uint)ap->host_no);

		if( ap->raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xCC, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xCC, NULL, 0) ;
		goto resource_error_exit;

	}

	ap->mem_base_addr = (ulong)ioremap(base0, len0);
	if (!ap->mem_base_addr){
		HFC_DBGPRT(" can not remap memory mapped IO \n");
		if( ap->raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xCD, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xCD, NULL, 0) ;
		goto iospace_error_exit;
	}
	
	HFC_DBGPRT(" hfcldd : hfc_attach - config pci devices end \n"); 
	HFC_DBGPRT( "  scsi(%ld): Success to reserve PCI_MEMORY_SPACE\n",
				(ulong)ap->host_no);
	
	memset(&ap->pkg, 0, sizeof(ap->pkg));                           /* FCLNX-0*** */

	ap->pkg.map = (struct pkg_map *) &hfc_pkg_map[1];		/* FCLNX-03xx */
	
	if ( hfc_mlpf_setup_lparmode(ap) == HFC_MLPF_DISABLE )
		return (-ENODEV);
	
	/* initialize PCI config register */ 
	if ( (!(HFC_MMODE_CHECK_SHARED(ap)) || HFC_MMODE_CHECK_SHADOW(ap) ) )
	{
		pci_reg = (ushort) hfc_read_cnfg (ap, HFC_HOST_STAT_CMD, 0x2);
		hfc_write_cnfg(ap, HFC_HOST_STAT_CMD, 0x2, (uint64_t)(pci_reg|HFC_HSC_MSS|HFC_HSC_MB));

		if ( (ushort) hfc_read_cnfg (ap, HFC_HOST_VENDER_ID, 0x2) == 0x1054 )	/* FIVE FCWIN-0217 */
		{
			ushort device_id = (ushort) hfc_read_cnfg (ap, HFC_HOST_DEVICE_ID, 0x2);
		
			if ( (device_id == 0x300A)
			  || (device_id == 0x300B)
			  || (device_id == 0x300C)
			  || (device_id == 0x300D) )
			{
				ushort      wkc;
				
				uchar lsi_rev = (uchar)hfc_read_cnfg(ap, 0x4a, 0x01);
				lsi_rev &= 0x07;
				if ( lsi_rev == 0x02 )
				{
				
					// Set registers for FIVE Pass2				/* @@ */

					// Set MCW HFMToCKDSBL
					wkc = (uchar)hfc_read_reg_ext(ap, 0x217, 0x01);
					wkc |= 0x40;
					hfc_write_reg_ext(ap, 0x217, 0x01, wkc);
				
					// Set MCW HMCWBUREQSEL
					wkc = (uchar)hfc_read_reg_ext(ap, 0x229, 0x01);
					wkc |= 0x08;
					hfc_write_reg_ext(ap, 0x229, 0x01, wkc);		/* @@ */
				}
				// Set MCW HMCWBG(0-1)								/* FCLNX-0367 */
				wkc = (uchar)hfc_read_reg_ext(ap, 0x228, 0x01);
				wkc |= 0x30;
				hfc_write_reg_ext(ap, 0x228, 0x01, wkc);			/* FCLNX-0367 */
			}
		}
	}
																	/* @MLPF */

	HFC_EXIT("hfc_pci_conf");
	
    return (0);
    
iospace_error_exit:
        pci_release_regions(pdev);

resource_error_exit:

	return (-ENOMEM);
}

extern int hfc_allocate_memory(struct adap_info *ap, int internal);
extern int hfc_allocate_dma(struct adap_info *ap);
extern int hfc_free_memory(struct adap_info *ap, int internal);


int hfc_attach(struct adap_info *ap)
{
	HFC_ENTRY("hfc_attach");

	/* set adap_info */
	HFC_DBGPRT( "  hfcldd : hfc_attach - init waitqueue head\n"); 
	init_waitqueue_head(&ap->mb_event);
	init_waitqueue_head(&ap->ioctl_event);
	init_waitqueue_head(&ap->ioexe_event);
	init_waitqueue_head(&ap->mck_event);
	init_waitqueue_head(&ap->init_event);																/* FCLNX_007 */

	init_waitqueue_head(&ap->mb_lock_event);
	init_MUTEX(&ap->ioctl_sem) ;
	init_MUTEX(&ap->sem) ;

	/* Initialize spinlock */
	HFC_DBGPRT( "  hfcldd : hfc_attach - init spin_lock\n"); 
	spin_lock_init(&ap->adap_lock);

	/* Allocate table area (Non DMA area)  */
	HFC_DBGPRT( "  hfcldd : hfc_attach - allocate memory area\n"); 
	if ( hfc_allocate_memory(ap, FALSE) ) goto attach_error_exit;

	/* allocate init_table, xob, xrb, mailbox, seg_info, soft_log (DMA area) */

	HFC_DBGPRT( "  hfcldd : hfc_attach - allocate DMA area\n"); 
	if ( hfc_allocate_dma(ap) )    goto attach_error_exit;
	
	HFC_EXIT("hfc_attach");
	return (0);
	
attach_error_exit:
    hfc_free_memory(ap, FALSE);

	HFC_EXIT("hfc_attach");
	return (1);
}


int hfc_allocate_memory(struct adap_info *ap, int internal)
{
	int 	i;

	HFC_EXIT("hfc_allocate_memory");

	HFC_DBGPRT(" hfcldd : hfc_allocate_memory - allocate hardware trace area\n"); 

	if (ap->trc_ptr == NULL) {
		ap->trc_ptr = (struct hfctrace *)kmalloc((sizeof(struct hfctrace) * ap->trc_max),GFP_ATOMIC); /* FCLNX-XX */
		if (ap->trc_ptr == NULL) {
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x3E, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x3E, NULL, 0) ;
			
			goto attachmem_error_exit;
		}
	}
	memset( ap->trc_ptr, 0, (sizeof(struct hfctrace) * ap->trc_max) );
	
	/* get target_scan */
	HFC_DBGPRT(" hfcldd : hfc_allocate_memory - allocate target_scan\n"); 
	if(ap->target_scan == NULL) {
		ap->target_scan = (struct target_scan *)kmalloc( (sizeof(struct target_scan)*MAX_TARGET_PROBE), GFP_ATOMIC);
		if(ap->target_scan == NULL) {
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x4B, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x4B, NULL, 0) ;
			goto attachmem_error_exit;
		}
	}
	memset( ap->target_scan, 0, (sizeof(struct target_scan)*MAX_TARGET_PROBE) );

	/* get bind_target_id */
	HFC_DBGPRT(" hfcldd : hfc_allocate_memory - allocate bind_target_id \n"); 
	if(ap->bind_target_id == NULL) {
		ap->bind_target_id = (struct _bind_target_id *)kmalloc( sizeof(struct _bind_target_id), GFP_ATOMIC);
		if(ap->bind_target_id == NULL) {
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x4C, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x4C, NULL, 0) ;
			goto attachmem_error_exit;
		}
	}
	memset( ap->bind_target_id, 0, sizeof(struct _bind_target_id) );

	/* allocate hfc_pkt */
	HFC_DBGPRT(" hfcldd : hfc_allocate_memory - allocate hfc_pkt \n"); 
	if (ap->pkt_pool[0] == NULL) {															/* FCLNX-0521 */
		for (i=0;i<HFC_PKT_POOL_NUM;i++) {
			ap->pkt_pool[i] = (struct hfc_pkt *)kmalloc( (sizeof(struct hfc_pkt)*HFC_PKT_POOL_SIZE), GFP_ATOMIC);

			if(ap->pkt_pool[i] == NULL) {
				if( ap->raslog_install )
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x4C, NULL, 0) ;
				else
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x4C, NULL, 0) ;
				goto attachmem_error_exit;
			}
			memset( ap->pkt_pool[i], 0, sizeof(struct hfc_pkt)*HFC_PKT_POOL_SIZE );
		}
	}

	/* allocate target_info */
	HFC_DBGPRT(" hfcldd : hfc_allocate_memory - allocate target_info \n"); 
	if (internal != TRUE) {
	    for (i=0;i<ap->max_target;i++) {
			if (ap -> target_arg[i] == NULL) {
				ap -> target_arg[i] = (struct target_info *)kmalloc(sizeof(struct target_info), GFP_ATOMIC);
				if (ap->target_arg[i] == NULL){
					if( ap->raslog_install )
						hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x4B, NULL, 0) ;
					else
						hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x4B, NULL, 0) ;
					goto attachmem_error_exit;
				}
			}
			memset( ap->target_arg[i], 0, sizeof(struct target_info) );
		}
	}

	/* allocate cmnd_num */
	HFC_DBGPRT(" hfcldd : hfc_allocate_memory - allocate scsi_cmnd \n"); 
	ap->cmnd_pool = (struct scsi_cmnd *)kmalloc( (sizeof(struct scsi_cmnd)*ap->cmnd_num), GFP_ATOMIC);
	if(ap->cmnd_pool == NULL){
		if( ap->raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x3C, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x3C, NULL, 0) ;
		goto attachmem_error_exit;
	}
	memset( ap->cmnd_pool, 0, sizeof(struct scsi_cmnd)*ap->cmnd_num );


	HFC_DBGPRT( "  hfcldd : hfc_allocate_memory - allocate DMA area\n"); 

	HFC_EXIT("hfc_allocate_memory");

	return (0); 

attachmem_error_exit:
	hfc_free_memory(ap, FALSE);

	return (1); /* fail */
}



int hfc_allocate_dma(struct adap_info *ap)
{
	int 				an = 0;
	dma_addr_t			bus_addr_table[10];
	void				*vir_addr_table[10];
	int					size_table[10];
	int					err;	
	int				sn = 0; 
	struct pci_dev		*pdev = NULL;
	int 				seg_info_cnt, i, num, cnt;
	uint64_t			work_adr;
	struct seg_info			*seg;	

	HFC_ENTRY("hfc_allocate_dma");

	pdev = ap->pci_cfginf;
	
	/* allocate f/w init table (4096 Bytes) */

	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate fw_init_table \n");
	size_table[an] = sizeof(struct fw_init_tbl);

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->fw_init_p = (struct fw_init_tbl *)vir_addr_table[an];
	err=0x38;
	if (ap->fw_init_p == NULL) { 	/* failed? */
		goto alloc_error_exit;
	}                                                                                         

	HFC_DBGPRT("  (init table) logical=%lx, physical=%lx, size = %d \n",
			(ulong)ap->fw_init_p, (ulong)ap->padr_init, size_table[an]);
			
	memset((char *)ap->fw_init_p, 0, (int)size_table[an]);
	ap->padr_init = bus_addr_table[an]; 
	an++;

	
	/* allocate xob table (4096 * 4 Bytes) */

	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate xob \n");
	size_table[an] = (sizeof(struct xob)) * ap->xob_max;

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->xob = (struct xob *)vir_addr_table[an];
	err=0x39;
	if (ap->xob == NULL) {	/* failed ? */
		goto alloc_error_exit;
	}

	memset((char *)ap->xob, 0, (int)size_table[an]);
	ap->fw_init_p->xob_num = ap->xob_max;
	ap->phys_xob = bus_addr_table[an];

	/* set each page top address to f/w init table */

	cnt=0;
	work_adr = (uint64_t)ap->phys_xob;
	for (i=0;i<(int)(ap->xob_max/(HFC_PAGE_SIZE/sizeof(struct xob)));i++) {
		hfc_write_val( ap->fw_init_p->fw_bus_addr[cnt], work_adr );
		work_adr += HFC_PAGE_SIZE;
		cnt++;
	}

	HFC_DBGPRT("  (xob) logical=%lx, physical=%lx, size = %d \n",
			(ulong)ap->xob, (ulong)ap->phys_xob, size_table[an]);
	an++;
	
	/* allocate for xrb table */

	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate xrb \n");

	size_table[an] = (sizeof(struct xrb)) * ap->xrb_max;

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->xrb = (struct xrb *)vir_addr_table[an];
	err=0x3A;
	if (ap->xrb == NULL) {
		goto alloc_error_exit;
	}

	memset((char *)ap->xrb, 0, (int)size_table[an]);
	ap->fw_init_p->xrb_num = ap->xrb_max;
	ap->phys_xrb = bus_addr_table[an];


	/* set each page top address to f/w init table */
	
	work_adr = (uint64_t)ap->phys_xrb;
	for (i=0;i<(int)(ap->xrb_max/(HFC_PAGE_SIZE/sizeof(struct xrb)));i++) {
		hfc_write_val( ap->fw_init_p->fw_bus_addr[cnt], work_adr );
		work_adr += HFC_PAGE_SIZE;
		cnt++;
	}

	HFC_DBGPRT("  (xrb) logical=%lx, physical=%lx, size = %d \n",
			(ulong)ap->xrb, (ulong)ap->phys_xrb, size_table[an]);
	an++;

	/* allocate mailbox */

	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate mailbox \n");
	size_table[an] = sizeof(struct mailbox);

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->mb = (struct mailbox *)vir_addr_table[an];
	err=0x3B;
	if (ap->mb == NULL) {
		goto alloc_error_exit;
	}

	memset((char *)ap->mb, 0, (int)size_table[an]);
	ap->phys_mb = bus_addr_table[an];

	HFC_DBGPRT("  (mailbox) logical=%lx, physical=%lx, size = %d \n",
			(ulong)ap->mb, (ulong)ap->phys_mb, size_table[an]);
	an++;

	/* allocate mailbox parameter */
	
 	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate mailbox parameter \n");
	size_table[an] = 4096 * 2;

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->mb_parm = (uchar *)vir_addr_table[an];
	err=0x3C;
	if (ap->mb_parm == NULL) {
		goto alloc_error_exit;
	}
	ap->phys_mb_parm = bus_addr_table[an];
	memset((char *)ap->mb_parm, 0, 4096 * 2);	
	HFC_DBGPRT("  (init table) logical=%lx, physical=%lx, size = %d \n",
			(ulong)ap->xob, (ulong)ap->phys_xob, size_table[an]);
	an++;

	/* allocate soft log area */

 	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate soft log area \n");
	for (i=0; i<ap->slog_max ;i++) {									/* FCWIN_0013 */

		ap->slog_addr[i] = (uchar *)dma_alloc_coherent(&pdev->dev, (uint)HFC_PAGE_SIZE, &ap->phys_slog[i],GFP_ATOMIC);

		if (ap->slog_addr[i] == NULL) {
			err = 0x3D;
			goto alloc_error_exit;
		}		
		memset((char *)ap->slog_addr[i], 0, HFC_PAGE_SIZE);		
		hfc_write_val( ap->fw_init_p->fw_bus_addr[cnt], (uint64_t)ap->phys_slog[i] );

		HFC_DBGPRT("  (soft log area [%d]) logical=%lx, physical=%lx\n",
		i, (ulong)ap->slog_addr[i], (ulong)ap->phys_slog[i]);
		
		cnt++;
	}
	sn=i;
	ap->slog = (uchar *)ap->slog_addr[0];
	
	
	HFC_DBGPRT("  (soft log area) logical=%lx, physical=%lx, size = %d \n",
		(ulong)ap->slog, (ulong)ap->phys_slog[i], HFC_PAGE_SIZE);
	
	/* allocate seg_info */

 	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - allocate seg_info \n");
	seg_info_cnt = (ap->dma_max / HFC_PAGE_SIZE) +
         			((ap->dma_max/ HFC_PAGE_SIZE) /  (HFC_PAGE_SIZE / sizeof(struct seg_info)));
	size_table[an] =	HFC_ALIGNPG (seg_info_cnt * sizeof(struct seg_info) );

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->seg_info = (struct seg_info *)vir_addr_table[an];
	err=0x55;
	if (ap->seg_info == NULL) {
		goto alloc_error_exit;
	}
	
	memset((char *)ap->seg_info, 0, (int)size_table[an]);
	ap->seg_phys_addr =bus_addr_table[an];

	HFC_DBGPRT("  (seg_info) logical=%lx, physical=%lx, size = %d seg_info_cnt=%d \n",
			(ulong)ap->seg_info, (ulong)ap->seg_phys_addr, size_table[an], seg_info_cnt);

	an++;

	/* allocate iov_map */

	size_table[an] = (ap->dma_max/HFC_PAGE_SIZE/8); 

	vir_addr_table[an] = dma_alloc_coherent(&pdev->dev, (uint)size_table[an], &bus_addr_table[an],GFP_ATOMIC);

	ap->iov_map = (uint *)vir_addr_table[an];
	err=0x56;
	if (ap->iov_map == NULL) {
			goto alloc_error_exit;
	}
	memset((char *)ap->iov_map, 0, (int)size_table[an]); /* clear iov_map */
	ap->phys_iov = bus_addr_table[an];
	HFC_DBGPRT(" (iov_map) logical = %lx, physical = %lx, size = %d cnt = %d\n",
		(ulong)ap->iov_map, (ulong)bus_addr_table[an], size_table[an], ap->iov_map_cnt);

	an++;

	/* initialize iov_map */

 	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - initialize iov_map \n");
	for ( i = 0; i < ap->iov_map_cnt; i++ ){
		ap->iov_map[i/32] |= (0x80000000 >> (i%32));
	}

	/* initialize seg_info */

 	HFC_DBGPRT( "  hfcldd : hfc_alloc_dma - initialize seg_info \n");
	num = HFC_PAGE_SIZE / sizeof(struct seg_info);
	cnt = ap->iov_map_cnt + (ap->iov_map_cnt / num);
	cnt /= num;

	HFC_DBGPRT( "hfcldd : seg_info  cnt = %d, num = %d \n", cnt, num);

	for(i=1;i<=cnt;i++) {
		seg = &ap->seg_info[(i*num)-1]; 
		seg->seg_len = HFC_SEG_LEN_F;		/* FCWIN-0079 */
		hfc_write_val(seg->seg_addr, (uint64_t)(ap->seg_phys_addr + (i * HFC_PAGE_SIZE)));
	}

	HFC_EXIT("hfc_allocate_dma");

	return (0); /* Normal end */

	
alloc_error_exit:
 	HFC_DBGPRT(" hfcldd : hfc_alloc_dma - exit with error (alloc_error_exit) \n");
	if( ap->raslog_install )
		hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, err, NULL, 0) ;
	else
		hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, err, NULL, 0) ;
	hfc_free_dma(ap);
	HFC_EXIT("hfc_allocate_dma (error) ");
	return (1);

}

void hfc_free_mpap(struct adap_info *ap)
{
	struct mp_adap_info *mpap;

	HFC_ENTRY("hfc_free_mpap");

	/* Remove adap_info from mp_adap_info */
	mpap = ap->mp_adap_info;

	if(mpap == NULL) return;
	
	
	HFC_ADAP_LOCK(mpap,HFC_MP_ADAP_BUSY);
	
	if ( mpap->port_cnt == 1) {
		mpap->ap = NULL;
		mpap->port_cnt--;

		/* Remove mp_adap_info from hfc_manage_info */
		if ( hfc_manage_info.mp_adap_cnt == 1) {
			if(hfc_manage_info.hfcldd_mp_mod) {
				hfc_manage_info.npubp->hfc_free_mp_table();
			}
			hfc_manage_info.mp_adap_info = NULL;
		}
		else {
			if (hfc_manage_info.mp_adap_info == mpap) {
				hfc_manage_info.mp_adap_info = mpap->next;
			}
			else {
				mpap->prev->next = mpap->next;
			}
		}

		hfc_manage_info.mp_adap_cnt--;
		kfree(mpap->hw_log);
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
		kfree(mpap);
	}
	else {
		if (mpap->ap == ap)
			mpap->ap = ap->next;
		else
			ap->prev->next = ap->next;

		mpap->port_cnt--;

		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
	}
	

}


int hfc_free_memory(struct adap_info *ap, int internal)
{
	int i;
	struct target_info *target;
	
	/* Release trace, bind_target_id, hfc_pkt, target_id */
	if (ap->cmnd_pool != NULL) {
		memset( ap->cmnd_pool, 0, sizeof(struct scsi_cmnd)*ap->cmnd_num );
		kfree(ap->cmnd_pool);
		ap->cmnd_pool = NULL;
	}

	if (internal != TRUE) {
		for (i=0;i<MAX_TARGET_PROBE;i++) {
			
			target = ap -> target_arg[i];
			if (target != NULL) {
				if (hfc_manage_info.hfcldd_mp_mod) {
					hfc_manage_info.npubp->hfc_free_dev(target);
				}
				
				memset( ap -> target_arg[i], 0, sizeof(struct target_info) );
				kfree(ap->target_arg[i]);
				ap->target_arg[i] = NULL;
			}
		}
	}
	
	for (i=0;i<HFC_PKT_POOL_NUM;i++) {			/* FCLNX-0521 */
		if (ap->pkt_pool[i] != NULL) {
			memset( ap->pkt_pool[i], 0, sizeof(struct hfc_pkt)*HFC_PKT_POOL_SIZE );
			kfree(ap->pkt_pool[i]);
			ap->pkt_pool[i] = NULL;
		}
	}
	
	if (ap->bind_target_id != NULL) {
		memset(ap->bind_target_id, 0, sizeof(struct _bind_target_id) );
		kfree(ap->bind_target_id);
		ap->bind_target_id = NULL;
	}
	
	if (ap->target_scan != NULL) {
		memset(ap->target_scan, 0, (sizeof(struct target_scan)*MAX_TARGET_PROBE) );
		kfree(ap->target_scan);
		ap->target_scan = NULL;
	}
	
	if (ap->trc_ptr != NULL) {
		memset(ap->trc_ptr, 0, (sizeof(struct hfctrace) * ap->trc_max) );
		kfree(ap->trc_ptr);
		ap->trc_ptr = NULL;
	}
	
	return(0);
}


int hfc_free_dma(struct adap_info *ap)
{
	struct pci_dev		*pdev = NULL;
	int 				i;
	int 				seg_info_cnt;

	size_t size;

	pdev = ap->pci_cfginf;

	HFC_ENTRY("hfc_free_dma");	

	if ( ap->fw_init_p != NULL ) {
		/* Release f/w init table (4096 Bytes) */
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free fw_init_tbl \n");
		memset((char *)ap->fw_init_p, 0, (int)sizeof(struct fw_init_tbl));

		dma_free_coherent(&pdev->dev, (size_t)sizeof(struct fw_init_tbl),(void *)ap->fw_init_p, (dma_addr_t)ap->padr_init);

		ap->fw_init_p = NULL;
	}
	
	if (ap->xob != NULL) {
		/* Release xob table (4096 * 4 Bytes) */
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free xob_tbl \n");
		memset((char *)ap->xob, 0, (int)(sizeof(struct xob) * ap->xob_max));

		dma_free_coherent(&pdev->dev, (size_t)(sizeof(struct xob) * ap->xob_max),(void *)ap->xob, (dma_addr_t)ap->phys_xob);

		ap->xob = NULL;
	}
	
	if (ap->xrb != NULL) {
		/* Release xrb table */
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free xrb tbl \n");
		memset((char *)ap->xrb, 0, (int)(sizeof(struct xrb) * ap->xrb_max));

		dma_free_coherent(&pdev->dev, (size_t)(sizeof(struct xrb) * ap->xrb_max),(void *)ap->xrb, (dma_addr_t)ap->phys_xrb);

		ap->xrb = NULL;
	}
	
	if (ap->mb != NULL) {
		/* Release mailbox */
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free fw_mailbox \n");
		memset((char *)ap->mb, 0, (int)sizeof(struct mailbox));

		dma_free_coherent(&pdev->dev, (size_t)sizeof(struct mailbox), (void *)ap->mb, (dma_addr_t)ap->phys_mb);

		ap->mb = NULL;
	}
	
	if (ap->mb_parm != NULL) {
		/* Release mailbox parameter */
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free mailbox parameter \n");
		memset((char *)ap->mb_parm, 0, (int)(4096 * 2) );

		dma_free_coherent(&pdev->dev, (size_t)(4096 * 2), (void *)ap->mb_parm, (dma_addr_t)ap->phys_mb_parm);

		ap->mb_parm = NULL;
	}
	
	/* Release soft log area */
	HFC_DBGPRT(" hfcldd : hfc_free_dma - free soft log area \n");
	for (i=0; i<ap->slog_max; i++) { 													/* FCLNX_0013 */
		if (ap->slog_addr[i] != NULL) {
			memset((char *)ap->slog_addr[i], 0, (int)HFC_PAGE_SIZE );

			dma_free_coherent(&pdev->dev, (size_t)HFC_PAGE_SIZE, (void *)ap->slog_addr[i], (dma_addr_t)ap->phys_slog[i] );

			ap->slog_addr[i] = NULL;
		}
	}

	if (ap->seg_info != NULL) {
		/* Release seg_info */
		seg_info_cnt = (ap->dma_max / HFC_PAGE_SIZE) +
    	     			((ap->dma_max/ HFC_PAGE_SIZE) /  (HFC_PAGE_SIZE / sizeof(struct seg_info)));
		size =	HFC_ALIGNPG (seg_info_cnt * sizeof(struct seg_info) );
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free seg_info \n");
		memset((char *)ap->seg_info, 0, (int)size);

		dma_free_coherent(&pdev->dev, (size_t)size, (void *)ap->seg_info, (dma_addr_t)ap->seg_phys_addr);

		ap->seg_info = NULL;
	}
	
	if (ap->iov_map != NULL) {
		/* Release iov_map */
		size = ap->dma_max/HFC_PAGE_SIZE/8;
		HFC_DBGPRT(" hfcldd : hfc_free_dma - free iov_map \n");
		memset((char *)ap->iov_map, 0, (int)size );

		dma_free_coherent(&pdev->dev, (size_t)size, (void *)ap->iov_map, (dma_addr_t)ap->phys_iov);

		ap->iov_map = NULL;
	}

	HFC_EXIT("hfc_free_dma");

	return (0); /* Normal end */
}


void hfc_detach(struct adap_info *ap)
{
	/* Release DMA memory area */
	hfc_free_dma(ap);
	
	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
		hfc_manage_info.npubp->hfc_remove_lgpath(ap);
	}
	
	/* Release adapter memory area */
	hfc_free_memory(ap, FALSE);
}

void hfc_set_fw_init_tbl(struct adap_info *ap)
{
	int		i;
	uchar			pre_conf=0;

	HFC_DBGPRT( "hfcldd : set_fw_init_tbl - HFC_ENTRY \n");
	HFC_ENTRY("hfc_set_hw_init_tbl");
	
	/* +00-01 max exchange */
	hfc_write_val( ap->fw_init_p->max_exchange, MAX_EXCHANGE);

	/* +02-03 max port */
	hfc_write_val( ap->fw_init_p->max_port,     ap->max_target);

	/* +04 revision	*/
	hfc_write_val( ap->fw_init_p->init_tbl_rev, 0);
	hfc_write_val( ap->fw_init_p->dd_support_info, HFC_DDSP_OPTERR9E);		/* FCLNX-0273 */

	if ( ap->pkg.type == HFC_PKTYPE_FPP ) 		pre_conf = (uchar) hfc_read_cnfg (ap, 0x4B, 0x1);
	else if( ap->pkg.type == HFC_PKTYPE_FIVE )	pre_conf = (uchar) hfc_read_cnfg (ap, 0xAF, 0x1);

	
	hfc_write_val( ap->fw_init_p->init_tbl_rev, 1);
	
	hfc_write_val( ap->fw_init_p->fw_rnid.n_port_name, ap->ww_name);
	hfc_write_val( ap->fw_init_p->fw_rnid.node_name, ap->node_name);
																			/* FCLNX-0299 */	


	/* +18-1F pointer to Mailbox top */
	hfc_write_val( ap->fw_init_p->mb_addr, (uint64_t)ap->phys_mb);
	
	/* +20-21 XOB Queue HFC_ENTRY number */
	hfc_write_val( ap->fw_init_p->xob_num,  ap->xob_max);
	/* +24-25 XRB Queue HFC_ENTRY number */
	hfc_write_val( ap->fw_init_p->xrb_num,  ap->xrb_max);
	/* +28-29 Soft Log page number */
	hfc_write_val( ap->fw_init_p->slog_num, (ap->slog_max / (HFC_PAGE_SIZE / HFC_SLOG_LEN)));
	/* +2A-2B Soft Log HFC_ENTRY length */
	hfc_write_val( ap->fw_init_p->slog_len, HFC_SLOG_LEN);

	/* +180 trace information  */
	hfc_write_val( ap->fw_init_p->trc_info.trc_num, 0x01 );
	hfc_write_val( ap->fw_init_p->trc_info.inst,    0x04 );

	for (i=0;i<4;i++) {
		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].seg_no,        i );
		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].resv0,         0x00 );

		if ((i==0) && !(ap->fw_parm & HFC_FWP_SEGTRC_V))														/* FCWIN-0192 */
				hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].mode, 	 0x60 );	/* FCWIN-0192 */
		else
				hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].mode, 	 0xf8 );

		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].ctl,           0x00 );
		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].top_port_id,   0x00000000 );
		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].bottom_port_id,0x00ffffff );
		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[i].resv1,         0x00000000 );
	}
	hfc_write_val( ap->fw_init_p->trc_info.trc_seg[0].seg_no, 0x80 );	

	HFC_EXIT("hfc_set_hw_init_tbl");



}

#define HFC_VPD_WAITCNT 			5000000
#define CHECKSUM_FPP_ADD_ADDR		0x55
#define CHECKSUM_FIVE_ADD_ADDR		 0x63
#define VPDDATA_LEN                 0x80
#define VPD_ADDRESS_FLAG_MASK       0x80
#define HFC_CAPABILITIES_POINTER    0x34
#define HFC_CAPABILITIES_ADDR       0x40


int hfc_get_vpd(struct adap_info *ap, uchar *vpd_buf)
{

	char  *vpd_ptr;
	uchar capabilities_list_addr = 0;
	uint i=0;
	ushort offset=0,adr=0;
	uchar sum=0;
	uint  nwords=0;
	uint value,value_bak;
	int err;
	uint chk_sum = CHECKSUM_FPP_ADD_ADDR;
	
	HFC_ENTRY("hfc_get_vpd");

	if  ( (!(HFC_MMODE_CHECK_SHARED(ap)) || HFC_MMODE_CHECK_SHADOW(ap) ) )
	{
		/* Read status register */
		value = (uint) hfc_read_cnfg(ap, HFC_HOST_STAT_CMD, 4);
		HFC_4B_TO_4L(value_bak,value);

		if (!(value & 0x00100000)) {	/* bit 4(capabilities list) = 0? */
			/* VPD is not supported */
			err = 0x71;
			goto getvpd_error_exit;
		}

		/* Read capabilities list(0x34) */
		value = (uint) hfc_read_cnfg(ap, HFC_CAPABILITIES_POINTER, 4 );
		HFC_4B_TO_4L(value_bak,value);

		if ((value & 0x000000FF) != HFC_CAPABILITIES_ADDR) {
			/* Abnormal end (data is not 0x40) */
			err = 0x72;
			goto getvpd_error_exit;
		}

		capabilities_list_addr = (value & 0xFF);

		/* Read VPD data */
		value = (uint) hfc_read_cnfg(ap, capabilities_list_addr, 4);
		HFC_4B_TO_4L(value_bak,value);

		/* ID != VPD(3)? */
		if ((value & 0x000000FF) != 3) {
			err = 0x73;
			goto getvpd_error_exit;
		}

		vpd_ptr = (char *)vpd_buf; 
		offset = 0;
		do {
			value = 0;
			adr = 0x0000 | offset;
			hfc_write_cnfg(ap, (capabilities_list_addr + 0x2), 2, adr);

			i = 0;
			while (i != HFC_VPD_WAITCNT) {
				value = (uint) hfc_read_cnfg(ap, capabilities_list_addr, 4);
				HFC_4B_TO_4L(value_bak,value);

				if (value & 0x80000000) {
					break;
				}

			    udelay(10);
				i++;
			}
			if (i == HFC_VPD_WAITCNT) {
				/* Flag has not changed */
				err = 0x74;
				goto getvpd_error_exit;
			}

			value = 0;
			value = (uint) hfc_read_cnfg(ap, (capabilities_list_addr + 0x4), 4);
			HFC_4B_TO_4L(value_bak,value);

			/* Store data in reverse order */
			*vpd_ptr++ = value         & 0xFF;
			*vpd_ptr++ = (value >> 8)  & 0xFF;
			*vpd_ptr++ = (value >> 16) & 0xFF;
			*vpd_ptr++ = (value >> 24) & 0xFF;
			
			offset += 4;
		} while (offset != VPDDATA_LEN);

		/* Caliculate checksum */			  
		if ( ap->pkg.type == HFC_PKTYPE_FPP )
		{
			chk_sum = CHECKSUM_FPP_ADD_ADDR;
		}
		else if( ap->pkg.type == HFC_PKTYPE_FIVE )
		{
			chk_sum = CHECKSUM_FIVE_ADD_ADDR;
		}
		
		vpd_ptr = (char *)vpd_buf;
		for (sum = 0, nwords = 0; nwords <= chk_sum; nwords++) {
			sum += *vpd_ptr++;
		}

		/* Checksum = 0? */
		if (sum != 0) {
			err = 0x75;
			goto getvpd_error_exit;
		}
	}
	else /* This case is considered for Guest LPAR */
	{
		hfc_mlpf_get_mmio_hg(ap, ap->mp_adap_info->vpd_buf, HFC_IOHGSPC_VPDAREA, HFC_IOHGSPC_VPDAREA_LEN);
	}

	HFC_EXIT("hfc_get_vpd");

	return(0);

getvpd_error_exit:
	if( ap->raslog_install )
		hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERRF, err, NULL, 0) ;
	else
		hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERRF, err, NULL, 0) ;

	/* Issue initial reset command */
	hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDRES,( char )0x1,(char)0x04);

	/* Wait 2ms */
	mdelay(2);

	/* Force link-status into disable state */
	if ( ap->pkg.type == HFC_PKTYPE_FPP )
	{
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDFCIF,( char )0x1,(char)0x80);
		hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0x00000000 );
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDLED,( char )0x1,HFC_WAKE_UP_FAILURE);
	}
	else if( ap->pkg.type == HFC_PKTYPE_FIVE )
	{
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDFCIF,( char )0x4,(char)0x80808080);
		hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0x00000000 );
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDLED,( char )0x4,HFC_WAKE_UP_FAILURE_FIVE);
	}

	return (1);

}


int hfc_config_hw_set(struct adap_info *ap, uint retry_maxcnt)
{
	uchar		stat_chkerrflg;
	uchar		post_chkerrflg;
	uint		retry_cnt;
	uchar		dmp_data[32];
	uint		errlog_data[4];
	union {
		uint	l;
		ushort	s[2];
		uchar	c[4];
	} value;

	uchar		err_code = 0;
	uint 		status_bak = 0;
	ushort		result;

	HFC_ENTRY("hfc_config_hw_set");


	memset(dmp_data, 0, sizeof(dmp_data));
    memset(errlog_data, 0, sizeof(errlog_data));

	/* Set 0x00 to CMD_RES (0x40) */
    hfc_write_reg(ap, HFC_IOSPACE_CMDRES, 0x1, 0x00);

	mdelay(250); /* 1ms wait */									/* FCLNX_0012 */


	for (retry_cnt = 0; retry_cnt < (retry_maxcnt + 1); retry_cnt++) {

		HFC_DBGPRT( "config_hw_set - retry loop = %d\n", retry_cnt);

		/* Initialize */
		stat_chkerrflg = 0;
		post_chkerrflg = 0;
		err_code = 0;
		
		value.l = 0;
		status_bak = 0;
		/* Read status information (4 bytes) */
		status_bak = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS0, 0x4 );
		HFC_4B_TO_4L(value.l,status_bak);
		
		HFC_DBGPRT( "hfc_read_reg HFC_IOSPACE_STATUS[%x] = 0x%08x.\n",
					HFC_IOSPACE_STATUS0, status_bak );

		if (status_bak != 0x80000000) { /* status is normal? */

			HFC_DBGPRT( "config_hw_set - firmware status error\n");
		
			/* Still executing reset process? */
			if ((value.c[0] & HFC_PCI_RESETCHK) != 0x00) {
				
				HFC_DBGPRT( "config_hw_set - wakeup failure (HFC_PCI_RESETCHK) \n");

				/* Set LED to Wake up Falure */
				hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
				hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_WAKE_UP_FAILURE);
				stat_chkerrflg = 1;
				err_code = 1;
			}
			else {
				/* Detect PCI error? */
				if ((value.c[1] & HFC_PCI_PCIERR_DETECTED) == HFC_PCI_PCIERR_DETECTED) {
					
					/* Set LED to Wake up Falure */
					
					HFC_DBGPRT( "config_hw_set - pci error was detected (HFC_PCI_PCIERR_DETECTED) \n");	

					hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
					hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_WAKE_UP_FAILURE);
					stat_chkerrflg = 1;
					err_code = 2;
				}
				else {
					/* EXGMCK and BOOTRUN check ? */
					if ((value.c[1] & HFC_PCI_PCICHK) == HFC_PCI_PCICHK) {
						
						/* Set LED to Wake up Falure */
						HFC_DBGPRT( "config_hw_set - pci check was detected (HFC_PCI_PCICHK) \n");	

						hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
						hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_WAKE_UP_FAILURE);
						stat_chkerrflg = 1;
						err_code = 3;
					}
					else {
						/* EXGMCK check ? */
						if ((value.c[1] & HFC_PCI_EXGMCK) == HFC_PCI_EXGMCK) {

							/* Set LED to POST up Falure */
							HFC_DBGPRT( "config_hw_set - exigent machine check was detected (HFC_PCI_EXGMCK) \n");	

							hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
							hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_POST_FAILURE);
							stat_chkerrflg = 1;
							err_code = 4;
						}
						else {
							/* BOOTRUN check? */
							if ((value.c[1] & HFC_PCI_BOOTRUN) == HFC_PCI_BOOTRUN) {
								
								HFC_DBGPRT( "config_hw_set - bootrun check was detected (HFC_PCI_BOOTRUN) \n");	
								
								hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
								hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_WAKE_UP_FAILURE);
								stat_chkerrflg = 1;
								err_code = 5;
							}
							else {
								/* FUNC_STOP check? */
								if ((value.c[1] & HFC_PCI_FCNSTOP) == HFC_PCI_FCNSTOP) {

									HFC_DBGPRT( "config_hw_set - function stop was detected (HFC_PCI_FCNSTOP) \n");	
			
									hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
									hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_WAKE_UP_FAILURE);
									stat_chkerrflg = 1;
									err_code = 6;
								}
								else {
									/* CH not Ready check? */
									if ((value.c[2] & HFC_PCI_CH_NOT_READY) == HFC_PCI_CH_NOT_READY) {

										/* Set LED to POST up Falure */

										HFC_DBGPRT( "config_hw_set - CH was not ready (HFC_PCI_CH_NOT_READY) \n");	

										hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
										hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_POST_FAILURE);
										stat_chkerrflg = 1;
										err_code = 7;
									}
								}
							}
						}
					}
				}
			}
		}
		/* Normal case */
		if (stat_chkerrflg == 0) {
			
			/* Read first 2bytes of POST result */
			result = (ushort) hfc_read_reg(ap, HFC_IOSPACE_CA_POSTRESULT, 0x2);

			HFC_DBGPRT( "config_hw_set - Port result = %02x \n", result);
			
			if ((result & 0xff00) == 0x8000) { /* Is first byte 0x80? */
				
				if ((result & 0x00ff) == 0x00FF) { /* Normal end? */
					
					HFC_DBGPRT( "config_hw_set - Port status is normal \n");
					break;
				} else {							/* Otherwise error */
					post_chkerrflg = 1;
					err_code = 9;
				}
			}
			else {
				/* Set LED to POST up Falure */
				HFC_DBGPRT( "config_hw_set - Port status is not normal \n");

				hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
				hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x1, HFC_POST_FAILURE);
				post_chkerrflg = 1;
				err_code = 8;
			}
		}

		HFC_DBGPRT( "config_hw_set() - err_code_2 = 0x%x.",err_code);
		if(err_code != 0){ 
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x35, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x35, NULL, 0) ;
		}			
		/* Retry count reaches retry maxcnt */
		if (retry_cnt == retry_maxcnt) {
			
			/* Status information error? */
			if (stat_chkerrflg == 1) {
				HFC_DBGPRT( "config_hw_set() - stat_chkerrflg=1, errcode = 0x%x.",err_code );

				memset(dmp_data, 0, sizeof(dmp_data));
				memset(errlog_data, 0, sizeof(errlog_data));

				/* Collect error log information (16 bytes from STATUS) */
				errlog_data[0] = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS0, 0x4);
				errlog_data[1] = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS1, 0x4);
				errlog_data[2] = (uint) hfc_read_reg(ap, HFC_IOSPACE_ERRDETAIL0, 0x4);
				errlog_data[3] = (uint) hfc_read_reg(ap, HFC_IOSPACE_ERRDETAIL1, 0x4);

				/* Create error log data */
				dmp_data[0]  = (uchar)(errlog_data[0] >> 24);
				dmp_data[1]  = (uchar)(errlog_data[0] >> 16);
				dmp_data[2]  = (uchar)(errlog_data[0] >> 8 );
				dmp_data[3]  = (uchar)(errlog_data[0]      );
				dmp_data[4]  = (uchar)(errlog_data[1] >> 24);
				dmp_data[5]  = (uchar)(errlog_data[1] >> 16);
				dmp_data[6]  = (uchar)(errlog_data[1] >> 8 );
				dmp_data[7]  = (uchar)(errlog_data[1]      );
				dmp_data[8]  = (uchar)(errlog_data[2] >> 24);
				dmp_data[9]  = (uchar)(errlog_data[2] >> 16);
				dmp_data[10] = (uchar)(errlog_data[2] >> 8 );
				dmp_data[11] = (uchar)(errlog_data[2]      );
				dmp_data[12] = (uchar)(errlog_data[3] >> 24);
				dmp_data[13] = (uchar)(errlog_data[3] >> 16);
				dmp_data[14] = (uchar)(errlog_data[3] >> 8 );
				dmp_data[15] = (uchar)(errlog_data[3]      );

				
				hfc_logout(ap, (uint)0x35, HFC_ERRLOG_TYPE_IMLLOG);
				
				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_IMLLOG, ERRID_HFCP_ERRF, 0x35, dmp_data, 16) ;
			}
			/* Post-check error? */
			else if (post_chkerrflg == 1) {
				HFC_DBGPRT(  "config_hw_set() - post_chkerrflg=1, errcode = 0x%x.", err_code );

				memset(dmp_data, 0, sizeof(dmp_data));
				memset(errlog_data, 0, sizeof(errlog_data));

				/* Read POST information */
				errlog_data[0] = (uint) hfc_read_reg(ap, HFC_IOSPACE_CA_POSTRESULT, 4);

				/* Create error log data */
				dmp_data[0]  = (uchar)(errlog_data[0] >> 24) ;
				dmp_data[1]  = (uchar)(errlog_data[0] >> 16) ;
				dmp_data[2]  = (uchar)(errlog_data[0] >> 8 ) ;
				dmp_data[3]  = (uchar)(errlog_data[0]	   ) ;

				
				hfc_logout(ap, (uint)0x36, HFC_ERRLOG_TYPE_IMLLOG);
				
				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_IMLLOG, ERRID_HFCP_ERRF, 0x36, dmp_data, 16) ;
			}

			/* Abnormal End */
			return(-1);
		}
		/* Forced MCK */
		hfc_write_reg(ap, HFC_IOSPACE_CMDCTL, 0x1, 0x08);

		/* CTLRES */
		hfc_write_reg(ap, HFC_IOSPACE_CMDRES, 0x1, 0x02);

		/* Wait 1ms*/
		mdelay(1);
		
		/* Initiate reboot */
		hfc_write_reg(ap, HFC_IOSPACE_CMDBOOT, 0x1, 0x40);

		/* Wait 500ms */
		mdelay(500);	/* FCLNX_0012 */
    }

	HFC_EXIT("hfc_config_hw_set");

    return(0);
}

 
int hfc_config_hw_set_five(struct adap_info *ap, uint retry_maxcnt)
{
	uchar		stat_chkerrflg;
	uchar		post_chkerrflg;
	uint		retry_cnt;
	uchar		dmp_data[32];
	uint		errlog_data[4];
	union {
		uint	l;
		ushort	s[2];
		uchar	c[4];
	} value;

	uchar		err_code = 0;
	uint 		status_bak = 0;
	uchar       exe_reset = 0;

	HFC_ENTRY("hfc_config_hw_set_five");
	
	/* Initialize */
	memset(dmp_data, 0, sizeof(dmp_data));
	memset(errlog_data, 0, sizeof(errlog_data));

	/* Set 0x00 to CMD_RES (0x30) */
	hfc_write_reg(ap, HFC_IOSPACE_CMDRES, 0x1, 0x00);

	mdelay(1000); /* Wait 1000ms */

//	if ( ( hfc_manage_info.hfcplus_enable ) && 							/* FCLNX-GPL-394 */
	if ( ( hfc_manage_info.hfcldd_mp_mod) &&
		 ( ap->isol_force == HFC_CHKSTP_FRC_ISOL )){ /* - FCLNX-546 - *//* FCLNX-GPL-147 */ /* FCLNX-GPL-349 */
		/* Stop optical transmission */
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDFCIF,( char )0x4,(char)0x80808080);
		/* Turn LED (Yellow and Green) off */
		hfc_write_reg(ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x4,HFC_WAKE_UP_FAILURE_FIVE);
		return 0;
	} /* FCLNX-GPL-147 */												/* FCLNX-GPL-394 */

//skip_cmd_res : 

	for (retry_cnt = 0; retry_cnt < (retry_maxcnt + 1); retry_cnt++) {

		stat_chkerrflg = 0;
		post_chkerrflg = 0;
		err_code = 0;
		value.l = 0;
		status_bak = 0;
		
		/* Read status information (4 bytes) */
		status_bak = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS0, 0x4 );
		HFC_4B_TO_4L(value.l,status_bak);

		HFC_DBGPRT( "hfc_read_reg HFC_IOSPACE_STATUS[%x] = 0x%08x.",
					HFC_IOSPACE_STATUS0, status_bak);
		
		if (status_bak != 0x80000000) { /* status is normal? */

			/* Still executing reset process? */
			if ((value.c[0] & HFC_PCI_RESETCHK) != 0x00) {

				hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_WAKE_UP_FAILURE_FIVE);
				stat_chkerrflg = 1;
				err_code = 1;
			}
			else {
				/* Detect PCI error? */
				if (((value.c[1] & HFC_PCI_PCIERR_DETECTED) == HFC_PCI_PCIERR_DETECTED) && ( exe_reset != 1)){

					/* PCI error status reset */                      /*  FCWIN-0223 */
					hfc_write_cnfg(ap, 0x06, 0x2, 0xffff);
					hfc_write_cnfg(ap, 0x6c, 0x4, 0xffffffff);
					hfc_write_cnfg(ap, 0x70, 0x4, 0x0fffffff);
					exe_reset = 1;                                      /*  FCWIN-0223 */

					/* Set LED to Wake up Falure */
					hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_WAKE_UP_FAILURE_FIVE);
					stat_chkerrflg = 1;
					err_code = 2;
				}
				else {
					/* EXGMCK and BOOTRUN check ? */
					if ((value.c[1] & HFC_PCI_PCICHK) == HFC_PCI_PCICHK) {

						/* Set LED to Wake up Falure */
						hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_WAKE_UP_FAILURE_FIVE);
						stat_chkerrflg = 1;
						err_code = 3;
					}
					else {
						/* EXGMCK check */
						if ((value.c[1] & HFC_PCI_EXGMCK) == HFC_PCI_EXGMCK) {

							/* Set LED to POST up Falure */
							hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_POST_FAILURE_FIVE);
							stat_chkerrflg = 1;
							err_code = 4;
						}
						else {
							/* BOOTRUN check */
							if ((value.c[1] & HFC_PCI_BOOTRUN) == HFC_PCI_BOOTRUN) {

								/* Set LED to Wake up Falure */
								hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_WAKE_UP_FAILURE_FIVE);
								stat_chkerrflg = 1;
								err_code = 5;
							}
							else {
								/* FUNC_STOP check */
								if ((value.c[1] & HFC_PCI_FCNSTOP) == HFC_PCI_FCNSTOP) {

									/* Set LED to Wake up Falure */
									hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_WAKE_UP_FAILURE_FIVE);
									stat_chkerrflg = 1;
									err_code = 6;
								}
								else {
									/* CH not Ready check */
									if ((value.c[2] & HFC_PCI_CH_NOT_READY) == HFC_PCI_CH_NOT_READY) {

										/* Set LED to POST up Falure */
										hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_POST_FAILURE_FIVE);
										stat_chkerrflg = 1;
										err_code = 7;
									}
								}
							}
						}
					}
				}
			}
		}
		/* Normal case */
		if (stat_chkerrflg == 0) {
			ushort result;

			/* Read first 2bytes of POST result */
			result = (ushort) hfc_read_reg(ap, HFC_IOSPACE_CA_POSTRESULT, 0x2);
			if ((result & 0xff00) == 0x8000) {/* Is first byte 0x80? */

				if ((result & 0x00ff) == 0x00FF) {/* Normal end? */

					break;
				} else { /* Otherwise error */
					post_chkerrflg = 1;
					err_code = 9;
				}
			}
			else {

				/* Set LED to POST up Falure */
				hfc_write_reg(ap, HFC_IOSPACE_CMDLED, 0x4, HFC_POST_FAILURE_FIVE);
				post_chkerrflg = 1;
				err_code = 8;
			}
		}

		HFC_DBGPRT(  "config_hw_set_five() - err_code_2 = 0x%x.",err_code);

		/* Retry count reaches retry maxcnt */
		if (retry_cnt == retry_maxcnt) {

			/* Status information error? */
			if (stat_chkerrflg == 1) {
				HFC_DBGPRT(  "CONFIG_HW_set_five() - stat_chkerrflg=1, errcode = 0x%x.",err_code );

				memset(dmp_data, 0, sizeof(dmp_data));
				memset(errlog_data, 0, sizeof(errlog_data));

				errlog_data[0] = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS0, 0x4);
				errlog_data[1] = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS1, 0x4);
				errlog_data[2] = (uint) hfc_read_reg(ap, HFC_IOSPACE_ERRDETAIL0, 0x4);

				dmp_data[0]  = (uchar)(errlog_data[0] >> 24);
				dmp_data[1]  = (uchar)(errlog_data[0] >> 16);
				dmp_data[2]  = (uchar)(errlog_data[0] >> 8 );
				dmp_data[3]  = (uchar)(errlog_data[0]      );
				dmp_data[4]  = (uchar)(errlog_data[1] >> 24);
				dmp_data[5]  = (uchar)(errlog_data[1] >> 16);
				dmp_data[6]  = (uchar)(errlog_data[1] >> 8 );
				dmp_data[7]  = (uchar)(errlog_data[1]      );
				dmp_data[8]  = (uchar)(errlog_data[2] >> 24);
				dmp_data[9]  = (uchar)(errlog_data[2] >> 16);
				dmp_data[10] = (uchar)(errlog_data[2] >> 8 );
				dmp_data[11] = (uchar)(errlog_data[2]      );
				dmp_data[12] = (uchar)(errlog_data[3] >> 24);
				dmp_data[13] = (uchar)(errlog_data[3] >> 16);
				dmp_data[14] = (uchar)(errlog_data[3] >> 8 );
				dmp_data[15] = (uchar)(errlog_data[3]      );


				hfc_logout(ap, (uint)0x35, HFC_ERRLOG_TYPE_IMLLOG);

				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_IMLLOG, ERRID_HFCP_ERRF, 0x35, dmp_data, 16) ;
			}
			/* Post-check error? */
			else if (post_chkerrflg == 1) {
				HFC_DBGPRT(  "CONFIG_HW_set_five() - post_chkerrflg=1, errcode = 0x%x.", err_code );

				memset(dmp_data, 0, sizeof(dmp_data));
				memset(errlog_data, 0, sizeof(errlog_data));
				
				/* Read POST information */
				errlog_data[0] = (uint) hfc_read_reg(ap, HFC_IOSPACE_CA_POSTRESULT, 0x32);

				/* Create error log data */
				dmp_data[0]  = (uchar)(errlog_data[0] >> 24) ;
				dmp_data[1]  = (uchar)(errlog_data[0] >> 16) ;
				dmp_data[2]  = (uchar)(errlog_data[0] >> 8 ) ;
				dmp_data[3]  = (uchar)(errlog_data[0]	   ) ;

				hfc_logout(ap, (uint)0x36, HFC_ERRLOG_TYPE_IMLLOG);

				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_IMLLOG, ERRID_HFCP_ERRF, 0x36, dmp_data, 16) ;
			}

			/* Abnormal End */
			return(-1);
		}
		/* Forced MCK */
		hfc_write_reg(ap, HFC_IOSPACE_CMDCTL, 0x1, 0x08);

		/* Reset all interrupt factor */
		hfc_write_reg( ap, ( uint )HFC_IOSPACE_INTA_RST,( char )0x4, 0xffffffff );      /*  FCLNX-0172 */

		/* CTLRES */
		hfc_write_reg(ap, HFC_IOSPACE_CMDRES, 0x1, 0x02);

		/* 1ms wait */
		mdelay(1);

		/* Initiate reboot */
		hfc_write_reg(ap, HFC_IOSPACE_CMDBOOT, 0x1, 0x40);

		if( HFC_MMODE_CHECK_MLPF(ap) )                                  /* FCLNX-0428 */
			hfc_reset_start(ap, HFC_SET_MLPF_MODE);

		/* Wait 500ms */
		mdelay(500);
    }

	exe_reset = 0;

	HFC_EXIT("hfc_config_hw_set_five");
    return(0);
}


int hfc_query_pktype(struct adap_info *ap)
{
	uchar p;

	HFC_ENTRY("hfc_query_pktype");
	
	HFC_DBGPRT( " memory map base addr = %lx\n", ap->mem_base_addr);

	/* Read package code from adr = 0x005	*/
	ap->pkg.code = (uchar) hfc_read_reg_ext(ap, 0x005, 0x1);

	HFC_DBGPRT( " package code = %01x\n", ap->pkg.code);

	/* Decode package code and set package type to adap_info */
	switch ( ap->pkg.code ) {
		case 0x80:
			ap->pkg.type = HFC_PKTYPE_FPP;
			ap->pkg.map  = (struct pkg_map *) &hfc_pkg_map[0];
			ap->pkg.port = 1;
			break;

		case 0x85: /* FIVE 2G     */			/* FCLNX-0263 */
		case 0x82: /* FIVE 4G     */
		case 0x86: /* FIVE 4G     */
		case 0x87: /* FIVE 10G    */
		case 0x88: /* PCIe Combo  */
		case 0x89: /* FCSWM Combo */
		case 0x8a: /* Mezanine    */			/* FCLNX-0263 */
		case 0x8b: /* Enhanced Mezanine    */	/* FCLNX-GPL-040 */
		case 0x8f: /* FC-GW       */			/* FC-GW */
		case 0x84: /* FIVE 4G PCIe */		/* FCLNX-GPL-054 */
			ap->pkg.type = HFC_PKTYPE_FIVE;
			ap->pkg.map  = (struct pkg_map *) &hfc_pkg_map[1];

			/* Read port number from addr = 0x01E */
			p = (uint) hfc_read_reg_ext(ap, 0x01E, 0x1);
			p = (p & 0x30) >> 4;

			switch (p) {
				case 0x0: ap->pkg.port = 1; break;
				case 0x1: ap->pkg.port = 2; break;
				case 0x3: ap->pkg.port = 4; break;
				default : return(-1);
			}

			break;

		default :
			ap->pkg.map  = (struct pkg_map *) &hfc_pkg_map[1];	/* FCLNX-GPL-040 */

			/* Read port number from addr = 0x01E */
			p = (uint) hfc_read_reg_ext(ap, 0x01E, 0x1);
			p = (p & 0x30) >> 4;

			switch (p) {
				case 0x0: ap->pkg.port = 1; break;
				case 0x1: ap->pkg.port = 2; break;
				case 0x3: ap->pkg.port = 4; break;
				default : return(-1);
			}													/* FCLNX-GPL-040 */
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xCE, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xCE, NULL, 0) ;
//			return(-1);				/* FCLNX-GPL-040 */
			break;
	}

	ap->pkg.vender_id = (ushort) hfc_read_cnfg (ap, HFC_HOST_VENDER_ID, 0x2);
	ap->pkg.device_id = (ushort) hfc_read_cnfg (ap, HFC_HOST_DEVICE_ID, 0x2);
	
	HFC_EXIT("hfc_query_pktype");
	
	return(0);

}


int hfc_release_adp(struct adap_info *ap)
{

	uint				i;
	int exec;
	struct target_info	*target;
//	struct wtimer		*w_timer;	/* FCLNX-GPL-394 */
//	struct dev_info		*dev;		/* FCLNX-GPL-394 */

	HFC_ENTRY("hfc_release_adp");

	/* A stop check of a target */
	for (i=0;i<ap->max_target;i++) {
		target = ap->target_arg[i];

		if (test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags)) {
			exec = hfc_clear_target_info( ap, target, TRUE, FALSE );
			
			if (exec)
				return(exec);											/* FCLNX-0459 */
		}
	}

	if (test_bit(HFC_WAIT_LINK_INIT, (ulong *)&ap->status)	|| 
		test_bit(HFC_WAIT_NMSRV, (ulong *)&ap->status)		||
		test_bit(HFC_WAIT_GPNID, (ulong *)&ap->status)		|| 
		test_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status) ) {
		return(9);														/* FCLNX-0459 */
	}

	if(!(test_bit(HFC_ISOL, (ulong *)&ap->status))){	/* FCLNX-GPL-306 *//* FCLNX-GPL-394 */
		/* Set interrupt mask OFF */
		hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0 );

		/* Set close(offline) request (Set 0x42000000 to E_FRAMEA) */
		hfc_write_reg(ap, HFC_IOSPACE_FRAMEA, 0x4, HFC_FRAMEA_OFFLINE);
	}																		/* FCLNX-GPL-394 */

	/* Cancel mailbox request */
	unlock_mailbox(ap);									/* FCLNX-GPL-318 */	/* FCLNX-GPL-394 */

	/* Stop timer (adap_info) */
	hfc_reset_all_timer(ap);							/* FCLNX-GPL-353 */

//	set_bit(HFC_DETACH, (ulong *)&ap->attach_status);
//	clear_bit(HFC_ATTACH, (ulong *)&ap->attach_status);						/* FCLNX-GPL-394 */

	if ( test_bit(HFC_ENABLE, (ulong *)&ap->status) ) {
		ap->status = 0;
		set_bit(HFC_ENABLE, (ulong *)&ap->status);
	}
	else {
		ap->status = 0;
	}
	
	HFC_EXIT("hfc_release_adp");
	return (0);

}


int hfc_release(struct Scsi_Host *host)
{
	ulong				flags = 0;	/* FCLNX-0274 */
	struct adap_info	*ap;
	struct pci_dev		*pdev = NULL;
	int wait=0;

	HFC_ENTRY("hfc_release");
	
	ap = (struct adap_info *)host->hostdata;
	/* If ap is not configured or opened, return (abend) */
	if ( ap == NULL ) {										/* FCWIN_0014 */
		return 0;
	}
	pdev = ap->pci_cfginf;

	HFC_ADAPLOCK_IRQSAVE(flags);

	/* Closing process is in progress */
	set_bit(HFC_WAIT_CLOSE, (ulong *)&ap->status);
	ap->initialize = 0;
	
	set_bit(HFC_DETACH, (ulong *)&ap->attach_status);		/* FCLNX-0459 */
	clear_bit(HFC_ATTACH, (ulong *)&ap->attach_status);		/* FCLNX-0459 */
	
	do {													/* FCLNX-0459 */
		wait=0;
		
		if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info)
			wait = hfc_manage_info.npubp->hfc_wait_mp_ioend(ap);
		
		if (!wait)
			wait = hfc_release_adp(ap);
		
		/* sleep for a while */
		HFC_ADAPUNLOCK_IRQRESTORE(flags);
		msleep(1);
		HFC_ADAPLOCK_IRQSAVE(flags);
	} while (wait);											/* FCLNX-0459 */
	
	/* Release interrupts */
	if (host->irq)    free_irq(pdev->irq, ap);
	if (ap->msi_flag) pci_disable_msi(pdev);
	//	if(hfc_manage_info.hfcplus_enable)						/* FCLNX-0506 *//* FCLNX-GPL-394 */
	if (hfc_manage_info.hfcldd_mp_mod)						/* FCLNX-GPL-349 */
		hfc_manage_info.npubp->hfc_free_errcnt_info(ap);	/* FCLNX-0506 */	/* FCLNX-GPL-394 */

	/* Release memory area */
	hfc_detach(ap);

	HFC_ADAPUNLOCK_IRQRESTORE(flags);

#if defined(__x86_64) && !defined(_HFC_FCGW) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	/* unregister ioctl32 */
	x86_64_32_unreg_increasing();
#endif /*__x86_64 && !_HFC_FCGW */

	pci_release_regions(ap->pci_cfginf);
	iounmap((void *)(ap->mem_base_addr));

	/* Remove adap_info from mp_adap_info */
	hfc_free_mpap(ap);

	set_bit(HFC_CLOSED, (ulong *)&ap->open_status);

	hfc_manage_info.adap_info_arg[ap->dev_minor] = NULL;					/* FCLNX-0317 */

	if(hfc_manage_info.mp_adap_info == NULL){
		unregister_chrdev(hfc_manage_info.major, "hfcldd");
	}

	HFC_EXIT("hfc_release");
	
	return (0);

}


int hfc_start_adapter(struct adap_info *ap)
{
	struct mp_adap_info *mpap, *mpap_prev, *new_mpap;
	uint i;
	uchar	pkrev_format[17] = "ABCDEFGHJKLMNPQR",adap_id[16];					/* FCLNX-0274 */
	struct	hfc_vpd			*vpd_ptr=NULL;
	struct	hfc_vpd_five	*vpdf_ptr=NULL;
	unsigned long 			wk_buf;
	unsigned long long		wkp;
	uchar 					pkrev_data, buf[16], fail=0;
	int						dma_size;
	int 		err = 0;
	uint		status_bak = 0;
					 	
	HFC_ENTRY("hfc_start_adapter");

	/* Set fw_init_tbl */
	HFC_DBGPRT( " set hw init table \n");
	hfc_set_fw_init_tbl(ap);

	/* Initialize adap_id */
	HFC_DBGPRT( " initialize adap_id \n");
																	/* @MLPF  */
	if ( HFC_MMODE_CHECK_SHARED(ap) )
		hfc_mlpf_get_mmio_hg(ap, adap_id, HFC_IOHGSPC_ADAPID0, HFC_IOHGSPC_ADAPID_LEN);
	else {
		if (ap->pkg.code == 0x8f) {
			hfc_read_flash(ap, 0x20018, 8, &adap_id[0]); /* only FC-GW FCLNX-0405 */
			hfc_read_flash(ap, 0x20028, 8, &adap_id[8]); /* only FC-GW FCLNX-0405 */
		}
		else {
			hfc_read_flash(ap, 0x10000, 16, adap_id);
		}
	}																/* @MLPF  */

	/* Initialize mp_adap_info */

	HFC_DBGPRT( " start hfc manage info settings \n");
	mpap_prev = NULL;
	mpap = hfc_manage_info.mp_adap_info;

	while (mpap != NULL) {
		if ( !memcmp( mpap->adap_id, adap_id, sizeof(adap_id)) )
			break;

		mpap_prev = mpap;
		mpap = mpap->next;
	}

	HFC_DBGPRT( " ** mpap null case \n");
	if ( mpap == NULL ) {

		/* Create new mp_adap_info area */
		HFC_DBGPRT( "allocate new mp adap info area\n");

		for (i=0;i<100;i++) {
			if ((new_mpap = (void *) kmalloc(sizeof(struct mp_adap_info), GFP_ATOMIC)) )
				break;
			
			/* Retry after 100ms if allocation failed */
				mdelay(100); 
		}

		if (new_mpap == NULL) {
			/* Allocation failed after 100 times retries */
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x9B, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x9B, NULL, 0) ;
			return (1); 
		}
		HFC_DBGPRT( "hfcl_manage_info pointer = %lx\n", (ulong) new_mpap);

		/* Clear mp_adap_info */
		memset(new_mpap, 0, sizeof(struct mp_adap_info));
		init_waitqueue_head(&new_mpap->lock_event);

		HFC_DBGPRT( "allocate hw log area \n");
		/* Allocate hw_log area */
		for (i=0;i<100;i++) {
			if ((new_mpap->hw_log = ((uint *) kmalloc(HFC_HWLOG_SIZE, GFP_ATOMIC))) )
				break;

			/* Retry after 100ms when allocation failed */
			mdelay(100); 

		}

		if (new_mpap->hw_log == NULL) {
		/* Allocation failed after 100 times retries */
			kfree(new_mpap);						
			return (1); 
		}

		memset(new_mpap->hw_log, 0, HFC_HWLOG_SIZE);		/* FCWIN-0107 */
		HFC_DBGPRT( "hw_log = %lx\n", (ulong) new_mpap->hw_log);

		/* Copy adap_id */
		memcpy(new_mpap->adap_id, adap_id, sizeof(adap_id));

		if (mpap_prev == NULL) {
			hfc_manage_info.mp_adap_info = new_mpap;
		}
		else {
			mpap_prev->next = new_mpap;
			new_mpap->prev = mpap_prev;
		}
		hfc_manage_info.mp_adap_cnt++;
		mpap = new_mpap;
	}

	HFC_DBGPRT( " ** enquee adap info to mp_adap_info \n");
	/* Enqueue adap_info into mp_adap_info */
	if (mpap->ap != NULL) {
		struct adap_info *wk_ap;
		wk_ap = mpap->ap;
		while (wk_ap->next != NULL) {

			if (wk_ap == ap)
				break;

			wk_ap = wk_ap->next;
		}

		if (wk_ap != ap) {
			wk_ap->next = ap;
			ap->prev = wk_ap;
			ap->next = NULL;
			mpap->port_cnt++;
		}
		else {/* Already enqueued? */	
			HFC_DBGPRT("hfcldd%d : Execute internal online.\n", ap->dev_minor);
		}
	}
	else {
		mpap->ap = ap;
		ap->prev = NULL;
		ap->next = NULL;
		mpap->port_cnt++;
	}

	ap->mp_adap_info = mpap;

	/* Check every 1ms during mp_adap_info is using */
	i=0;
	while (mpap->lock != HFC_NON_STATUS) {
		mdelay(1); 	/* 1ms wait */

		if (i > 6000) {		/* Warning when 1 minutes passed */
			HFC_DBGPRT( "FindAdapter() - mp_adap_info status busy <%d>.", mpap->status);
			i = 0;
		}
		i++;
	}

	if (!test_bit(HFC_HWINIT_COMP, (ulong *)&mpap->status)) {	/* Initialization finished? */
		/* Lock mp_adap_info, set HW_INITIALIZE and release hfc_lock */
		set_bit(HFC_HW_INITIALIZE, (ulong *)&mpap->lock);
		mpap->locked_ap = ap;

		/* Set sys_rev */
																	/* @MLPF STR */
		if ( HFC_MMODE_CHECK_SHARED(ap) ) {
			uint wkint_sysrev1;
			uint wkint_sysrev2;

			wkint_sysrev1 = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_SYSREV0, 0x4 );
			HFC_4L_TO_4B(wkint_sysrev2,wkint_sysrev1);
			HFC_MEMCPY(buf,(uchar*)&wkint_sysrev2,4);
			
			HFC_4B_TO_4L(mpap->sys_rev, (*(uint*)buf));
		}
		else {
			hfc_read_flash(ap, 0, 4, buf);
			HFC_4B_TO_4L(mpap->sys_rev, (*(uint*)buf));
		}
																	/* @MLPF END */

		/* Set vpd information */
		{
		int skip=0;

																	/* @MLPF STR */
			if (!skip)
			{
				err = hfc_get_vpd( ap, mpap->vpd_buf );
				if(err) goto start_adapter_error_exit; 
				if (HFC_MMODE_CHECK_SHADOW(ap))
				{
					/* Shadow LPAR read VPD DATA from mp_adap_info and set it to MMIO-HG */
					hfc_mlpf_set_mmio_hg(ap, mpap->vpd_buf, HFC_IOHGSPC_VPDAREA, HFC_IOHGSPC_VPDAREA_LEN);
				}
					
			}
																	/* @MLPF END */
		}

		fail=0;
		if (ap->pkg.type == HFC_PKTYPE_FPP) {							/* FPP */
			/* Update vpd information */
			vpd_ptr = (struct hfc_vpd *)mpap->vpd_buf;
			strcpy(vpd_ptr->driver_ver, hfc_manage_info.package_ver);
			vpd_ptr->driver_len = (uchar) strlen(vpd_ptr->driver_ver);

			/* Read PK-REV (addr = 0x007) */
			pkrev_data = (uchar) hfc_read_reg(ap, HFC_IOSPACE_PKREV, 0x1);
			/* Store PK_REV into ec_value */
			vpd_ptr->ec_value[0] = pkrev_format[pkrev_data];
			vpd_ptr->fw_ver=mpap->sys_rev;
			strncpy(mpap->model_name,"HFC0201",16);						/* FCLNX-0292 */
			
			if ((hfc_config_hw_set(ap, HFC_CONFIG_HW_CHECK_RETRY)) != 0)
				fail=1;

			vpd_ptr->ww_name  = ap->ww_name;							/* FCLNX-0299 */
		}
		else if (ap->pkg.type == HFC_PKTYPE_FIVE) {	/* FIVE */
			uchar parts_value[HFC_IOHGSPC_PARTSNUM_LEN];
			memset(parts_value, 0,HFC_IOHGSPC_PARTSNUM_LEN);
			/* Update vpd information */
			vpdf_ptr = (struct hfc_vpd_five *)mpap->vpd_buf;
			strcpy(vpdf_ptr->driver_ver, hfc_manage_info.package_ver);
			vpdf_ptr->driver_len = (uchar) strlen(vpdf_ptr->driver_ver);

			/* Read PK-REV (addr = 0x007) */
			pkrev_data = (uchar) hfc_read_reg(ap, HFC_IOSPACE_PKREV, 0x1);
			vpdf_ptr->ec_level = pkrev_format[pkrev_data];
	
			i=0;
			while (cr_pn[i].pk_code && cr_pn[i].pk_port) {
				struct cr_PartsNumber *pn_ptr = (struct cr_PartsNumber *) &cr_pn[i];
				
				if ( (ap->pkg.code == pn_ptr->pk_code)
				  && (ap->pkg.port == pn_ptr->pk_port) ) {
					memset(buf,0,sizeof(buf));
					strncpy(buf, pn_ptr->header, 1);
					strncpy(buf+1, vpdf_ptr->pn_value, 5);
					strncpy(buf+6, pn_ptr->pk, 1);
					strncpy(buf+7, vpdf_ptr->pn_value+6, 2);
					strncpy(buf+9, pn_ptr->hwinf, 2);
					strncpy(vpdf_ptr->pn_value, buf, sizeof(vpdf_ptr->pn_value));
					strncpy(mpap->model_name, pn_ptr->model_name, 16);	/* FCLNX-0292 */
					break;
				}
				i++;
			}

			if (!cr_pn[i].pk_code || !cr_pn[i].pk_port) {	/* Not hit ? */				/* FCLNX-0263 */
				if( ap->raslog_install )
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xCF, NULL, 0) ;
				else
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xCF, NULL, 0) ;
				goto start_adapter_error_exit;											/* FCLNX-0263 */
			}
			
			vpdf_ptr->fw_ver = mpap->sys_rev;
			
			if (HFC_MMODE_CHECK_SHADOW(ap))
			{
				strncpy(parts_value, vpdf_ptr->pn_value, HFC_IOHGSPC_PARTSNUM_LEN);
				hfc_mlpf_set_mmio_hg(ap, parts_value, HFC_IOHGSPC_PARTSNUM0, HFC_IOHGSPC_PARTSNUM_LEN);
			}
		
			if(hfc_manage_info.instance == 0) {
				HFC_DBGPRT(" mck_point = %d\n", ap->mck_point);
				if(ap->mck_point == HFC_BEFORE_POSTCHK) {        /* FCLNX-0533 */
					HFC_DBGPRT("call hfc_occurred_mck \n");
					hfc_occurred_mck(ap,ap->mck_point);
				}       /* FCLNX-0533 */
			}
	
			                                                                            /* FCLNX-0370 */
			if( !HFC_MMODE_CHECK_SHARED(ap) || 
			   ( HFC_MMODE_CHECK_SHADOW(ap) && ! HFC_MMODE_CHECK_REBOOT(ap) ) )
			{
				if ((hfc_config_hw_set_five(ap, HFC_CONFIG_HW_CHECK_RETRY)) != 0)
					fail=1;
			}                                                                           /* FCLNX-0370 */
			
			if ( HFC_MMODE_CHECK_SHADOW(ap) && HFC_MMODE_CHECK_REBOOT(ap) )             /* FCLNX-0386 */
			{
				uint                        hyp_status;
				
				hyp_status = hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4);
				/* Read HYPER status before reset int_a_reg.     */
				if( hyp_status & ( HFC_HG_HYPSTATUS_FCSTP | HFC_HG_HYPSTATUS_FCSTP_IML | HFC_HG_HYPSTATUS_FMCK ) )
					hfc_mlpf_hwerr_int(ap, hyp_status);
			}
			
			if ( HFC_MMODE_CHECK_SHARED(ap) )                                           /* FCLNX-0384 */
			{
				if ( !hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_ENABLE, HFC_CHECK_HYPER_STATE) )
					fail = 1;
			}
			vpdf_ptr->ww_name = ap->ww_name;							/* FCLNX-0299 */
		}
		else {
			HFC_DBGPRT( "FindAdapter() - Invalid Package Type.");
			fail=1;
		}

		if (!fail) {
			dma_size = sizeof(dma_addr_t);
			wkp = ap->padr_init;
			if(dma_size == 8){
				wkp >>=32;
			}
			else{
				wkp = 0;
			}
			HFC_DBGPRT(" ** wkp = %llx, %llx dma_size = %d\n",
				(unsigned long long)wkp, (unsigned long long)ap->padr_init, dma_size);

			hfc_write_reg(ap, HFC_IOSPACE_CA_INIT_ADDR0, 0x4, wkp);
			hfc_write_reg(ap, HFC_IOSPACE_CA_INIT_ADDR1, 0x4, ap->padr_init);

			wk_buf = (uint) hfc_read_reg(ap, HFC_IOSPACE_CA_INIT_ADDR0, 0x04);
			HFC_DBGPRT( "init addr = %lx",wk_buf);
			wk_buf = (uint) hfc_read_reg(ap, HFC_IOSPACE_CA_INIT_ADDR1, 0x04);
			HFC_DBGPRT( "** %lx \n",wk_buf);

			hfc_write_reg(ap, HFC_IOSPACE_CA_FLAG, 0x1, 0x00);
			
			ap->fw_support = (uint) hfc_read_reg(ap, HFC_IOSPACE_FW_SUPPORT, 0x1);		/* FCLNX-GPL-311 */

			if ( HFC_MMODE_CHECK_SHADOW(ap) && HFC_MMODE_CHECK_REBOOT(ap) )
				hfc_write_reg(ap, HFC_IOSPACE_FRAMEA, 0x4, HFC_FRAMEA_SHADOW_UP);
			else
			{
				if (  ( !hfc_manage_info.hfcplus_enable ) || /* - FCLNX-546 - */
				     ((  hfc_manage_info.hfcplus_enable ) &&  ( ap->isol_force == HFC_NO_FRC_ISOL )) ){
					/* Initiate firmware */
					hfc_write_reg(ap, HFC_IOSPACE_FRAMEA, 0x4, HFC_FRAMEA_FW_START);
				}
			}

			/* Normal end */
			set_bit(HFC_HWINIT_COMP, (ulong *)&mpap->status);
		}
		else {
			/* Abnormal end */
			set_bit(HFC_HWINIT_COMP, (ulong *)&mpap->status);
			set_bit(HFC_HWINIT_FAIL, (ulong *)&mpap->status);
		}

		mpap->lock = HFC_NON_STATUS;
		mpap->locked_ap = NULL;
	}
	else {
		/* FIVE (multiport) case */

		if( ap->pkg.type != HFC_PKTYPE_FPP)
		{
			status_bak = (uint) hfc_read_reg(ap, HFC_IOSPACE_STATUS0, 0x4 );
			
			if (status_bak & 0x00800000)
			{
				hfc_write_cnfg(ap, 0x06, 0x2, 0xffff);
				hfc_write_cnfg(ap, 0x6c, 0x4, 0xffffffff);
				hfc_write_cnfg(ap, 0x70, 0x4, 0x0fffffff);
				HFC_DBGPRT("exec pci reset.");
			}
			
			/* Reset interrupt factor */
			if( !HFC_MMODE_CHECK_SHARED(ap) || 
			   ( HFC_MMODE_CHECK_SHADOW(ap) && ! HFC_MMODE_CHECK_REBOOT(ap) ) )     /* FCLNX-0391 */
				hfc_write_reg( ap, ( uint )HFC_IOSPACE_INTA_RST,( char )0x4, 0xffffffff );
			
			vpdf_ptr = (struct hfc_vpd_five *)mpap->vpd_buf;
			vpdf_ptr->ww_name = ap->ww_name;
		}
		
		if ( HFC_MMODE_CHECK_SHADOW(ap) && HFC_MMODE_CHECK_REBOOT(ap) )             /* FCLNX-0386 */
		{
			uint                        hyp_status;
			
			hyp_status = hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4);
			/* read HYPER status before reset int_a_reg.     */
			if( hyp_status & ( HFC_HG_HYPSTATUS_FCSTP | HFC_HG_HYPSTATUS_FCSTP_IML | HFC_HG_HYPSTATUS_FMCK ) )
				hfc_mlpf_hwerr_int(ap, hyp_status);
		}
		
		if ( HFC_MMODE_CHECK_SHARED(ap) )                                           /* FCLNX-0384 */
		{
			if ( !hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_ENABLE, HFC_CHECK_HYPER_STATE) )
				fail = 1;
		}
		
		if (!fail) {															/* FCLNX-0228 STR */
			dma_size = sizeof(dma_addr_t);
			wkp = ap->padr_init;
			if(dma_size == 8){
				wkp >>=32;
			}
			else{
				wkp = 0;
			}
			hfc_write_reg(ap, HFC_IOSPACE_CA_INIT_ADDR0, 0x4, wkp);
			hfc_write_reg(ap, HFC_IOSPACE_CA_INIT_ADDR1, 0x4, ap->padr_init);

			wk_buf = (uint) hfc_read_reg(ap, HFC_IOSPACE_CA_INIT_ADDR0, 0x04);
			HFC_DBGPRT( "init addr = %lx",wk_buf);
			wk_buf = (uint) hfc_read_reg(ap, HFC_IOSPACE_CA_INIT_ADDR1, 0x04);
			HFC_DBGPRT( "** %lx \n",wk_buf);

			hfc_write_reg(ap, HFC_IOSPACE_CA_FLAG, 0x1, 0x00);
			
			ap->fw_support = (uint) hfc_read_reg(ap, HFC_IOSPACE_FW_SUPPORT, 0x1);	/* FCLNX-GPL-311 */

			if ( HFC_MMODE_CHECK_SHADOW(ap) && HFC_MMODE_CHECK_REBOOT(ap) )     /* FCLNX-0328 */
				hfc_write_reg(ap, HFC_IOSPACE_FRAMEA, 0x4, HFC_FRAMEA_SHADOW_UP);
			else		/* FCLNX-GPL-394 */
			{
//				if (  ( !hfc_manage_info.hfcplus_enable ) || /* - FCLNX-546 - */
//				     ((  hfc_manage_info.hfcplus_enable ) &&  ( ap->isol_force == HFC_NO_FRC_ISOL )) ){
				if (  ( !hfc_manage_info.hfcldd_mp_mod ) || /* FCLNX-GPL-349 */
				     ((  hfc_manage_info.hfcldd_mp_mod ) &&  ( ap->isol_force == HFC_NO_FRC_ISOL )) ){
					/* Initiate firmware */
					hfc_write_reg(ap, HFC_IOSPACE_FRAMEA, 0x4, HFC_FRAMEA_FW_START);
				}
			}			/* FCLNX-GPL-394 */

		}																		/* FCLNX-0228 END */

	}

//	if ( (!hfc_manage_info.hfcplus_enable) ||
//	    (( hfc_manage_info.hfcplus_enable ) && ( ap->isol_force == HFC_NO_FRC_ISOL )) ){  /* - FCLNX-546 - */
	if (  ( !hfc_manage_info.hfcldd_mp_mod ) || /* FCLNX-GPL-349 */	/* FCLNX-GPL-394 */
		((  hfc_manage_info.hfcldd_mp_mod ) &&  ( ap->isol_force == HFC_NO_FRC_ISOL )) ){
	    
		set_bit(HFC_NEED_LINK_INIT, (ulong *)&ap->status);
	}																/* FCLNX-GPL-394 */

	
	if ( test_bit(HFC_HWINIT_FAIL, (ulong *)&mpap->status)){

		HFC_DBGPRT( " hfcldd : hfc_start_adapter - disable interrupt\n");
		hfc_write_reg( ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, 0 ); 
		
		if( HFC_MMODE_CHECK_SHADOW(ap) )
		{
			if( ap->raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xD0, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xD0, NULL, 0) ;
			hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_ENABLE, HFC_DISABLE_HYPER_STATE );
			hfc_write_hg_reg(ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_CSTPEND);     /* FCLNX-0388 */
		}

	}
	else {
		set_bit(HFC_ATTACH, (ulong *)&ap->attach_status);
	}																			/* FCLNX-0228 END */

	HFC_EXIT("hfc_start_adapter");

    return (0); 

start_adapter_error_exit:
	hfc_free_mpap(ap);
    return (1); 

}

int hfc_initialize(struct adap_info *ap, int immdt_cmd)
{
	unsigned long flags = 0;

	HFC_ENTRY("hfc_initialize");
	
	HFC_ADAPLOCK_IRQSAVE(flags);
	
	if ( !test_bit(HFC_ENABLE, (ulong *)&ap->status) ) { /* HBA initialization has not finished? */
		HFC_DBGPRT( "hfcldd : hfc_initialize- HFC_ENABLE=0.");
		HFC_ADAPUNLOCK_IRQRESTORE(flags);
		return TRUE;
	}

	if ( !test_bit(HFC_NEED_LINK_INIT, (ulong *)&ap->status) ) { /* Need link initialization? */
		HFC_DBGPRT( "hfcldd : hfc_initialize - HFC_NEED_LINK_INIT=0.\n");
		HFC_ADAPUNLOCK_IRQRESTORE(flags);
		return TRUE;
	}

	/* Start target detection */
	HFC_DBGPRT(" hfcldd : hfc_initialize - start target detection adap_status = %lx\n",(ulong)ap->status); 

	clear_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status);
	hfc_w_stop( ap, HFC_LINKUP_TMR );
	hfc_w_stop( ap, HFC_LINKUP2_TMR );						/* FCLNX-0241 */

	ap->initialize = 1;
	atomic_set(&ap->int_a_poll, 0); 						/* FCLNX_0029 */
	if ( hfc_issue_linkini(ap) ) {
		HFC_DBGPRT(" hfcldd : hfc_initialize - issue link initialize fail.\n");
		HFC_ADAPUNLOCK_IRQRESTORE(flags);
		return TRUE;
	}

	HFC_ADAPUNLOCK_IRQRESTORE(flags);

	if(hfc_manage_info.instance == 1) {
		if(ap->mck_point == HFC_BEFORE_LINKINITIALIZE) {	/* FCLNX-0533 */
			hfc_occurred_mck(ap,ap->mck_point);
		}													/* FCLNX-0533 */
	}

	/* Enable intterupt */
	HFC_DBGPRT( " hfcldd : hfc_initialize - enable interrupt\n");
																	/* @MLPF STR */
	if (HFC_MMODE_CHECK_SHARED(ap))
		hfc_write_reg( ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )HFC_INTA_MASK_MLPF );
	else
		hfc_write_reg( ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )HFC_INTA_MASK );
																	/* @MLPF END */

	HFC_DBGPRT(" hfcldd : hfc_initialize - interruptible sleep on (mailbox event) \n");

	if(!immdt_cmd){ /* FCLNX-0514 */
		hfc_sleep_on(&ap->init_event, &ap->int_a_poll );								/* FCLNX-0269 */
	}
	else{
		ap->initialize = 0;
	}
	
	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
		hfc_manage_info.npubp->hfc_mp_scan_dev(ap);			/* FC-GW 0331 */
	}

	if(hfc_manage_info.instance == 1) {
		if(ap->mck_point == HFC_AFTER_LINKINITIALIZE) {		/* FCLNX-0533 */
			hfc_occurred_mck(ap,ap->mck_point);
		}													/* FCLNX-0533 */
	}

	HFC_ADAPLOCK_IRQSAVE(flags);
	atomic_set(&ap->int_a_poll, 0);							/* FCLNX_0029 */
	ap->initialize = 0;		

	HFC_ADAPUNLOCK_IRQRESTORE(flags);

	HFC_EXIT("hfc_initialize");
	
	return TRUE;
	
}


void hfc_wwnverify_linkup (struct adap_info *ap, struct target_info *target, uint mb_resp_status, uint64_t ww_name )
{
	int i,j;
	int login_req = 0;
	uchar count;											
	int sc;
	uchar hit;
				
	HFC_ENTRY("hfc_wwnverify_linkup");

	if (mb_resp_status) {
		/* Link initialization has failed */
		if(ap->initialize != 0){
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		}
		return;
	}

	for(i=0 ; i<MAX_TARGET_PROBE ; i++)					/* FCWIN-0082STR*/
	{
		target = hfc_hash_target_valid(ap, i);
		if (target != NULL)
		{
			set_bit(HFC_NEED_LOGIN, (ulong *)&target->status);
			clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */

			if ( (ap->connect_type == HFC_SWITCH ) 			/* FCWIN-0146 */
				|| ((ap->connect_type == HFC_AL) && (ap -> scsi_id & 0x00ffff00))) {/* FCWIN-0185 */
				set_bit(HFC_NEED_GIDPN, (ulong *)&target->status);
				ap->next_gidpn = TRUE;

				/* FCLNX-GPL-038 */
//				hfc_issue_gidpn( ap, target );	/* Issue GID_PN */			
			}
			else {
				set_bit(HFC_WWN_VALID, (ulong *)&target->flags);				/* ??? */
				hfc_enque_login_req(ap, target);			/* FCWIN-0146 */
			}
		}
	}														/* FCWIN-0082END*/

	set_bit(HFC_LOGIN_DELAY, (ulong *)&ap->status );								/* FCLNX-0279 */
	hfc_watchdog_enter( ap, NULL, NULL, 0, HFC_LOGIN_DELAY_TMR, 0, 1);				/* FCLNX-0279 */
	hfc_watchdog_enter( ap, NULL, NULL, 0, HFC_LOGIN_DELAY_TMR, ap->login_wait, 0 );/* FCLNX-0279 */

	/* Clear target scan */
	HFC_DBGPRT( "target_scan = %lx\n",(ulong)ap->target_scan );
	memset(ap->target_scan,0, (sizeof(struct target_scan)*MAX_TARGET_PROBE) );

	switch (ap->connect_type) {
		case HFC_SWITCH :	/* FC-SW */
			HFC_DBGPRT("wwnverify_linkup() - Connect FC-SW.");
			set_bit(HFC_NEED_NMSRV, (ulong *)&ap->status);							/* request GID_FT */

			break;

		case HFC_AL :	/* FC-AL (without SW) */

			if (ap->scsi_id & 0x00ffff00) {											/** FCWIN-0185 **/
				HFC_DBGPRT("wwnverify_linkup() - Connect FC-AL (FC-SW).");
				set_bit(HFC_NEED_NMSRV, (ulong *)&ap->status);	/* request GID_FT */
				break;
			}																		/*@ FCWIN-0185 @*/

			HFC_DBGPRT("wwnverify_linkup() - Connect FC-AL(Non SW).");
			/* Create the list of SCSI_ID (HFC_WWN_VALID = 0 or 1) */
			if ( (hfc_read_val( ap->fw_init_p->fw_iocinfo.flag ) & HFC_POSMAP_VALID )
			  && (!(hfc_read_val( ap->fw_init_p->fw_iocinfo.flag ) & HFC_POSMAP_LISA)) ) {
				HFC_DBGPRT("wwnverify_linkup() - ***\n");
				sc=0;
				/* Set position_map valid */
				count = (int) hfc_read_val(ap->fw_init_p->pos_map[0]);
				for (i=0;i<count; i++) {
					if (ap->scsi_id != ap->fw_init_p->pos_map[i+1]) {
						ap->target_scan[sc].flags |= HFC_SCAN_SCV;
						ap->target_scan[sc].scsi_id = ap->fw_init_p->pos_map[i+1];
						sc++;
					}
				}
			}
			else {
				sc=0;
				HFC_DBGPRT("wwnverify_linkup() - @@@\n");
				/* Register all AL_PN number */
				for (i=0;i<127;i++) {
					if (ap->scsi_id != posmap_lisa[i+1]) {
						ap->target_scan[sc].flags  |= HFC_SCAN_SCV;
						ap->target_scan[sc].scsi_id = posmap_lisa[i+1];
						sc++;
					}
				}
			}
			login_req = FALSE;
			for (i=0;i<MAX_TARGET_PROBE;i++) {
				hit=0;

				if (!(ap->target_scan[i].flags & HFC_SCAN_SCV))
					continue;
				
				for (j=0;j<(int) ap->max_target;j++) {

					target = ap->target_arg[j];
					if(target != NULL) {
						if (!test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags))
							continue;

						HFC_DBGPRT( "wwnverify_login() - target_arg = %d, %d flags = %d\n",
										i, j, (uint)target->flags);

						if (ap->target_scan[i].scsi_id == target->scsi_id) {
							/* Target_info has already exsist */
							HFC_DBGPRT( "wwnverify_linkup() - target info has already existed*** \n");
							HFC_DBGPRT( "(wwpn) ww_name (from mailbox) %llx, target_name = %llx\n",
													(unsigned long long)ww_name, (unsigned long long)target->ww_name );

							if( ww_name == 0){
								HFC_DBGPRT(" wwnverify_linkup() - ww-name has not changed \n");
								ap->target_scan[i].flags &= ~HFC_SCAN_SCV;
								hit=1;
								break; 
							}
							if( (ww_name != 0) && (target->ww_name == ww_name) ){
								HFC_DBGPRT(" wwnverify_linkup() - ww-name has changed \n");
								ap->target_scan[i].flags &= ~HFC_SCAN_SCV;
								hit=1;
								break; 
							}
						}
					}
				}

				if (!hit) {
					
					HFC_DBGPRT("wwnverify_linkup() - target did not hit \n");
					target = hfc_add_target_info( ap, ap->target_scan[i].scsi_id );
					if(target != NULL){
					/* request LOGIN */
					
					HFC_DBGPRT("wwnverify_linkup() - target existed \n");
						set_bit(HFC_NEED_LOGIN, (ulong *)&target->status);
						clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
						hfc_enque_login_req(ap,target);
						login_req = TRUE;
					}
					else{
						HFC_DBGPRT("wwnverify_linkup() - target info does not exist\n"); 
						/* There is no space for target_info */
						ap->target_scan[i].flags &= ~HFC_SCAN_SCV;      /* FCWIN-0148 */
					}
				}
				else {
					if ( !test_bit(HFC_WWN_VALID, (ulong *)&target->flags) ){
						set_bit(HFC_NEED_LOGIN, (ulong *)&target->status);
						clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
						hfc_enque_login_req(ap,target);
						login_req = TRUE;
					}
				}
			}
			if(login_req == FALSE){
				if(ap->initialize != 0){
					hfc_wake_up(&ap->init_event,&ap->int_a_poll);
				}
				HFC_DBGPRT(" ap->initialize = %d\n", ap->initialize);
			}
			break;

		case HFC_PT2PT :							/* P2P */
			HFC_DBGPRT("wwnverify_linkup() - Connect P2P.");

			if (!ap->target_cnt) {
				/* Create target_info */
				target = hfc_add_target_info(ap, hfc_read_val( ap->fw_init_p->fw_iocinfo.p2p_port_id));

				set_bit(HFC_NEED_LOGIN, (ulong *)&target->status);
				clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
				hfc_enque_login_req(ap,target);
			}
			else {
				target = ap->target_arg[0];
			}

			break;

		default :
			HFC_DBGPRT( "wwnverify_linkup() - Invalid connect_type=%d.",ap->connect_type);
	}

	HFC_EXIT("hfc_wwnverify_linkup");

}


void hfc_wwnverify_linkup_timeout(struct adap_info *ap, struct target_info *target, uint mb_resp_status)
{															  /* FCWIN-0082 */
	int i,rsp;

	HFC_ENTRY("hfc_wwnverify_linkup_timeout");

	if ( target != NULL ) {
		HFC_DBGPRT(  "wwnverify_linkup_timeout() - delete target (pseq#=%d).",target->pseq);

		/* Release specified target_info */
		if (test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags)) {
			rsp = hfc_clear_target_info( ap, target, TRUE, TRUE );  			

		}
	}
	else {
		HFC_DBGPRT(  "wwnverify_linkup_timeout() - delete all target.");

		/* Release all target_info */
		for (i=0;i<(int)ap->max_target;i++) {
			target = ap->target_arg[i];
			if(target != NULL) {
				if (test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags)) {
					rsp = hfc_clear_target_info( ap, target, TRUE, TRUE );
					
				}
			}
		}
	}

	HFC_EXIT("hfc_wwnverify_linkup_timeout");

}


void hfc_wwnverify_gidft(struct adap_info *ap, struct target_info *target, uint mb_resp_status)
{
	struct FS_ACC *fs_acc = (struct FS_ACC *) ap->mb_parm;
	uint	scsi_id,i,port_num;
	int    no_target=1;

	HFC_ENTRY("hfc_wwnverify_gidft");

	/* Clear target_can */
	memset(ap->target_scan,0,(sizeof(struct target_scan)*MAX_TARGET_PROBE));

	if (mb_resp_status) {
		/* GID_FT has failed. Impossible to detect target */
		if(ap->initialize != 0){
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		}
		HFC_DBGPRT( "wwnverify_gidft() - error end<mb_resp_status=0x%x>.",mb_resp_status);
		return;
	}

	/* Read device number */
	port_num = (uint) hfc_read_val( ap->mb->mb_resp.type.drvioctl2.gid_ft.port_number);
	port_num = ( port_num < 508 ) ? port_num : 508 ;

	if (!port_num) {
		/* No target for this FC-SW */
		if(ap->initialize != 0){
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		}
		HFC_DBGPRT( "wwnverify_gidft() - non target");
		return;
	}

	HFC_DBGPRT(" port_num = %d\n",port_num);

	for (i=0;i<port_num;i++) {
		scsi_id = (uint) hfc_read_val( fs_acc->port_id[i] )& 0x00ffffff;        /* FCWIN-0276 */
		
		if(ap->scsi_id != scsi_id) {
			ap->target_scan[i].flags  |= HFC_SCAN_SCV | HFC_SCAN_NEED;	
			ap->target_scan[i].scsi_id = scsi_id;                           /* FCWIN-0276 */
			set_bit(HFC_NEED_GPNID, (ulong *)&ap->status);
			no_target=0;
			if (port_num >= MAX_TARGET_PROBE)
				break;

		}

	}

	if(no_target){	/* FCLNX-0165 */
		HFC_DBGPRT(" target is not found\n");
		if(ap->initialize != 0){
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		}
	}


	HFC_EXIT("hfc_wwnverify_gidft");

}


void hfc_wwnverify_gpnid(struct adap_info *ap, struct target_info *target, uint mb_resp_status)
{															  /* FCWIN-0082 */
	struct target_scan *tscan,*next_scan=NULL;
	int hit=0,i,j;
	int no_target=0;

	HFC_DBGPRT("hfc_wwnverify_gpnid");

	HFC_DBGPRT("gpnid ** mb_resp_status = %x\n",mb_resp_status);

	for (i=0;i<MAX_TARGET_PROBE;i++) {
		tscan = &ap->target_scan[i];

		if ( (tscan->flags & (HFC_SCAN_SCV | HFC_SCAN_WAIT))
							 == (HFC_SCAN_SCV | HFC_SCAN_WAIT) ) {

			HFC_DBGPRT("gpnid ** tscan->flags = %x\n", tscan->flags);
			
			if ( hfc_read_val( ap->mb->mb_init.type.drvioctl1.un.gpn_id.port_id )
					== tscan->scsi_id) {
				tscan->flags &= ~(HFC_SCAN_NEED | HFC_SCAN_WAIT);
				tscan->flags |= HFC_SCAN_COMP;
				
				if (!mb_resp_status) {		

					tscan->wwpn = hfc_read_val( 
						ap->mb->mb_resp.type.drvioctl1.gpn_id.port_name_hi );
					tscan->wwpn <<= 32;
					tscan->wwpn |= hfc_read_val( 
						ap->mb->mb_resp.type.drvioctl1.gpn_id.port_name_lo );
					HFC_DBGPRT("gpnid ** tscan->wwpn = %llx\n", (unsigned long long)tscan->wwpn);

				}
				else {
					/* Target is invalid */
					tscan->flags |= HFC_SCAN_FAIL;
					tscan->flags &= ~HFC_SCAN_SCV;				/* FCLNX-GPL-292 */
					HFC_DBGPRT("gpnid ** HFC_SCAN_FAIL\n");
				}
				hit++;
			}
		}

		if ( (tscan->flags & (HFC_SCAN_SCV | HFC_SCAN_NEED))
				== (HFC_SCAN_SCV | HFC_SCAN_NEED)) {
			/* Store pointer of target_scan which will issue GPN_ID next */
			next_scan = tscan;
			set_bit(HFC_NEED_GPNID, (ulong *)&ap->status);
		}
	}

	HFC_DBGPRT(" hit = %d",hit);


	if ( !next_scan ) {	/* Find target to issue GPN_ID next */
		HFC_DBGPRT("next_scan finished \n");
		no_target=1;

		for (i=0;i<MAX_TARGET_PROBE;i++) {
			tscan = &ap->target_scan[i];

			if ( (tscan->flags & (HFC_SCAN_SCV | HFC_SCAN_COMP | HFC_SCAN_FAIL))
							  == (HFC_SCAN_SCV | HFC_SCAN_COMP)) {
				
				int new=TRUE;

				for (j=0;j<(int)(ap->max_target);j++) {
					target = ap->target_arg[j];

					if(target != NULL){
						/* If target has already exists, no need to make new target. */
						if ( test_bit(HFC_DEVFLG_VALID, (ulong *)&target->flags) 
							&& ( target->ww_name == tscan->wwpn ) ) {
						
							tscan->flags &= ~HFC_SCAN_SCV;
							new = FALSE;
							HFC_DBGPRT("target hit @@ \n");

							if (target->scsi_id != tscan->scsi_id)					/* FCWIN-0166 */
							{														/* FCWIN-0166 */
								set_bit(HFC_NEED_GIDPN, (ulong *)&target->status);	/* FCWIN-0166 */
								ap -> next_gidpn = TRUE;							/* FCWIN-0166 */
								no_target = 0;
							}														/* FCWIN-0166 */
							else if(!test_bit(HFC_WWN_VALID, (ulong *)&target->flags)){	/* ??? */
								set_bit(HFC_NEED_LOGIN, (ulong *)&target->status);
								clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);					/* FCLNX-GPL-038 */
								
								clear_bit(HFC_SCN_WLINKUP, (ulong *)&target->status);					/* FCLNX-GPL-038 */
								hfc_watchdog_enter(ap, target, NULL, 0, HFC_SCN_LINKUP_TMR, 0, TRUE);	/* FCLNX-GPL-038 */
								hfc_enque_login_req(ap,target);
								no_target = 0; 
							}
							
							break;
						}

						/* If this target is in creating process, no need to create new target */
						if (  test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags) &&
							  !test_bit(HFC_DEVFLG_VALID, (ulong *)&target->flags) 
							&& ( target->scsi_id == tscan->scsi_id ) ){
							tscan->flags &= ~HFC_SCAN_SCV;
							new = FALSE;
							no_target = 0;
							break;
						}	
					}
				}

				if ( new == TRUE ) {/* Make new target if there is no target with the same WWN */
					
					HFC_DBGPRT("add target scsi_id=%d wwpn= %llx\n", tscan->scsi_id, (unsigned long long)tscan->wwpn);

					if ( (target = hfc_add_target_info(ap, tscan->scsi_id)) != NULL ) {
						set_bit(HFC_NEED_LOGIN, (ulong *)&target->status);
						hfc_enque_login_req(ap,target);
						no_target = 0;
					} 
				}

			}
		}
	}

	HFC_DBGPRT("no_target = %d\n",no_target);

	if(no_target){
		HFC_DBGPRT(" verify_gpnid no_target; target is not found\n");
		if(ap->initialize != 0){
			HFC_DBGPRT("wake up detect forcefully\n");
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		}
	}

	HFC_DBGPRT("hfc_wwnverify_gpnid");
}


void hfc_wwnverify_login(struct adap_info *ap, struct target_info *target, uint mb_resp_status, uint64_t ww_name)
{
	int i=0,j,rtn,hit,tid=0,tid_hit=0,empty_hit=0,entry;
	char buf1[32];
	struct target_info *wk_target;
	uint64_t parm=0;

	HFC_ENTRY("hfc_wwnverify_login");

	HFC_DBGPRT( "wwnverify_login() login normal end \n");

	for (j=0;j<MAX_TARGET_PROBE;j++) {										/* FCWIN-0148 */
		if ( (ap->target_scan[j].flags & HFC_SCAN_SCV)
		  && (ap->target_scan[j].scsi_id == target->scsi_id) ){			  
			ap->target_scan[j].flags |= HFC_SCAN_LOGIN;			/* LOGIN has finished */
			HFC_DBGPRT( "wwnverify_login() - target has already logined");
		}
	}


	switch (mb_resp_status) {
	case 0 :

		HFC_DBGPRT( "wwnverify_login() check login timing \n");

		/* Is this target in making process?*/
		if ( test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags) &&
			 !test_bit(HFC_DEVFLG_VALID, (ulong *)&target->flags) ){
			tid_hit=0;
			empty_hit=0;

			if (ap->narrowmap != 2) {	/* ap->narrowmap <> narrowmap(WWPN mode) is OFF ? */
				if(ap->automap == 0){ /* Is automap off? */
					
					for (i=0;i<MAX_TARGET_PROBE;i++) {
						hit=0;

						switch ( ap->bind_target_id->type[i] ) {
						case HFC_TID_WWPN :
							parm = hfc_read_val(ap->mb->mb_resp.type.drvioctl1.login.port_name);
							hit=1;
							
							HFC_DBGPRT(" bind_target_id type = %d, parm = %llx\n",
											ap->bind_target_id->type[i], 
											(unsigned long long)ap->bind_target_id->parm[i]);
							break;
							
						case HFC_TID_WWNN :
							parm = hfc_read_val(ap->mb->mb_resp.type.drvioctl1.login.node_name);
							hit=1;
							
							HFC_DBGPRT(" bind_target_id type = %d, parm = %llx\n",
											ap->bind_target_id->type[i], 
											(unsigned long long)ap->bind_target_id->parm[i]);
							break;
							
						case HFC_TID_DID  :
							parm = hfc_read_val(ap->mb->mb_init.type.drvioctl1.adr.d_id);
							hit=1;
							
							HFC_DBGPRT(" bind_target_id type = %d, parm = %llx\n",
											ap->bind_target_id->type[i], 
											(unsigned long long)ap->bind_target_id->parm[i]);
							break;
						}
						
						if (hit && (ap->bind_target_id->parm[i] == parm)) {		/* FCWIN-XXXX */
							/* This target is reserved with persistent binding  */
							
							HFC_DBGPRT(" bind_target_id -- persistent bindinf set \n");
							HFC_DBGPRT("   (from mailbox)=%llx, binding=%llx\n", (unsigned long long)parm, (unsigned long long)ap->bind_target_id->parm[i]);						
							
							tid_hit=1;
							tid = i;
							break;
						}
					}
#if defined(__x86_64) && defined(_HFC_FCGW)							/* FCLNX-0405 */
					if (!tid_hit) {
						if ( !((ap->connect_type == HFC_SWITCH) 
							|| ((ap->connect_type == HFC_AL) && (ap -> scsi_id & 0x00ffff00))) ) {
							tid_hit=2;
							tid = ap->hosts->this_id;
						}
					}
#endif /*__x86_64 && _HFC_FCGW*/										/* FCLNX-0405 */
				}
				else {	/* automap is ON */
					/* If this target is not reserved with persistent binding, assign empty ID */
					HFC_DBGPRT( " allocate vacant id (without persistent binding) \n");
					
					for (i=0;i<MAX_TARGET_PROBE;i++) {
						
						if ( !ap->bind_target_id->type[i] ) { /* No binding information */
							if ( !hfc_hash_target_valid(ap, i) ) {	/* Not allocated */
								empty_hit=1;
								tid = i;
								break;
							}
						}
					}
				}
			}
			else {	/* narrowmap(WWPN mode) is ON */
				parm = hfc_read_val(ap->mb->mb_resp.type.drvioctl1.login.port_name);
				
				hit=0;													/* FC-GW rev.4 */
				for (entry=0;entry<8;entry++) {
					if (ap->boot_priority[entry].ww_name == parm) {
						hit = 1;
						break;
					}
				}
				

				if (hit) {
					hit=0;
					
					if ( (!hfc_narrow_dev.ap.ww_name)								/* check 1 */
					 || (hfc_narrow_dev.ap.bus > ap->pci_cfginf->bus->number)		/* check 2 */
					 || ((hfc_narrow_dev.ap.bus == ap->pci_cfginf->bus->number)		/* check 3 */
					     && (hfc_narrow_dev.ap.devfn > ap->pci_cfginf->devfn))
					 || ((hfc_narrow_dev.ap.bus == ap->pci_cfginf->bus->number)		/* check 4 */
					     && (hfc_narrow_dev.ap.devfn == ap->pci_cfginf->devfn)
						 && (hfc_narrow_dev.priority > entry)) ) {
						
						/* remove other target */
						wk_target = hfc_hash_target_valid(ap, 0);
						
						if ((wk_target != NULL) && (wk_target != target)) {
							hfc_clear_target_info(ap, wk_target, FALSE, FALSE);
							hfc_release_seq_num(ap, wk_target->pseq);
							wk_target->flags=0;
							ap->target_cnt--;
						}
						
						hfc_narrow_dev.ap.ww_name   = ap->ww_name;
						hfc_narrow_dev.ap.bus       = ap->pci_cfginf->bus->number;
						hfc_narrow_dev.ap.devfn     = ap->pci_cfginf->devfn;
						hfc_narrow_dev.priority     = entry;
						hfc_narrow_dev.tgt.ww_name  = parm;
						hfc_narrow_dev.tgt.lun      = 0;
						tid_hit=3;
						tid = 0;
						
						sprintf(buf1,"%llx", (unsigned long long)hfc_narrow_dev.tgt.ww_name);
						HFC_INFPRT("hfcldd%d : narrow mode(wwpn) - HBA WWPN=%llx(%02x:%02x.%x), Priority=%d, Target WWPN=%s\n",
								ap->dev_minor,
								(unsigned long long) hfc_narrow_dev.ap.ww_name,
								ap->pci_cfginf->bus->number, 
								PCI_SLOT(ap->pci_cfginf->devfn),
								PCI_FUNC(ap->pci_cfginf->devfn),
								hfc_narrow_dev.priority,
								buf1 );
					}
				}
			}

			if (tid_hit || empty_hit) {
				target->target_id      = tid;
				target->ww_name        = hfc_read_val(ap->mb->mb_resp.type.drvioctl1.login.port_name);
				target->node_name      = hfc_read_val(ap->mb->mb_resp.type.drvioctl1.login.node_name);
				target->fc_class_mask  = (ushort) hfc_read_val(ap -> mb -> mb_resp.type.drvioctl1.login.class_mask);
				target->device_flags   = (ushort) hfc_read_val(ap -> mb -> mb_resp.type.drvioctl1.login.parameter );
				target->max_frame_size = (ushort) hfc_read_val(ap -> mb -> mb_resp.type.drvioctl1.login.max_frame_size);
				set_bit(HFC_DEVFLG_VALID, (ulong *)&target->flags);
				ap->tid_map[tid]       = target->pseq;
				for (i=0;i<8;i++) {
					if (target->fc_class_mask & (0x80>>i))
						target->fc_class = 8-i;
				}
				
				target->ap             = ap;
				target->dev            = NULL;
				target->group_id       = 0xff;
				target->attribute      = 0xff;
				target->path_id        = 0xff;
				target->lg_target      = NULL;
				
				if (tid_hit == 2)																/* FCLNX-0405 */
					set_bit(HFC_TARGET_GHOST, (ulong *)&target->flags);							/* FCLNX-0405 */

				HFC_DBGPRT(" tid_hit or empty_hit : tid=%d, ww_name = %llx, node_name=%llx flag=%x, preq=%d\n",
						target->target_id, (unsigned long long)target->ww_name, (unsigned long long)target->node_name,
						target->flags, target->pseq);
				
				if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
					hfc_manage_info.npubp->hfc_write_retries(ap, target);
				}
				
				if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
					hfc_manage_info.npubp->hfc_update_attribute(ap, target);
				}
			}
			else {
				/* LOGIN succeeded, but there is no target with specified target ID. */
				HFC_DBGPRT( "wwnverify_login() login succeeded, but the target id does not exist\n");

				if( (ap->automap == 0) && (tid_hit == 0) ){ /* Persistent binding info is mismatch */
					HFC_DBGPRT(" wwnverify_login() persistent binding info is not match \n");
					HFC_DBGPRT("    tid=%d ww_name=%llx, node_name=%llx, flag=%x, pseq=%d\n",
								target->target_id, (unsigned long long)target->ww_name, (unsigned long long)target->node_name, target->flags, target->pseq);
					hfc_clear_target_info(ap, target, TRUE, TRUE);
					target->flags=0;
					hfc_release_seq_num(ap, target->pseq);
				}
				else{
					HFC_DBGPRT(" wwnverify_login() target ww_name has changed \n");
					HFC_DBGPRT("    tid=%d ww_name=%llx, node_name=%llx, flag=%x, pseq=%d\n",
								target->target_id, (unsigned long long)target->ww_name, (unsigned long long)target->node_name, target->flags, target->pseq);
					rtn = hfc_clear_target_info( ap, target, TRUE, TRUE );				
				}
			}
		}																		/* FCWIN-0148 */
		break;


	case SCS_LOGIN_WWCHG :							/* WWN mismatch */
		HFC_DBGPRT("wwnverify_login() - login ww_name change.");

		/* LOGIN succeeded, but target->ww_name does not match with WNN in LOGIN parameter */
		rtn = hfc_clear_target_info( ap, target, TRUE, TRUE );

		hfc_wwnverify_linkup(ap, NULL, 0, ww_name);			/* Search target once more */
		break;

	default :
		/* Remove target_info due to abnormal end of LOGIN process*/
		HFC_DBGPRT( "wwnverify_login() - login fail.");
		hfc_clear_target_info( ap, target, TRUE, TRUE );	
	}

	hit = 0;
	for (j=0;j<MAX_TARGET_PROBE;j++) {
		if ( (ap->target_scan[j].flags & (HFC_SCAN_SCV | HFC_SCAN_LOGIN)) == HFC_SCAN_SCV)
			hit = 1;	/* There remain targets needs LOGIN */
	}

	if (!hit){
		HFC_DBGPRT(  "set ap initialize = 0\n");
		if(ap->initialize != 0){
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		}
	}
	
	HFC_EXIT("hfc_wwnverify_login");

}


void hfc_wwnverify_scn(struct adap_info *ap, struct target_info *target, uint mb_resp_status)
{

	HFC_ENTRY("hfc_wwnverify_scn");

	if (    (ap->connect_type == HFC_SWITCH ) /* FC-SW ? */
		|| ((ap->connect_type == HFC_AL) && (ap -> scsi_id & 0x00ffff00)) ) {/* FCWIN-0185 */
		if (!test_bit(HFC_WAIT_NMSRV, (ulong *)&ap->status) ) {
			set_bit(HFC_NEED_NMSRV, (ulong *)&ap->status);		/* Reserve GID_FT */
		}
	}

	HFC_EXIT("hfc_wwnverify_scn");
}


struct target_info *hfc_add_target_info( struct adap_info *ap, uint64_t scsi_id )
{
	struct target_info *target;
	uint pseq;

	HFC_ENTRY("hfc_add_target_info");

	if ( (pseq = hfc_uniq_seq_num( ap )) == 0xFFFFFFFF )
		return (NULL);

	target = ap->target_arg[pseq];

	memset( target, 0, sizeof(struct target_info) );
	set_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags);
	target -> pseq      = (uchar) pseq;
	target -> scsi_id   = (uint64_t) scsi_id;
	ap->target_cnt++;


	HFC_DBGPRT( " target exist target = %lx\n", *((ulong *)&target));
	HFC_DBGPRT( "target  = %d, %d, %d \n",(uint)target->flags, (uint)target->pseq, (uint)target->scsi_id);

	HFC_EXIT("hfc_add_target_info");

	return (target);
}



int hfc_clear_target_info( struct adap_info *ap, struct target_info *target, int check, int pmsg )
{
	uint i,tid,pseq;

	HFC_ENTRY("hfc_clear_target_info");

	tid  = target->target_id ;
	pseq = target->pseq ;
	
	set_bit(HFC_WAIT_TARGET_STOP, (ulong *)&target->status);	/* Set target-stop-waiting bit */

	if (check == TRUE) {
		for (i=0;i<MAX_DEV_CNT;i++) {
			if ( target->lustat[i] ) {

				HFC_DBGPRT("clear_target_info() - lustat[%d] = 0x%x.", i, target->lustat[i]);
				return (1);
			}
		}

		if ( target -> we_que_cnt ) {

			HFC_DBGPRT("clear_target_info() - we_que_cnt = 0x%x.", target -> we_que_cnt);
			return (2);		/* SCSI initiation process is in progress */
		}

		if ( target -> wx_que_cnt ) {

			HFC_DBGPRT("clear_target_info() - wx_que_cnt = 0x%x.", target -> we_que_cnt);
			return (3);		/* Waiting SCSI response */
		}


		while ( test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status)		|| 
				test_bit(HFC_WAIT_CANCEL, (ulong *)&target->status)		|| 		/* FCLNX-GPL-038 */
				test_bit(HFC_WAIT_PDISC, (ulong *)&target->status)		|| 
				test_bit(HFC_WAIT_TARGET_RESET, (ulong *)&target->status) ||
				test_bit(HFC_WAIT_GIDPN, (ulong *)&target->status) ){
				
			HFC_DBGPRT( "clear_target_info() - target->status = 0x%x.", target->status);
			return (4);		/* Other process is using mailbox now */
		}

	}

	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
		hfc_manage_info.npubp->hfc_fo_check_and_offline(target, pmsg);
	}

	/* Delete Target_info */
	hfc_watchdog_enter(ap, target, NULL, 0, HFC_DELAY_TMR,      0, TRUE);	/* Cancel Timer */
	hfc_watchdog_enter(ap, target, NULL, 0, HFC_SCN_LINKUP_TMR, 0, TRUE);	/* Cancel Timer */

	hfc_deque_login_req( ap, target );			/* Release LOGIN waiting state */
	hfc_deque_pdisc_req( ap, target );			/* Release PDISC waiting state */
	target->status = 0 ;
	clear_bit( HFC_NEED_TARGET_RESET, (ulong *)&target->target_reset );		/* FCLNX-GPL-036 */
	clear_bit(HFC_WWN_VALID, (ulong *)&target->flags); /* temp */

#ifdef HFC_SYSFS_FCGW 													/* FC-GW */
	hfc_rport_delete(target);							/* FC-GW */
#endif /* HFC_SYSFS_FCGW */				/* FC-GW */

	if ( (test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags)
		 && !test_bit(HFC_DEVFLG_VALID, (ulong *)&target->flags))
		|| test_bit(HFC_TARGET_GHOST, (ulong *)&target->flags) )					/* FCLNX-0405 */
	{
		hfc_release_seq_num(ap, target->pseq);
		target->flags = 0;
		ap->target_cnt--;
	}

	HFC_EXIT("hfc_clear_target_info");

	return (0);
}

/*
 * Function: hfc_info
 *
 * Purpose: Output driver infotmation to syslog
 *
 * Arguments:
 *  host  - Pointer to Scsi_Host
 *
 * Returns:
 *
 * Notes:   Pointer to output character string
 */
const char *hfc_info (struct Scsi_Host *host)
{
	static char buffer[512];
	struct hfc_vpd	*vpd_info;
	struct adap_info	*ap;

#if defined(__x86_64) && !defined(_HFC_FCGW) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	/* register ioctl32 */
	x86_64_32_reg_increasing();
#endif /*__x86_64 && !_HFC_FCGW */

	ap = (struct adap_info *)host->hostdata;

	buffer[0] = '\0';

	vpd_info = (struct hfc_vpd *)ap->mp_adap_info->vpd_buf;

	memset(&buffer[0], 0, sizeof(buffer));

	sprintf(&buffer[strlen(buffer)], "Hitachi PCI to Fibre Channel Host Adapter: device %02x:%02x.%02x IRQ %d \n",
				ap->pci_cfginf->bus->number, 
				PCI_SLOT(ap->pci_cfginf->devfn),
				PCI_FUNC(ap->pci_cfginf->devfn),
				 host->irq);
	sprintf(&buffer[strlen(buffer)], "      Firmware version %x, Driver version %s \n",
				ap->mp_adap_info->sys_rev, hfc_manage_info.package_ver);
	
	if(ap->automap == 0) sprintf(&buffer[strlen(buffer)], "       persistent binding is enabled");
	else				 sprintf(&buffer[strlen(buffer)], "       persistent binding is disabled");
	sprintf(&buffer[strlen(buffer)], "  (%016lx)\n",jiffies);									/* FCLNX-0266 */

	if(ap->defparam)     sprintf(&buffer[strlen(buffer)], "       default parameter is enabled\n");

	if(ap->narrowmap == 1)		sprintf(&buffer[strlen(buffer)], "       narrow mode(lun) is enabled\n");	/* DPM */
	else if(ap->narrowmap == 2)	sprintf(&buffer[strlen(buffer)], "       narrow mode(wwpn) is enabled\n");	/* FC-GW */

	sprintf(&buffer[strlen(buffer)], "        hfcl%d-wwpn=0x%llx\n",ap->instance,(unsigned long long)ap->ww_name);
	
	if ( hfc_manage_info.hfcldd_mp_mod ){		/* FCLNX-GPL-394 */
//	if ( hfc_manage_info.hfcplus_enable ){

		if ( ap->isol_force == HFC_PRT_FRC_ISOL ){
				sprintf(&buffer[strlen(buffer)], "        Adapter WWPN  0x%llx was forced to be isolated.\n",
				(unsigned long long)ap->ww_name);	
		}

		if ( ap->isol_force == HFC_CHKSTP_FRC_ISOL ){	/* FCLNX-GPL-147*/
				sprintf(&buffer[strlen(buffer)], "        Adapter WWPN  0x%llx was forced to be isolated.(CHK-STP).\n",
				(unsigned long long)ap->ww_name);
		}
	}											/* FCLNX-GPL-394 */

	return (buffer);
}

 
#undef SPRINTF
// #define SPRINTF(args...) { if (pos < (buffer + length)) pos += sprintf (pos, ## args); }
#define SPRINTF(args...)	{  	len = snprintf (buf, 256, ## args); \
								if (len > 256) { \
									len  = sprintf (buf, "No output: not enough space available to output to string character\n"); \
								} \
								if (pos + len > offset + length) \
									len = offset + length - pos; \
								if (pos + len < offset) { \
									pos += len; \
								} \
								else { \
									if (pos < offset) { \
										partial = offset - pos; \
										pos += partial; \
										len  -= partial; \
										if (len > 0) { \
											memcpy(data, buf+partial, len); \
											pos += len; \
											data += len; \
										} \
									} \
									else if (len > 0) { \
										memcpy(data, buf, len); \
										pos += len; \
										data += len; \
									} \
								} \
							}

int
hfc_proc_info(char *buffer, char **start, off_t offset, int length, int hostno, int inout)
{
	struct Scsi_Host 	*host;
	struct adap_info 	*ap;
	struct mp_adap_info	*mdap;

	host = NULL;
	/* Find the specified Host */
	for ( mdap = hfc_manage_info.mp_adap_info; mdap != NULL; mdap = mdap->next ){
		for (  ap = mdap->ap; ap != NULL; ap = ap->next){
			if( ap->hosts->host_no != hostno )  continue;
			host = ap->hosts;
			return hfc_proc_info_k26(host, buffer, start, offset, length, inout);
		}
	}
	
	return -ESRCH;

}	
			
/*
 * Function:    hfc_proc_info_k26
 *
 * Purpose:
 *
 * Arguments:
 *  host		- Pointer to Scsi_Host
 *  buffer		- Data input/output pointer
 *  start		- Data output pointer
 *  offset		- Offset from buffer start address
 *  length		- Buffer length
 *  inout		- TRUE (RD), FALSE (WR)
 *
 * Returns:   
 *    < 0		- Error number
 *    >=0       - Dara length
 * Notes:
 */
int
hfc_proc_info_k26(struct Scsi_Host *host, char *buffer, char **start, off_t offset, int length, int inout)
{

	struct hfc_vpd		*vpd_info=NULL;
	struct hfc_vpd_five	*vpdf_info=NULL;
	struct adap_info	*ap;
	struct target_info	*tp;
	short				vendor_id, device_id;
	int					i,j,hit;
	uchar				pre_conf=0;
	int					rtn;
	char				buf[256];
	char				*data = buffer;
	int					len, partial, pos = 0;
	struct target_info	*target;	/* FCLNX-GPL-394 */

	
	if (!host){ /* This host is not found */
		return (-EINVAL);
	}
	
	if (inout) {
		return (length);
	}
	

	ap = (struct adap_info *)host->hostdata;
	
	SPRINTF ("Hitachi PCI to Fibre Channel Host Bus Adapter\n");
	
	if(ap->pkg.type == HFC_PKTYPE_FPP){
		vpd_info = (struct hfc_vpd *)ap->mp_adap_info->vpd_buf;
		SPRINTF ("  Driver version %s  Firmware version %06x\n",vpd_info->driver_ver, vpd_info->fw_ver);
	
	}
	else{
		vpdf_info = (struct hfc_vpd_five *)ap->mp_adap_info->vpd_buf;
		SPRINTF ("  Driver version %s  Firmware version %06x\n",vpdf_info->driver_ver, vpdf_info->fw_ver);
	
	}
	SPRINTF ("  Package_ID              = 0x%02x\n",ap->pkg.code);
	SPRINTF ("  Special file name       = hfcldd%d\n", ap->dev_minor);
	
	SPRINTF ("  Major_number            = %d\n", ap->dev_major);
	SPRINTF ("  Minor_number            = %d\n", ap->dev_minor);			
	SPRINTF ("  Instance_number         = %d\n",ap->instance);						
	SPRINTF ("  Host# = %d, Unique id   = %d \n", host->host_no, host->unique_id);
	SPRINTF ("  PCI memory space address= 0x%08llx (%d)\n", (unsigned long long)ap->mem_base_addr, (int)sizeof(dma_addr_t));
	
	SPRINTF("  Adapter information \n");
	
	/* read config register (0x00 - 0x03) */
	vendor_id = (ushort) hfc_read_cnfg(ap, 0x00, 0x2);
	device_id = (ushort) hfc_read_cnfg(ap, 0x02, 0x2 );
	
	SPRINTF ("   Vender ID              =  %x\n", vendor_id);
	SPRINTF ("   Device ID              =  %x\n", device_id);
	SPRINTF ("   Port name              =  %llx \n", (unsigned long long)ap->ww_name);
	SPRINTF ("   Node name              =  %llx \n", (unsigned long long)ap->node_name);
	SPRINTF ("   DID                    =  %06llx \n", (unsigned long long)ap->scsi_id);
	
	if(ap->pkg.type == HFC_PKTYPE_FPP)			pre_conf = (uchar) hfc_read_cnfg(ap, 0x4B, 0x1);
	else if(ap->pkg.type == HFC_PKTYPE_FIVE)	pre_conf = (uchar) hfc_read_cnfg(ap, 0xAF, 0x1);
	
	if( (pre_conf == 0x01) || (pre_conf == 0x03) ){
		SPRINTF ("   Original port name     =  %llx \n", (unsigned long long)ap->org_ww_name);
		SPRINTF ("   Original node name     =  %llx \n", (unsigned long long)ap->org_node_name);
		SPRINTF ("   Add port name          =  %llx \n", (unsigned long long)ap->add_ww_name);
		SPRINTF ("   Add node name          =  %llx \n", (unsigned long long)ap->add_node_name);
		SPRINTF ("   hfc_pxe_boot           =  %d \n", hfc_pxe_boot);			
	}
	
	if ( HFC_MMODE_CHECK_MLPF(ap) ) {
		SPRINTF ("   MLPF MODE              =  %x \n", ap->mlpf_mode);	
		if ( !(HFC_MMODE_CHECK_DEDICATE(ap)) ) {
			SPRINTF ("   Original port name     =  %llx \n", (unsigned long long)ap->org_ww_name);
			SPRINTF ("   Original node name     =  %llx \n", (unsigned long long)ap->org_node_name);
		}
		SPRINTF ("   VFC port name          =  %llx \n", (unsigned long long)ap->vfc_ww_name);
		SPRINTF ("   VFC node name          =  %llx \n", (unsigned long long)ap->vfc_node_name);
		if ( !(HFC_MMODE_CHECK_DEDICATE(ap)) ) {
			SPRINTF ("   RID                    =  %x \n", ap->rid);
		}
	}
	
	SPRINTF ("   adapter ID             =  ");
	for(i=0; i<16; i++) SPRINTF("%02x",ap->mp_adap_info->adap_id[i]);
		SPRINTF("\n");
		
	SPRINTF ("   port number            =  %d\n", ap->port_no);
	
	if(ap->pkg.type == HFC_PKTYPE_FPP){
		char wkbuf[VPD_PN_LEN+13];  /* +13=NULL */						/* FCLNX-0337 *//* FCLNX-0368 */
		
		SPRINTF ("   manufacturer ID        =  ");
		for(i=0; i<vpd_info->mn_len;i++)
			 SPRINTF("%c",vpd_info->mn_value[i]);
		SPRINTF("\n");
		
		SPRINTF ("   parts number           =  ");
		memset(wkbuf,0,sizeof(wkbuf));									/* FCLNX-0337 *//* FCLNX-0368 */
		memcpy(wkbuf,vpd_info->pn_value,VPD_PN_LEN);					/* FCLNX-0337 *//* FCLNX-0368 */
		SPRINTF("%s\n",wkbuf);											/* FCLNX-0337 *//* FCLNX-0368 */
		
		SPRINTF ("   ec level               =  ");
		SPRINTF("%c",vpd_info->ec_value[0]);
		SPRINTF("\n");		
		
		SPRINTF ("   model name             =  %s \n", ap->mp_adap_info->model_name);	/* FCLNX-0329 */
		
	}else{
		char wkbuf[VPD_PN_LEN+13];  /* +13=NULL */						/* FCLNX-0337 *//* FCLNX-0368 */
		
		SPRINTF ("   manufacturer ID        =  ");
		for(i=0; i<vpdf_info->mn_len;i++)
				SPRINTF("%c",vpdf_info->mn_value[i]);
		SPRINTF("\n");
		
		SPRINTF ("   parts number           =  ");
		memset(wkbuf,0,sizeof(wkbuf));									/* FCLNX-0337 *//* FCLNX-0368 */
		memcpy(wkbuf,vpdf_info->pn_value,VPD_PN_LEN);					/* FCLNX-0337 *//* FCLNX-0368 */
		SPRINTF("%s\n",wkbuf);											/* FCLNX-0337 *//* FCLNX-0368 */
		
		SPRINTF ("   ec level               =  ");
		SPRINTF("%c",vpdf_info->ec_level);
		SPRINTF("\n");
		
		SPRINTF ("   model name             =  %s \n", ap->mp_adap_info->model_name);		/* FCLNX-0329 */
		SPRINTF ("   location               =  %02x:%02x.%02x\n", 							/* FCLNX-0404 */
												ap->pci_cfginf->bus->number,				/* FCLNX-0404 */
												PCI_SLOT(ap->pci_cfginf->devfn),			/* FCLNX-0404 */
												PCI_FUNC(ap->pci_cfginf->devfn));			/* FCLNX-0404 */
		
	}

	if( ap->raslog_install == 0 ){
		SPRINTF ("   Raslog version         =  raslog-%d.%d.%d-%d \n", ap->raslog_ver, ap->raslog_rev,
						ap->raslog_rver, ap->raslog_wver);
	}

	SPRINTF ("  Current Information \n");
	
	SPRINTF ("   Connection Type        = ");
	switch(ap->connect_type) {
		case HFC_SWITCH	:	if(test_bit(HFC_ONLINE, (ulong *)&ap->status)){
								SPRINTF (" Point to Point (fabric) \n");
							}
							else{
								SPRINTF (" - \n");
							}
							break;
		case HFC_AL		:	if(test_bit(HFC_ONLINE, (ulong *)&ap->status)){
								if (ap -> scsi_id & 0x00ffff00) {
									SPRINTF (" FC-AL (fabric) \n");
								}
								else{
									SPRINTF (" FC-AL \n");
								}
							}
							else{
								SPRINTF (" - \n");
							}
							break;
		
		case HFC_PT2PT	:	if(test_bit(HFC_ONLINE, (ulong *)&ap->status)){
								SPRINTF (" Point to Point \n");
							}
							else{
								SPRINTF (" - \n");
							}
							break;
		default			:	SPRINTF (" - \n");
	}
	
	SPRINTF ("   Link Speed             = ");
	if(test_bit(HFC_ONLINE, (ulong *)&ap->status)){
		SPRINTF (" %dGbps\n", (ap->max_data_rate/100));
	}
	else{
		SPRINTF (" - \n");
	}
	
	SPRINTF ("   Max Transfer Size      =  %dMB\n", (ap->dma_max/(1024*1024)) );
	SPRINTF ("   Link Down Time         =  %dsec\n", ap->linkup_tmo);
	SPRINTF ("   Reset Delay Time       =  %dsec\n", ap->scsi_reset_delay);
	SPRINTF ("   Machine Check Retry Count  =  %d\n", ap->max_mck_cnt);			/* FCLNX_011 */
	SPRINTF ("   Preferred AL-PA Number =  %02x\n", ap->pref_alpa);				/* FCLNX_011 */
	SPRINTF ("   Reset Timeout          =  %dsec\n", ap->target_reset_tmo);  	/* FCLNX_011 */
	SPRINTF ("   Abort timeout          =  %dsec\n", ap->abort_tmo);			/* FCLNX_011 */
	SPRINTF ("   Queue depth            =  %d\n", ap->queue_depth);
	SPRINTF ("   Enable Message         =  %d\n", ap->wmsg);
	SPRINTF ("   Link Down Time (MCK)   =  %dsec\n", ap->linkup2_tmo);				/* FCLNX-0241*/
	SPRINTF ("   Login delay            =  %dsec\n", ap->login_wait);				/* FCLNX-0243*/
	
	SPRINTF ("   Max target number      =  %d\n", ap->max_target);
	SPRINTF ("   Max xob number         =  %d\n", ap->xob_max);
	SPRINTF ("   Max xrb number         =  %d\n", ap->xrb_max);
	SPRINTF ("   Max soft log number    =  %d\n", ap->slog_max);
	SPRINTF ("   Max trace number       =  %d\n", ap->trc_max);
	SPRINTF ("   Max pkt number         =  %d\n", ap->pkt_num);
	SPRINTF ("   Max can queue          =  %d\n", ap->can_queue);
	SPRINTF ("   Max sg tablesize       =  %d\n", ap->sg_tblsize);
	SPRINTF ("   Max cmnd number        =  %d\n", ap->cmnd_num);
	SPRINTF ("   Minus timeout          =  %d\n", ap->minus_tout);
	SPRINTF ("   Scsi allowed           =  %d\n", ap->scsi_allowed);
	SPRINTF ("   Max cmd per lun        =  %d\n", ap->cmd_per_lun);
	SPRINTF ("   Max sectors            =  %d\n", ap->max_sectors);
	SPRINTF ("   Target reset flag      =  %d\n", ap->enable_tgtrst);
	SPRINTF ("   Scsi timeout parameter =  %d\n", ap->scsi_time_out);
	SPRINTF ("   Mailbox Time-Out Retry =  %d\n", ap->to_reset_retry);		/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */

#if _HFC_DEBUG_DET_00
	SPRINTF ("   Seg info trace mode    =  %d\n", ap->fw_parm);
#endif

	/*SPRINTF ("   Enable failover       = no \n");  */				/* FCLNX_011 */
	SPRINTF ("   Total number of the     \n");
	SPRINTF ("        target devices    =  %d \n", ap->target_cnt);

	SPRINTF ("   Current adapter status =  %08x (",ap->status);				/* FCLNX_011 */
	if(test_bit(HFC_ENABLE, (ulong *)&ap->status)) SPRINTF ("enable ");		/* FCLNX_011 */
	if(test_bit(HFC_ONLINE, (ulong *)&ap->status)) SPRINTF ("online ");		/* FCLNX_011 */
	if(test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) SPRINTF ("check stop");/* FCLNX_011 */
	SPRINTF (")\n");														/* FCLNX_011 */
	
	SPRINTF ("   FC Port Status         =  ");								/* FCLNX_0538 */
	switch(ap->isol_detail){
		case HFC_NO_ISOLATE:
			if( test_bit(HFC_ONLINE, (ulong *)&ap->status) ) {
				if( test_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status)
				 || test_bit(HFC_WAIT_LINK_INIT, (ulong *)&ap->status)){	/* FCLNX-0488 */
					SPRINTF ("WaitLinkUp");									/* FCLNX-0488 */
				}
				else{														/* FCLNX-0488 */
					SPRINTF ("LinkUp");										/* FCLNX-0488 */
				}
			}
			else{
				if(test_bit(HFC_CHK_STOP, (ulong *)&ap->status)){
#ifndef _HFC_FCGW /* FCLNX-699 */
					SPRINTF ("Isolate(CHK-STP)");						/* FCLNX-0592 *//* FCLNX-699 */
#else	/* _HFC_FCGW */ /* FCLNX-699 */
					SPRINTF ("LinkDown");								/* FCLNX-0699 */
#endif	/* _HFC_FCGW */ /* FCLNX-699 */
				}
				else if(test_bit(HFC_WAIT_LINK_INIT, (ulong *)&ap->status)){
					SPRINTF ("WaitLinkUp");
				}
				else{
					SPRINTF ("LinkDown");									/* FCLNX-0488 */
				}
			}
			break;
		case HFC_ISOLATE_PORT_C:
			SPRINTF ("Isolate(C)");											/* FCLNX-0488 */
			break;
		case HFC_ISOLATE_PORT_E:
			if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
				SPRINTF ("Isolate(CHK-STP)");								/* FCLNX-0592 */
			}
			else {
				SPRINTF ("Isolate(E)");										/* FCLNX-0488 */
			}
			break;
		case HFC_ISOLATE_SFPFAIL:
			if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
				SPRINTF ("Isolate(CHK-STP)");								/* FCLNX-0592 */
			}
			else {
				SPRINTF ("Isolate(SFP Fail)");								/* FCLNX-0488 */
			}
			break;
		case HFC_ISOLATE_SFPNOTSUPPORT:
			if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
				SPRINTF ("Isolate(CHK-STP)");								/* FCLNX-0592 */
			}
			else {
				SPRINTF ("Isolate(SFP not support)");							/* FCLNX-0488 */
			}
			break;
		case HFC_ISOLATE_SFPDOWN:
			if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
				SPRINTF ("Isolate(CHK-STP)");								/* FCLNX-0592 */
			}
			else {
				SPRINTF ("Isolate(SFP Down)");									/* FCLNX-0488 */
			}
			break;
		case HFC_ISOLATE_CHKSTP:
			SPRINTF ("Isolate(CHK-STP)");									/* FCLNX-0488 */
			break;
		default:
			SPRINTF ("Unknown status");										/* FCLNX-0488 */
			break;
	}
	SPRINTF ("\n");
	
	SPRINTF ("   Path Change Queue Count =  %08x   \n",ap->retry_hfcp_count);
	
	SPRINTF ("  \n");
	SPRINTF ("  Device Information \n");
	i=0;
	
	for(j=0; j<(ap->max_target); j++){
	
		tp = ap->target_arg[j];
		while( tp != NULL){

			if( test_bit(HFC_TARGETINF_VALID, (ulong *)&tp->flags)){

				SPRINTF ("    target id [");
				if(test_bit(HFC_WWN_VALID, (ulong *)&tp->flags)){
					SPRINTF("%d",tp->target_id);
				}
				else{ 
					SPRINTF("-");
				}
				SPRINTF ("] :");
				if(test_bit(HFC_WWN_VALID, (ulong *)&tp->flags)){
					SPRINTF (" port name = %llx node name = %llx DID = %02llx ",
						(unsigned long long)tp->ww_name, (unsigned long long)tp->node_name, (unsigned long long)tp->scsi_id );
				}
				else{
					SPRINTF (" port name =    -    node name =   -     DID = %02llx",(unsigned long long)tp->scsi_id );                            
				}
				
				i++;
			
				SPRINTF (" (pseq = %02d flags=%08x status=%08x)\n", tp->pseq, tp->flags, tp->status);
			}
			
			tp = tp->next;
		}
	}
	
	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
		data = hfc_manage_info.npubp->hfc_proc_info_lun(ap, data, offset, length, &pos);
	}
	
	SPRINTF ("\n");
	SPRINTF ("  FC persistent binding information \n");
	SPRINTF ("   automap is ");
	if(ap->automap == 1){
		SPRINTF (" ON (find configuration automatically) \n");
		
		for(i=0; i<MAX_ADAP_CNT; i++){
			if(hfc_manage_info.adap_bind[i] != -1){
				SPRINTF("   hfcl%d-wwpn=%llx\n",i,(unsigned long long)hfc_manage_info.adap_bind[i]);
			}
		}
	}
	else{
	
		SPRINTF (" OFF (establish configuration based on the presistent binding information\n");			
		SPRINTF ("  \n");

		hit=0;
		for(i=0; i<MAX_ADAP_CNT;i++){
			if(hfc_manage_info.adap_bind[i] != -1){
				SPRINTF ("   hfcl%d-wwpn=%llx\n", i, (unsigned long long)hfc_manage_info.adap_bind[i]);
				hit++;
			}
		}
		
		hit=0;
		for(j=0; j<(ap->max_target); j++){
			if( ap->bind_target_id->type[j] != 0){ 
				hit++;
				switch(ap->bind_target_id->type[j]){
					case HFC_TID_WWPN : 
						SPRINTF ("   hfcl%d-tid-%d-wwpn=%llx\n", ap->instance, j, (unsigned long long)ap->bind_target_id->parm[j]);
						break;
					case HFC_TID_WWNN : 
						SPRINTF ("   hfcl%d-tid-%d-wwnn=%llx\n", ap->instance, j, (unsigned long long)ap->bind_target_id->parm[j]);
						break;
					case HFC_TID_DID : 
						SPRINTF ("   hfcl%d-tid-%d-did=%llx\n", ap->instance, j, (unsigned long long)ap->bind_target_id->parm[j]);
						break;
				}					 
			}
		}
		if(hit==0) SPRINTF("   None of target bindings are specified \n");
	}
	
	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
		data = hfc_manage_info.npubp->hfc_proc_info_option(data, offset, length, &pos);
	}
	else
	{
		SPRINTF ("   HFC-PCM                       = OFF\n");
		if (hfc_manage_info.hfcplus_enable)                /* FCLNX-0642 */
                        SPRINTF("   E-Option                      = ON\n");             /* FCLNX-0642 */
	}
	
	if((ap->ld_err_limit_s)||(ap->if_err_limit )||(ap->to_err_limit )||(ap->rt_err_enable)){	/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */
		SPRINTF ("   Isolate setting of HBA port:\n");
		if (ap->hba_isolation){
			SPRINTF ("    HBA Isolation                = On\n");
		}
		else{
			SPRINTF ("    HBA Isolation                = Off\n");
		}																/* FCLNX-GPL-349 */
		if (ap->ld_err_limit_s) {
			SPRINTF ("    Linkdown Error(S)            Limit:%d  Count:%d\n", ap->ld_err_limit_s, ap->ld_err_count_s);
		}
		if (ap->if_err_limit ) {
			SPRINTF ("    Interface Error              Limit:%d  Count:%d\n", ap->if_err_limit, ap->if_err_count);
		}
		if (ap->to_err_limit ) {
			SPRINTF ("    TimeOut Error                Limit:%d  Count:%d\n", ap->to_err_limit, ap->to_err_count);
		}
		if (ap->rt_err_enable) {
			if(ap->c_err == HFC_ISOLATE_RT){
				SPRINTF ("    TimeOutReset Error           = Enable (Error occurred)\n");
			}
			else{
				SPRINTF ("    TimeOutReset Error           = Enable (No error)\n");
			}
		}
			
		if((ap->connect_type == HFC_SWITCH)||((ap->connect_type == HFC_AL) && (ap -> scsi_id & 0x00ffff00))) {
			for(i=0; i<(ap->max_target); i++){	
				target = ap->target_arg[i];
				if( target != NULL){
					if( test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags)){
						if (ap->ld_err_limit_s ){
							SPRINTF ("   Isolate setting of Target port(Target: %d,  WWPN: %llx):\n", target->target_id, (unsigned long long)target->ww_name);
							SPRINTF ("    Linkdown Error(S)            Limit:%d  Count:%d\n", ap->ld_err_limit_s, target->tgt_ld_err_count_s);
						}
					}
				}
			}
		}
	}															/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */


	SPRINTF ("  --- \n");

	
    /*
     * Calculate start of next buffer, and return value.
     */

	if (start) {
		*start = buffer;
	}


	rtn = pos > offset ? pos - offset : 0;


	return (rtn);

}

int hfc_chk_conf_val(int min, int max, int val)
{
	if( val == -1) return 0;  /* not defined */
	if( (min <= val) && (val <= max) ){
		return (1);
	}

	return 0;

}


int hfc_chk_conf_ls(int min, int max, int val)
{
	if( val == -1) return 0;  /* not defined */
	if( (min <= val) && (val <= max) ){
		if(val == 0) return (1); /* 0 :  Auto */
		if(val == 1) return (1); /* 1 : 1Gbps */
		if(val == 2) return (1); /* 2 : 2Gbps */
		if(val == 4) return (1); /* 4 : 4Gbps */
	}
	return 0;
}



int hfc_chk_conf_mt(int min, int max, int val)
{
	if( val == -1) return 0;  /* not defined */
	if( (min <= val) && (val <= max) ){
		if(val == 1) return (1); /* 1 :  1MB */
		if(val == 4) return (1); /* 4 :  4MB */
		if(val == 8) return (1); /* 8 :  8MB */
		if(val ==16) return (1); /*16 : 16MB */
	}
	return 0;
}



void hfc_set_topology(struct adap_info *ap)
{
	uchar cnv[3]={HFC_UNKN,HFC_PT2PT,HFC_AL}; /* connection type */

	if(ap->defparam) return;

    /* 
	   Attention!! : The BIOS data might already been set. 
                                                           */

	if(hfc_chk_conf_val(1,2,hfc_connection_type)){ /* in case global parameter is set */
		ap->topology  = cnv[hfc_connection_type];	/* 1:P2P, 3:FC-AL */
	}

	/* in case local parameter is set */
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(1,2, hfc0_connection_type) ) ap->topology  =  cnv[hfc0_connection_type]; break;
		case  1 : if( hfc_chk_conf_val(1,2, hfc1_connection_type) ) ap->topology  =  cnv[hfc1_connection_type]; break;
		case  2 : if( hfc_chk_conf_val(1,2, hfc2_connection_type) ) ap->topology  =  cnv[hfc2_connection_type]; break;
		case  3 : if( hfc_chk_conf_val(1,2, hfc3_connection_type) ) ap->topology  =  cnv[hfc3_connection_type]; break;
		case  4 : if( hfc_chk_conf_val(1,2, hfc4_connection_type) ) ap->topology  =  cnv[hfc4_connection_type]; break;
		case  5 : if( hfc_chk_conf_val(1,2, hfc5_connection_type) ) ap->topology  =  cnv[hfc5_connection_type]; break;
		case  6 : if( hfc_chk_conf_val(1,2, hfc6_connection_type) ) ap->topology  =  cnv[hfc6_connection_type]; break;
		case  7 : if( hfc_chk_conf_val(1,2, hfc7_connection_type) ) ap->topology  =  cnv[hfc7_connection_type]; break;
		case  8 : if( hfc_chk_conf_val(1,2, hfc8_connection_type) ) ap->topology  =  cnv[hfc8_connection_type]; break;
		case  9 : if( hfc_chk_conf_val(1,2, hfc9_connection_type) ) ap->topology  =  cnv[hfc9_connection_type]; break;
		case 10 : if( hfc_chk_conf_val(1,2,hfc10_connection_type) ) ap->topology  = cnv[hfc10_connection_type]; break;
		case 11 : if( hfc_chk_conf_val(1,2,hfc11_connection_type) ) ap->topology  = cnv[hfc11_connection_type]; break;
		case 12 : if( hfc_chk_conf_val(1,2,hfc12_connection_type) ) ap->topology  = cnv[hfc12_connection_type]; break;
		case 13 : if( hfc_chk_conf_val(1,2,hfc13_connection_type) ) ap->topology  = cnv[hfc13_connection_type]; break;
		case 14 : if( hfc_chk_conf_val(1,2,hfc14_connection_type) ) ap->topology  = cnv[hfc14_connection_type]; break;
		case 15 : if( hfc_chk_conf_val(1,2,hfc15_connection_type) ) ap->topology  = cnv[hfc15_connection_type]; break;
		case 16 : if( hfc_chk_conf_val(1,2,hfc16_connection_type) ) ap->topology  = cnv[hfc16_connection_type]; break;
		case 17 : if( hfc_chk_conf_val(1,2,hfc17_connection_type) ) ap->topology  = cnv[hfc17_connection_type]; break;
		case 18 : if( hfc_chk_conf_val(1,2,hfc18_connection_type) ) ap->topology  = cnv[hfc18_connection_type]; break;
		case 19 : if( hfc_chk_conf_val(1,2,hfc19_connection_type) ) ap->topology  = cnv[hfc19_connection_type]; break;
		case 20 : if( hfc_chk_conf_val(1,2,hfc20_connection_type) ) ap->topology  = cnv[hfc20_connection_type]; break;
		case 21 : if( hfc_chk_conf_val(1,2,hfc21_connection_type) ) ap->topology  = cnv[hfc21_connection_type]; break;
		case 22 : if( hfc_chk_conf_val(1,2,hfc22_connection_type) ) ap->topology  = cnv[hfc22_connection_type]; break;
		case 23 : if( hfc_chk_conf_val(1,2,hfc23_connection_type) ) ap->topology  = cnv[hfc23_connection_type]; break;
		case 24 : if( hfc_chk_conf_val(1,2,hfc24_connection_type) ) ap->topology  = cnv[hfc24_connection_type]; break;
		case 25 : if( hfc_chk_conf_val(1,2,hfc25_connection_type) ) ap->topology  = cnv[hfc25_connection_type]; break;
		case 26 : if( hfc_chk_conf_val(1,2,hfc26_connection_type) ) ap->topology  = cnv[hfc26_connection_type]; break;
		case 27 : if( hfc_chk_conf_val(1,2,hfc27_connection_type) ) ap->topology  = cnv[hfc27_connection_type]; break;
		case 28 : if( hfc_chk_conf_val(1,2,hfc28_connection_type) ) ap->topology  = cnv[hfc28_connection_type]; break;
		case 29 : if( hfc_chk_conf_val(1,2,hfc29_connection_type) ) ap->topology  = cnv[hfc29_connection_type]; break;
		case 30 : if( hfc_chk_conf_val(1,2,hfc30_connection_type) ) ap->topology  = cnv[hfc30_connection_type]; break;
		case 31 : if( hfc_chk_conf_val(1,2,hfc31_connection_type) ) ap->topology  = cnv[hfc31_connection_type]; break;
	}

	/* in case local parameter is set */
	if( hfc_chk_conf_val( 1,2,hfcmp_connection_type[ap->instance]) ) {
		ap->topology  = cnv[ hfcmp_connection_type[ap->instance] ];
		HFC_DBGPRT("connect_type <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("connect_type = %d\n", ap->topology);

}


void hfc_set_linkspeed(struct adap_info *ap)
{


	if(ap->defparam) return;

	if(hfc_chk_conf_ls(1,4,hfc_link_speed)){	/* in case global parameter is set */
		ap->linkspeed = hfc_link_speed;			/* 1:1Gbps, 2:2Gbps, 4:4Gbps */
	}

	switch(ap->instance){
		case  0 : if( hfc_chk_conf_ls(1,4, hfc0_link_speed) ) ap->linkspeed =  hfc0_link_speed; break;
		case  1 : if( hfc_chk_conf_ls(1,4, hfc1_link_speed) ) ap->linkspeed =  hfc1_link_speed; break;
		case  2 : if( hfc_chk_conf_ls(1,4, hfc2_link_speed) ) ap->linkspeed =  hfc2_link_speed; break;
		case  3 : if( hfc_chk_conf_ls(1,4, hfc3_link_speed) ) ap->linkspeed =  hfc3_link_speed; break;
		case  4 : if( hfc_chk_conf_ls(1,4, hfc4_link_speed) ) ap->linkspeed =  hfc4_link_speed; break;
		case  5 : if( hfc_chk_conf_ls(1,4, hfc5_link_speed) ) ap->linkspeed =  hfc5_link_speed; break;
		case  6 : if( hfc_chk_conf_ls(1,4, hfc6_link_speed) ) ap->linkspeed =  hfc6_link_speed; break;
		case  7 : if( hfc_chk_conf_ls(1,4, hfc7_link_speed) ) ap->linkspeed =  hfc7_link_speed; break;
		case  8 : if( hfc_chk_conf_ls(1,4, hfc8_link_speed) ) ap->linkspeed =  hfc8_link_speed; break;
		case  9 : if( hfc_chk_conf_ls(1,4, hfc9_link_speed) ) ap->linkspeed =  hfc9_link_speed; break;
		case 10 : if( hfc_chk_conf_ls(1,4,hfc10_link_speed) ) ap->linkspeed = hfc10_link_speed; break;
		case 11 : if( hfc_chk_conf_ls(1,4,hfc11_link_speed) ) ap->linkspeed = hfc11_link_speed; break;
		case 12 : if( hfc_chk_conf_ls(1,4,hfc12_link_speed) ) ap->linkspeed = hfc12_link_speed; break;
		case 13 : if( hfc_chk_conf_ls(1,4,hfc13_link_speed) ) ap->linkspeed = hfc13_link_speed; break;
		case 14 : if( hfc_chk_conf_ls(1,4,hfc14_link_speed) ) ap->linkspeed = hfc14_link_speed; break;
		case 15 : if( hfc_chk_conf_ls(1,4,hfc15_link_speed) ) ap->linkspeed = hfc15_link_speed; break;
		case 16 : if( hfc_chk_conf_ls(1,4,hfc16_link_speed) ) ap->linkspeed = hfc16_link_speed; break;
		case 17 : if( hfc_chk_conf_ls(1,4,hfc17_link_speed) ) ap->linkspeed = hfc17_link_speed; break;
		case 18 : if( hfc_chk_conf_ls(1,4,hfc18_link_speed) ) ap->linkspeed = hfc18_link_speed; break;
		case 19 : if( hfc_chk_conf_ls(1,4,hfc19_link_speed) ) ap->linkspeed = hfc19_link_speed; break;
		case 20 : if( hfc_chk_conf_ls(1,4,hfc20_link_speed) ) ap->linkspeed = hfc20_link_speed; break;
		case 21 : if( hfc_chk_conf_ls(1,4,hfc21_link_speed) ) ap->linkspeed = hfc21_link_speed; break;
		case 22 : if( hfc_chk_conf_ls(1,4,hfc22_link_speed) ) ap->linkspeed = hfc22_link_speed; break;
		case 23 : if( hfc_chk_conf_ls(1,4,hfc23_link_speed) ) ap->linkspeed = hfc23_link_speed; break;
		case 24 : if( hfc_chk_conf_ls(1,4,hfc24_link_speed) ) ap->linkspeed = hfc24_link_speed; break;
		case 25 : if( hfc_chk_conf_ls(1,4,hfc25_link_speed) ) ap->linkspeed = hfc25_link_speed; break;
		case 26 : if( hfc_chk_conf_ls(1,4,hfc26_link_speed) ) ap->linkspeed = hfc26_link_speed; break;
		case 27 : if( hfc_chk_conf_ls(1,4,hfc27_link_speed) ) ap->linkspeed = hfc27_link_speed; break;
		case 28 : if( hfc_chk_conf_ls(1,4,hfc28_link_speed) ) ap->linkspeed = hfc28_link_speed; break;
		case 29 : if( hfc_chk_conf_ls(1,4,hfc29_link_speed) ) ap->linkspeed = hfc29_link_speed; break;
		case 30 : if( hfc_chk_conf_ls(1,4,hfc30_link_speed) ) ap->linkspeed = hfc30_link_speed; break;
		case 31 : if( hfc_chk_conf_ls(1,4,hfc31_link_speed) ) ap->linkspeed = hfc31_link_speed; break;
	}

	if( hfc_chk_conf_ls(1,4,hfcmp_link_speed[ap->instance] ) ) {
		ap->linkspeed = hfcmp_link_speed[ap->instance];
		HFC_DBGPRT("link speed <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("link speed = %d\n", ap->linkspeed);
}


void hfc_set_max_transfer(struct adap_info *ap)
{


	ap->max_transfer = HFC_MAX_TRANSFER; /* AUTO : default */

	/* max transfer */
	if(ap->defparam){
		ap->dma_max = ap->max_transfer*0x100000;
		return;
	}

	if(hfc_chk_conf_mt(0,16,hfc_max_transfer)){ /* in case global parameter is set */
		ap->max_transfer = hfc_max_transfer; 
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_mt(0,16, hfc0_max_transfer) ) ap->max_transfer =  hfc0_max_transfer; break;
		case  1 : if( hfc_chk_conf_mt(0,16, hfc1_max_transfer) ) ap->max_transfer =  hfc1_max_transfer; break;
		case  2 : if( hfc_chk_conf_mt(0,16, hfc2_max_transfer) ) ap->max_transfer =  hfc2_max_transfer; break;
		case  3 : if( hfc_chk_conf_mt(0,16, hfc3_max_transfer) ) ap->max_transfer =  hfc3_max_transfer; break;
		case  4 : if( hfc_chk_conf_mt(0,16, hfc4_max_transfer) ) ap->max_transfer =  hfc4_max_transfer; break;
		case  5 : if( hfc_chk_conf_mt(0,16, hfc5_max_transfer) ) ap->max_transfer =  hfc5_max_transfer; break;
		case  6 : if( hfc_chk_conf_mt(0,16, hfc6_max_transfer) ) ap->max_transfer =  hfc6_max_transfer; break;
		case  7 : if( hfc_chk_conf_mt(0,16, hfc7_max_transfer) ) ap->max_transfer =  hfc7_max_transfer; break;
		case  8 : if( hfc_chk_conf_mt(0,16, hfc8_max_transfer) ) ap->max_transfer =  hfc8_max_transfer; break;
		case  9 : if( hfc_chk_conf_mt(0,16, hfc9_max_transfer) ) ap->max_transfer =  hfc9_max_transfer; break;
		case 10 : if( hfc_chk_conf_mt(0,16,hfc10_max_transfer) ) ap->max_transfer = hfc10_max_transfer; break;
		case 11 : if( hfc_chk_conf_mt(0,16,hfc11_max_transfer) ) ap->max_transfer = hfc11_max_transfer; break;
		case 12 : if( hfc_chk_conf_mt(0,16,hfc12_max_transfer) ) ap->max_transfer = hfc12_max_transfer; break;
		case 13 : if( hfc_chk_conf_mt(0,16,hfc13_max_transfer) ) ap->max_transfer = hfc13_max_transfer; break;
		case 14 : if( hfc_chk_conf_mt(0,16,hfc14_max_transfer) ) ap->max_transfer = hfc14_max_transfer; break;
		case 15 : if( hfc_chk_conf_mt(0,16,hfc15_max_transfer) ) ap->max_transfer = hfc15_max_transfer; break;
		case 16 : if( hfc_chk_conf_mt(0,16,hfc16_max_transfer) ) ap->max_transfer = hfc16_max_transfer; break;
		case 17 : if( hfc_chk_conf_mt(0,16,hfc17_max_transfer) ) ap->max_transfer = hfc17_max_transfer; break;
		case 18 : if( hfc_chk_conf_mt(0,16,hfc18_max_transfer) ) ap->max_transfer = hfc18_max_transfer; break;
		case 19 : if( hfc_chk_conf_mt(0,16,hfc19_max_transfer) ) ap->max_transfer = hfc19_max_transfer; break;
		case 20 : if( hfc_chk_conf_mt(0,16,hfc20_max_transfer) ) ap->max_transfer = hfc20_max_transfer; break;
		case 21 : if( hfc_chk_conf_mt(0,16,hfc21_max_transfer) ) ap->max_transfer = hfc21_max_transfer; break;
		case 22 : if( hfc_chk_conf_mt(0,16,hfc22_max_transfer) ) ap->max_transfer = hfc22_max_transfer; break;
		case 23 : if( hfc_chk_conf_mt(0,16,hfc23_max_transfer) ) ap->max_transfer = hfc23_max_transfer; break;
		case 24 : if( hfc_chk_conf_mt(0,16,hfc24_max_transfer) ) ap->max_transfer = hfc24_max_transfer; break;
		case 25 : if( hfc_chk_conf_mt(0,16,hfc25_max_transfer) ) ap->max_transfer = hfc25_max_transfer; break;
		case 26 : if( hfc_chk_conf_mt(0,16,hfc26_max_transfer) ) ap->max_transfer = hfc26_max_transfer; break;
		case 27 : if( hfc_chk_conf_mt(0,16,hfc27_max_transfer) ) ap->max_transfer = hfc27_max_transfer; break;
		case 28 : if( hfc_chk_conf_mt(0,16,hfc28_max_transfer) ) ap->max_transfer = hfc28_max_transfer; break;
		case 29 : if( hfc_chk_conf_mt(0,16,hfc29_max_transfer) ) ap->max_transfer = hfc29_max_transfer; break;
		case 30 : if( hfc_chk_conf_mt(0,16,hfc30_max_transfer) ) ap->max_transfer = hfc30_max_transfer; break;
		case 31 : if( hfc_chk_conf_mt(0,16,hfc31_max_transfer) ) ap->max_transfer = hfc31_max_transfer; break;
	}

	if( hfc_chk_conf_mt(0,16,hfcmp_max_transfer[ap->instance] ) ) {
		ap->max_transfer = hfcmp_max_transfer[ap->instance];
		HFC_DBGPRT("max_transfer <-- hfcldd.conf\n");
	}

	ap->dma_max = ap->max_transfer*0x100000;
	HFC_DBGPRT("max_transfer = %d, dma_max = %d\n", ap->max_transfer, ap->dma_max);

}



void hfc_set_linkup_tmo(struct adap_info *ap)
{

	ap->linkup_tmo = HFC_LINKUP_TO; /* default = 15s */

	if(ap->defparam) return;

	/* link down timer  */
	if(hfc_chk_conf_val(0,60,hfc_link_down)){ /* in case global parameter is set */
		ap->linkup_tmo = hfc_link_down; 

	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,60, hfc0_link_down) ) ap->linkup_tmo =  hfc0_link_down; break;
		case  1 : if( hfc_chk_conf_val(0,60, hfc1_link_down) ) ap->linkup_tmo =  hfc1_link_down; break;
		case  2 : if( hfc_chk_conf_val(0,60, hfc2_link_down) ) ap->linkup_tmo =  hfc2_link_down; break;
		case  3 : if( hfc_chk_conf_val(0,60, hfc3_link_down) ) ap->linkup_tmo =  hfc3_link_down; break;
		case  4 : if( hfc_chk_conf_val(0,60, hfc4_link_down) ) ap->linkup_tmo =  hfc4_link_down; break;
		case  5 : if( hfc_chk_conf_val(0,60, hfc5_link_down) ) ap->linkup_tmo =  hfc5_link_down; break;
		case  6 : if( hfc_chk_conf_val(0,60, hfc6_link_down) ) ap->linkup_tmo =  hfc6_link_down; break;
		case  7 : if( hfc_chk_conf_val(0,60, hfc7_link_down) ) ap->linkup_tmo =  hfc7_link_down; break;
		case  8 : if( hfc_chk_conf_val(0,60, hfc8_link_down) ) ap->linkup_tmo =  hfc8_link_down; break;
		case  9 : if( hfc_chk_conf_val(0,60, hfc9_link_down) ) ap->linkup_tmo =  hfc9_link_down; break;
		case 10 : if( hfc_chk_conf_val(0,60,hfc10_link_down) ) ap->linkup_tmo = hfc10_link_down; break;
		case 11 : if( hfc_chk_conf_val(0,60,hfc11_link_down) ) ap->linkup_tmo = hfc11_link_down; break;
		case 12 : if( hfc_chk_conf_val(0,60,hfc12_link_down) ) ap->linkup_tmo = hfc12_link_down; break;
		case 13 : if( hfc_chk_conf_val(0,60,hfc13_link_down) ) ap->linkup_tmo = hfc13_link_down; break;
		case 14 : if( hfc_chk_conf_val(0,60,hfc14_link_down) ) ap->linkup_tmo = hfc14_link_down; break;
		case 15 : if( hfc_chk_conf_val(0,60,hfc15_link_down) ) ap->linkup_tmo = hfc15_link_down; break;
		case 16 : if( hfc_chk_conf_val(0,60,hfc16_link_down) ) ap->linkup_tmo = hfc16_link_down; break;
		case 17 : if( hfc_chk_conf_val(0,60,hfc17_link_down) ) ap->linkup_tmo = hfc17_link_down; break;
		case 18 : if( hfc_chk_conf_val(0,60,hfc18_link_down) ) ap->linkup_tmo = hfc18_link_down; break;
		case 19 : if( hfc_chk_conf_val(0,60,hfc19_link_down) ) ap->linkup_tmo = hfc19_link_down; break;
		case 20 : if( hfc_chk_conf_val(0,60,hfc20_link_down) ) ap->linkup_tmo = hfc20_link_down; break;
		case 21 : if( hfc_chk_conf_val(0,60,hfc21_link_down) ) ap->linkup_tmo = hfc21_link_down; break;
		case 22 : if( hfc_chk_conf_val(0,60,hfc22_link_down) ) ap->linkup_tmo = hfc22_link_down; break;
		case 23 : if( hfc_chk_conf_val(0,60,hfc23_link_down) ) ap->linkup_tmo = hfc23_link_down; break;
		case 24 : if( hfc_chk_conf_val(0,60,hfc24_link_down) ) ap->linkup_tmo = hfc24_link_down; break;
		case 25 : if( hfc_chk_conf_val(0,60,hfc25_link_down) ) ap->linkup_tmo = hfc25_link_down; break;
		case 26 : if( hfc_chk_conf_val(0,60,hfc26_link_down) ) ap->linkup_tmo = hfc26_link_down; break;
		case 27 : if( hfc_chk_conf_val(0,60,hfc27_link_down) ) ap->linkup_tmo = hfc27_link_down; break;
		case 28 : if( hfc_chk_conf_val(0,60,hfc28_link_down) ) ap->linkup_tmo = hfc28_link_down; break;
		case 29 : if( hfc_chk_conf_val(0,60,hfc29_link_down) ) ap->linkup_tmo = hfc29_link_down; break;
		case 30 : if( hfc_chk_conf_val(0,60,hfc30_link_down) ) ap->linkup_tmo = hfc30_link_down; break;
		case 31 : if( hfc_chk_conf_val(0,60,hfc31_link_down) ) ap->linkup_tmo = hfc31_link_down; break;
	}

	if( hfc_chk_conf_val(0,60,hfcmp_link_down[ap->instance]) ) {
		ap->linkup_tmo = hfcmp_link_down[ap->instance];
		HFC_DBGPRT("linkup_timeout <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("linkup_timeout = %d \n", ap->linkup_tmo);
}


void hfc_set_linkup2_tmo(struct adap_info *ap)
{

	ap->linkup2_tmo = HFC_LINKUP2_TO; /* default = 15s */
	if(ap->defparam) return;

	/* link down timer  */
	if(hfc_chk_conf_val(0,60,hfc_link_down2)){ /* in case global parameter is set */
		ap->linkup2_tmo = hfc_link_down2; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,60, hfc0_link_down2) ) ap->linkup2_tmo =  hfc0_link_down2; break;
		case  1 : if( hfc_chk_conf_val(0,60, hfc1_link_down2) ) ap->linkup2_tmo =  hfc2_link_down2; break;
		case  3 : if( hfc_chk_conf_val(0,60, hfc3_link_down2) ) ap->linkup2_tmo =  hfc3_link_down2; break;
		case  4 : if( hfc_chk_conf_val(0,60, hfc4_link_down2) ) ap->linkup2_tmo =  hfc4_link_down2; break;
		case  5 : if( hfc_chk_conf_val(0,60, hfc5_link_down2) ) ap->linkup2_tmo =  hfc5_link_down2; break;
		case  6 : if( hfc_chk_conf_val(0,60, hfc6_link_down2) ) ap->linkup2_tmo =  hfc6_link_down2; break;
		case  7 : if( hfc_chk_conf_val(0,60, hfc7_link_down2) ) ap->linkup2_tmo =  hfc7_link_down2; break;
		case  8 : if( hfc_chk_conf_val(0,60, hfc8_link_down2) ) ap->linkup2_tmo =  hfc8_link_down2; break;
		case  9 : if( hfc_chk_conf_val(0,60, hfc9_link_down2) ) ap->linkup2_tmo =  hfc9_link_down2; break;
		case 10 : if( hfc_chk_conf_val(0,60,hfc10_link_down2) ) ap->linkup2_tmo = hfc10_link_down2; break;
		case 11 : if( hfc_chk_conf_val(0,60,hfc11_link_down2) ) ap->linkup2_tmo = hfc11_link_down2; break;
		case 12 : if( hfc_chk_conf_val(0,60,hfc12_link_down2) ) ap->linkup2_tmo = hfc12_link_down2; break;
		case 13 : if( hfc_chk_conf_val(0,60,hfc13_link_down2) ) ap->linkup2_tmo = hfc13_link_down2; break;
		case 14 : if( hfc_chk_conf_val(0,60,hfc14_link_down2) ) ap->linkup2_tmo = hfc14_link_down2; break;
		case 15 : if( hfc_chk_conf_val(0,60,hfc15_link_down2) ) ap->linkup2_tmo = hfc15_link_down2; break;
		case 16 : if( hfc_chk_conf_val(0,60,hfc16_link_down2) ) ap->linkup2_tmo = hfc16_link_down2; break;
		case 17 : if( hfc_chk_conf_val(0,60,hfc17_link_down2) ) ap->linkup2_tmo = hfc17_link_down2; break;
		case 18 : if( hfc_chk_conf_val(0,60,hfc18_link_down2) ) ap->linkup2_tmo = hfc18_link_down2; break;
		case 19 : if( hfc_chk_conf_val(0,60,hfc19_link_down2) ) ap->linkup2_tmo = hfc19_link_down2; break;
		case 20 : if( hfc_chk_conf_val(0,60,hfc20_link_down2) ) ap->linkup2_tmo = hfc20_link_down2; break;
		case 21 : if( hfc_chk_conf_val(0,60,hfc21_link_down2) ) ap->linkup2_tmo = hfc21_link_down2; break;
		case 22 : if( hfc_chk_conf_val(0,60,hfc22_link_down2) ) ap->linkup2_tmo = hfc22_link_down2; break;
		case 23 : if( hfc_chk_conf_val(0,60,hfc23_link_down2) ) ap->linkup2_tmo = hfc23_link_down2; break;
		case 24 : if( hfc_chk_conf_val(0,60,hfc24_link_down2) ) ap->linkup2_tmo = hfc24_link_down2; break;
		case 25 : if( hfc_chk_conf_val(0,60,hfc25_link_down2) ) ap->linkup2_tmo = hfc25_link_down2; break;
		case 26 : if( hfc_chk_conf_val(0,60,hfc26_link_down2) ) ap->linkup2_tmo = hfc26_link_down2; break;
		case 27 : if( hfc_chk_conf_val(0,60,hfc27_link_down2) ) ap->linkup2_tmo = hfc27_link_down2; break;
		case 28 : if( hfc_chk_conf_val(0,60,hfc28_link_down2) ) ap->linkup2_tmo = hfc28_link_down2; break;
		case 29 : if( hfc_chk_conf_val(0,60,hfc29_link_down2) ) ap->linkup2_tmo = hfc29_link_down2; break;
		case 30 : if( hfc_chk_conf_val(0,60,hfc30_link_down2) ) ap->linkup2_tmo = hfc30_link_down2; break;
		case 31 : if( hfc_chk_conf_val(0,60,hfc31_link_down2) ) ap->linkup2_tmo = hfc31_link_down2; break;
	}

	if( hfc_chk_conf_val(0,60,hfcmp_link_down2[ap->instance]) ) {
		ap->linkup2_tmo = hfcmp_link_down2[ap->instance];
		HFC_DBGPRT("linkup_timeout2 <-- hfcldd.conf\n");
	}

}


void hfc_set_reset_delay(struct adap_info *ap)
{

	ap->scsi_reset_delay = HFC_DELAY_TO; /* default = 10s */

	if(ap->defparam) return;

	/* link reset delay timer  */

	if(hfc_chk_conf_val(0,60,hfc_reset_delay)){ /* in case global parameter is set */
		ap->scsi_reset_delay = hfc_reset_delay; 

	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,60, hfc0_reset_delay) ) ap->scsi_reset_delay =  hfc0_reset_delay; break;
		case  1 : if( hfc_chk_conf_val(0,60, hfc1_reset_delay) ) ap->scsi_reset_delay =  hfc1_reset_delay; break;
		case  2 : if( hfc_chk_conf_val(0,60, hfc2_reset_delay) ) ap->scsi_reset_delay =  hfc2_reset_delay; break;
		case  3 : if( hfc_chk_conf_val(0,60, hfc3_reset_delay) ) ap->scsi_reset_delay =  hfc3_reset_delay; break;
		case  4 : if( hfc_chk_conf_val(0,60, hfc4_reset_delay) ) ap->scsi_reset_delay =  hfc4_reset_delay; break;
		case  5 : if( hfc_chk_conf_val(0,60, hfc5_reset_delay) ) ap->scsi_reset_delay =  hfc5_reset_delay; break;
		case  6 : if( hfc_chk_conf_val(0,60, hfc6_reset_delay) ) ap->scsi_reset_delay =  hfc6_reset_delay; break;
		case  7 : if( hfc_chk_conf_val(0,60, hfc7_reset_delay) ) ap->scsi_reset_delay =  hfc7_reset_delay; break;
		case  8 : if( hfc_chk_conf_val(0,60, hfc8_reset_delay) ) ap->scsi_reset_delay =  hfc8_reset_delay; break;
		case  9 : if( hfc_chk_conf_val(0,60, hfc9_reset_delay) ) ap->scsi_reset_delay =  hfc9_reset_delay; break;
		case 10 : if( hfc_chk_conf_val(0,60,hfc10_reset_delay) ) ap->scsi_reset_delay = hfc10_reset_delay; break;
		case 11 : if( hfc_chk_conf_val(0,60,hfc11_reset_delay) ) ap->scsi_reset_delay = hfc11_reset_delay; break;
		case 12 : if( hfc_chk_conf_val(0,60,hfc12_reset_delay) ) ap->scsi_reset_delay = hfc12_reset_delay; break;
		case 13 : if( hfc_chk_conf_val(0,60,hfc13_reset_delay) ) ap->scsi_reset_delay = hfc13_reset_delay; break;
		case 14 : if( hfc_chk_conf_val(0,60,hfc14_reset_delay) ) ap->scsi_reset_delay = hfc14_reset_delay; break;
		case 15 : if( hfc_chk_conf_val(0,60,hfc15_reset_delay) ) ap->scsi_reset_delay = hfc15_reset_delay; break;
		case 16 : if( hfc_chk_conf_val(0,60,hfc16_reset_delay) ) ap->scsi_reset_delay = hfc16_reset_delay; break;
		case 17 : if( hfc_chk_conf_val(0,60,hfc17_reset_delay) ) ap->scsi_reset_delay = hfc17_reset_delay; break;
		case 18 : if( hfc_chk_conf_val(0,60,hfc18_reset_delay) ) ap->scsi_reset_delay = hfc18_reset_delay; break;
		case 19 : if( hfc_chk_conf_val(0,60,hfc19_reset_delay) ) ap->scsi_reset_delay = hfc19_reset_delay; break;
		case 20 : if( hfc_chk_conf_val(0,60,hfc20_reset_delay) ) ap->scsi_reset_delay = hfc20_reset_delay; break;
		case 21 : if( hfc_chk_conf_val(0,60,hfc21_reset_delay) ) ap->scsi_reset_delay = hfc21_reset_delay; break;
		case 22 : if( hfc_chk_conf_val(0,60,hfc22_reset_delay) ) ap->scsi_reset_delay = hfc22_reset_delay; break;
		case 23 : if( hfc_chk_conf_val(0,60,hfc23_reset_delay) ) ap->scsi_reset_delay = hfc23_reset_delay; break;
		case 24 : if( hfc_chk_conf_val(0,60,hfc24_reset_delay) ) ap->scsi_reset_delay = hfc24_reset_delay; break;
		case 25 : if( hfc_chk_conf_val(0,60,hfc25_reset_delay) ) ap->scsi_reset_delay = hfc25_reset_delay; break;
		case 26 : if( hfc_chk_conf_val(0,60,hfc26_reset_delay) ) ap->scsi_reset_delay = hfc26_reset_delay; break;
		case 27 : if( hfc_chk_conf_val(0,60,hfc27_reset_delay) ) ap->scsi_reset_delay = hfc27_reset_delay; break;
		case 28 : if( hfc_chk_conf_val(0,60,hfc28_reset_delay) ) ap->scsi_reset_delay = hfc28_reset_delay; break;
		case 29 : if( hfc_chk_conf_val(0,60,hfc29_reset_delay) ) ap->scsi_reset_delay = hfc29_reset_delay; break;
		case 30 : if( hfc_chk_conf_val(0,60,hfc30_reset_delay) ) ap->scsi_reset_delay = hfc30_reset_delay; break;
		case 31 : if( hfc_chk_conf_val(0,60,hfc31_reset_delay) ) ap->scsi_reset_delay = hfc31_reset_delay; break;
	}
	
	if( hfc_chk_conf_val(0,60,hfcmp_reset_delay[ap->instance]) ) {
		ap->scsi_reset_delay = hfcmp_reset_delay[ap->instance];
		HFC_DBGPRT("reset_delay <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("reset_delay = %d \n", ap->scsi_reset_delay);
}



void hfc_set_mck_count(struct adap_info *ap)
{
	ap->max_mck_cnt = HFC_MCKERR_CNT; /* default = 8 */
	
	if(ap->defparam) return;
	
	/* machine check recovery retry count */

	if( hfc_chk_conf_val(0,10, hfc_mck_retry)){ /* in case global parameter is set */
		ap->max_mck_cnt = hfc_mck_retry; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,10, hfc0_mck_retry) ) ap->max_mck_cnt =  hfc0_mck_retry; break;
		case  1 : if( hfc_chk_conf_val(0,10, hfc1_mck_retry) ) ap->max_mck_cnt =  hfc1_mck_retry; break;
		case  2 : if( hfc_chk_conf_val(0,10, hfc2_mck_retry) ) ap->max_mck_cnt =  hfc2_mck_retry; break;
		case  3 : if( hfc_chk_conf_val(0,10, hfc3_mck_retry) ) ap->max_mck_cnt =  hfc3_mck_retry; break;
		case  4 : if( hfc_chk_conf_val(0,10, hfc4_mck_retry) ) ap->max_mck_cnt =  hfc4_mck_retry; break;
		case  5 : if( hfc_chk_conf_val(0,10, hfc5_mck_retry) ) ap->max_mck_cnt =  hfc5_mck_retry; break;
		case  6 : if( hfc_chk_conf_val(0,10, hfc6_mck_retry) ) ap->max_mck_cnt =  hfc6_mck_retry; break;
		case  7 : if( hfc_chk_conf_val(0,10, hfc7_mck_retry) ) ap->max_mck_cnt =  hfc7_mck_retry; break;
		case  8 : if( hfc_chk_conf_val(0,10, hfc8_mck_retry) ) ap->max_mck_cnt =  hfc8_mck_retry; break;
		case  9 : if( hfc_chk_conf_val(0,10, hfc9_mck_retry) ) ap->max_mck_cnt =  hfc9_mck_retry; break;
		case 10 : if( hfc_chk_conf_val(0,10,hfc10_mck_retry) ) ap->max_mck_cnt = hfc10_mck_retry; break;
		case 11 : if( hfc_chk_conf_val(0,10,hfc11_mck_retry) ) ap->max_mck_cnt = hfc11_mck_retry; break;
		case 12 : if( hfc_chk_conf_val(0,10,hfc12_mck_retry) ) ap->max_mck_cnt = hfc12_mck_retry; break;
		case 13 : if( hfc_chk_conf_val(0,10,hfc13_mck_retry) ) ap->max_mck_cnt = hfc13_mck_retry; break;
		case 14 : if( hfc_chk_conf_val(0,10,hfc14_mck_retry) ) ap->max_mck_cnt = hfc14_mck_retry; break;
		case 15 : if( hfc_chk_conf_val(0,10,hfc15_mck_retry) ) ap->max_mck_cnt = hfc15_mck_retry; break;
		case 16 : if( hfc_chk_conf_val(0,10,hfc16_mck_retry) ) ap->max_mck_cnt = hfc16_mck_retry; break;
		case 17 : if( hfc_chk_conf_val(0,10,hfc17_mck_retry) ) ap->max_mck_cnt = hfc17_mck_retry; break;
		case 18 : if( hfc_chk_conf_val(0,10,hfc18_mck_retry) ) ap->max_mck_cnt = hfc18_mck_retry; break;
		case 19 : if( hfc_chk_conf_val(0,10,hfc19_mck_retry) ) ap->max_mck_cnt = hfc19_mck_retry; break;
		case 20 : if( hfc_chk_conf_val(0,10,hfc20_mck_retry) ) ap->max_mck_cnt = hfc20_mck_retry; break;
		case 21 : if( hfc_chk_conf_val(0,10,hfc21_mck_retry) ) ap->max_mck_cnt = hfc21_mck_retry; break;
		case 22 : if( hfc_chk_conf_val(0,10,hfc22_mck_retry) ) ap->max_mck_cnt = hfc22_mck_retry; break;
		case 23 : if( hfc_chk_conf_val(0,10,hfc23_mck_retry) ) ap->max_mck_cnt = hfc23_mck_retry; break;
		case 24 : if( hfc_chk_conf_val(0,10,hfc24_mck_retry) ) ap->max_mck_cnt = hfc24_mck_retry; break;
		case 25 : if( hfc_chk_conf_val(0,10,hfc25_mck_retry) ) ap->max_mck_cnt = hfc25_mck_retry; break;
		case 26 : if( hfc_chk_conf_val(0,10,hfc26_mck_retry) ) ap->max_mck_cnt = hfc26_mck_retry; break;
		case 27 : if( hfc_chk_conf_val(0,10,hfc27_mck_retry) ) ap->max_mck_cnt = hfc27_mck_retry; break;
		case 28 : if( hfc_chk_conf_val(0,10,hfc28_mck_retry) ) ap->max_mck_cnt = hfc28_mck_retry; break;
		case 29 : if( hfc_chk_conf_val(0,10,hfc29_mck_retry) ) ap->max_mck_cnt = hfc29_mck_retry; break;
		case 30 : if( hfc_chk_conf_val(0,10,hfc30_mck_retry) ) ap->max_mck_cnt = hfc30_mck_retry; break;
		case 31 : if( hfc_chk_conf_val(0,10,hfc31_mck_retry) ) ap->max_mck_cnt = hfc31_mck_retry; break;
	}

	if( hfc_chk_conf_val(0,10,hfcmp_mck_retry[ap->instance]) ) {
		ap->max_mck_cnt = hfcmp_mck_retry[ap->instance];
		HFC_DBGPRT("max machine check count <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("max machine check count = %d \n", ap->max_mck_cnt);
}



void hfc_set_pref_alpa(struct adap_info *ap)
{
	ap->pref_alpa = 0x01; /* default = 0x01 */
	
	if(ap->defparam) return;
		
	/* preferred ALPA */
	
	if(hfc_chk_conf_val(0x01,0xef,hfc_preferred_alpa)){ /* in case global parameter is set */
		ap->pref_alpa = hfc_preferred_alpa; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0x01,0xef, hfc0_preferred_alpa) ) ap->pref_alpa =  hfc0_preferred_alpa; break;
		case  1 : if( hfc_chk_conf_val(0x01,0xef, hfc1_preferred_alpa) ) ap->pref_alpa =  hfc1_preferred_alpa; break;
		case  2 : if( hfc_chk_conf_val(0x01,0xef, hfc2_preferred_alpa) ) ap->pref_alpa =  hfc2_preferred_alpa; break;
		case  3 : if( hfc_chk_conf_val(0x01,0xef, hfc3_preferred_alpa) ) ap->pref_alpa =  hfc3_preferred_alpa; break;
		case  4 : if( hfc_chk_conf_val(0x01,0xef, hfc4_preferred_alpa) ) ap->pref_alpa =  hfc4_preferred_alpa; break;
		case  5 : if( hfc_chk_conf_val(0x01,0xef, hfc5_preferred_alpa) ) ap->pref_alpa =  hfc5_preferred_alpa; break;
		case  6 : if( hfc_chk_conf_val(0x01,0xef, hfc6_preferred_alpa) ) ap->pref_alpa =  hfc6_preferred_alpa; break;
		case  7 : if( hfc_chk_conf_val(0x01,0xef, hfc7_preferred_alpa) ) ap->pref_alpa =  hfc7_preferred_alpa; break;
		case  8 : if( hfc_chk_conf_val(0x01,0xef, hfc8_preferred_alpa) ) ap->pref_alpa =  hfc8_preferred_alpa; break;
		case  9 : if( hfc_chk_conf_val(0x01,0xef, hfc9_preferred_alpa) ) ap->pref_alpa =  hfc9_preferred_alpa; break;
		case 10 : if( hfc_chk_conf_val(0x01,0xef,hfc10_preferred_alpa) ) ap->pref_alpa = hfc10_preferred_alpa; break;
		case 11 : if( hfc_chk_conf_val(0x01,0xef,hfc11_preferred_alpa) ) ap->pref_alpa = hfc11_preferred_alpa; break;
		case 12 : if( hfc_chk_conf_val(0x01,0xef,hfc12_preferred_alpa) ) ap->pref_alpa = hfc12_preferred_alpa; break;
		case 13 : if( hfc_chk_conf_val(0x01,0xef,hfc13_preferred_alpa) ) ap->pref_alpa = hfc13_preferred_alpa; break;
		case 14 : if( hfc_chk_conf_val(0x01,0xef,hfc14_preferred_alpa) ) ap->pref_alpa = hfc14_preferred_alpa; break;
		case 15 : if( hfc_chk_conf_val(0x01,0xef,hfc15_preferred_alpa) ) ap->pref_alpa = hfc15_preferred_alpa; break;
		case 16 : if( hfc_chk_conf_val(0x01,0xef,hfc16_preferred_alpa) ) ap->pref_alpa = hfc16_preferred_alpa; break;
		case 17 : if( hfc_chk_conf_val(0x01,0xef,hfc17_preferred_alpa) ) ap->pref_alpa = hfc17_preferred_alpa; break;
		case 18 : if( hfc_chk_conf_val(0x01,0xef,hfc18_preferred_alpa) ) ap->pref_alpa = hfc18_preferred_alpa; break;
		case 19 : if( hfc_chk_conf_val(0x01,0xef,hfc19_preferred_alpa) ) ap->pref_alpa = hfc19_preferred_alpa; break;
		case 20 : if( hfc_chk_conf_val(0x01,0xef,hfc20_preferred_alpa) ) ap->pref_alpa = hfc20_preferred_alpa; break;
		case 21 : if( hfc_chk_conf_val(0x01,0xef,hfc21_preferred_alpa) ) ap->pref_alpa = hfc21_preferred_alpa; break;
		case 22 : if( hfc_chk_conf_val(0x01,0xef,hfc22_preferred_alpa) ) ap->pref_alpa = hfc22_preferred_alpa; break;
		case 23 : if( hfc_chk_conf_val(0x01,0xef,hfc23_preferred_alpa) ) ap->pref_alpa = hfc23_preferred_alpa; break;
		case 24 : if( hfc_chk_conf_val(0x01,0xef,hfc24_preferred_alpa) ) ap->pref_alpa = hfc24_preferred_alpa; break;
		case 25 : if( hfc_chk_conf_val(0x01,0xef,hfc25_preferred_alpa) ) ap->pref_alpa = hfc25_preferred_alpa; break;
		case 26 : if( hfc_chk_conf_val(0x01,0xef,hfc26_preferred_alpa) ) ap->pref_alpa = hfc26_preferred_alpa; break;
		case 27 : if( hfc_chk_conf_val(0x01,0xef,hfc27_preferred_alpa) ) ap->pref_alpa = hfc27_preferred_alpa; break;
		case 28 : if( hfc_chk_conf_val(0x01,0xef,hfc28_preferred_alpa) ) ap->pref_alpa = hfc28_preferred_alpa; break;
		case 29 : if( hfc_chk_conf_val(0x01,0xef,hfc29_preferred_alpa) ) ap->pref_alpa = hfc29_preferred_alpa; break;
		case 30 : if( hfc_chk_conf_val(0x01,0xef,hfc30_preferred_alpa) ) ap->pref_alpa = hfc30_preferred_alpa; break;
		case 31 : if( hfc_chk_conf_val(0x01,0xef,hfc31_preferred_alpa) ) ap->pref_alpa = hfc31_preferred_alpa; break;
	}
	
	if( hfc_chk_conf_val(0x01,0xef,hfcmp_preferred_alpa[ap->instance]) ) {
		ap->pref_alpa = hfcmp_preferred_alpa[ap->instance];
		HFC_DBGPRT("preferred alpa <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("preferred alpa = %d \n", ap->pref_alpa);
}


void hfc_set_target_timeout(struct adap_info *ap)
{
	ap->target_reset_tmo = HFC_TARGET_RST_TO; /* default = 20s */

	/* target reset timeout */
	
	if(ap->defparam) return;
	
	if(hfc_chk_conf_val(0,60,hfc_reset_timeout)){ /* in case global parameter is set */
		ap->target_reset_tmo = hfc_reset_timeout;
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,60, hfc0_reset_timeout) ) ap->target_reset_tmo =  hfc0_reset_timeout; break;
		case  1 : if( hfc_chk_conf_val(0,60, hfc1_reset_timeout) ) ap->target_reset_tmo =  hfc1_reset_timeout; break;
		case  2 : if( hfc_chk_conf_val(0,60, hfc2_reset_timeout) ) ap->target_reset_tmo =  hfc2_reset_timeout; break;
		case  3 : if( hfc_chk_conf_val(0,60, hfc3_reset_timeout) ) ap->target_reset_tmo =  hfc3_reset_timeout; break;
		case  4 : if( hfc_chk_conf_val(0,60, hfc4_reset_timeout) ) ap->target_reset_tmo =  hfc4_reset_timeout; break;
		case  5 : if( hfc_chk_conf_val(0,60, hfc5_reset_timeout) ) ap->target_reset_tmo =  hfc5_reset_timeout; break;
		case  6 : if( hfc_chk_conf_val(0,60, hfc6_reset_timeout) ) ap->target_reset_tmo =  hfc6_reset_timeout; break;
		case  7 : if( hfc_chk_conf_val(0,60, hfc7_reset_timeout) ) ap->target_reset_tmo =  hfc7_reset_timeout; break;
		case  8 : if( hfc_chk_conf_val(0,60, hfc8_reset_timeout) ) ap->target_reset_tmo =  hfc8_reset_timeout; break;
		case  9 : if( hfc_chk_conf_val(0,60, hfc9_reset_timeout) ) ap->target_reset_tmo =  hfc9_reset_timeout; break;
		case 10 : if( hfc_chk_conf_val(0,60,hfc10_reset_timeout) ) ap->target_reset_tmo = hfc10_reset_timeout; break;
		case 11 : if( hfc_chk_conf_val(0,60,hfc11_reset_timeout) ) ap->target_reset_tmo = hfc11_reset_timeout; break;
		case 12 : if( hfc_chk_conf_val(0,60,hfc12_reset_timeout) ) ap->target_reset_tmo = hfc12_reset_timeout; break;
		case 13 : if( hfc_chk_conf_val(0,60,hfc13_reset_timeout) ) ap->target_reset_tmo = hfc13_reset_timeout; break;
		case 14 : if( hfc_chk_conf_val(0,60,hfc14_reset_timeout) ) ap->target_reset_tmo = hfc14_reset_timeout; break;
		case 15 : if( hfc_chk_conf_val(0,60,hfc15_reset_timeout) ) ap->target_reset_tmo = hfc15_reset_timeout; break;
		case 16 : if( hfc_chk_conf_val(0,60,hfc16_reset_timeout) ) ap->target_reset_tmo = hfc16_reset_timeout; break;
		case 17 : if( hfc_chk_conf_val(0,60,hfc17_reset_timeout) ) ap->target_reset_tmo = hfc17_reset_timeout; break;
		case 18 : if( hfc_chk_conf_val(0,60,hfc18_reset_timeout) ) ap->target_reset_tmo = hfc18_reset_timeout; break;
		case 19 : if( hfc_chk_conf_val(0,60,hfc19_reset_timeout) ) ap->target_reset_tmo = hfc19_reset_timeout; break;
		case 20 : if( hfc_chk_conf_val(0,60,hfc20_reset_timeout) ) ap->target_reset_tmo = hfc20_reset_timeout; break;
		case 21 : if( hfc_chk_conf_val(0,60,hfc21_reset_timeout) ) ap->target_reset_tmo = hfc21_reset_timeout; break;
		case 22 : if( hfc_chk_conf_val(0,60,hfc22_reset_timeout) ) ap->target_reset_tmo = hfc22_reset_timeout; break;
		case 23 : if( hfc_chk_conf_val(0,60,hfc23_reset_timeout) ) ap->target_reset_tmo = hfc23_reset_timeout; break;
		case 24 : if( hfc_chk_conf_val(0,60,hfc24_reset_timeout) ) ap->target_reset_tmo = hfc24_reset_timeout; break;
		case 25 : if( hfc_chk_conf_val(0,60,hfc25_reset_timeout) ) ap->target_reset_tmo = hfc25_reset_timeout; break;
		case 26 : if( hfc_chk_conf_val(0,60,hfc26_reset_timeout) ) ap->target_reset_tmo = hfc26_reset_timeout; break;
		case 27 : if( hfc_chk_conf_val(0,60,hfc27_reset_timeout) ) ap->target_reset_tmo = hfc27_reset_timeout; break;
		case 28 : if( hfc_chk_conf_val(0,60,hfc28_reset_timeout) ) ap->target_reset_tmo = hfc28_reset_timeout; break;
		case 29 : if( hfc_chk_conf_val(0,60,hfc29_reset_timeout) ) ap->target_reset_tmo = hfc29_reset_timeout; break;
		case 30 : if( hfc_chk_conf_val(0,60,hfc30_reset_timeout) ) ap->target_reset_tmo = hfc30_reset_timeout; break;
		case 31 : if( hfc_chk_conf_val(0,60,hfc31_reset_timeout) ) ap->target_reset_tmo = hfc31_reset_timeout; break;
	}

	if( hfc_chk_conf_val(0,60,hfcmp_reset_timeout[ap->instance]) ) {
		ap->target_reset_tmo = hfcmp_reset_timeout[ap->instance];
		HFC_DBGPRT("target reset timeout <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("target reset timeout = %d \n", ap->target_reset_tmo);
}



void hfc_set_abort_timeout(struct adap_info *ap)
{
	
	ap->abort_tmo = HFC_ABORT_ACA_TO;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,60,hfc_abort_timeout)){ /* in case global parameter is set */
		ap->abort_tmo = hfc_abort_timeout; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,60, hfc0_abort_timeout) ) ap->abort_tmo =  hfc0_abort_timeout; break;
		case  1 : if( hfc_chk_conf_val(0,60, hfc1_abort_timeout) ) ap->abort_tmo =  hfc1_abort_timeout; break;
		case  2 : if( hfc_chk_conf_val(0,60, hfc2_abort_timeout) ) ap->abort_tmo =  hfc2_abort_timeout; break;
		case  3 : if( hfc_chk_conf_val(0,60, hfc3_abort_timeout) ) ap->abort_tmo =  hfc3_abort_timeout; break;
		case  4 : if( hfc_chk_conf_val(0,60, hfc4_abort_timeout) ) ap->abort_tmo =  hfc4_abort_timeout; break;
		case  5 : if( hfc_chk_conf_val(0,60, hfc5_abort_timeout) ) ap->abort_tmo =  hfc5_abort_timeout; break;
		case  6 : if( hfc_chk_conf_val(0,60, hfc6_abort_timeout) ) ap->abort_tmo =  hfc6_abort_timeout; break;
		case  7 : if( hfc_chk_conf_val(0,60, hfc7_abort_timeout) ) ap->abort_tmo =  hfc7_abort_timeout; break;
		case  8 : if( hfc_chk_conf_val(0,60, hfc8_abort_timeout) ) ap->abort_tmo =  hfc8_abort_timeout; break;
		case  9 : if( hfc_chk_conf_val(0,60, hfc9_abort_timeout) ) ap->abort_tmo =  hfc9_abort_timeout; break;
		case 10 : if( hfc_chk_conf_val(0,60,hfc10_abort_timeout) ) ap->abort_tmo = hfc10_abort_timeout; break;
		case 11 : if( hfc_chk_conf_val(0,60,hfc11_abort_timeout) ) ap->abort_tmo = hfc11_abort_timeout; break;
		case 12 : if( hfc_chk_conf_val(0,60,hfc12_abort_timeout) ) ap->abort_tmo = hfc12_abort_timeout; break;
		case 13 : if( hfc_chk_conf_val(0,60,hfc13_abort_timeout) ) ap->abort_tmo = hfc13_abort_timeout; break;
		case 14 : if( hfc_chk_conf_val(0,60,hfc14_abort_timeout) ) ap->abort_tmo = hfc14_abort_timeout; break;
		case 15 : if( hfc_chk_conf_val(0,60,hfc15_abort_timeout) ) ap->abort_tmo = hfc15_abort_timeout; break;
		case 16 : if( hfc_chk_conf_val(0,60,hfc16_abort_timeout) ) ap->abort_tmo = hfc16_abort_timeout; break;
		case 17 : if( hfc_chk_conf_val(0,60,hfc17_abort_timeout) ) ap->abort_tmo = hfc17_abort_timeout; break;
		case 18 : if( hfc_chk_conf_val(0,60,hfc18_abort_timeout) ) ap->abort_tmo = hfc18_abort_timeout; break;
		case 19 : if( hfc_chk_conf_val(0,60,hfc19_abort_timeout) ) ap->abort_tmo = hfc19_abort_timeout; break;
		case 20 : if( hfc_chk_conf_val(0,60,hfc20_abort_timeout) ) ap->abort_tmo = hfc20_abort_timeout; break;
		case 21 : if( hfc_chk_conf_val(0,60,hfc21_abort_timeout) ) ap->abort_tmo = hfc21_abort_timeout; break;
		case 22 : if( hfc_chk_conf_val(0,60,hfc22_abort_timeout) ) ap->abort_tmo = hfc22_abort_timeout; break;
		case 23 : if( hfc_chk_conf_val(0,60,hfc23_abort_timeout) ) ap->abort_tmo = hfc23_abort_timeout; break;
		case 24 : if( hfc_chk_conf_val(0,60,hfc24_abort_timeout) ) ap->abort_tmo = hfc24_abort_timeout; break;
		case 25 : if( hfc_chk_conf_val(0,60,hfc25_abort_timeout) ) ap->abort_tmo = hfc25_abort_timeout; break;
		case 26 : if( hfc_chk_conf_val(0,60,hfc26_abort_timeout) ) ap->abort_tmo = hfc26_abort_timeout; break;
		case 27 : if( hfc_chk_conf_val(0,60,hfc27_abort_timeout) ) ap->abort_tmo = hfc27_abort_timeout; break;
		case 28 : if( hfc_chk_conf_val(0,60,hfc28_abort_timeout) ) ap->abort_tmo = hfc28_abort_timeout; break;
		case 29 : if( hfc_chk_conf_val(0,60,hfc29_abort_timeout) ) ap->abort_tmo = hfc29_abort_timeout; break;
		case 30 : if( hfc_chk_conf_val(0,60,hfc30_abort_timeout) ) ap->abort_tmo = hfc30_abort_timeout; break;
		case 31 : if( hfc_chk_conf_val(0,60,hfc31_abort_timeout) ) ap->abort_tmo = hfc31_abort_timeout; break;
	}

	if( hfc_chk_conf_val(0,60,hfcmp_abort_timeout[ap->instance]) ) {
		ap->abort_tmo = hfcmp_abort_timeout[ap->instance];
		HFC_DBGPRT("abort timeout <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("abort timeout = %d \n", ap->abort_tmo );

}


void hfc_set_seg_trace(struct adap_info *ap)
{
	
	ap->fw_parm = 0;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,1,hfc_seg_trace)){ /* in case global parameter is set */
		ap->fw_parm = hfc_seg_trace; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,1, hfc0_seg_trace) ) ap->fw_parm =  hfc0_seg_trace; break;
		case  1 : if( hfc_chk_conf_val(0,1, hfc1_seg_trace) ) ap->fw_parm =  hfc1_seg_trace; break;
		case  2 : if( hfc_chk_conf_val(0,1, hfc2_seg_trace) ) ap->fw_parm =  hfc2_seg_trace; break;
		case  3 : if( hfc_chk_conf_val(0,1, hfc3_seg_trace) ) ap->fw_parm =  hfc3_seg_trace; break;
		case  4 : if( hfc_chk_conf_val(0,1, hfc4_seg_trace) ) ap->fw_parm =  hfc4_seg_trace; break;
		case  5 : if( hfc_chk_conf_val(0,1, hfc5_seg_trace) ) ap->fw_parm =  hfc5_seg_trace; break;
		case  6 : if( hfc_chk_conf_val(0,1, hfc6_seg_trace) ) ap->fw_parm =  hfc6_seg_trace; break;
		case  7 : if( hfc_chk_conf_val(0,1, hfc7_seg_trace) ) ap->fw_parm =  hfc7_seg_trace; break;
		case  8 : if( hfc_chk_conf_val(0,1, hfc8_seg_trace) ) ap->fw_parm =  hfc8_seg_trace; break;
		case  9 : if( hfc_chk_conf_val(0,1, hfc9_seg_trace) ) ap->fw_parm =  hfc9_seg_trace; break;
		case 10 : if( hfc_chk_conf_val(0,1,hfc10_seg_trace) ) ap->fw_parm = hfc10_seg_trace; break;
		case 11 : if( hfc_chk_conf_val(0,1,hfc11_seg_trace) ) ap->fw_parm = hfc11_seg_trace; break;
		case 12 : if( hfc_chk_conf_val(0,1,hfc12_seg_trace) ) ap->fw_parm = hfc12_seg_trace; break;
		case 13 : if( hfc_chk_conf_val(0,1,hfc13_seg_trace) ) ap->fw_parm = hfc13_seg_trace; break;
		case 14 : if( hfc_chk_conf_val(0,1,hfc14_seg_trace) ) ap->fw_parm = hfc14_seg_trace; break;
		case 15 : if( hfc_chk_conf_val(0,1,hfc15_seg_trace) ) ap->fw_parm = hfc15_seg_trace; break;
		case 16 : if( hfc_chk_conf_val(0,1,hfc16_seg_trace) ) ap->fw_parm = hfc16_seg_trace; break;
		case 17 : if( hfc_chk_conf_val(0,1,hfc17_seg_trace) ) ap->fw_parm = hfc17_seg_trace; break;
		case 18 : if( hfc_chk_conf_val(0,1,hfc18_seg_trace) ) ap->fw_parm = hfc18_seg_trace; break;
		case 19 : if( hfc_chk_conf_val(0,1,hfc19_seg_trace) ) ap->fw_parm = hfc19_seg_trace; break;
		case 20 : if( hfc_chk_conf_val(0,1,hfc20_seg_trace) ) ap->fw_parm = hfc20_seg_trace; break;
		case 21 : if( hfc_chk_conf_val(0,1,hfc21_seg_trace) ) ap->fw_parm = hfc21_seg_trace; break;
		case 22 : if( hfc_chk_conf_val(0,1,hfc22_seg_trace) ) ap->fw_parm = hfc22_seg_trace; break;
		case 23 : if( hfc_chk_conf_val(0,1,hfc23_seg_trace) ) ap->fw_parm = hfc23_seg_trace; break;
		case 24 : if( hfc_chk_conf_val(0,1,hfc24_seg_trace) ) ap->fw_parm = hfc24_seg_trace; break;
		case 25 : if( hfc_chk_conf_val(0,1,hfc25_seg_trace) ) ap->fw_parm = hfc25_seg_trace; break;
		case 26 : if( hfc_chk_conf_val(0,1,hfc26_seg_trace) ) ap->fw_parm = hfc26_seg_trace; break;
		case 27 : if( hfc_chk_conf_val(0,1,hfc27_seg_trace) ) ap->fw_parm = hfc27_seg_trace; break;
		case 28 : if( hfc_chk_conf_val(0,1,hfc28_seg_trace) ) ap->fw_parm = hfc28_seg_trace; break;
		case 29 : if( hfc_chk_conf_val(0,1,hfc29_seg_trace) ) ap->fw_parm = hfc29_seg_trace; break;
		case 30 : if( hfc_chk_conf_val(0,1,hfc30_seg_trace) ) ap->fw_parm = hfc30_seg_trace; break;
		case 31 : if( hfc_chk_conf_val(0,1,hfc31_seg_trace) ) ap->fw_parm = hfc31_seg_trace; break;
	}

	if( hfc_chk_conf_val(0,1,hfcmp_seg_trace[ap->instance]) ) {
		ap->fw_parm = hfcmp_seg_trace[ap->instance];
		HFC_DBGPRT("seg info trace flag <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("seg info trace flag = %d \n", ap->fw_parm );

	if(ap->fw_parm != 0){
		hfc_write_val( ap->fw_init_p->trc_info.trc_seg[0].mode, 0xf8);
	}

}


void hfc_set_queue_depth(struct adap_info *ap)
{

	ap->queue_depth = HFC_DEFAULT_QUEUE_DEPTH;

	if(ap->defparam) return;

	/* abort task set timeout */

	if(hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc_queue_depth)){ /* in case global parameter is set */
		ap->queue_depth = hfc_queue_depth; 

	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc0_queue_depth) ) ap->queue_depth =  hfc0_queue_depth; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc1_queue_depth) ) ap->queue_depth =  hfc1_queue_depth; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc2_queue_depth) ) ap->queue_depth =  hfc2_queue_depth; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc3_queue_depth) ) ap->queue_depth =  hfc3_queue_depth; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc4_queue_depth) ) ap->queue_depth =  hfc4_queue_depth; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc5_queue_depth) ) ap->queue_depth =  hfc5_queue_depth; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc6_queue_depth) ) ap->queue_depth =  hfc6_queue_depth; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc7_queue_depth) ) ap->queue_depth =  hfc7_queue_depth; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc8_queue_depth) ) ap->queue_depth =  hfc8_queue_depth; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH, hfc9_queue_depth) ) ap->queue_depth =  hfc9_queue_depth; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc10_queue_depth) ) ap->queue_depth = hfc10_queue_depth; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc11_queue_depth) ) ap->queue_depth = hfc11_queue_depth; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc12_queue_depth) ) ap->queue_depth = hfc12_queue_depth; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc13_queue_depth) ) ap->queue_depth = hfc13_queue_depth; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc14_queue_depth) ) ap->queue_depth = hfc14_queue_depth; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc15_queue_depth) ) ap->queue_depth = hfc15_queue_depth; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc16_queue_depth) ) ap->queue_depth = hfc16_queue_depth; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc17_queue_depth) ) ap->queue_depth = hfc17_queue_depth; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc18_queue_depth) ) ap->queue_depth = hfc18_queue_depth; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc19_queue_depth) ) ap->queue_depth = hfc19_queue_depth; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc20_queue_depth) ) ap->queue_depth = hfc20_queue_depth; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc21_queue_depth) ) ap->queue_depth = hfc21_queue_depth; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc22_queue_depth) ) ap->queue_depth = hfc22_queue_depth; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc23_queue_depth) ) ap->queue_depth = hfc23_queue_depth; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc24_queue_depth) ) ap->queue_depth = hfc24_queue_depth; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc25_queue_depth) ) ap->queue_depth = hfc25_queue_depth; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc26_queue_depth) ) ap->queue_depth = hfc26_queue_depth; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc27_queue_depth) ) ap->queue_depth = hfc27_queue_depth; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc28_queue_depth) ) ap->queue_depth = hfc28_queue_depth; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc29_queue_depth) ) ap->queue_depth = hfc29_queue_depth; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc30_queue_depth) ) ap->queue_depth = hfc30_queue_depth; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfc31_queue_depth) ) ap->queue_depth = hfc31_queue_depth; break;
	}

	if( hfc_chk_conf_val(0,HFC_MAX_QUEUE_DEPTH,hfcmp_queue_depth[ap->instance]) ) {
		ap->queue_depth = hfcmp_queue_depth[ap->instance];
		HFC_DBGPRT("queue depth <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("queue depth = %d \n", ap->queue_depth );
}


void hfc_set_enable_target_reset(struct adap_info *ap)
{
	
	ap->enable_tgtrst = HFC_DEFAULT_TARGET_RESET;

	if(ap->defparam) return;

	if(hfc_chk_conf_val(0,1,hfc_enable_tgtrst)){ /* in case global parameter is set */
		ap->enable_tgtrst = hfc_enable_tgtrst; 
	}

	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,1, hfc0_enable_tgtrst) ) ap->enable_tgtrst =  hfc0_enable_tgtrst; break;
		case  1 : if( hfc_chk_conf_val(0,1, hfc1_enable_tgtrst) ) ap->enable_tgtrst =  hfc1_enable_tgtrst; break;
		case  2 : if( hfc_chk_conf_val(0,1, hfc2_enable_tgtrst) ) ap->enable_tgtrst =  hfc2_enable_tgtrst; break;
		case  3 : if( hfc_chk_conf_val(0,1, hfc3_enable_tgtrst) ) ap->enable_tgtrst =  hfc3_enable_tgtrst; break;
		case  4 : if( hfc_chk_conf_val(0,1, hfc4_enable_tgtrst) ) ap->enable_tgtrst =  hfc4_enable_tgtrst; break;
		case  5 : if( hfc_chk_conf_val(0,1, hfc5_enable_tgtrst) ) ap->enable_tgtrst =  hfc5_enable_tgtrst; break;
		case  6 : if( hfc_chk_conf_val(0,1, hfc6_enable_tgtrst) ) ap->enable_tgtrst =  hfc6_enable_tgtrst; break;
		case  7 : if( hfc_chk_conf_val(0,1, hfc7_enable_tgtrst) ) ap->enable_tgtrst =  hfc7_enable_tgtrst; break;
		case  8 : if( hfc_chk_conf_val(0,1, hfc8_enable_tgtrst) ) ap->enable_tgtrst =  hfc8_enable_tgtrst; break;
		case  9 : if( hfc_chk_conf_val(0,1, hfc9_enable_tgtrst) ) ap->enable_tgtrst =  hfc9_enable_tgtrst; break;
		case 10 : if( hfc_chk_conf_val(0,1,hfc10_enable_tgtrst) ) ap->enable_tgtrst = hfc10_enable_tgtrst; break;
		case 11 : if( hfc_chk_conf_val(0,1,hfc11_enable_tgtrst) ) ap->enable_tgtrst = hfc11_enable_tgtrst; break;
		case 12 : if( hfc_chk_conf_val(0,1,hfc12_enable_tgtrst) ) ap->enable_tgtrst = hfc12_enable_tgtrst; break;
		case 13 : if( hfc_chk_conf_val(0,1,hfc13_enable_tgtrst) ) ap->enable_tgtrst = hfc13_enable_tgtrst; break;
		case 14 : if( hfc_chk_conf_val(0,1,hfc14_enable_tgtrst) ) ap->enable_tgtrst = hfc14_enable_tgtrst; break;
		case 15 : if( hfc_chk_conf_val(0,1,hfc15_enable_tgtrst) ) ap->enable_tgtrst = hfc15_enable_tgtrst; break;
		case 16 : if( hfc_chk_conf_val(0,1,hfc16_enable_tgtrst) ) ap->enable_tgtrst = hfc16_enable_tgtrst; break;
		case 17 : if( hfc_chk_conf_val(0,1,hfc17_enable_tgtrst) ) ap->enable_tgtrst = hfc17_enable_tgtrst; break;
		case 18 : if( hfc_chk_conf_val(0,1,hfc18_enable_tgtrst) ) ap->enable_tgtrst = hfc18_enable_tgtrst; break;
		case 19 : if( hfc_chk_conf_val(0,1,hfc19_enable_tgtrst) ) ap->enable_tgtrst = hfc19_enable_tgtrst; break;
		case 20 : if( hfc_chk_conf_val(0,1,hfc20_enable_tgtrst) ) ap->enable_tgtrst = hfc20_enable_tgtrst; break;
		case 21 : if( hfc_chk_conf_val(0,1,hfc21_enable_tgtrst) ) ap->enable_tgtrst = hfc21_enable_tgtrst; break;
		case 22 : if( hfc_chk_conf_val(0,1,hfc22_enable_tgtrst) ) ap->enable_tgtrst = hfc22_enable_tgtrst; break;
		case 23 : if( hfc_chk_conf_val(0,1,hfc23_enable_tgtrst) ) ap->enable_tgtrst = hfc23_enable_tgtrst; break;
		case 24 : if( hfc_chk_conf_val(0,1,hfc24_enable_tgtrst) ) ap->enable_tgtrst = hfc24_enable_tgtrst; break;
		case 25 : if( hfc_chk_conf_val(0,1,hfc25_enable_tgtrst) ) ap->enable_tgtrst = hfc25_enable_tgtrst; break;
		case 26 : if( hfc_chk_conf_val(0,1,hfc26_enable_tgtrst) ) ap->enable_tgtrst = hfc26_enable_tgtrst; break;
		case 27 : if( hfc_chk_conf_val(0,1,hfc27_enable_tgtrst) ) ap->enable_tgtrst = hfc27_enable_tgtrst; break;
		case 28 : if( hfc_chk_conf_val(0,1,hfc28_enable_tgtrst) ) ap->enable_tgtrst = hfc28_enable_tgtrst; break;
		case 29 : if( hfc_chk_conf_val(0,1,hfc29_enable_tgtrst) ) ap->enable_tgtrst = hfc29_enable_tgtrst; break;
		case 30 : if( hfc_chk_conf_val(0,1,hfc30_enable_tgtrst) ) ap->enable_tgtrst = hfc30_enable_tgtrst; break;
		case 31 : if( hfc_chk_conf_val(0,1,hfc31_enable_tgtrst) ) ap->enable_tgtrst = hfc31_enable_tgtrst; break;
	}

	if( hfc_chk_conf_val(0,1,hfcmp_enable_tgtrst[ap->instance]) ) {
		ap->enable_tgtrst = hfcmp_enable_tgtrst[ap->instance];
		HFC_DBGPRT("enable target reset <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("enable target reset = %d \n", ap->enable_tgtrst );

}


void hfc_set_max_target(struct adap_info *ap)
{

	ap->max_target = MAX_TARGET_PROBE;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc_max_target)){ /* in case global parameter is set */
		ap->max_target = hfc_max_target; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc0_max_target) ) ap->max_target =  hfc0_max_target; break;
		case  1 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc1_max_target) ) ap->max_target =  hfc1_max_target; break;
		case  2 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc2_max_target) ) ap->max_target =  hfc2_max_target; break;
		case  3 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc3_max_target) ) ap->max_target =  hfc3_max_target; break;
		case  4 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc4_max_target) ) ap->max_target =  hfc4_max_target; break;
		case  5 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc5_max_target) ) ap->max_target =  hfc5_max_target; break;
		case  6 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc6_max_target) ) ap->max_target =  hfc6_max_target; break;
		case  7 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc7_max_target) ) ap->max_target =  hfc7_max_target; break;
		case  8 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc8_max_target) ) ap->max_target =  hfc8_max_target; break;
		case  9 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE, hfc9_max_target) ) ap->max_target =  hfc9_max_target; break;
		case 10 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc10_max_target) ) ap->max_target = hfc10_max_target; break;
		case 11 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc11_max_target) ) ap->max_target = hfc11_max_target; break;
		case 12 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc12_max_target) ) ap->max_target = hfc12_max_target; break;
		case 13 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc13_max_target) ) ap->max_target = hfc13_max_target; break;
		case 14 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc14_max_target) ) ap->max_target = hfc14_max_target; break;
		case 15 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc15_max_target) ) ap->max_target = hfc15_max_target; break;
		case 16 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc16_max_target) ) ap->max_target = hfc16_max_target; break;
		case 17 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc17_max_target) ) ap->max_target = hfc17_max_target; break;
		case 18 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc18_max_target) ) ap->max_target = hfc18_max_target; break;
		case 19 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc19_max_target) ) ap->max_target = hfc19_max_target; break;
		case 20 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc20_max_target) ) ap->max_target = hfc20_max_target; break;
		case 21 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc21_max_target) ) ap->max_target = hfc21_max_target; break;
		case 22 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc22_max_target) ) ap->max_target = hfc22_max_target; break;
		case 23 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc23_max_target) ) ap->max_target = hfc23_max_target; break;
		case 24 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc24_max_target) ) ap->max_target = hfc24_max_target; break;
		case 25 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc25_max_target) ) ap->max_target = hfc25_max_target; break;
		case 26 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc26_max_target) ) ap->max_target = hfc26_max_target; break;
		case 27 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc27_max_target) ) ap->max_target = hfc27_max_target; break;
		case 28 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc28_max_target) ) ap->max_target = hfc28_max_target; break;
		case 29 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc29_max_target) ) ap->max_target = hfc29_max_target; break;
		case 30 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc30_max_target) ) ap->max_target = hfc30_max_target; break;
		case 31 : if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfc31_max_target) ) ap->max_target = hfc31_max_target; break;
	}

	if( hfc_chk_conf_val(0,MAX_TARGET_PROBE,hfcmp_max_target[ap->instance]) ) {
		ap->max_target =  hfcmp_max_target[ap->instance];
		HFC_DBGPRT("max target number <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("max target number = %d \n", ap->max_target );

}



void hfc_set_xob_max(struct adap_info *ap)
{
	
	ap->xob_max = MAX_XOB_NUM;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,MAX_XOB_NUM,hfc_xob_max)){ /* in case global parameter is set */
		ap->xob_max = hfc_xob_max; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc0_xob_max) ) ap->xob_max =  hfc0_xob_max; break;
		case  1 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc1_xob_max) ) ap->xob_max =  hfc1_xob_max; break;
		case  2 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc2_xob_max) ) ap->xob_max =  hfc2_xob_max; break;
		case  3 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc3_xob_max) ) ap->xob_max =  hfc3_xob_max; break;
		case  4 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc4_xob_max) ) ap->xob_max =  hfc4_xob_max; break;
		case  5 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc5_xob_max) ) ap->xob_max =  hfc5_xob_max; break;
		case  6 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc6_xob_max) ) ap->xob_max =  hfc6_xob_max; break;
		case  7 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc7_xob_max) ) ap->xob_max =  hfc7_xob_max; break;
		case  8 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc8_xob_max) ) ap->xob_max =  hfc8_xob_max; break;
		case  9 : if( hfc_chk_conf_val(0,MAX_XOB_NUM, hfc9_xob_max) ) ap->xob_max =  hfc9_xob_max; break;
		case 10 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc10_xob_max) ) ap->xob_max = hfc10_xob_max; break;
		case 11 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc11_xob_max) ) ap->xob_max = hfc11_xob_max; break;
		case 12 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc12_xob_max) ) ap->xob_max = hfc12_xob_max; break;
		case 13 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc13_xob_max) ) ap->xob_max = hfc13_xob_max; break;
		case 14 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc14_xob_max) ) ap->xob_max = hfc14_xob_max; break;
		case 15 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc15_xob_max) ) ap->xob_max = hfc15_xob_max; break;
		case 16 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc16_xob_max) ) ap->xob_max = hfc16_xob_max; break;
		case 17 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc17_xob_max) ) ap->xob_max = hfc17_xob_max; break;
		case 18 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc18_xob_max) ) ap->xob_max = hfc18_xob_max; break;
		case 19 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc19_xob_max) ) ap->xob_max = hfc19_xob_max; break;
		case 20 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc20_xob_max) ) ap->xob_max = hfc20_xob_max; break;
		case 21 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc21_xob_max) ) ap->xob_max = hfc21_xob_max; break;
		case 22 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc22_xob_max) ) ap->xob_max = hfc22_xob_max; break;
		case 23 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc23_xob_max) ) ap->xob_max = hfc23_xob_max; break;
		case 24 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc24_xob_max) ) ap->xob_max = hfc24_xob_max; break;
		case 25 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc25_xob_max) ) ap->xob_max = hfc25_xob_max; break;
		case 26 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc26_xob_max) ) ap->xob_max = hfc26_xob_max; break;
		case 27 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc27_xob_max) ) ap->xob_max = hfc27_xob_max; break;
		case 28 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc28_xob_max) ) ap->xob_max = hfc28_xob_max; break;
		case 29 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc29_xob_max) ) ap->xob_max = hfc29_xob_max; break;
		case 30 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc30_xob_max) ) ap->xob_max = hfc30_xob_max; break;
		case 31 : if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfc31_xob_max) ) ap->xob_max = hfc31_xob_max; break;
	}

	if( hfc_chk_conf_val(0,MAX_XOB_NUM,hfcmp_xob_max[ap->instance]) ) {
		ap->xob_max = hfcmp_xob_max[ap->instance];
		HFC_DBGPRT("max xob number <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("max xob number = %d \n", ap->xob_max );

}


void hfc_set_xrb_max(struct adap_info *ap)
{

	ap->xrb_max = MAX_XRB_NUM;

	if(ap->defparam) return;

	/* abort task set timeout */

	if(hfc_chk_conf_val(0,MAX_XRB_NUM,hfc_xrb_max)){ /* in case global parameter is set */
		ap->xrb_max = hfc_xrb_max; 

	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc0_xrb_max) ) ap->xrb_max =  hfc0_xrb_max; break;
		case  1 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc1_xrb_max) ) ap->xrb_max =  hfc1_xrb_max; break;
		case  2 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc2_xrb_max) ) ap->xrb_max =  hfc2_xrb_max; break;
		case  3 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc3_xrb_max) ) ap->xrb_max =  hfc3_xrb_max; break;
		case  4 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc4_xrb_max) ) ap->xrb_max =  hfc4_xrb_max; break;
		case  5 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc5_xrb_max) ) ap->xrb_max =  hfc5_xrb_max; break;
		case  6 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc6_xrb_max) ) ap->xrb_max =  hfc6_xrb_max; break;
		case  7 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc7_xrb_max) ) ap->xrb_max =  hfc7_xrb_max; break;
		case  8 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc8_xrb_max) ) ap->xrb_max =  hfc8_xrb_max; break;
		case  9 : if( hfc_chk_conf_val(0,MAX_XRB_NUM, hfc9_xrb_max) ) ap->xrb_max =  hfc9_xrb_max; break;
		case 10 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc10_xrb_max) ) ap->xrb_max = hfc10_xrb_max; break;
		case 11 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc11_xrb_max) ) ap->xrb_max = hfc11_xrb_max; break;
		case 12 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc12_xrb_max) ) ap->xrb_max = hfc12_xrb_max; break;
		case 13 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc13_xrb_max) ) ap->xrb_max = hfc13_xrb_max; break;
		case 14 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc14_xrb_max) ) ap->xrb_max = hfc14_xrb_max; break;
		case 15 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc15_xrb_max) ) ap->xrb_max = hfc15_xrb_max; break;
		case 16 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc16_xrb_max) ) ap->xrb_max = hfc16_xrb_max; break;
		case 17 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc17_xrb_max) ) ap->xrb_max = hfc17_xrb_max; break;
		case 18 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc18_xrb_max) ) ap->xrb_max = hfc18_xrb_max; break;
		case 19 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc19_xrb_max) ) ap->xrb_max = hfc19_xrb_max; break;
		case 20 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc20_xrb_max) ) ap->xrb_max = hfc20_xrb_max; break;
		case 21 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc21_xrb_max) ) ap->xrb_max = hfc21_xrb_max; break;
		case 22 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc22_xrb_max) ) ap->xrb_max = hfc22_xrb_max; break;
		case 23 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc23_xrb_max) ) ap->xrb_max = hfc23_xrb_max; break;
		case 24 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc24_xrb_max) ) ap->xrb_max = hfc24_xrb_max; break;
		case 25 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc25_xrb_max) ) ap->xrb_max = hfc25_xrb_max; break;
		case 26 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc26_xrb_max) ) ap->xrb_max = hfc26_xrb_max; break;
		case 27 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc27_xrb_max) ) ap->xrb_max = hfc27_xrb_max; break;
		case 28 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc28_xrb_max) ) ap->xrb_max = hfc28_xrb_max; break;
		case 29 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc29_xrb_max) ) ap->xrb_max = hfc29_xrb_max; break;
		case 30 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc30_xrb_max) ) ap->xrb_max = hfc30_xrb_max; break;
		case 31 : if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfc31_xrb_max) ) ap->xrb_max = hfc31_xrb_max; break;
	}

	if( hfc_chk_conf_val(0,MAX_XRB_NUM,hfcmp_xrb_max[ap->instance]) ) {
		ap->xrb_max = hfcmp_xrb_max[ap->instance];
		HFC_DBGPRT("max xrb number <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("max xrb number = %d \n", ap->xrb_max );

}



void hfc_set_slog_max(struct adap_info *ap)
{
	
	ap->slog_max = HFC_SOFT_LOG_PAGE;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc_slog_max)){ /* in case global parameter is set */
		ap->slog_max = hfc_slog_max; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc0_slog_max) ) ap->slog_max =  hfc0_slog_max; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc1_slog_max) ) ap->slog_max =  hfc1_slog_max; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc2_slog_max) ) ap->slog_max =  hfc2_slog_max; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc3_slog_max) ) ap->slog_max =  hfc3_slog_max; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc4_slog_max) ) ap->slog_max =  hfc4_slog_max; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc5_slog_max) ) ap->slog_max =  hfc5_slog_max; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc6_slog_max) ) ap->slog_max =  hfc6_slog_max; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc7_slog_max) ) ap->slog_max =  hfc7_slog_max; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc8_slog_max) ) ap->slog_max =  hfc8_slog_max; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE, hfc9_slog_max) ) ap->slog_max =  hfc9_slog_max; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc10_slog_max) ) ap->slog_max = hfc10_slog_max; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc11_slog_max) ) ap->slog_max = hfc11_slog_max; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc12_slog_max) ) ap->slog_max = hfc12_slog_max; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc13_slog_max) ) ap->slog_max = hfc13_slog_max; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc14_slog_max) ) ap->slog_max = hfc14_slog_max; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc15_slog_max) ) ap->slog_max = hfc15_slog_max; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc16_slog_max) ) ap->slog_max = hfc16_slog_max; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc17_slog_max) ) ap->slog_max = hfc17_slog_max; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc18_slog_max) ) ap->slog_max = hfc18_slog_max; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc19_slog_max) ) ap->slog_max = hfc19_slog_max; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc20_slog_max) ) ap->slog_max = hfc20_slog_max; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc21_slog_max) ) ap->slog_max = hfc21_slog_max; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc22_slog_max) ) ap->slog_max = hfc22_slog_max; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc23_slog_max) ) ap->slog_max = hfc23_slog_max; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc24_slog_max) ) ap->slog_max = hfc24_slog_max; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc25_slog_max) ) ap->slog_max = hfc25_slog_max; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc26_slog_max) ) ap->slog_max = hfc26_slog_max; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc27_slog_max) ) ap->slog_max = hfc27_slog_max; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc28_slog_max) ) ap->slog_max = hfc28_slog_max; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc29_slog_max) ) ap->slog_max = hfc29_slog_max; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc30_slog_max) ) ap->slog_max = hfc30_slog_max; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfc31_slog_max) ) ap->slog_max = hfc31_slog_max; break;
	}

	if( hfc_chk_conf_val(0,HFC_SOFT_LOG_PAGE,hfcmp_slog_max[ap->instance]) ) {
		ap->slog_max =  hfcmp_slog_max[ap->instance];
		HFC_DBGPRT("slog max <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("slog max = %d \n", ap->slog_max );
}



void hfc_set_trc_max(struct adap_info *ap)
{
	
	ap->trc_max = HFC_MAX_TRCCNT;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc_trc_max)){ /* in case global parameter is set */
		ap->trc_max = hfc_trc_max; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc0_trc_max) ) ap->trc_max =  hfc0_trc_max; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc1_trc_max) ) ap->trc_max =  hfc1_trc_max; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc2_trc_max) ) ap->trc_max =  hfc2_trc_max; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc3_trc_max) ) ap->trc_max =  hfc3_trc_max; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc4_trc_max) ) ap->trc_max =  hfc4_trc_max; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc5_trc_max) ) ap->trc_max =  hfc5_trc_max; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc6_trc_max) ) ap->trc_max =  hfc6_trc_max; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc7_trc_max) ) ap->trc_max =  hfc7_trc_max; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc8_trc_max) ) ap->trc_max =  hfc8_trc_max; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT, hfc9_trc_max) ) ap->trc_max =  hfc9_trc_max; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc10_trc_max) ) ap->trc_max = hfc10_trc_max; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc11_trc_max) ) ap->trc_max = hfc11_trc_max; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc12_trc_max) ) ap->trc_max = hfc12_trc_max; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc13_trc_max) ) ap->trc_max = hfc13_trc_max; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc14_trc_max) ) ap->trc_max = hfc14_trc_max; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc15_trc_max) ) ap->trc_max = hfc15_trc_max; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc16_trc_max) ) ap->trc_max = hfc16_trc_max; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc17_trc_max) ) ap->trc_max = hfc17_trc_max; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc18_trc_max) ) ap->trc_max = hfc18_trc_max; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc19_trc_max) ) ap->trc_max = hfc19_trc_max; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc20_trc_max) ) ap->trc_max = hfc20_trc_max; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc21_trc_max) ) ap->trc_max = hfc21_trc_max; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc22_trc_max) ) ap->trc_max = hfc22_trc_max; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc23_trc_max) ) ap->trc_max = hfc23_trc_max; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc24_trc_max) ) ap->trc_max = hfc24_trc_max; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc25_trc_max) ) ap->trc_max = hfc25_trc_max; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc26_trc_max) ) ap->trc_max = hfc26_trc_max; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc27_trc_max) ) ap->trc_max = hfc27_trc_max; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc28_trc_max) ) ap->trc_max = hfc28_trc_max; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc29_trc_max) ) ap->trc_max = hfc29_trc_max; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc30_trc_max) ) ap->trc_max = hfc30_trc_max; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfc31_trc_max) ) ap->trc_max = hfc31_trc_max; break;
	}

	if( hfc_chk_conf_val(0,HFC_MAX_TRCCNT,hfcmp_trc_max[ap->instance]) ) {
		ap->trc_max =  hfcmp_trc_max[ap->instance];
		HFC_DBGPRT("trace max <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("trace max = %d \n", ap->trc_max );
}



void hfc_set_pkt_num(struct adap_info *ap)
{
	
	ap->pkt_num = HFC_PKT_NUM;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(1,HFC_PKT_NUM,hfc_pkt_num)){ /* in case global parameter is set */
		ap->pkt_num = hfc_pkt_num; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc0_pkt_num) ) ap->pkt_num =  hfc0_pkt_num; break;
		case  1 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc1_pkt_num) ) ap->pkt_num =  hfc1_pkt_num; break;
		case  2 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc2_pkt_num) ) ap->pkt_num =  hfc2_pkt_num; break;
		case  3 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc3_pkt_num) ) ap->pkt_num =  hfc3_pkt_num; break;
		case  4 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc4_pkt_num) ) ap->pkt_num =  hfc4_pkt_num; break;
		case  5 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc5_pkt_num) ) ap->pkt_num =  hfc5_pkt_num; break;
		case  6 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc6_pkt_num) ) ap->pkt_num =  hfc6_pkt_num; break;
		case  7 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc7_pkt_num) ) ap->pkt_num =  hfc7_pkt_num; break;
		case  8 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc8_pkt_num) ) ap->pkt_num =  hfc8_pkt_num; break;
		case  9 : if( hfc_chk_conf_val(1,HFC_PKT_NUM, hfc9_pkt_num) ) ap->pkt_num =  hfc9_pkt_num; break;
		case 10 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc10_pkt_num) ) ap->pkt_num = hfc10_pkt_num; break;
		case 11 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc11_pkt_num) ) ap->pkt_num = hfc11_pkt_num; break;
		case 12 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc12_pkt_num) ) ap->pkt_num = hfc12_pkt_num; break;
		case 13 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc13_pkt_num) ) ap->pkt_num = hfc13_pkt_num; break;
		case 14 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc14_pkt_num) ) ap->pkt_num = hfc14_pkt_num; break;
		case 15 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc15_pkt_num) ) ap->pkt_num = hfc15_pkt_num; break;
		case 16 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc16_pkt_num) ) ap->pkt_num = hfc16_pkt_num; break;
		case 17 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc17_pkt_num) ) ap->pkt_num = hfc17_pkt_num; break;
		case 18 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc18_pkt_num) ) ap->pkt_num = hfc18_pkt_num; break;
		case 19 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc19_pkt_num) ) ap->pkt_num = hfc19_pkt_num; break;
		case 20 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc20_pkt_num) ) ap->pkt_num = hfc20_pkt_num; break;
		case 21 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc21_pkt_num) ) ap->pkt_num = hfc21_pkt_num; break;
		case 22 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc22_pkt_num) ) ap->pkt_num = hfc22_pkt_num; break;
		case 23 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc23_pkt_num) ) ap->pkt_num = hfc23_pkt_num; break;
		case 24 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc24_pkt_num) ) ap->pkt_num = hfc24_pkt_num; break;
		case 25 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc25_pkt_num) ) ap->pkt_num = hfc25_pkt_num; break;
		case 26 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc26_pkt_num) ) ap->pkt_num = hfc26_pkt_num; break;
		case 27 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc27_pkt_num) ) ap->pkt_num = hfc27_pkt_num; break;
		case 28 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc28_pkt_num) ) ap->pkt_num = hfc28_pkt_num; break;
		case 29 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc29_pkt_num) ) ap->pkt_num = hfc29_pkt_num; break;
		case 30 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc30_pkt_num) ) ap->pkt_num = hfc30_pkt_num; break;
		case 31 : if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfc31_pkt_num) ) ap->pkt_num = hfc31_pkt_num; break;
	}

	if( hfc_chk_conf_val(1,HFC_PKT_NUM,hfcmp_pkt_num[ap->instance]) ) {
		ap->pkt_num =  hfcmp_pkt_num[ap->instance];
		HFC_DBGPRT("hfc_pkt_number <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("hfc_pkt_number = %d \n", ap->pkt_num );
}



void hfc_set_can_queue(struct adap_info *ap)
{
	
	ap->can_queue = HFC_DEFAULT_CAN_QUEUE;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc_can_queue)){ /* in case global parameter is set */
		ap->can_queue = hfc_can_queue; 
	}

	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc0_can_queue) ) ap->can_queue =  hfc0_can_queue; break;
		case  1 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc1_can_queue) ) ap->can_queue =  hfc1_can_queue; break;
		case  2 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc2_can_queue) ) ap->can_queue =  hfc2_can_queue; break;
		case  3 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc3_can_queue) ) ap->can_queue =  hfc3_can_queue; break;
		case  4 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc4_can_queue) ) ap->can_queue =  hfc4_can_queue; break;
		case  5 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc5_can_queue) ) ap->can_queue =  hfc5_can_queue; break;
		case  6 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc6_can_queue) ) ap->can_queue =  hfc6_can_queue; break;
		case  7 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc7_can_queue) ) ap->can_queue =  hfc7_can_queue; break;
		case  8 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc8_can_queue) ) ap->can_queue =  hfc8_can_queue; break;
		case  9 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE, hfc9_can_queue) ) ap->can_queue =  hfc9_can_queue; break;
		case 10 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc10_can_queue) ) ap->can_queue = hfc10_can_queue; break;
		case 11 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc11_can_queue) ) ap->can_queue = hfc11_can_queue; break;
		case 12 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc12_can_queue) ) ap->can_queue = hfc12_can_queue; break;
		case 13 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc13_can_queue) ) ap->can_queue = hfc13_can_queue; break;
		case 14 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc14_can_queue) ) ap->can_queue = hfc14_can_queue; break;
		case 15 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc15_can_queue) ) ap->can_queue = hfc15_can_queue; break;
		case 16 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc16_can_queue) ) ap->can_queue = hfc16_can_queue; break;
		case 17 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc17_can_queue) ) ap->can_queue = hfc17_can_queue; break;
		case 18 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc18_can_queue) ) ap->can_queue = hfc18_can_queue; break;
		case 19 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc19_can_queue) ) ap->can_queue = hfc19_can_queue; break;
		case 20 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc20_can_queue) ) ap->can_queue = hfc20_can_queue; break;
		case 21 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc21_can_queue) ) ap->can_queue = hfc21_can_queue; break;
		case 22 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc22_can_queue) ) ap->can_queue = hfc22_can_queue; break;
		case 23 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc23_can_queue) ) ap->can_queue = hfc23_can_queue; break;
		case 24 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc24_can_queue) ) ap->can_queue = hfc24_can_queue; break;
		case 25 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc25_can_queue) ) ap->can_queue = hfc25_can_queue; break;
		case 26 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc26_can_queue) ) ap->can_queue = hfc26_can_queue; break;
		case 27 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc27_can_queue) ) ap->can_queue = hfc27_can_queue; break;
		case 28 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc28_can_queue) ) ap->can_queue = hfc28_can_queue; break;
		case 29 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc29_can_queue) ) ap->can_queue = hfc29_can_queue; break;
		case 30 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc30_can_queue) ) ap->can_queue = hfc30_can_queue; break;
		case 31 : if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfc31_can_queue) ) ap->can_queue = hfc31_can_queue; break;
	}

	if( hfc_chk_conf_val(1,HFC_DEFAULT_CAN_QUEUE,hfcmp_can_queue[ap->instance]) ) {
		ap->can_queue =  hfcmp_can_queue[ap->instance];
		HFC_DBGPRT("max can queue <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("max can_queue = %d \n", ap->can_queue );
}



void hfc_set_sg_tblsize(struct adap_info *ap)
{
	
	ap->sg_tblsize = HFC_SG_TABLESIZE;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc_sg_tblsize)){ /* in case global parameter is set */
		ap->sg_tblsize = hfc_sg_tblsize; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc0_sg_tblsize) ) ap->sg_tblsize =  hfc0_sg_tblsize; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc1_sg_tblsize) ) ap->sg_tblsize =  hfc1_sg_tblsize; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc2_sg_tblsize) ) ap->sg_tblsize =  hfc2_sg_tblsize; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc3_sg_tblsize) ) ap->sg_tblsize =  hfc3_sg_tblsize; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc4_sg_tblsize) ) ap->sg_tblsize =  hfc4_sg_tblsize; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc5_sg_tblsize) ) ap->sg_tblsize =  hfc5_sg_tblsize; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc6_sg_tblsize) ) ap->sg_tblsize =  hfc6_sg_tblsize; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc7_sg_tblsize) ) ap->sg_tblsize =  hfc7_sg_tblsize; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc8_sg_tblsize) ) ap->sg_tblsize =  hfc8_sg_tblsize; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE, hfc9_sg_tblsize) ) ap->sg_tblsize =  hfc9_sg_tblsize; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc10_sg_tblsize) ) ap->sg_tblsize = hfc10_sg_tblsize; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc11_sg_tblsize) ) ap->sg_tblsize = hfc11_sg_tblsize; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc12_sg_tblsize) ) ap->sg_tblsize = hfc12_sg_tblsize; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc13_sg_tblsize) ) ap->sg_tblsize = hfc13_sg_tblsize; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc14_sg_tblsize) ) ap->sg_tblsize = hfc14_sg_tblsize; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc15_sg_tblsize) ) ap->sg_tblsize = hfc15_sg_tblsize; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc16_sg_tblsize) ) ap->sg_tblsize = hfc16_sg_tblsize; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc17_sg_tblsize) ) ap->sg_tblsize = hfc17_sg_tblsize; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc18_sg_tblsize) ) ap->sg_tblsize = hfc18_sg_tblsize; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc19_sg_tblsize) ) ap->sg_tblsize = hfc19_sg_tblsize; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc20_sg_tblsize) ) ap->sg_tblsize = hfc20_sg_tblsize; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc21_sg_tblsize) ) ap->sg_tblsize = hfc21_sg_tblsize; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc22_sg_tblsize) ) ap->sg_tblsize = hfc22_sg_tblsize; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc23_sg_tblsize) ) ap->sg_tblsize = hfc23_sg_tblsize; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc24_sg_tblsize) ) ap->sg_tblsize = hfc24_sg_tblsize; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc25_sg_tblsize) ) ap->sg_tblsize = hfc25_sg_tblsize; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc26_sg_tblsize) ) ap->sg_tblsize = hfc26_sg_tblsize; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc27_sg_tblsize) ) ap->sg_tblsize = hfc27_sg_tblsize; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc28_sg_tblsize) ) ap->sg_tblsize = hfc28_sg_tblsize; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc29_sg_tblsize) ) ap->sg_tblsize = hfc29_sg_tblsize; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc30_sg_tblsize) ) ap->sg_tblsize = hfc30_sg_tblsize; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfc31_sg_tblsize) ) ap->sg_tblsize = hfc31_sg_tblsize; break;
	}

	if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfcmp_sg_tblsize[ap->instance]) ) {
		ap->sg_tblsize =  hfcmp_sg_tblsize[ap->instance];
		HFC_DBGPRT("sg tablesize <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("sg tablesize = %d \n", ap->sg_tblsize );
}


void hfc_set_cmnd_num(struct adap_info *ap)
{
	
	ap->cmnd_num = HFC_CMND_NUM;
	if(ap->defparam) return;


	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_CMND_NUM,hfc_cmnd_num)){ /* in case global parameter is set */
		ap->cmnd_num = hfc_cmnd_num; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc0_cmnd_num) ) ap->cmnd_num =  hfc0_cmnd_num; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc1_cmnd_num) ) ap->cmnd_num =  hfc1_cmnd_num; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc2_cmnd_num) ) ap->cmnd_num =  hfc2_cmnd_num; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc3_cmnd_num) ) ap->cmnd_num =  hfc3_cmnd_num; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc4_cmnd_num) ) ap->cmnd_num =  hfc4_cmnd_num; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc5_cmnd_num) ) ap->cmnd_num =  hfc5_cmnd_num; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc6_cmnd_num) ) ap->cmnd_num =  hfc6_cmnd_num; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc7_cmnd_num) ) ap->cmnd_num =  hfc7_cmnd_num; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc8_cmnd_num) ) ap->cmnd_num =  hfc8_cmnd_num; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_CMND_NUM, hfc9_cmnd_num) ) ap->cmnd_num =  hfc9_cmnd_num; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc10_cmnd_num) ) ap->cmnd_num = hfc10_cmnd_num; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc11_cmnd_num) ) ap->cmnd_num = hfc11_cmnd_num; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc12_cmnd_num) ) ap->cmnd_num = hfc12_cmnd_num; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc13_cmnd_num) ) ap->cmnd_num = hfc13_cmnd_num; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc14_cmnd_num) ) ap->cmnd_num = hfc14_cmnd_num; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc15_cmnd_num) ) ap->cmnd_num = hfc15_cmnd_num; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc16_cmnd_num) ) ap->cmnd_num = hfc16_cmnd_num; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc17_cmnd_num) ) ap->cmnd_num = hfc17_cmnd_num; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc18_cmnd_num) ) ap->cmnd_num = hfc18_cmnd_num; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc19_cmnd_num) ) ap->cmnd_num = hfc19_cmnd_num; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc20_cmnd_num) ) ap->cmnd_num = hfc10_cmnd_num; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc21_cmnd_num) ) ap->cmnd_num = hfc21_cmnd_num; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc22_cmnd_num) ) ap->cmnd_num = hfc22_cmnd_num; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc23_cmnd_num) ) ap->cmnd_num = hfc23_cmnd_num; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc24_cmnd_num) ) ap->cmnd_num = hfc24_cmnd_num; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc25_cmnd_num) ) ap->cmnd_num = hfc25_cmnd_num; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc26_cmnd_num) ) ap->cmnd_num = hfc26_cmnd_num; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc27_cmnd_num) ) ap->cmnd_num = hfc27_cmnd_num; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc28_cmnd_num) ) ap->cmnd_num = hfc28_cmnd_num; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc29_cmnd_num) ) ap->cmnd_num = hfc29_cmnd_num; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc30_cmnd_num) ) ap->cmnd_num = hfc30_cmnd_num; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfc31_cmnd_num) ) ap->cmnd_num = hfc31_cmnd_num; break;
	}

	if( hfc_chk_conf_val(0,HFC_CMND_NUM,hfcmp_cmnd_num[ap->instance]) ) {
		ap->cmnd_num =  hfcmp_cmnd_num[ap->instance];
		HFC_DBGPRT("cmnd num <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("cmnd num = %d \n", ap->cmnd_num );
}


/*
 * Function:    hfc_set_minus_tout
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_set_minus_tout(struct adap_info *ap)
{
	
	ap->minus_tout = HFC_MINUS_TIMOUT;
	if(ap->defparam) return;


	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc_minus_tout)){ /* in case global parameter is set */
		ap->minus_tout = hfc_minus_tout; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc0_minus_tout) ) ap->minus_tout =  hfc0_minus_tout; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc1_minus_tout) ) ap->minus_tout =  hfc1_minus_tout; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc2_minus_tout) ) ap->minus_tout =  hfc2_minus_tout; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc3_minus_tout) ) ap->minus_tout =  hfc3_minus_tout; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc4_minus_tout) ) ap->minus_tout =  hfc4_minus_tout; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc5_minus_tout) ) ap->minus_tout =  hfc5_minus_tout; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc6_minus_tout) ) ap->minus_tout =  hfc6_minus_tout; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc7_minus_tout) ) ap->minus_tout =  hfc7_minus_tout; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc8_minus_tout) ) ap->minus_tout =  hfc8_minus_tout; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT, hfc9_minus_tout) ) ap->minus_tout =  hfc9_minus_tout; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc10_minus_tout) ) ap->minus_tout = hfc10_minus_tout; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc11_minus_tout) ) ap->minus_tout = hfc11_minus_tout; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc12_minus_tout) ) ap->minus_tout = hfc12_minus_tout; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc13_minus_tout) ) ap->minus_tout = hfc13_minus_tout; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc14_minus_tout) ) ap->minus_tout = hfc14_minus_tout; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc15_minus_tout) ) ap->minus_tout = hfc15_minus_tout; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc16_minus_tout) ) ap->minus_tout = hfc16_minus_tout; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc17_minus_tout) ) ap->minus_tout = hfc17_minus_tout; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc18_minus_tout) ) ap->minus_tout = hfc18_minus_tout; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc19_minus_tout) ) ap->minus_tout = hfc19_minus_tout; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc20_minus_tout) ) ap->minus_tout = hfc20_minus_tout; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc21_minus_tout) ) ap->minus_tout = hfc21_minus_tout; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc22_minus_tout) ) ap->minus_tout = hfc22_minus_tout; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc23_minus_tout) ) ap->minus_tout = hfc23_minus_tout; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc24_minus_tout) ) ap->minus_tout = hfc24_minus_tout; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc25_minus_tout) ) ap->minus_tout = hfc25_minus_tout; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc26_minus_tout) ) ap->minus_tout = hfc26_minus_tout; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc27_minus_tout) ) ap->minus_tout = hfc27_minus_tout; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc28_minus_tout) ) ap->minus_tout = hfc28_minus_tout; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc29_minus_tout) ) ap->minus_tout = hfc29_minus_tout; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc30_minus_tout) ) ap->minus_tout = hfc30_minus_tout; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfc31_minus_tout) ) ap->minus_tout = hfc31_minus_tout; break;
	}

	if( hfc_chk_conf_val(0,HFC_MINUS_TIMOUT,hfcmp_minus_tout[ap->instance]) ) {
		ap->minus_tout =  hfcmp_minus_tout[ap->instance];
		HFC_DBGPRT("minus timeout <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("minus timeout = %d \n", ap->minus_tout );
	
}


void hfc_set_scsi_allowed(struct adap_info *ap)
{
	
	ap->scsi_allowed = HFC_SCSI_ALLOWED;
	if(ap->defparam) return;
	
	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,30,hfc_scsi_allowed)){ /* in case global parameter is set */
		ap->scsi_allowed = hfc_scsi_allowed; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,30, hfc0_scsi_allowed) ) ap->scsi_allowed =  hfc0_scsi_allowed; break;
		case  1 : if( hfc_chk_conf_val(0,30, hfc1_scsi_allowed) ) ap->scsi_allowed =  hfc1_scsi_allowed; break;
		case  2 : if( hfc_chk_conf_val(0,30, hfc2_scsi_allowed) ) ap->scsi_allowed =  hfc2_scsi_allowed; break;
		case  3 : if( hfc_chk_conf_val(0,30, hfc3_scsi_allowed) ) ap->scsi_allowed =  hfc3_scsi_allowed; break;
		case  4 : if( hfc_chk_conf_val(0,30, hfc4_scsi_allowed) ) ap->scsi_allowed =  hfc4_scsi_allowed; break;
		case  5 : if( hfc_chk_conf_val(0,30, hfc5_scsi_allowed) ) ap->scsi_allowed =  hfc5_scsi_allowed; break;
		case  6 : if( hfc_chk_conf_val(0,30, hfc6_scsi_allowed) ) ap->scsi_allowed =  hfc6_scsi_allowed; break;
		case  7 : if( hfc_chk_conf_val(0,30, hfc7_scsi_allowed) ) ap->scsi_allowed =  hfc7_scsi_allowed; break;
		case  8 : if( hfc_chk_conf_val(0,30, hfc8_scsi_allowed) ) ap->scsi_allowed =  hfc8_scsi_allowed; break;
		case  9 : if( hfc_chk_conf_val(0,30, hfc9_scsi_allowed) ) ap->scsi_allowed =  hfc9_scsi_allowed; break;
		case 10 : if( hfc_chk_conf_val(0,30,hfc10_scsi_allowed) ) ap->scsi_allowed = hfc10_scsi_allowed; break;
		case 11 : if( hfc_chk_conf_val(0,30,hfc11_scsi_allowed) ) ap->scsi_allowed = hfc11_scsi_allowed; break;
		case 12 : if( hfc_chk_conf_val(0,30,hfc12_scsi_allowed) ) ap->scsi_allowed = hfc12_scsi_allowed; break;
		case 13 : if( hfc_chk_conf_val(0,30,hfc13_scsi_allowed) ) ap->scsi_allowed = hfc13_scsi_allowed; break;
		case 14 : if( hfc_chk_conf_val(0,30,hfc14_scsi_allowed) ) ap->scsi_allowed = hfc14_scsi_allowed; break;
		case 15 : if( hfc_chk_conf_val(0,30,hfc15_scsi_allowed) ) ap->scsi_allowed = hfc15_scsi_allowed; break;
		case 16 : if( hfc_chk_conf_val(0,30,hfc16_scsi_allowed) ) ap->scsi_allowed = hfc16_scsi_allowed; break;
		case 17 : if( hfc_chk_conf_val(0,30,hfc17_scsi_allowed) ) ap->scsi_allowed = hfc17_scsi_allowed; break;
		case 18 : if( hfc_chk_conf_val(0,30,hfc18_scsi_allowed) ) ap->scsi_allowed = hfc18_scsi_allowed; break;
		case 19 : if( hfc_chk_conf_val(0,30,hfc19_scsi_allowed) ) ap->scsi_allowed = hfc19_scsi_allowed; break;
		case 20 : if( hfc_chk_conf_val(0,30,hfc20_scsi_allowed) ) ap->scsi_allowed = hfc20_scsi_allowed; break;
		case 21 : if( hfc_chk_conf_val(0,30,hfc21_scsi_allowed) ) ap->scsi_allowed = hfc21_scsi_allowed; break;
		case 22 : if( hfc_chk_conf_val(0,30,hfc22_scsi_allowed) ) ap->scsi_allowed = hfc22_scsi_allowed; break;
		case 23 : if( hfc_chk_conf_val(0,30,hfc23_scsi_allowed) ) ap->scsi_allowed = hfc23_scsi_allowed; break;
		case 24 : if( hfc_chk_conf_val(0,30,hfc24_scsi_allowed) ) ap->scsi_allowed = hfc24_scsi_allowed; break;
		case 25 : if( hfc_chk_conf_val(0,30,hfc25_scsi_allowed) ) ap->scsi_allowed = hfc25_scsi_allowed; break;
		case 26 : if( hfc_chk_conf_val(0,30,hfc26_scsi_allowed) ) ap->scsi_allowed = hfc26_scsi_allowed; break;
		case 27 : if( hfc_chk_conf_val(0,30,hfc27_scsi_allowed) ) ap->scsi_allowed = hfc27_scsi_allowed; break;
		case 28 : if( hfc_chk_conf_val(0,30,hfc28_scsi_allowed) ) ap->scsi_allowed = hfc28_scsi_allowed; break;
		case 29 : if( hfc_chk_conf_val(0,30,hfc29_scsi_allowed) ) ap->scsi_allowed = hfc29_scsi_allowed; break;
		case 30 : if( hfc_chk_conf_val(0,30,hfc30_scsi_allowed) ) ap->scsi_allowed = hfc30_scsi_allowed; break;
		case 31 : if( hfc_chk_conf_val(0,30,hfc31_scsi_allowed) ) ap->scsi_allowed = hfc31_scsi_allowed; break;
	}

	if( hfc_chk_conf_val(0,30,hfcmp_scsi_allowed[ap->instance]) ) {
		ap->scsi_allowed =  hfcmp_scsi_allowed[ap->instance];
		HFC_DBGPRT("scsi_allowed <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("scsi_allowed = %d \n", ap->scsi_allowed );
}



void hfc_set_cmd_per_lun(struct adap_info *ap)															/* FCLNX-283 STR*/
{
	
	ap->cmd_per_lun = HFC_CMD_PER_LUN;

	if(ap->defparam) return;
	
	if(hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc_cmd_per_lun)){ /* in case global parameter is set */
		ap->cmd_per_lun = hfc_cmd_per_lun; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc0_cmd_per_lun) ) ap->cmd_per_lun =  hfc0_cmd_per_lun; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc1_cmd_per_lun) ) ap->cmd_per_lun =  hfc1_cmd_per_lun; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc2_cmd_per_lun) ) ap->cmd_per_lun =  hfc2_cmd_per_lun; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc3_cmd_per_lun) ) ap->cmd_per_lun =  hfc3_cmd_per_lun; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc4_cmd_per_lun) ) ap->cmd_per_lun =  hfc4_cmd_per_lun; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc5_cmd_per_lun) ) ap->cmd_per_lun =  hfc5_cmd_per_lun; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc6_cmd_per_lun) ) ap->cmd_per_lun =  hfc6_cmd_per_lun; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc7_cmd_per_lun) ) ap->cmd_per_lun =  hfc7_cmd_per_lun; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc8_cmd_per_lun) ) ap->cmd_per_lun =  hfc8_cmd_per_lun; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN, hfc9_cmd_per_lun) ) ap->cmd_per_lun =  hfc9_cmd_per_lun; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc10_cmd_per_lun) ) ap->cmd_per_lun = hfc10_cmd_per_lun; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc11_cmd_per_lun) ) ap->cmd_per_lun = hfc11_cmd_per_lun; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc12_cmd_per_lun) ) ap->cmd_per_lun = hfc12_cmd_per_lun; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc13_cmd_per_lun) ) ap->cmd_per_lun = hfc13_cmd_per_lun; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc14_cmd_per_lun) ) ap->cmd_per_lun = hfc14_cmd_per_lun; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc15_cmd_per_lun) ) ap->cmd_per_lun = hfc15_cmd_per_lun; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc16_cmd_per_lun) ) ap->cmd_per_lun = hfc16_cmd_per_lun; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc17_cmd_per_lun) ) ap->cmd_per_lun = hfc17_cmd_per_lun; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc18_cmd_per_lun) ) ap->cmd_per_lun = hfc18_cmd_per_lun; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc19_cmd_per_lun) ) ap->cmd_per_lun = hfc19_cmd_per_lun; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc20_cmd_per_lun) ) ap->cmd_per_lun = hfc20_cmd_per_lun; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc21_cmd_per_lun) ) ap->cmd_per_lun = hfc21_cmd_per_lun; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc22_cmd_per_lun) ) ap->cmd_per_lun = hfc22_cmd_per_lun; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc23_cmd_per_lun) ) ap->cmd_per_lun = hfc23_cmd_per_lun; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc24_cmd_per_lun) ) ap->cmd_per_lun = hfc24_cmd_per_lun; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc25_cmd_per_lun) ) ap->cmd_per_lun = hfc25_cmd_per_lun; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc26_cmd_per_lun) ) ap->cmd_per_lun = hfc26_cmd_per_lun; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc27_cmd_per_lun) ) ap->cmd_per_lun = hfc27_cmd_per_lun; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc28_cmd_per_lun) ) ap->cmd_per_lun = hfc28_cmd_per_lun; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc29_cmd_per_lun) ) ap->cmd_per_lun = hfc29_cmd_per_lun; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc30_cmd_per_lun) ) ap->cmd_per_lun = hfc30_cmd_per_lun; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_MAX_CMD_PER_LUN,hfc31_cmd_per_lun) ) ap->cmd_per_lun = hfc31_cmd_per_lun; break;
	}

	if( hfc_chk_conf_val(0,HFC_MAX_SG_TABLESIZE,hfcmp_cmd_per_lun[ap->instance]) ) {
		ap->cmd_per_lun =  hfcmp_cmd_per_lun[ap->instance];
		HFC_DBGPRT("cmd_per_lun <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("cmd_per_lun = %d \n", ap->cmd_per_lun );
}																										/* FCLNX-283 END */



void hfc_set_max_sectors(struct adap_info *ap)															/* FCLNX-283 STR */
{
	
	ap->max_sectors = HFC_MAX_SECTORS;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc_max_sectors)){ /* in case global parameter is set */
		ap->max_sectors = hfc_max_sectors; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc0_max_sectors) ) ap->max_sectors =  hfc0_max_sectors; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc1_max_sectors) ) ap->max_sectors =  hfc1_max_sectors; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc2_max_sectors) ) ap->max_sectors =  hfc2_max_sectors; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc3_max_sectors) ) ap->max_sectors =  hfc3_max_sectors; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc4_max_sectors) ) ap->max_sectors =  hfc4_max_sectors; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc5_max_sectors) ) ap->max_sectors =  hfc5_max_sectors; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc6_max_sectors) ) ap->max_sectors =  hfc6_max_sectors; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc7_max_sectors) ) ap->max_sectors =  hfc7_max_sectors; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc8_max_sectors) ) ap->max_sectors =  hfc8_max_sectors; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS, hfc9_max_sectors) ) ap->max_sectors =  hfc9_max_sectors; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc10_max_sectors) ) ap->max_sectors = hfc10_max_sectors; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc11_max_sectors) ) ap->max_sectors = hfc11_max_sectors; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc12_max_sectors) ) ap->max_sectors = hfc12_max_sectors; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc13_max_sectors) ) ap->max_sectors = hfc13_max_sectors; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc14_max_sectors) ) ap->max_sectors = hfc14_max_sectors; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc15_max_sectors) ) ap->max_sectors = hfc15_max_sectors; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc16_max_sectors) ) ap->max_sectors = hfc16_max_sectors; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc17_max_sectors) ) ap->max_sectors = hfc17_max_sectors; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc18_max_sectors) ) ap->max_sectors = hfc18_max_sectors; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc19_max_sectors) ) ap->max_sectors = hfc19_max_sectors; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc20_max_sectors) ) ap->max_sectors = hfc20_max_sectors; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc21_max_sectors) ) ap->max_sectors = hfc21_max_sectors; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc22_max_sectors) ) ap->max_sectors = hfc22_max_sectors; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc23_max_sectors) ) ap->max_sectors = hfc23_max_sectors; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc24_max_sectors) ) ap->max_sectors = hfc24_max_sectors; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc25_max_sectors) ) ap->max_sectors = hfc25_max_sectors; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc26_max_sectors) ) ap->max_sectors = hfc26_max_sectors; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc27_max_sectors) ) ap->max_sectors = hfc27_max_sectors; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc28_max_sectors) ) ap->max_sectors = hfc28_max_sectors; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc29_max_sectors) ) ap->max_sectors = hfc29_max_sectors; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc30_max_sectors) ) ap->max_sectors = hfc30_max_sectors; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfc31_max_sectors) ) ap->max_sectors = hfc31_max_sectors; break;
	}

	if( hfc_chk_conf_val(0,HFC_MAX_MAX_SECTORS,hfcmp_max_sectors[ap->instance]) ) {
		ap->max_sectors =  hfcmp_max_sectors[ap->instance];
		HFC_DBGPRT("max_sectors <-- hfcldd.conf\n");
	}
	
	if( ap->max_sectors > 0 ){
		 ap->hosts->max_sectors = ap->max_sectors;
	}

	HFC_DBGPRT("max_sectors = %d \n", ap->max_sectors );
}																										/* FCLNX-283 END */


void hfc_set_vary_io(struct adap_info *ap)																/* FCLNX-283 STR */
{
	
	ap->vary_io = 0;

	if(ap->defparam) return;

	/* abort task set timeout */
	
	if(hfc_chk_conf_val(0,HFC_VARY_IO,hfc_vary_io)){ /* in case global parameter is set */
		ap->vary_io = hfc_vary_io; 
		
	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc0_vary_io) ) ap->vary_io =  hfc0_vary_io; break;
		case  1 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc1_vary_io) ) ap->vary_io =  hfc1_vary_io; break;
		case  2 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc2_vary_io) ) ap->vary_io =  hfc2_vary_io; break;
		case  3 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc3_vary_io) ) ap->vary_io =  hfc3_vary_io; break;
		case  4 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc4_vary_io) ) ap->vary_io =  hfc4_vary_io; break;
		case  5 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc5_vary_io) ) ap->vary_io =  hfc5_vary_io; break;
		case  6 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc6_vary_io) ) ap->vary_io =  hfc6_vary_io; break;
		case  7 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc7_vary_io) ) ap->vary_io =  hfc7_vary_io; break;
		case  8 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc8_vary_io) ) ap->vary_io =  hfc8_vary_io; break;
		case  9 : if( hfc_chk_conf_val(0,HFC_VARY_IO, hfc9_vary_io) ) ap->vary_io =  hfc9_vary_io; break;
		case 10 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc10_vary_io) ) ap->vary_io = hfc10_vary_io; break;
		case 11 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc11_vary_io) ) ap->vary_io = hfc11_vary_io; break;
		case 12 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc12_vary_io) ) ap->vary_io = hfc12_vary_io; break;
		case 13 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc13_vary_io) ) ap->vary_io = hfc13_vary_io; break;
		case 14 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc14_vary_io) ) ap->vary_io = hfc14_vary_io; break;
		case 15 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc15_vary_io) ) ap->vary_io = hfc15_vary_io; break;
		case 16 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc16_vary_io) ) ap->vary_io = hfc16_vary_io; break;
		case 17 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc17_vary_io) ) ap->vary_io = hfc17_vary_io; break;
		case 18 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc18_vary_io) ) ap->vary_io = hfc18_vary_io; break;
		case 19 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc19_vary_io) ) ap->vary_io = hfc19_vary_io; break;
		case 20 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc20_vary_io) ) ap->vary_io = hfc20_vary_io; break;
		case 21 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc21_vary_io) ) ap->vary_io = hfc21_vary_io; break;
		case 22 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc22_vary_io) ) ap->vary_io = hfc22_vary_io; break;
		case 23 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc23_vary_io) ) ap->vary_io = hfc23_vary_io; break;
		case 24 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc24_vary_io) ) ap->vary_io = hfc24_vary_io; break;
		case 25 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc25_vary_io) ) ap->vary_io = hfc25_vary_io; break;
		case 26 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc26_vary_io) ) ap->vary_io = hfc26_vary_io; break;
		case 27 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc27_vary_io) ) ap->vary_io = hfc27_vary_io; break;
		case 28 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc28_vary_io) ) ap->vary_io = hfc28_vary_io; break;
		case 29 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc29_vary_io) ) ap->vary_io = hfc29_vary_io; break;
		case 30 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc30_vary_io) ) ap->vary_io = hfc30_vary_io; break;
		case 31 : if( hfc_chk_conf_val(0,HFC_VARY_IO,hfc31_vary_io) ) ap->vary_io = hfc31_vary_io; break;
	}

	if( hfc_chk_conf_val(0,HFC_VARY_IO,hfcmp_vary_io[ap->instance]) ) {
		ap->vary_io =  hfcmp_vary_io[ap->instance];
		HFC_DBGPRT("vary_io <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("vary_io = %d \n", ap->vary_io );
}																										/* FCLNX-283 END */

void hfc_set_lun_reset_delay(struct adap_info *ap)/* FCLNX-0506 */ 	/* FCLNX-GPL-038 */
{

	ap->lun_reset_delay = HFC_LUN_DELAY; /* default = 0s */

	if(ap->defparam) return;

	/* link reset delay timer  */

	if(hfc_chk_conf_val(0,60,hfc_lun_reset_delay)){ /* in case global parameter is set */
		ap->lun_reset_delay = hfc_lun_reset_delay; 

	}
	switch(ap->instance){
		case  0 : if( hfc_chk_conf_val(0,60, hfc0_lun_reset_delay) ) ap->lun_reset_delay =  hfc0_lun_reset_delay; break;
		case  1 : if( hfc_chk_conf_val(0,60, hfc1_lun_reset_delay) ) ap->lun_reset_delay =  hfc1_lun_reset_delay; break;
		case  2 : if( hfc_chk_conf_val(0,60, hfc2_lun_reset_delay) ) ap->lun_reset_delay =  hfc2_lun_reset_delay; break;
		case  3 : if( hfc_chk_conf_val(0,60, hfc3_lun_reset_delay) ) ap->lun_reset_delay =  hfc3_lun_reset_delay; break;
		case  4 : if( hfc_chk_conf_val(0,60, hfc4_lun_reset_delay) ) ap->lun_reset_delay =  hfc4_lun_reset_delay; break;
		case  5 : if( hfc_chk_conf_val(0,60, hfc5_lun_reset_delay) ) ap->lun_reset_delay =  hfc5_lun_reset_delay; break;
		case  6 : if( hfc_chk_conf_val(0,60, hfc6_lun_reset_delay) ) ap->lun_reset_delay =  hfc6_lun_reset_delay; break;
		case  7 : if( hfc_chk_conf_val(0,60, hfc7_lun_reset_delay) ) ap->lun_reset_delay =  hfc7_lun_reset_delay; break;
		case  8 : if( hfc_chk_conf_val(0,60, hfc8_lun_reset_delay) ) ap->lun_reset_delay =  hfc8_lun_reset_delay; break;
		case  9 : if( hfc_chk_conf_val(0,60, hfc9_lun_reset_delay) ) ap->lun_reset_delay =  hfc9_lun_reset_delay; break;
		case 10 : if( hfc_chk_conf_val(0,60,hfc10_lun_reset_delay) ) ap->lun_reset_delay = hfc10_lun_reset_delay; break;
		case 11 : if( hfc_chk_conf_val(0,60,hfc11_lun_reset_delay) ) ap->lun_reset_delay = hfc11_lun_reset_delay; break;
		case 12 : if( hfc_chk_conf_val(0,60,hfc12_lun_reset_delay) ) ap->lun_reset_delay = hfc12_lun_reset_delay; break;
		case 13 : if( hfc_chk_conf_val(0,60,hfc13_lun_reset_delay) ) ap->lun_reset_delay = hfc13_lun_reset_delay; break;
		case 14 : if( hfc_chk_conf_val(0,60,hfc14_lun_reset_delay) ) ap->lun_reset_delay = hfc14_lun_reset_delay; break;
		case 15 : if( hfc_chk_conf_val(0,60,hfc15_lun_reset_delay) ) ap->lun_reset_delay = hfc15_lun_reset_delay; break;
		case 16 : if( hfc_chk_conf_val(0,60,hfc16_lun_reset_delay) ) ap->lun_reset_delay = hfc16_lun_reset_delay; break;
		case 17 : if( hfc_chk_conf_val(0,60,hfc17_lun_reset_delay) ) ap->lun_reset_delay = hfc17_lun_reset_delay; break;
		case 18 : if( hfc_chk_conf_val(0,60,hfc18_lun_reset_delay) ) ap->lun_reset_delay = hfc18_lun_reset_delay; break;
		case 19 : if( hfc_chk_conf_val(0,60,hfc19_lun_reset_delay) ) ap->lun_reset_delay = hfc19_lun_reset_delay; break;
		case 20 : if( hfc_chk_conf_val(0,60,hfc20_lun_reset_delay) ) ap->lun_reset_delay = hfc20_lun_reset_delay; break;
		case 21 : if( hfc_chk_conf_val(0,60,hfc21_lun_reset_delay) ) ap->lun_reset_delay = hfc21_lun_reset_delay; break;
		case 22 : if( hfc_chk_conf_val(0,60,hfc22_lun_reset_delay) ) ap->lun_reset_delay = hfc22_lun_reset_delay; break;
		case 23 : if( hfc_chk_conf_val(0,60,hfc23_lun_reset_delay) ) ap->lun_reset_delay = hfc23_lun_reset_delay; break;
		case 24 : if( hfc_chk_conf_val(0,60,hfc24_lun_reset_delay) ) ap->lun_reset_delay = hfc24_lun_reset_delay; break;
		case 25 : if( hfc_chk_conf_val(0,60,hfc25_lun_reset_delay) ) ap->lun_reset_delay = hfc25_lun_reset_delay; break;
		case 26 : if( hfc_chk_conf_val(0,60,hfc26_lun_reset_delay) ) ap->lun_reset_delay = hfc26_lun_reset_delay; break;
		case 27 : if( hfc_chk_conf_val(0,60,hfc27_lun_reset_delay) ) ap->lun_reset_delay = hfc27_lun_reset_delay; break;
		case 28 : if( hfc_chk_conf_val(0,60,hfc28_lun_reset_delay) ) ap->lun_reset_delay = hfc28_lun_reset_delay; break;
		case 29 : if( hfc_chk_conf_val(0,60,hfc29_lun_reset_delay) ) ap->lun_reset_delay = hfc29_lun_reset_delay; break;
		case 30 : if( hfc_chk_conf_val(0,60,hfc30_lun_reset_delay) ) ap->lun_reset_delay = hfc30_lun_reset_delay; break;
		case 31 : if( hfc_chk_conf_val(0,60,hfc31_lun_reset_delay) ) ap->lun_reset_delay = hfc31_lun_reset_delay; break;
	}
	
	if( hfc_chk_conf_val(0,60,hfcmp_lun_rst_delay[ap->instance]) ) {
		ap->lun_reset_delay = hfcmp_lun_rst_delay[ap->instance];
		HFC_DBGPRT("lun_reset_delay <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("lun_reset_delay = %d \n", ap->lun_reset_delay);
}																	/* FCLNX-GPL-038 */


void hfc_set_abort_t_restrain(struct adap_info *ap) /* FCLNX-0506 */
{
	ap->abort_t_restrain = 0;

	if(ap->defparam) return;

	if(hfc_chk_conf_val(0,1,hfc_abort_t_restrain)){ /* in case global parameter is set*/
			ap->abort_t_restrain = hfc_abort_t_restrain;
	}

	if( hfc_chk_conf_val(0,1,hfcmp_abort_t_restrain[ap->instance]) ) {
		ap->abort_t_restrain =	hfcmp_abort_t_restrain[ap->instance];
		HFC_DBGPRT("abort_t_restrain <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("abort_t_restrain = %d \n", ap->abort_t_restrain );
} /* FCLNX-0506 */

void hfc_set_login_restrain(struct adap_info *ap) /* FCLNX-0506 */
{
	ap->login_restrain = 0;

	if(ap->defparam) return;

	if(hfc_chk_conf_val(0,1,hfc_login_restrain)){ /* in case global parameter is set*/
			ap->login_restrain = hfc_login_restrain;
	}

	if( hfc_chk_conf_val(0,1,hfcmp_login_restrain[ap->instance]) ) {
		ap->login_restrain =	hfcmp_login_restrain[ap->instance];
		HFC_DBGPRT("login_restrain <-- hfcldd.conf\n");
	}

	HFC_DBGPRT("login_restrain = %d \n", ap->login_restrain );
} /* FCLNX-0506 */ 

void hfc_set_mck_point(struct adap_info *ap){   /* FCLNX-0533 */

        int mck_point_valid = 0;
        int value;

        ap->mck_point = HFC_NO_MCK_POINT;

        if (hfc_param_search("hfc_mck_point", &value)) hfc_mck_point = value;

        mck_point_valid =       hfc_chk_conf_val(HFC_NO_MCK_POINT,HFC_MAX_MCK_POINT, hfc_mck_point);

        if(ap->defparam) return;

        if (mck_point_valid == 1){
                ap->mck_point = hfc_mck_point;
        }

        HFC_DBGPRT( "hfc_mck_point=%d\n",ap->mck_point);
        return;
}       /* FCLNX-0533 */

/* FCLNX-GPL-394 */
/* Set HBA Isolation" */		/* FCLNX-GPL-349 */
void hfc_set_hba_isolation(struct adap_info *ap)
{	
	/* Set Default Parameter */
	ap->hba_isolation = HFC_ISOL_START;

	/* in case Force_Default_Parameter is ON */
	if(ap->defparam){
		return;
	}
	/* in case global parameter is set*/
	if(hfc_chk_conf_val(HFC_ISOL_STOP,HFC_ISOL_START,hfc_hba_isolation)){
		ap->hba_isolation = hfc_hba_isolation;
	}
	return;
}								/* FCLNX-GPL-349 */

/* Set Linkdown(s) Limit  */	/* FCLNX-GPL-349 */
void hfc_set_ld_err_limit_s(struct adap_info *ap)
{
	/* Set Default Parameter */
	ap->ld_err_limit_s = HFC_MIN_LD_ERR_LIMIT_S;
	/* in case Force_Default_Parameter is ON */
	if(ap->defparam) return;
	/* in case global parameter is set */
	if(hfc_chk_conf_val(HFC_MIN_LD_ERR_LIMIT_S,HFC_MAX_LD_ERR_LIMIT_S,hfc_ld_err_limit_s)){
		ap->ld_err_limit_s = hfc_ld_err_limit_s;
	}
	/* in case local parameter is set */
	if( hfc_chk_conf_val(HFC_MIN_LD_ERR_LIMIT_S,HFC_MAX_LD_ERR_LIMIT_S,hfcmp_ld_err_limit_s[ap->instance]) ) {
		ap->ld_err_limit_s = hfcmp_ld_err_limit_s[ap->instance];
	}
}								/* FCLNX-GPL-349 */

/* Set interface Error Limit */	/* FCLNX-GPL-349 */
void hfc_set_if_err_limit(struct adap_info *ap)
{
	/* Set Default Parameter */
	ap->if_err_limit = HFC_MIN_IF_ERR_LIMIT;
	/* in case Force_Default_Parameter is ON */
	if(ap->defparam) return;
	/* in case global parameter is set */
	if(hfc_chk_conf_val(HFC_MIN_IF_ERR_LIMIT,HFC_MAX_IF_ERR_LIMIT,hfc_if_err_limit)){
		ap->if_err_limit = hfc_if_err_limit;
	}
	/* in case local parameter is set */
	if( hfc_chk_conf_val(HFC_MIN_IF_ERR_LIMIT,HFC_MAX_IF_ERR_LIMIT,hfcmp_if_err_limit[ap->instance]) ) {
		ap->if_err_limit = hfcmp_if_err_limit[ap->instance];
	}
}								/* FCLNX-GPL-349 */

/* Set Time-Out Error Limit */	/* FCLNX-GPL-349 */
void hfc_set_to_err_limit(struct adap_info *ap)
{
	/* Set Default Parameter */
	ap->to_err_limit = HFC_MIN_TO_ERR_LIMIT;
	/* in case Force_Default_Parameter is ON */
	if(ap->defparam) return;
	/* in case global parameter is set */
	if(hfc_chk_conf_val(HFC_MIN_TO_ERR_LIMIT,HFC_MAX_TO_ERR_LIMIT,hfc_to_err_limit)){
		ap->to_err_limit = hfc_to_err_limit;
	}
	/* in case local parameter is set */
	if( hfc_chk_conf_val(HFC_MIN_TO_ERR_LIMIT,HFC_MAX_TO_ERR_LIMIT,hfcmp_to_err_limit[ap->instance]) ) {
		ap->to_err_limit = hfcmp_to_err_limit[ap->instance];
	}
}								/* FCLNX-GPL-349 */

/* Set Mailbox Time-Out Retry  *//* FCLNX-GPL-349 */
void hfc_set_to_reset_retry(struct adap_info *ap)
{
	/* Set Default Parameter */
	ap->to_reset_retry = HFC_TO_RESET_RETRY;
	/* in case Force_Default_Parameter is ON */
	if(ap->defparam) return;
	/* in case global parameter is set */
	if(hfc_chk_conf_val(HFC_TO_RESET_RETRY_MIN,HFC_TO_RESET_RETRY_MAX,hfc_to_reset_retry)){
		ap->to_reset_retry = hfc_to_reset_retry;
	}
	/* in case local parameter is set */
	if( hfc_chk_conf_val(HFC_TO_RESET_RETRY_MIN,HFC_TO_RESET_RETRY_MAX,hfcmp_to_reset_retry[ap->instance]) ) {
		ap->to_reset_retry = hfcmp_to_reset_retry[ap->instance];
	}
}								/* FCLNX-GPL-349 */

/* Set Time-Out Reset Error */	/* FCLNX-GPL-349 */
void hfc_set_rt_err_enable(struct adap_info *ap)
{
	/* Set Default Parameter */
	ap->rt_err_enable = HFC_RT_ERR_NOT_SPPRTD;
	/* in case Force_Default_Parameter is ON */
	if(ap->defparam) return;
	/* in case global parameter is set */
	if(hfc_chk_conf_val(HFC_RT_ERR_NOT_SPPRTD,HFC_RT_ERR_SPPRTD,hfc_rt_err_enable)){
		ap->rt_err_enable = hfc_rt_err_enable;
	}
	/* in case local parameter is set */
	if( hfc_chk_conf_val(HFC_RT_ERR_NOT_SPPRTD,HFC_RT_ERR_SPPRTD,hfcmp_rt_err_enable[ap->instance]) ) {
		ap->rt_err_enable = hfcmp_rt_err_enable[ap->instance];
	}
}								/* FCLNX-GPL-349 */
/* FCLNX-GPL-394 */

void hfc_conf_setup(struct adap_info *ap)
{

	hfc_set_topology(ap);			/* get topology					ap->topology */
	hfc_set_linkspeed(ap);			/* get linkspeed				ap->linkspeed */
	hfc_set_max_transfer(ap);		/* get max tarnsfer length		ap->max_transfer, ap->dma_max */
	ap->iov_map_cnt = ap->dma_max / HFC_PAGE_SIZE;

	hfc_set_linkup_tmo(ap); 		/* get linkup timeout time		ap->linkup_tmo;*/
	hfc_set_linkup2_tmo(ap);		/* get linkup timeout time after MCK    ap->linkup2_tmo;*/ /* FCLNX-0241 */
	hfc_set_reset_delay(ap);		/* get reset delay				ap->scsi_reset_delay */
	hfc_set_mck_count(ap);			/* get machine check count		ap->max_mck_cnt */

	hfc_set_pref_alpa(ap);			/* get preferred alpa			ap->pref_alpa */
	ap->host_alpa = ap->pref_alpa;

	hfc_set_target_timeout(ap); 	/* get target reset timeout 	ap->target_reset_tmo */
	hfc_set_abort_timeout(ap);		/* get abort timeout			ap->abort_tmo*/

	hfc_set_seg_trace(ap);			/* get seg info trace mode		ap->fw_parm */
	hfc_set_queue_depth(ap);		/* get queue_depth 				ap->queue_depth */
	hfc_set_enable_target_reset(ap);/* get enable target reset mode ap->enable_tgtrst */

	hfc_set_max_target(ap);			/* max target number			ap->max_target */
	hfc_set_xob_max(ap);			/* xob max						ap->xob_max */
	hfc_set_xrb_max(ap);			/* xrb max						ap->xrb_max */
	hfc_set_slog_max(ap);			/* softlog max					ap->slog_max */
	hfc_set_trc_max(ap);			/* trace max					ap->trc_max */
	hfc_set_pkt_num(ap);			/* hfc_pkt max					ap->pkt_num */
	hfc_set_can_queue(ap);			/* can queue depth				ap->can_queue */
	hfc_set_sg_tblsize(ap);			/* sg table size				ap->sg_tablesize */
	hfc_set_cmnd_num(ap);			/* cmnd num						ap->cmnd_num */
	hfc_set_minus_tout(ap);			/* minus timeout				ap->minus_timeout */
	hfc_set_scsi_allowed(ap);		/* Scsi_Cmnd->allowed 			ap->scsi_allowed */
	hfc_set_cmd_per_lun(ap);		/* cmd_per_lun					ap->cmd_per_lun */
	hfc_set_max_sectors(ap);		/* max_sectors					ap->max_sectors */
	hfc_set_lun_reset_delay(ap);	/* Delay after LUN Reset 		ap->tmt_delay			*/ /* FCLNX-0506 */	/* FCLNX-GPL-038 */
	hfc_set_abort_t_restrain(ap);	/* Restrain of AbortT.S issue	ap->abort_t_restrain	*/ /* FCLNX-0506 */
	hfc_set_login_restrain(ap);		/* Restrain of Login isuue		ap->login_restrain		*/ /* FCLNX-0506 */
	hfc_set_mck_point(ap);  /* FCLNX-0533 */
	
	/* FCLNX-GPL-394 */
	hfc_set_hba_isolation(ap);		/* Set HBA isolation	 		ap->hba_isolation	*/	/* FCLNX-GPL-349 */
	if (!( hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info ))				/* FCLNX-GPL-349 */
	{
		hfc_set_ld_err_limit_s(ap);		/* Set Linkdown(s) Limit 		ap->ld_err_limit_s	*/
		hfc_set_if_err_limit(ap);		/* Set interface Error Limit 	ap->if_err_limit	*/
		hfc_set_to_err_limit(ap);		/* Set Time-Out Error Limit		ap->to_err_limit	*/
		hfc_set_to_reset_retry(ap);		/* Set Mailbox Time-Out Retry	ap->to_reset_retry	*/
		hfc_set_rt_err_enable(ap);		/* Set Time-Out Reset Error		ap->rt_err_enable	*/
	}																						/* FCLNX-GPL-349 */
	/* FCLNX-GPL-394 */

	ap->wmsg = hfc_message_enable;
	ap->errlog_max	= HFC_MAX_ERRLOG_CNT;

	ap->scsi_time_out = hfc_scsi_time_out;

	/* originally set by modules.conf. This part is still tentative */
	ap->hosts->max_id 		= 255; /* modules.conf ??? */
	ap->hosts->max_lun 		= 256; /* modules.conf ??? */
#ifdef _HFC_FCGW														/* FC-GW */
	ap->hosts->max_channel 	= 0;
#else
	ap->hosts->max_channel 	= 1;
#endif
	ap->hosts->max_cmd_len 	= 16;	
	ap->hosts->this_id 		= HFC_MAX_THIS_ID;		/* FCLNX-0651 */
	ap->hosts->can_queue 	= ap->can_queue;
	ap->hosts->n_io_port		= 0xff;
	ap->hosts->base		= (unsigned long)ap->mem_base_addr;
	ap->hosts->cmd_per_lun  = ap->cmd_per_lun;		/* FCLNX-283 */
	ap->hosts->sg_tablesize  = ap->sg_tblsize;		/* FCLNX-283 */

	ap->hosts->max_cmd_len = 255;

	HFC_EXIT("hfc_conf_setup");

	return;	
}



int hfc_get_target_bindings(struct adap_info *ap)
{
	char wwpn[32],wwnn[32],did[32];
	uint64_t value1;
	uint64_t value2;
	int i,j, hit=0;
	int hit1, hit2;

	HFC_ENTRY("hfc_get_target_bindings");

	hit=0;

	i = ap->instance;

	HFC_DBGPRT("adapter number = %d\n",ap->instance);

	if((hfclddopts == NULL) && (hfclddconf == NULL)) return (hit);

	if(hfc_manage_info.adap_bind[ap->instance] == -1) return (hit);	

	for(j=0; j<MAX_TARGET_PROBE; j++){
		
		ap->bind_target_id->type[j]=0;
		ap->bind_target_id->parm[j]=0;

		/* Get wwpn from strings */
		if(hfclddconf == NULL) {
			sprintf(wwpn,"hfcl%d-tid-%d-wwpn",i,j);
			/* Get wwpn from modules.conf */
			if( hfc_parse_string(hfclddopts, wwpn, &value1) ){
				ap->bind_target_id->type[j]=HFC_TID_WWPN;
				ap->bind_target_id->parm[j]=value1;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d wwpn = %llx  in modules.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
		else if(hfclddopts == NULL) {
			sprintf(wwpn,"hfcl%d-tid-%d-wwpn",i,j);
			/* Get wwpn from hfcldd.conf */
			if( hfc_parse_string(hfclddconf, wwpn, &value2) ){
				ap->bind_target_id->type[j]=HFC_TID_WWPN;
				ap->bind_target_id->parm[j]=value2;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d did = %llx  in hfcldd.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
		else {
			sprintf(wwpn,"hfcl%d-tid-%d-wwpn",i,j);
			/* Get wwpn from modules.conf */
			hit1 = hfc_parse_string(hfclddopts, wwpn, &value1);
			/* Get wwpn from hfcldd.conf */
			hit2 = hfc_parse_string(hfclddconf, wwpn, &value2);
	
			if ((hit1 != 0) && (hit2 == 0)) {
				ap->bind_target_id->type[j]=HFC_TID_WWPN;
				ap->bind_target_id->parm[j]=value1;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d wwpn = %llx  in modules.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
			else if ( hit1 || hit2 ) {
				ap->bind_target_id->type[j]=HFC_TID_WWPN;
				ap->bind_target_id->parm[j]=value2;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d did = %llx  in hfcldd.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}

		/* Get wwnn from strings */
		if(hfclddconf == NULL) {
			sprintf(wwnn,"hfcl%d-tid-%d-wwnn",i,j);
			/* Get wwnn from modules.conf */
			if( hfc_parse_string(hfclddopts, wwnn, &value1) ){
				ap->bind_target_id->type[j]=HFC_TID_WWNN;
				ap->bind_target_id->parm[j]=value1;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d wwnn = %llx  in modules.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
		else if(hfclddopts == NULL) {
			sprintf(wwnn,"hfcl%d-tid-%d-wwnn",i,j);
			/* Get wwnn from hfcldd.conf */
			if( hfc_parse_string(hfclddconf, wwnn, &value2) ){
				ap->bind_target_id->type[j]=HFC_TID_WWNN;
				ap->bind_target_id->parm[j]=value2;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d wwnn = %llx  in hfcldd.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
		else {
			sprintf(wwnn,"hfcl%d-tid-%d-wwnn",i,j);
			/* Get wwnn from modules.conf */
			hit1 = hfc_parse_string(hfclddopts, wwnn, &value1);
			/* Get wwnn from hfcldd.conf */
			hit2 = hfc_parse_string(hfclddconf, wwnn, &value2);
	
			if ((hit1 != 0) && (hit2 == 0)) {
				ap->bind_target_id->type[j]=HFC_TID_WWNN;
				ap->bind_target_id->parm[j]=value1;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d wwnn = %llx  in modules.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
			else if ( hit1 || hit2 ) {
				ap->bind_target_id->type[j]=HFC_TID_WWNN;
				ap->bind_target_id->parm[j]=value2;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d wwnn = %llx  in hfcldd.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}

		/* Get did from strings */
		if(hfclddconf == NULL) {
			sprintf(did,"hfcl%d-tid-%d-did",i,j);
			/* Get did from modules.conf */
			if( hfc_parse_string(hfclddopts, did, &value1) ){
				ap->bind_target_id->type[j]=HFC_TID_DID;
				ap->bind_target_id->parm[j]=value1;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d did = %llx  in modules.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
		else if(hfclddopts == NULL) {
			sprintf(did,"hfcl%d-tid-%d-did",i,j);
			/* Get did from hfcldd.conf */
			if( hfc_parse_string(hfclddconf, did, &value2) ){
				ap->bind_target_id->type[j]=HFC_TID_DID;
				ap->bind_target_id->parm[j]=value2;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d did = %llx  in hfcldd.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
		else {
			sprintf(did,"hfcl%d-tid-%d-did",i,j);
			/* Get did from modules.conf */
			hit1 = hfc_parse_string(hfclddopts, did, &value1);
			/* Get did from hfcldd.conf */
			hit2 = hfc_parse_string(hfclddconf, did, &value2);
	
			if ((hit1 != 0) && (hit2 == 0)) {
				ap->bind_target_id->type[j]=HFC_TID_DID;
				ap->bind_target_id->parm[j]=value1;
				HFC_DBGPRT("** instance=%d tid=%d type=%d did = %llx  - modules.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
			else if ( hit1 || hit2 ) {
				ap->bind_target_id->type[j]=HFC_TID_DID;
				ap->bind_target_id->parm[j]=value2;
				HFC_DBGPRT("hit instance=%d tid=%d type=%d did = %llx  in hfcldd.conf\n",
					i,j,ap->bind_target_id->type[j],(unsigned long long)ap->bind_target_id->parm[j]);
				
				hit++;
			}
		}
	}	

	HFC_EXIT("hfc_get_target_bindings");

	return (hit);

}

int hfc_search_adapter_number(struct adap_info *ap)
{
	int i;
	uchar buf[64];
	uint wk;
	uchar pre_conf;
	uchar wkchar_wwn0 = 0;
	uchar wkchar_wwn1 = 0;

	uint64_t pxe_add_ww_name;	/* FCLNX-GPL-056 */
		
	HFC_ENTRY("hfc_set_config");
	
	HFC_DBGPRT("pkg code=%d , port = %d \n", ap->pkg.code, ap->pkg.port );

 	if(ap->pkg.type == HFC_PKTYPE_FPP){ /* FPP? */
		pre_conf = (uchar) hfc_read_cnfg (ap, 0x4B, 0x1);
 	}
 	else{ /* FIVE */
		pre_conf = (uchar) hfc_read_cnfg (ap, 0xAF, 0x1);
	}

	HFC_DBGPRT("function number = %d\n",ap->port_no);
		
	if ( HFC_MMODE_CHECK_BASIC(ap) )
	{															/* FCLNX-GPL-076 */
		hfc_read_flash(ap, 0x10000 + 16*ap->port_no, 16, buf);
	
		/* Read WWPN and WWNN from flash. */
		HFC_4B_TO_4L(wk, (*(uint*)(&buf[0])));
		ap->ww_name = wk;
		ap->ww_name <<= 32;
		HFC_4B_TO_4L(wk, (*(uint*)(&buf[4])));
		ap->ww_name |= wk;
		HFC_4B_TO_4L(wk, (*(uint*)(&buf[8])));
		ap->node_name = wk;
		ap->node_name <<= 32;
		HFC_4B_TO_4L(wk, (*(uint*)(&buf[12])));
		ap->node_name |= wk;

		HFC_DBGPRT("ww_name=%llx, node_name=%llx\n",(unsigned long long)ap->ww_name, (unsigned long long)ap->node_name);

		ap->org_ww_name = ap->ww_name;
		ap->org_node_name = ap->node_name;

		if( (pre_conf == 0x01) || (pre_conf == 0x03) ){

//			if(buf[0] == 0x50) {
			hfc_read_flash(ap, 0x20018 + 16*ap->port_no, 8, buf);

//			if(buf[0] == 0x50) { /* FCLNX-GPL-059 */ /* FCLNX-GPL-076 */
			
			HFC_4B_TO_4L(wk, (*(uint*)(&buf[0])));
			ap->add_ww_name = wk;
			ap->add_ww_name <<= 32;
			HFC_4B_TO_4L(wk, (*(uint*)(&buf[4])));
			ap->add_ww_name |= wk;
			ap->add_node_name=ap->add_ww_name + 1;		

			if( ((buf[0]&0xf0) == 0x50) || ((buf[0]&0xf0) == 0x20) ){ /* FCLNX-GPL-076 */

				ap->ww_name   = ap->add_ww_name;				/* FCLNX-0299 */
				ap->node_name = ap->add_node_name;				/* FCLNX-0299 */
			}
			else {
				HFC_ERRPRT("hfcldd : 'Additional WWPN is invalid ' addWWPN = 0x%llx orgWWPN = 0x%llx\n", 
					(unsigned long long)ap->add_ww_name, (unsigned long long)ap->org_ww_name);	/* FCLNX-GPL-076 */

				ap->add_ww_name   = ap->ww_name;
				ap->add_node_name = ap->node_name;

				return (MAX_ADAP_CNT + 1);                          /* FCLNX-0376 */
			}
		}
	}
	else {															/* @MLPF */
		hfc_mlpf_setup_wwn(ap);
		
		wkchar_wwn0 = (uchar)(ap->vfc_ww_name >> 56);
		wkchar_wwn1 = (uchar)(ap->vfc_node_name >> 56);
		
		wkchar_wwn0 &= 0xf0;										/* FCLNX-0338 */
		wkchar_wwn1 &= 0xf0;										/* FCLNX-0338 */
		
		if( ( (wkchar_wwn0 != 0x50) && (wkchar_wwn0 != 0x20) ) ||
			( (wkchar_wwn1 != 0x50) && (wkchar_wwn1 != 0x20) ) )
		{
			HFC_DBGPRT("hfcldd @MLPF: WWN check error \n"); 
			return (MAX_ADAP_CNT + 1);								/* FCLNX-0376 */
		}
	}																/* @MLPF */

	/* --------------------------------------------------------------------- FCLNX-XXX */
	if(hfc_pxe_boot != 0){

		hfc_read_flash(ap, 0x20018 + 16*ap->port_no, 8, buf);

		HFC_4B_TO_4L(wk, (*(uint*)(&buf[0])));
		pxe_add_ww_name = wk;
		pxe_add_ww_name <<= 32;
		HFC_4B_TO_4L(wk, (*(uint*)(&buf[4])));
		pxe_add_ww_name |= wk;

		wkchar_wwn0 = (uchar)(pxe_add_ww_name >> 56);
		wkchar_wwn0 &= 0xf0;

		if ( (wkchar_wwn0 != 0x50) && (wkchar_wwn0 != 0x20) ){ /* Error case */
			ap->ww_name = ap->org_ww_name;
			ap->node_name = ap->org_node_name;
		}

	}
	/* --------------------------------------------------------------------- FCLNX-XXX */


	for(i=0; i<MAX_ADAP_CNT; i++) {
		if(ap->ww_name == hfc_manage_info.adap_bind[i]){
			HFC_DBGPRT("adapter info is hit, i = %d, wwpn=%llx\n", i, (unsigned long long)ap->ww_name);
			return (i); /* hit */
		} 
	}

	HFC_EXIT("hfc_set_config");
	return (-1); 
}


int hfc_get_adapter_port_no(struct adap_info *ap)
{
	int func_no;
	ulong pci_status;
	
	HFC_ENTRY("hfc_get_adapter_port_no");
	
 	if(ap->pkg.type == HFC_PKTYPE_FPP) { /* FPP? */
		ap->port_no = 0; /* set port number to '0' */
 	}
 	else { /* FIVE */
 		pci_status = (ulong) hfc_read_cnfg (ap, 0x6C, 0x4);
 		HFC_DBGPRT("pci status = %x\n",(uint)pci_status);
		func_no = (pci_status & 0x00000007);
		ap->port_no = func_no; 
	}
	
	HFC_EXIT("hfc_set_config");
	return (ap->port_no); 
}


int hfc_get_adapter_bindings(void)
{
	char path[32];
	uint64_t value1;
	uint64_t value2;
	int i, hit=0, hit_max=-1;
	int hit1, hit2;

	HFC_ENTRY("hfc_get_adapter_bindings");


	for(i=0;i<MAX_ADAP_CNT;i++) hfc_manage_info.adap_bind[i] = -1;

	if((hfclddopts == NULL) && (hfclddconf == NULL)) return (-1);

	/* Make sure adapter number has already registered. */
	
	if(hfclddconf == NULL) {
		for (i=0;i<MAX_ADAP_CNT;i++) {
			sprintf(path,"hfcl%d-wwpn",i);
			/* Get wwpn from modules.conf */
			if( hfc_parse_string(hfclddopts, path, &value1) ){
				hfc_manage_info.adap_bind[i] = value1;
				HFC_DBGPRT("hit adapter no=%d  wwpn %llx  in modules.conf\n",
				i, (unsigned long long)hfc_manage_info.adap_bind[i] );
				hit_max = i;
				hit++;
			}
		}
	}
	else if(hfclddopts == NULL) {
		for (i=0;i<MAX_ADAP_CNT;i++) {
			sprintf(path,"hfcl%d-wwpn",i);
			/* Get wwpn from hfcldd.conf */
			if( hfc_parse_string(hfclddconf, path, &value2) ){
				hfc_manage_info.adap_bind[i] = value2;
				HFC_DBGPRT("hit adapter no=%d  wwpn %llx  in hfcldd.conf\n",
				i, (unsigned long long)hfc_manage_info.adap_bind[i] );
				hit_max = i;
				hit++;
			}
		}
	}
	else {
		for (i=0;i<MAX_ADAP_CNT;i++) {
			sprintf(path,"hfcl%d-wwpn",i);
			/* Get wwpn from modules.conf */
			hit1 = hfc_parse_string(hfclddopts, path, &value1);
			/* Get wwpn from hfcldd.conf */
			hit2 = hfc_parse_string(hfclddconf, path, &value2);
			
			if ((hit1 != 0) && (hit2 == 0)) {
				hfc_manage_info.adap_bind[i] = value1;
				HFC_DBGPRT("hit adapter no=%d  wwpn %llx  in modules.conf\n",
				i, (unsigned long long)hfc_manage_info.adap_bind[i] );
				hit_max = i;
				hit++;
			}
			else if ( hit1 || hit2 ) {
				hfc_manage_info.adap_bind[i] = value2;

				HFC_DBGPRT("hit adapter no=%d  wwpn %llx  in hfcldd.conf\n",
				i, (unsigned long long)hfc_manage_info.adap_bind[i] );
				hit_max = i;
				hit++;
			}
		}
	}

	HFC_EXIT("hfc_get_adapter_bindings");

	return (hit_max);

}



void hfc_param_setup(void)
{
	int value=0;
	int i;
	char buf[32];

	HFC_ENTRY("hfc_param_setup");

	/* "hfc<x>_connection_type" */
	if (hfc_param_search("hfc_connection_type", &value)) hfc_connection_type = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_connection_type[i] = -1;
		sprintf(buf,"hfc%d_connection_type",i);
		if (hfc_param_search(buf, &value)) hfcmp_connection_type[i] = value;
	}

	/* "hfc<x>_link_speed" */
	if (hfc_param_search("hfc_link_speed", &value)) hfc_link_speed = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_link_speed[i] = -1;
		sprintf(buf,"hfc%d_link_speed",i);
		if (hfc_param_search(buf, &value)) hfcmp_link_speed[i] = value;
	}
	
	/* "hfc<x>_max_transfer" */
	if (hfc_param_search("hfc_max_transfer", &value)) hfc_max_transfer = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_max_transfer[i] = -1;
		sprintf(buf,"hfc%d_max_transfer",i);
		if (hfc_param_search(buf, &value)) hfcmp_max_transfer[i] = value;
	}

	/* "hfc<x>_link_down" */
	if (hfc_param_search("hfc_link_down", &value)) hfc_link_down = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_link_down[i] = -1;
		sprintf(buf,"hfc%d_link_down",i);
		if (hfc_param_search(buf, &value)) hfcmp_link_down[i] = value;
	}

	/* "hfc<x>_link_down" */					/* FCLNX-241 */
	if (hfc_param_search("hfc_link_down2", &value)) hfc_link_down2 = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_link_down2[i] = -1;
		sprintf(buf,"hfc%d_link_down2",i);
		if (hfc_param_search(buf, &value)) hfcmp_link_down2[i] = value;
	}								/* FCLNX-0241 */

	/* "hfc<x>_reset_delay" */
	if (hfc_param_search("hfc_reset_delay", &value)) hfc_reset_delay = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_reset_delay[i] = -1;
		sprintf(buf,"hfc%d_reset_delay",i);
		if (hfc_param_search(buf, &value)) hfcmp_reset_delay[i] = value;
	}

	/* "hfc<x>_mck_retry" */
	if (hfc_param_search("hfc_mck_retry", &value)) hfc_mck_retry = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_mck_retry[i] = -1;
		sprintf(buf,"hfc%d_mck_retry",i);
		if (hfc_param_search(buf, &value)) hfcmp_mck_retry[i] = value;
	}

	/* "hfc<x>_preferred_alpa" */
	if (hfc_param_search("hfc_preferred_alpa", &value)) hfc_preferred_alpa = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_preferred_alpa[i] = -1;
		sprintf(buf,"hfc%d_preferred_alpa",i);
		if (hfc_param_search(buf, &value)) hfcmp_preferred_alpa[i] = value;
	}

	/* "hfc<x>_reset_timeout" */
	if (hfc_param_search("hfc_reset_timeout", &value)) hfc_reset_timeout = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_reset_timeout[i] = -1;
		sprintf(buf,"hfc%d_reset_timeout",i);
		if (hfc_param_search(buf, &value)) hfcmp_reset_timeout[i] = value;
	}

	/* "hfc<x>_abort_timeout" */
	if (hfc_param_search("hfc_abort_timeout", &value)) hfc_abort_timeout = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_abort_timeout[i] = -1;
		sprintf(buf,"hfc%d_abort_timeout",i);
		if (hfc_param_search(buf, &value)) hfcmp_abort_timeout[i] = value;
	}

	/* "hfc<x>_enable_tgtrst" */
	if (hfc_param_search("hfc_enable_tgtrst", &value)) hfc_enable_tgtrst = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_enable_tgtrst[i] = -1;
		sprintf(buf,"hfc%d_enable_tgtrst",i);
		if (hfc_param_search(buf, &value)) hfcmp_enable_tgtrst[i] = value;
	}

	/* "hfc<x>_queue_depth" */
	if (hfc_param_search("hfc_queue_depth", &value)) hfc_queue_depth = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_queue_depth[i] = -1;
		sprintf(buf,"hfc%d_queue_depth",i);
		if (hfc_param_search(buf, &value)) hfcmp_queue_depth[i] = value;
	}

	/* "hfc<x>_seg_trace" */
	if (hfc_param_search("hfc_seg_trace", &value)) hfc_seg_trace = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_seg_trace[i] = -1;
		sprintf(buf,"hfc%d_seg_trace",i);
		if (hfc_param_search(buf, &value)) hfcmp_seg_trace[i] = value;
	}

	/* "hfc<x>_max_target" */
	if (hfc_param_search("hfc_max_target", &value)) hfc_max_target = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_max_target[i] = -1;
		sprintf(buf,"hfc%d_max_target",i);
		if (hfc_param_search(buf, &value)) hfcmp_max_target[i] = value;
	}

	/* "hfc<x>_xob_max" */
	if (hfc_param_search("hfc_xob_max", &value)) hfc_xob_max = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_xob_max[i] = -1;
		sprintf(buf,"hfc%d_xob_max",i);
		if (hfc_param_search(buf, &value)) hfcmp_xob_max[i] = value;
	}

	/* "hfc<x>_xrb_max" */
	if (hfc_param_search("hfc_xrb_max", &value)) hfc_xrb_max = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_xrb_max[i] = -1;
		sprintf(buf,"hfc%d_xrb_max",i);
		if (hfc_param_search(buf, &value)) hfcmp_xrb_max[i] = value;
	}

	/* "hfc<x>_slog_max" */
	if (hfc_param_search("hfc_slog_max", &value)) hfc_slog_max = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_slog_max[i] = -1;
		sprintf(buf,"hfc%d_slog_max",i);
		if (hfc_param_search(buf, &value)) hfcmp_slog_max[i] = value;
	}

	/* "hfc<x>_trc_max" */
	if (hfc_param_search("hfc_trc_max", &value)) hfc_trc_max = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_trc_max[i] = -1;
		sprintf(buf,"hfc%d_trc_max",i);
		if (hfc_param_search(buf, &value)) hfcmp_trc_max[i] = value;
	}

	/* "hfc<x>_pkt_num" */
	if (hfc_param_search("hfc_pkt_num", &value)) hfc_pkt_num = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_pkt_num[i] = -1;
		sprintf(buf,"hfc%d_pkt_num",i);
		if (hfc_param_search(buf, &value)) hfcmp_pkt_num[i] = value;
	}

	/* "hfc<x>_can_queue" */
	if (hfc_param_search("hfc_can_queue", &value)) hfc_can_queue = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_can_queue[i] = -1;
		sprintf(buf,"hfc%d_can_queue",i);
		if (hfc_param_search(buf, &value)) hfcmp_can_queue[i] = value;
	}

	/* "hfc<x>_sg_tblsize" */
	if (hfc_param_search("hfc_sg_tblsize", &value)) hfc_sg_tblsize = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_sg_tblsize[i] = -1;
		sprintf(buf,"hfc%d_sg_tblsize",i);
		if (hfc_param_search(buf, &value)) hfcmp_sg_tblsize[i] = value;
	}

	/* "hfc<x>_cmnd_num" */
	if (hfc_param_search("hfc_cmnd_num", &value)) hfc_cmnd_num = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_cmnd_num[i] = -1;
		sprintf(buf,"hfc%d_cmnd_num",i);
		if (hfc_param_search(buf, &value)) hfcmp_cmnd_num[i] = value;
	}

	/* "hfc<x>_minus_tout" */
	if (hfc_param_search("hfc_minus_tout", &value)) hfc_minus_tout = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_minus_tout[i] = -1;
		sprintf(buf,"hfc%d_minus_tout",i);
		if (hfc_param_search(buf, &value)) hfcmp_minus_tout[i] = value;
	}

	/* "hfc<x>_scsi_allowed" */
	if (hfc_param_search("hfc_scsi_allowed", &value)) hfc_scsi_allowed = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_scsi_allowed[i] = -1;
		sprintf(buf,"hfc%d_scsi_allowed",i);
		if (hfc_param_search(buf, &value)) hfcmp_scsi_allowed[i] = value;
	}

	/* "hfc<x>_cmd_per_lun" */													/* FCLNX-0283 */
	if (hfc_param_search("hfc_cmd_per_lun", &value)) hfc_cmd_per_lun = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_cmd_per_lun[i] = -1;
		sprintf(buf,"hfc%d_cmd_per_lun",i);
		if (hfc_param_search(buf, &value)) hfcmp_cmd_per_lun[i] = value;
	}

	/* "hfc<x>_max_sectors" */
	if (hfc_param_search("hfc_max_sectors", &value)) hfc_max_sectors = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_max_sectors[i] = -1;
		sprintf(buf,"hfc%d_max_sectors",i);
		if (hfc_param_search(buf, &value)) hfcmp_max_sectors[i] = value;
	}
	/* "hfc<x>_abort_t_restrain" */
	if (hfc_param_search("hfc_abort_t_restrain", &value)) hfc_abort_t_restrain = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_abort_t_restrain[i] = -1;
		sprintf(buf,"hfc%d_abort_t_restrain",i);
		if (hfc_param_search(buf, &value)) hfcmp_abort_t_restrain[i] = value;
	}
	
	/* "hfc<x>_login_restrain" */
	if (hfc_param_search("hfc_login_restrain", &value)) hfc_login_restrain = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_login_restrain[i] = -1;
		sprintf(buf,"hfc%d_login_restrain",i);
		if (hfc_param_search(buf, &value)) hfcmp_login_restrain[i] = value;
	}

	/* "hfc<x>_lun_reset_delay" */	/* FCLNX-GPL-038 */
	if (hfc_param_search("hfc_lun_reset_delay", &value)) hfc_lun_reset_delay = value;
	for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
		hfcmp_lun_rst_delay[i] = -1;
		sprintf(buf,"hfc%d_lun_reset_delay",i);
		if (hfc_param_search(buf, &value)) hfcmp_lun_rst_delay[i] = value;
	}								/* FCLNX-GPL-038 */

	/* FCLNX-GPL-394 */
	/* "hfc_hba_isolation" */								/* FCLNX-GPL-349 */
	if (hfc_param_search("hfc_hba_isolation", &value)) hfc_hba_isolation = value;
															/* FCLNX-GPL-349 */

	if (!( hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info )){	/* FCLNX-GPL-349 */

		/* "hfc_ld_err_limit_s", "hfc<x>_ld_err_limit_s" */	/* FCLNX-GPL-349 */
		if (hfc_param_search("hfc_ld_err_limit_s", &value)) hfc_ld_err_limit_s = value;
		for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
			hfcmp_ld_err_limit_s[i] = -1;
			sprintf(buf,"hfc%d_ld_err_limit_s",i);
			if (hfc_param_search(buf, &value)) hfcmp_ld_err_limit_s[i] = value;
		}													/* FCLNX-GPL-349 */
	
		/* "hfc_if_err_limit", "hfc<x>_if_err_limit" */		/* FCLNX-GPL-349 */
		if (hfc_param_search("hfc_if_err_limit", &value)) hfc_if_err_limit = value;
		for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
			hfcmp_if_err_limit[i] = -1;
			sprintf(buf,"hfc%d_if_err_limit",i);
			if (hfc_param_search(buf, &value)) hfcmp_if_err_limit[i] = value;
		}													/* FCLNX-GPL-349 */
	
		/* "hfc_to_err_limit", "hfc<x>_to_err_limit" */		/* FCLNX-GPL-349 */
		if (hfc_param_search("hfc_to_err_limit", &value)) hfc_to_err_limit = value;
		for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
			hfcmp_to_err_limit[i] = -1;
			sprintf(buf,"hfc%d_to_err_limit",i);
			if (hfc_param_search(buf, &value)) hfcmp_to_err_limit[i] = value;
		}													/* FCLNX-GPL-349 */
	
		/* "hfc_to_reset_retry", "hfc<x>_to_reset_retry" */	/* FCLNX-GPL-349 */
		if (hfc_param_search("hfc_to_reset_retry", &value)) hfc_to_reset_retry = value;
		for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
			hfcmp_to_reset_retry[i] = -1;
			sprintf(buf,"hfc%d_to_reset_retry",i);
			if (hfc_param_search(buf, &value)) hfcmp_to_reset_retry[i] = value;
		}													/* FCLNX-GPL-349 */
	
		/* "hfc_rt_err_enable", "hfc<x>_rt_err_enable" */	/* FCLNX-GPL-349 */
		if (hfc_param_search("hfc_rt_err_enable", &value)) hfc_rt_err_enable = value;
		for (i=0;i<HFC_MAX_INSTANCE_CNT;i++) {
			hfcmp_rt_err_enable[i] = -1;
			sprintf(buf,"hfc%d_rt_err_enable",i);
			if (hfc_param_search(buf, &value)) hfcmp_rt_err_enable[i] = value;
		}													/* FCLNX-GPL-349 */
	}														/* FCLNX-GPL-349 */
	/* FCLNX-GPL-394 */

	HFC_EXIT("hfc_param_setup");

}



int hfc_param_search(char *search_str, int *value)
{
	int      hit=0;
	uint64_t str_value;
	char     path[32];
	
	sprintf(path, search_str);
	if((hfclddconf!=NULL)&&(hfc_parse_string(hfclddconf, path, &str_value))) {
		hit = 1;
		*value = str_value;
		HFC_DBGPRT("hit %s = %d  in hfcldd.conf\n", search_str, *value);
	}
	
	return (hit);
}


void hfc_read_hfcbios(struct adap_info *ap)							/* hotplug */
{
	uchar disable_b[4];
	uint bios_ofs = 0xd8000 + (0x100 * ap->port_no);
	int i,entry;													/* FCLNX-0392 */
	uint wk;														/* FCLNX-0392 */
	uchar buf[16];													/* FCLNX-0392 */
																	/* @MLPF STR */
	uint wkint_bios_parm1;
	uint wkint_bios_parm2;

	if ( HFC_MMODE_CHECK_MLPF(ap) ) {								/* FCLNX-0365 */
		wkint_bios_parm1 = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_EFI_OP_TBL1, 0x4 );
		HFC_4L_TO_4B(wkint_bios_parm2,wkint_bios_parm1);
		HFC_MEMCPY(disable_b,(uchar*)&wkint_bios_parm2,4);
	}
	else
		hfc_read_flash(ap, (bios_ofs+4), 4, disable_b);
																	/* @MLPF END */

	ap->automap  = hfc_automap;
	ap->narrowmap = hfc_narrowmap;									/* FCLNX-0392 */
	
	ap->defparam = 0;
	if ( (disable_b[1] != 0xFF) && (disable_b[1] != 0x00) ) {
		HFC_DBGPRT("Force bindings disable state\n");

		if (disable_b[1] & 0x80) {	/* Forced Persistent Binding into disable state */
			ap->automap = 1;
			if (hfc_manage_info.hfcldd_mp_mod) hfc_manage_info.npubp->hfc_set_automap(ap);
		}

		if (disable_b[1] & 0x40) {	/* Forced default parameter into disable state */
			ap->defparam = 1;
		}
	}
	else {
		HFC_DBGPRT("binding depends on etc.module.conf\n");
	}

	/* Get LOGIN delay time */
	ap->login_wait = 2;	/* 2s */
	if (!ap->defparam) {
		if ( (disable_b[2] != 0xFF) && (disable_b[2] != 0x00) ) {
			HFC_DBGPRT("use LOGIN delay set by HBA BIOS\n");

			if(disable_b[2] & 0x80) {
				ap->login_wait = disable_b[2] & 0x7f;		/* FCLNX-0515 */
			}
		}
		else {
			HFC_DBGPRT("LOGIN delay time is set followed by modprobe.conf\n");
		}
	}
	
	ap->linkspeed = 0;	/* Auto */
	if (!ap->defparam) {
		/* Get Link Speed */
		if ( disable_b[0] != 0xFF ){
			HFC_DBGPRT("use Link Speed set by HBA BIOS\n");

			if ((disable_b[0] <= 2) || (disable_b[0] == 0x04)) {
				ap->linkspeed = disable_b[0];
			}
			else {
				if( ap->raslog_install )
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xD1, NULL, 0) ;
				else
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xD1, NULL, 0) ;
			}
		}
		else {
			HFC_DBGPRT("Link Speed is set followed by modprobe.conf\n");
		}
	}

																	/* @MLPF STR */
	if ( HFC_MMODE_CHECK_MLPF(ap) )									/* FCLNX-0365 */
	{
		wkint_bios_parm1 = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_EFI_OP_TBL0, 0x4 );
		HFC_4L_TO_4B(wkint_bios_parm2,wkint_bios_parm1);
		HFC_MEMCPY(disable_b,(uchar*)&wkint_bios_parm2,4);
	} else
		hfc_read_flash(ap, (bios_ofs+0), 4, disable_b);
																	/* @MLPF END */


	/* Get Connection Type */
	ap->topology = 0;	/* Auto */
	if (!ap->defparam) {
		if(disable_b[3] != 0xFF) {
			HFC_DBGPRT("Sets Connection Type from BIOS\n");
			
			if((disable_b[3] == 0x00) || (disable_b[3] == 0x01) || (disable_b[3] == 0x03) ) {
				 ap->topology = disable_b[3];
			}
			else {
				 HFC_ERRPRT("hfcldd : 'Connection Type Parameter set in HBA BIOS' = %d is invalid.\n", disable_b[3]);
			}
		}
		else {
			HFC_DBGPRT("Connection Type depends on etc.module.conf\n");
		}
	}
	
	/* Get Spinup Delay */														/* FCLNX-0404 */
	ap->spinup_delay = 0;			/* Disable */								/* FCLNX-0404 */
	if(disable_b[1] != 0xFF) {													/* FCLNX-0404 */
		HFC_DBGPRT("Spinup Delay from BIOS\n");									/* FCLNX-0404 */
		if((disable_b[1] & 0x20) == 0x20)										/* FCLNX-0404 */
			 ap->spinup_delay = 1;	/* Enable */								/* FCLNX-0404 */
	}																			/* FCLNX-0404 */
	
	HFC_DBGPRT("hfcldd%d : forced PB disable   = %d.\n",ap->dev_minor, ap->automap);
	HFC_DBGPRT("hfcldd%d : forced default parm = %d.\n",ap->dev_minor, ap->defparam);
	HFC_DBGPRT("hfcldd%d : login Delay time    = %d.\n",ap->dev_minor, ap->login_wait);
	HFC_DBGPRT("hfcldd%d : connection Type     = %d.\n",ap->dev_minor, ap->topology);
	HFC_DBGPRT("hfcldd%d : link speed          = %d.\n",ap->dev_minor, ap->linkspeed);
	HFC_DBGPRT("hfcldd%d : spinup delay        = %d.\n",ap->dev_minor, ap->spinup_delay);	/* FCLNX-0404 */
	
	/* Seve Boot Priority */
	memset(ap->boot_priority,0,sizeof(ap->boot_priority));						/* FCLNX-0392 */
	if (ap->narrowmap) {														/* FCLNX-0392 */

		if ( HFC_MMODE_CHECK_MLPF(ap) )											/* FCLNX-0410 */
		{
			wkint_bios_parm1 = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_EFI_OP_TBL0, 0x4 );
			HFC_4L_TO_4B(wkint_bios_parm2,wkint_bios_parm1);
			HFC_MEMCPY(disable_b,(uchar*)&wkint_bios_parm2,4);
		}
		else {																	/* FCLNX-0410 */
			hfc_read_flash(ap, (bios_ofs+0), 4, disable_b);
		}

		for (i=0;i<4;i++) {	
			if (disable_b[i] != 0xff)
				break;
		}
		
		if ((i < sizeof(disable_b))				 /* Check Byte0-3!=ALL'F' */
			&& ((disable_b[1] & 0x90) == 0x90)) {/* Valid Boot Priority   */
			entry=0;
			
			for (entry=0;entry<8;entry++) {

				if ( HFC_MMODE_CHECK_MLPF(ap) )									/* FCLNX-0410 */
				{
					for (i=0;i<16;i++) {
						buf[i] = hfc_read_reg_hg_ext(ap,
									 ap->lparmode.hg_map->iosp.reg[HFC_IOHGSPC_EFI_OP_TBL0]
									 +(0x10*(entry+1))+i, 1);
					}
					
					/* Get WWPN */
					ap->boot_priority[entry].ww_name = (*(uint*)(&buf[12]));	/* Attention!! */
					ap->boot_priority[entry].ww_name <<= 32;					/* Attention!! */
					ap->boot_priority[entry].ww_name |= (*(uint*)(&buf[8]));	/* Attention!! */
					
					/* Get LUN */
					ap->boot_priority[entry].lun = buf[6];						/* Attention!! */
				}
				else {															/* FCLNX-0410 */
					hfc_read_flash(ap, (bios_ofs+0x10*(entry+1)), 16, buf);
					
					/* Get WWPN */
					HFC_4B_TO_4L(wk, (*(uint*)(&buf[8])));
					ap->boot_priority[entry].ww_name = wk;
					ap->boot_priority[entry].ww_name <<= 32;
					HFC_4B_TO_4L(wk, (*(uint*)(&buf[12])));
					ap->boot_priority[entry].ww_name |= wk;

					/* Get LUN */
					ap->boot_priority[entry].lun = buf[7];
				}

			}
		}
	}																			/* FCLNX-0392 */
}																				/* hotplug */


char hfc_cnvc(char C)										  /* FCWIN-0081 */
{
	char c = C;
	if (c >= 'A' && c <= 'Z')
		return (c - 'A' + 'a');

	return c;
}



int hfc_parse_string(char *string, char *keyword, uint64_t *value)/* FCWIN-0081 */
{
	char *cptr,*kptr;
	int index;
	int stringlen=0,keywordlen=0;

	/* Get string's length */
	cptr = string;
	while (*cptr) {
		cptr++;
		stringlen++;
	}

	/* Get keyword length */
	cptr = keyword;
	while (*cptr) {
		cptr++;
		keywordlen++;
	}

	if (keywordlen > stringlen) {

		/* keyword length > string length */
		/*  That means this string does not include the specified keyword */ 
		return 0;
	}

	/* Now setup and start the compare */
	cptr = string;

ContinueSearch:

	/* Skip spaces and tabs */
	while (*cptr == ' ' || *cptr == '\t') {
		cptr++;
	}

	if (*cptr == '\0') {

		/* End of this string */
		return 0;
	}

	kptr = keyword;
	while (hfc_cnvc(*cptr) == hfc_cnvc(*kptr)) {
		
		cptr++;
		kptr++;

		if (*(cptr - 1) == '\0') {

			/* End of string */
	
			return 0;
		}
	}

	cptr++;
	kptr++;

	if (*(kptr - 1) == '\0') {

		/* May have a match backup and check for blank or equals */


		cptr--;
		while (*cptr == ' ' || *cptr == '\t') {
			cptr++;
		}

		/* Found a match.  Make sure there are equals */

		if (*cptr != '=') {

			/* Move to the next semicolon */

			while (*cptr) {
				if (*cptr++ == ';') {
					goto ContinueSearch;
				}
			}
			return 0;
		}

		/* Skip the equals sign */

		cptr++;

		/* Skip white space */

		while ((*cptr == ' ') || (*cptr == '\t')) {
			cptr++;
		}

		if (*cptr == '\0') {

			/* Early end of string, return not found */

			return 0;
		}

		if (*cptr == ';') {

			/* This isn't it either */

			cptr++;
			goto ContinueSearch;
		}

		*value = 0;
		if ((*cptr == '0') && (hfc_cnvc(*(cptr + 1)) == 'x')) {

			/* Value is in Hex.  Skip the "0x" */

			cptr += 2;
			for (index = 0; *(cptr + index); index++) {

				if (*(cptr + index) == ' ' ||
					*(cptr + index) == '\t' ||
					*(cptr + index) == ':') { /* changed */
					 break;
				}

				if ((*(cptr + index) >= '0') && (*(cptr + index) <= '9')) {
					*value = (16 * (*value)) + (*(cptr + index) - '0');
				} else {
					if ((hfc_cnvc(*(cptr + index)) >= 'a') && (hfc_cnvc(*(cptr + index)) <= 'f')) {
						*value = (16 * (*value)) + (hfc_cnvc(*(cptr + index)) - 'a') + 10;
					} else {

						/* Syntax error, return not found */

						return 0;
					}
				}
			}
		} else {

			/* Value is in decimal */

			for (index = 0; *(cptr + index); index++) {

				if (*(cptr + index) == ' ' ||
					*(cptr + index) == '\t' ||
					*(cptr + index) == ':') {  /* changed */
					break;
				}

				if ((*(cptr + index) >= '0') && (*(cptr + index) <= '9')) {
					*value = (10 * (*value)) + (*(cptr + index) - '0');
				} else {

					/* Syntax error return not found */

					return 0;
				}
			}
		}

		return 1;
	} else {

		/* Not a match check for ';' to continue search */

        while (*cptr) {
            if (*cptr++ == ':') {	/* changed */
                goto ContinueSearch;
            }
        }

        return 0;
    }
}



char *hfc_delete_space(char *string){	/* FCLNX-GPL-526 */
	
	char	*cptr = NULL;
	int		index = 0;
	
	cptr = string;
	
	while (*cptr) {
		cptr++;
		index++;
	}

	cptr = string;
	
	while  ((index != 0) && ((*(cptr+index) == ' ') || (*(cptr+index) == '\0'))){
		if (*(cptr+index) == ' ')
			*(cptr+index)  = '\0';
		index--;
	}
	return(string);
}										/* FCLNX-GPL-526 */



int hfc_convert_string(char *string, uint64_t *value)
{
	char *cptr;
	int index;

	cptr = string;

		*value = 0;
		if ((*cptr == '0') && (hfc_cnvc(*(cptr + 1)) == 'x')) {

			/* Value is in Hex.  Skip the "0x" */

			cptr += 2;
			for (index = 0; *(cptr + index); index++) {

				if (*(cptr + index) == ' ' ||
					*(cptr + index) == '\t' ||
					*(cptr + index) == ':') { /* changed */
					 break;
				}

				if ((*(cptr + index) >= '0') && (*(cptr + index) <= '9')) {
					*value = (16 * (*value)) + (*(cptr + index) - '0');
				} else {
					if ((hfc_cnvc(*(cptr + index)) >= 'a') && (hfc_cnvc(*(cptr + index)) <= 'f')) {
						*value = (16 * (*value)) + (hfc_cnvc(*(cptr + index)) - 'a') + 10;
					} else {

						/* Syntax error, return not found */

						return 0;
					}
				}
			}
		} else {

			/* Value is in Decimal */

			for (index = 0; *(cptr + index); index++) {

				if (*(cptr + index) == ' ' ||
					*(cptr + index) == '\t' ||
					*(cptr + index) == ':') {  /* changed */
					break;
				}

				if ((*(cptr + index) >= '0') && (*(cptr + index) <= '9')) {
					*value = (10 * (*value)) + (*(cptr + index) - '0');
				} else {

					/* Syntax error return not found */

					return 0;
				}
			}
		}

		return 1;

}


char *hfc_get_write_retries(void)
{
	return (write_retries);
}


void hfc_set_pub_symbol_list(void)
{

	struct pub_symbol_list		*pubp;
	
	HFC_ENTRY("hfc_set_pub_symbol_list");
	
	pubp  = hfc_get_pub_symbol_list();
	
	/* detect */
	pubp->hfc_param_search				=	hfc_param_search;
	pubp->hfc_cnvc						=	hfc_cnvc;
	pubp->hfc_parse_string				=	hfc_parse_string;
	pubp->hfc_convert_string			=	hfc_convert_string;
	pubp->hfc_get_write_retries			=	hfc_get_write_retries;
	pubp->hfc_initialize				=	hfc_initialize;				/* FCLNX-0526 */
	pubp->hfc_chk_stop					=	hfc_chk_stop;				/* FCLNX-0534 */
	pubp->hfc_issue_forced_mck			=	hfc_issue_forced_mck;		/* FCLNX-0534 */
	pubp->hfc_chk_conf_val				=	hfc_chk_conf_val;
	pubp->hfc_config_hw_set_five			=	hfc_config_hw_set_five;
	
	/* strategy */
	pubp->hfc_strategy_pg				=	hfc_strategy_pg;
	pubp->hfc_eh_abort_pg				=	hfc_eh_abort_pg;
	pubp->hfc_eh_device_reset_pg		=	hfc_eh_device_reset_pg;
	pubp->hfc_eh_bus_reset_pg			=	hfc_eh_bus_reset_pg;
	pubp->hfc_iodone					=	hfc_iodone;
	pubp->hfc_strategy_hfcp				=	hfc_strategy_hfcp;
	pubp->hfc_get_new_hfcp				=	hfc_get_new_hfcp;
	pubp->hfc_start						=	hfc_start;					/* FCLNX-0429 */
	pubp->hfc_cancel_scsi_cmd			=	hfc_cancel_scsi_cmd;		/* FCLNX-0429 */
	pubp->hfc_get_new_cmnd				=	hfc_get_new_cmnd;
	pubp->hfc_dummy_copy				=	hfc_dummy_copy;
	
	/* timer_recovery*/
	pubp->hfc_reset_adap_info			=	hfc_reset_adap_info;
	pubp->hfc_reset_start				= 	hfc_reset_start;
	pubp->hfc_errlog					=	hfc_errlog;
	pubp->hfc_watchdog					=	hfc_watchdog;
	pubp->hfc_force_linkdown			=	hfc_force_linkdown;
	pubp->hfc_force_linkdown_recovery	=	hfc_force_linkdown_recovery;
	
	/* top */
	pubp->hfc_hash_target_valid			=	hfc_hash_target_valid;
	pubp->hfc_hash_target_info			=	hfc_hash_target_info;
	pubp->hfc_hash_target_info_wwn		=	hfc_hash_target_info_wwn;
	pubp->hfc_read_tbl					=	hfc_read_tbl;
	pubp->hfc_issue_relogin				=	hfc_issue_relogin;			/* FCLNX-0429 */
	pubp->hfc_enque_login_req			=	hfc_enque_login_req;		/* FCLNX-0429 */
	pubp->lock_mailbox					=	lock_mailbox;				/* FCLNX-0526 */
	pubp->unlock_mailbox				=	unlock_mailbox;				/* FCLNX-0526 */
	pubp->hfc_write_reg_ext				=	hfc_write_reg_ext;			/* FCLNX-0526 */
	pubp->hfc_trace						=	hfc_trace;
	pubp->hfc_watchdog_enter			= 	hfc_watchdog_enter;

	/* ioctl */
	pubp->hfc_sciocmd					=	hfc_sciocmd;
	pubp->hfc_ioctl_iodone				=	hfc_ioctl_iodone;
	pubp->structdump					=	structdump;
#if defined(__x86_64) && !defined(_HFC_FCGW) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	pubp->x86_64_ioctl32_basic			=	x86_64_ioctl32_basic;
	pubp->x86_64_ioctl32_inquiry		=	x86_64_ioctl32_inquiry;
	pubp->x86_64_ioctl32_cdb			=	x86_64_ioctl32_cdb;
	pubp->x86_64_ioctl32_gidft			=	x86_64_ioctl32_gidft;
	pubp->x86_64_ioctl32_payload		=	x86_64_ioctl32_payload;
	pubp->x86_64_ioctl32_api			=	x86_64_ioctl32_api;
	pubp->x86_64_ioctl32_diag			=	x86_64_ioctl32_diag;
#endif /*__x86_64 && !_HFC_FCGW */

	pubp->hfc_reset_all_timer			=	hfc_reset_all_timer;
	pubp->_hfc_wake_up				=	_hfc_wake_up;

	HFC_EXIT("hfc_set_pub_symbol_list");
	

}



int hfc_check_nonpub_symbol(void)
{
	int	rtn = 0;
	
	HFC_ENTRY("hfc_check_nonpub_symbol");
	
	/* detect */
	if (hfc_manage_info.npubp->hfc_get_mp_manage_info == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_check_hfcmp == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_check_hop == NULL)						rtn++;	/* FCLNX-0429 */
	if (hfc_manage_info.npubp->hfc_mp_scan_dev == NULL)						rtn++;	/* FCLNX-0521 */
	if (hfc_manage_info.npubp->hfc_read_registry == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_alloc_lgpath == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_make_lgpath == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_remove_lgpath == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_update_lgtarget == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_update_lgdev == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_search_and_update_target == NULL)		rtn++;
	if (hfc_manage_info.npubp->hfc_search_and_update_dev == NULL)			rtn++;
	if (hfc_manage_info.npubp->hfc_search_dev_compare == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_scsi_rescan_check == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_scsi_host_rescan == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_free_mp_table == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_wait_mp_ioend == NULL)					rtn++;	/* FCLNX-0459 */
	if (hfc_manage_info.npubp->hfc_get_write_retries_param == NULL)			rtn++;
	if (hfc_manage_info.npubp->hfc_write_retries == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_update_attribute == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_proc_info_lun == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_proc_info_option == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_free_dev == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_fo_check_and_offline == NULL)			rtn++;
	if (hfc_manage_info.npubp->hfc_forced_offline_e == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_forced_offline_c == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_set_automap == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_queue_depth == NULL)					rtn++;	/* FCLNX-0521 */
	if (hfc_manage_info.npubp->hfc_isolparam_setup == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_isolconf_setup == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_allocate_errcnt_info == NULL)			rtn++;
	if (hfc_manage_info.npubp->hfc_free_errcnt_info == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_set_retry_cnt == NULL)					rtn++;
//	if (hfc_manage_info.npubp->hfc_check_errcount == NULL)					rtn++;	/* FCLNX-GPL-394 */
	if (hfc_manage_info.npubp->hfc_clear_errinfo == NULL)					rtn++;
	
	/* strategy */
	if (hfc_manage_info.npubp->hfc_strategy_mp == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_abort == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_device_reset == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_mp_bus_reset == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_retry_strategy == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_ioerror_check == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_queue_check == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_queue_count == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_iodone == NULL)						rtn++;
	
	/* top */
	if (hfc_manage_info.npubp->hfc_attribute_check == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_convert_rptluns == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_check_luconfig == NULL)					rtn++;	/* FCLNX-611 */
	
	/* timer_recovery */
	if (hfc_manage_info.npubp->hfc_errlog_mp == NULL)						rtn++;
#ifdef _HFC_FCGW
	if (hfc_manage_info.npubp->hfc_fcgw_simmsg == NULL)						rtn++;
#endif /* _HFC_FCGW */
	if (hfc_manage_info.npubp->hfc_watched_errcount == NULL)				rtn++;
	/* ioctl */
	if (hfc_manage_info.npubp->hfc_ioctl_mp == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_rd_param == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_wr_param == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_lu_map == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_setpath == NULL)						rtn++;
	if (hfc_manage_info.npubp->hfc_mp_path_health == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_mp_lgtarget_map == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_mp_lgpath_info1 == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_get_isolparam == NULL)					rtn++;
	if (hfc_manage_info.npubp->hfc_adp_pathchg == NULL)						rtn++;


#if defined(__x86_64) && !defined(_HFC_FCGW) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16) /* FC-GW */
	if (hfc_manage_info.npubp->hfc_x86_64_32_reg_mp == NULL)				rtn++;
	if (hfc_manage_info.npubp->hfc_x86_64_32_unreg_mp == NULL)				rtn++;
#endif
	if (hfc_manage_info.npubp->hfc_read_isolparam == NULL)                                  rtn++;
	if (hfc_manage_info.npubp->hfc_read_retry_cnt == NULL)
      rtn++;

	/* diag */
	if (hfc_manage_info.npubp->read_dev_info_mp == NULL)					rtn++;
	if (hfc_manage_info.npubp->read_lg_target_info_mp == NULL)				rtn++;
	if (hfc_manage_info.npubp->read_lg_dev_info_mp == NULL)					rtn++;
	if (hfc_manage_info.npubp->read_lg_path_info_mp == NULL)				rtn++;
	if (hfc_manage_info.npubp->read_lg_path_info1_mp == NULL)				rtn++;
	if (hfc_manage_info.npubp->read_lg_path_info2_mp == NULL)				rtn++;
	if (hfc_manage_info.npubp->read_failover_info_mp == NULL)				rtn++;
	
	/* version */
	if (hfc_manage_info.npubp->hfc_get_pcm_rcsid == NULL)					rtn++;
	
	HFC_DBGPRT("hfc_check_nonpub_symbol() -- null symbol = %d\n", rtn);
	HFC_EXIT("hfc_check_nonpub_symbol");	
	if ( rtn != 0 ){
		HFC_DBGPRT("hfc_check_nonpub_symbol() -- null symbol = %d\n", rtn);
	}
	else
	{
		HFC_INFPRT("Success at loading hfcldd_mp module.\n");
	}
	return (rtn);
}


static int
hfc_slave_alloc(struct scsi_device *sdev)
{
//	HFC_DBGPRT("hfc_slave_alloc scsi (%d:%d:%d:%d)\n",
//		device->host->host_no, device->channel, device->id, device->lun);
	return 0;
}


static void
hfc_slave_destroy(struct scsi_device *sdev)
{
//	HFC_DBGPRT("hfc_slave_destroy scsi (%d:%d:%d:%d)\n",
//		device->host->host_no, device->channel, device->id, device->lun);
	return; 
}


static int
hfc_slave_configure(struct scsi_device *sdev)
{

	struct adap_info	*ap;
	int queue_depth;

	ap = (struct adap_info *)sdev->host->hostdata;

	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info)				/* FCLNX-0521 */
		queue_depth = hfc_manage_info.npubp->hfc_mp_queue_depth(ap, ap->queue_depth);	/* FCLNX-0521 */
	else 
		queue_depth = ap->queue_depth;
	
	if(sdev->tagged_supported) {
		scsi_activate_tcq(sdev, queue_depth);
	}
	else{
		scsi_deactivate_tcq(sdev, queue_depth);
	}

	HFC_EXIT("hfc_slave_configure");	

	return (0);
}

static ssize_t
hfcldd_conf_show(struct class_device *cdev, char *buf)
{
//	printk ("hfcldd_conf_show()\n");
	return snprintf(buf, sizeof(char)*CFG_ENT_SIZE, "%s\n", hfclddconf);
}

static ssize_t
hfcldd_conf_store(struct class_device *cdev, const char *buf, size_t count)
{
	if(hfclddconf)		/* FCLNX-676*/
		memcpy(hfclddconf, buf, sizeof(char)*CFG_ENT_SIZE);
//	printk ("hfcldd_conf_store()\n");
	return strlen(buf);
}

static CLASS_DEVICE_ATTR(hfcldd_conf, S_IRUGO | S_IWUSR, hfcldd_conf_show, hfcldd_conf_store);

struct class_device_attribute *hfcldd_host_attrs[] = {
 &class_device_attr_hfcldd_conf,
 NULL,
};

static struct scsi_host_template hfcldd_driver_template = {

	.proc_dir=					NULL,					
	.proc_name=					"hfcldd",			
	.proc_info=					hfc_proc_info_k26,			
	.name=						"Hitachi PCI Fibre Channel Adapter",
	.info=						hfc_info,	
	.slave_configure=			hfc_slave_configure,
	.slave_alloc=				hfc_slave_alloc,
	.slave_destroy=				hfc_slave_destroy,
	.queuecommand=				hfc_strategy,
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,17) && !defined(HFC_X8664_SLES10) /* FCLNX-0309 */
	.eh_strategy_handler=		NULL,
#endif
	.eh_abort_handler=			hfc_eh_abort,	
	.eh_device_reset_handler=	hfc_eh_device_reset,
	.eh_bus_reset_handler=		hfc_eh_bus_reset,
	.eh_host_reset_handler=		NULL,	
	.bios_param=				hfc_biosparam,	
	.can_queue=					1,
	.this_id=					-1,	
	.sg_tablesize=				HFC_SG_TABLESIZE,
	.cmd_per_lun=				HFC_CMD_PER_LUN,
	.use_clustering=			ENABLE_CLUSTERING,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
	.shost_attrs=				hfcldd_host_attrs,
#endif /* KERNEL_VERSION(2,6,16) */
};



int
hfc_biosparam(struct scsi_device *sdev, struct block_device *bdev, sector_t capacity, int geom[])
{
	int heads, sectors, cylinders;

	heads = 64;
	sectors = 32;
	cylinders = (unsigned long)capacity / (heads * sectors);

	if(cylinders > 1024) {
		heads = 255;
		sectors = 63;
		cylinders = (unsigned long)capacity / (heads * sectors);
	}
 
	geom[0] = heads;
	geom[1] = sectors;
	geom[2] = cylinders;

	return 0;
}

/*
 * Function: hfc_probe_one
 *
 * Purpose: Driver initialization
 *
 * Arguments:
 *  pdev	 - Pointer to pdev 
 *  id		 - PCI device ID
 *
 * Returns: 
 *  0         - Normal end
 *  Otherwise - Error
 *
 * Notes:
 */
int __devinit
hfc_probe_one(struct pci_dev *pdev, const struct pci_device_id *id)
{
	struct Scsi_Host *host=NULL,*hsd_host=NULL;		/* FCLNX-0429 */
	int error = -ENODEV;
	int reg_chrdev = 0;
	int	rtn, mp=0;
	struct adap_info *ap = NULL;
	uint64_t dma_mask;
	struct pub_symbol_list			*pubp;
	struct nonpub_symbol_list		*npubp;
	struct mp_manage_info			*hfc_mp_manage_info;
	int	adapter_max = -1;
	struct mp_adap_info			*mpap;	
	ulong					flags = 0;      /* FCLNX-0506 */
	ushort  bind_err = FALSE;			/* FCLNX-0634 */

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)			/* FCLNX-0476 */
	int	i;
#endif

#ifdef HFC_SYSFS_FCGW
	int	lp;							/* FC-GW */
#endif /* HFC_SYSFS_FCGW */

	if(instance == 0){
#ifndef _HFC_NO_RASLOG
		hraslogopt.func_code = OPEN_RASLOG;
		raslog_install = _hraslogserv( &hraslogopt );
		if( raslog_install == 1 ){
			HFC_WRNPRT("hfcldd : HFC_ERR9 FC Adapter Driver error (ErrNo:0xC7) Failed to open hraslog\n");
		}
		else if( raslog_install == 2 ){
			HFC_INFPRT("hfcldd : Raslog module is not loaded.\n");
		}
		else if( raslog_install == 0 ){
			HFC_INFPRT("hfcldd : Raslog version is raslog-%d.%d.%d-%d.\n", hraslogopt.ver, hraslogopt.rev,
						hraslogopt.rver, hraslogopt.wver);
		}
#endif
		instance++;
		memset( &hfc_manage_info, 0, sizeof(hfc_manage_info) );
		memset( &hfc_narrow_dev, 0, sizeof(hfc_narrow_dev) );

		pubp  = hfc_get_pub_symbol_list();
		npubp = hfc_get_nonpub_symbol_list();
		hfc_manage_info.pubp  = pubp;
		hfc_manage_info.npubp = npubp;
		hfc_set_pub_symbol_list();
		if(!hfc_check_nonpub_symbol()) {
			hfc_manage_info.hfcldd_mp_mod = HFCLDD_MP_MOD_ENABLE;
			hfc_mp_manage_info = npubp->hfc_get_mp_manage_info();
			hfc_manage_info.mp_manage_info = hfc_mp_manage_info;
			hfc_mp_manage_info->manage_info = &hfc_manage_info;
			try_module_get(hfc_mp_manage_info->hfcldd_mp);
		}

		strcpy( hfc_manage_info.package_ver, package_ver);
		init_MUTEX(&hfc_manage_info.sem);
		spin_lock_init(&hfc_manage_info.hfcmp_lock);


		hfclddconf = (char *)hfcldd_cnf();

		if (!hfclddconf) {
			HFC_DBGPRT("hfclddconf pointer = 0.\n");
		}

		
		hfc_param_setup();
		adapter_max = hfc_get_adapter_bindings();
		
		if(hfc_manage_info.hfcldd_mp_mod) {
			if(hfc_manage_info.npubp->hfc_read_registry(hfc_automap, hfclddconf) )  {
				if( raslog_install )
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC1, NULL, 0) ;
				else
					hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC1, NULL, 0) ;
				goto hfc_probe_enable_error;
			}
			
			mp = hfc_manage_info.npubp->hfc_check_hfcmp(hfc_narrowmap);	/* FCLNX-0429 */
			
			if(mp) {
				if(hfc_manage_info.npubp->hfc_alloc_lgpath() )	{
					if( raslog_install )
						hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC2, NULL, 0) ;
					else
						hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC2, NULL, 0) ;
					goto hfc_probe_enable_error;	
				}
			}
			hfc_manage_info.hfcplus_enable = hfc_manage_info.npubp->hfc_check_hop(mp);		/* FCLNX-0429 */
			if(hfc_manage_info.hfcplus_enable){ 											/* FCLNX-0506 */
				if (mp) {
					/* FCLNX-0642 */
					if(hfc_manage_info.hfcplus_enable == 1){	/* FCLNX-0700 */
						HFC_INFPRT("hfcldd : HFC-PCM Premium Edition is loaded.");
					}
					else{
						HFC_INFPRT("hfcldd : HFC-PCM Enterprise Edition is loaded.");
					}										/* FCLNX-0700 */
				}
				else {
					HFC_INFPRT("hfcldd : E-Option is enabled."); 							/* FCLNX-0642 */
				}

				hfc_manage_info.npubp->hfc_isolparam_setup();								/* FCLNX-0506 */
			}
		}
		
		if(adapter_max>=0) hba_num = adapter_max+1;
		hfc_version();
	}
	else {

		hfclddconf = (char *)hfcldd_cnf();
		
		if (!hfclddconf) {
			HFC_DBGPRT("hfclddconf pointer =0\n");
		}

	}

	HFC_DBGPRT("hfcldd[%d] found on PCI bus %i, dev %i\n",
		(int)id->driver_data, pdev->bus->number, PCI_SLOT(pdev->devfn));

	if (pci_enable_device(pdev)) {
		if( raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x9A, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x9A, NULL, 0) ;
		goto hfc_probe_enable_error;
	}
	
	HFC_DBGPRT(" hfcldd : pci device is enabled irq=%d\n",pdev->irq);

	error = -ENOMEM;

	HFC_DBGPRT(" hfcldd : befire scsi_host_alloc\n");

	host = scsi_host_alloc(&hfcldd_driver_template, sizeof(struct adap_info));
	
	if ( host == NULL ){
		if( raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x37, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x37, NULL, 0) ;
		goto hfc_host_alloc_error;
	}

	HFC_DBGPRT( "  hfcldd : hfc_detect - adap_info pointer was set\n"); 
	ap = (struct adap_info *)host->hostdata;
	memset(ap, 0, sizeof(struct adap_info));

	ap->raslog_install = raslog_install;
#ifndef _HFC_NO_RASLOG
	if( raslog_install == 0 ){
		ap->raslog_ver = hraslogopt.ver;
		ap->raslog_rev = hraslogopt.rev;
		ap->raslog_rver = hraslogopt.rver;
		ap->raslog_wver = hraslogopt.wver;
	}
#endif

	/* set adap_info */

#ifdef __x86_64			
	/* set dma_mask */
	if (pci_set_dma_mask(pdev, 0xffffffffffffffffULL)==0) {
		HFC_DBGPRT(" hfcldd : Using 64bit DMA\n");
	} else if (pci_set_dma_mask(pdev, 0xffffffffUL)==0) {
		HFC_DBGPRT(" hfcldd : Using 32bit DMA\n");
	} else {
		if( raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC3, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC3, NULL, 0) ;
		goto hfc_probe_error;
	}
#endif			
			
	rtn=0;
	dma_mask = DMA_64BIT_MASK;
	if (sizeof(dma_addr_t) > 4) {
		if (pci_set_dma_mask(pdev, DMA_64BIT_MASK) == 0) {
			if (pci_set_consistent_dma_mask(pdev, DMA_64BIT_MASK)) {
				rtn=pci_set_consistent_dma_mask(pdev, DMA_32BIT_MASK);
				dma_mask = DMA_32BIT_MASK;
			}
		}
		else
		{
			dma_mask = DMA_32BIT_MASK;
			rtn=pci_set_dma_mask(pdev, DMA_32BIT_MASK);
		}
	}
	else
	{
		dma_mask = DMA_32BIT_MASK;
		rtn=pci_set_dma_mask(pdev, DMA_32BIT_MASK);
	}
	if(rtn){
		if( raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x9B, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x9B, NULL, 0) ;
		goto hfc_probe_error;
	}
	ap->dma_mask = dma_mask;	/* FCLNX-0671 */
	
	ap->manage_info = &hfc_manage_info;
	
	ap->hosts = host;
	ap->host_no = host->host_no;
	ap->pci_cfginf = pdev;
			
	if (hfc_shadow == 1)
		ap->mlpf_mode |= HFC_MMODE_SHADOW;
	
	rtn = hfc_pci_conf(ap);
	
	if ( HFC_MMODE_CHECK_SHARED(ap) ) {
		ap->lparmode.frame_cnt = 3;						/* FCLNX-0375 */
	}
	else {
		ap->lparmode.frame_cnt = 4;						/* FCLNX-0375 */
	}
	
	if (rtn != 0) {
		if( rtn != (-ENODEV) )															/* FCLNX-0357 */
		{
			if( raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC4, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC4, NULL, 0) ;
		}																				/* FCLNX-0357 */
		goto hfc_probe_error;
	}

	ap->irq = pdev->irq;
	host->irq = pdev->irq;

#ifdef HFC_SYSFS_FCGW
	host->transportt = hfc_fc_attach_transport;									/* FC-GW */
#endif /* HFC_SYSFS_FCGW */

	if(hfc_query_pktype(ap)){
		goto hfc_pktype_error;
	}

	hfc_get_adapter_port_no(ap);		/* Read adapter port# */
	hfc_read_hfcbios(ap);				/* Read hbabios data */

	rtn = hfc_search_adapter_number(ap);

	if ( rtn == MAX_ADAP_CNT + 1) { /* Incorrect Instance number is specified */
		 /* FCLNX-0376 */
		if ( !HFC_MMODE_CHECK_BASIC(ap) ) {
			hfc_mlpf_change_state(ap, HFC_HG_LPRSTATUS_ISVALID, HFC_ENABLE_LPAR_STATE);
			hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_SPACE, HFC_DISABLE_LPAR_STATE);
			hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_SHADOW_DED, HFC_ENABLE_LPAR_STATE);
		}
		goto hfc_attach_error;
	}
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16) 								/* FCLNX-0476 */
	else if (rtn == -1) {
		ap->instance = hba_num;
		hba_num++;
#else
	else if ( (rtn == -1) || (( ap->defparam == 1 )&&(ap->automap == 1)) ) { /* instance number is not specified or Force Default Setting */ /* FCLNX-0634 */
		for (i=0; i<MAX_ADAP_CNT; i++) {
			if(hfc_manage_info.adap_info_arg[i] == NULL){
				if((( ap->defparam == 1 )&&( ap->automap == 1 ))
				/* FCLNX-0634 */
				||(hfc_manage_info.adap_bind[i] == -1))
				break;
			}
		}
		if (i == MAX_ADAP_CNT)
			goto hfc_attach_error;
		ap->instance = i;
	}
	else if (hfc_manage_info.adap_info_arg[rtn] != NULL){	/*The instance number is also attached.*/ /* FCLNX-0634 */
		bind_err = TRUE;
		hfc_errlog(NULL,NULL,NULL,HFC_ERRLOG_TYPE_NONE,ERRID_HFCP_EVNT3,0x50,NULL,0) ;/* FCLNX-0634 */
		for (i=0; i<MAX_ADAP_CNT; i++) {
			if(hfc_manage_info.adap_info_arg[i] == NULL){
				break;
			}
		}
		if (i == MAX_ADAP_CNT)
			goto hfc_attach_error;
		ap->instance=i;
#endif
	}
										/* FCLNX-0476 */
	else {	 /* instance number is specified */
		ap->instance = rtn;
	}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	host->unique_id = hfc_manage_info.instance; 	/* FCLNX-0476 */
	ap->unique_id	= hfc_manage_info.instance; 	/* FCLNX-0476 */
	ap->dev_minor	= hfc_manage_info.instance; 	/* FCLNX-0476 */
#else
	if(( ap->defparam ==1 )&&( ap->automap== 1 )){	/* FCLNX-0630 */
		host->unique_id = hfc_manage_info.instance; 	/* FCLNX-0476 */
		ap->unique_id	= hfc_manage_info.instance; 	/* FCLNX-0476 */
		ap->dev_minor	= hfc_manage_info.instance; 	/* FCLNX-0476 */
	}
	else{	/* FCLNX-0630 */ 
		host->unique_id = ap->instance;                 /* FCLNX-0453 */
		ap->unique_id   = ap->instance;                 /* FCLNX-0453 */
		ap->dev_minor   = ap->instance;                 /* FCLNX-0453 */
	}  	/* FCLNX-0630 */
#endif

	/* FCLNX-0429 */	/* FCLNX-0651 */
	if (hfc_manage_info.npubp->hfc_mp_hsd_enable)	/* FCLNX-0651 */
	{	
		if ((hsd_host = hfc_manage_info.npubp->hfc_mp_search_hsd_host(host, ap)) == NULL)
			hsd_host = hfc_manage_info.npubp->hfc_mp_scsi_host_alloc(host, ap);
			
		if (hsd_host == NULL) {
			if( raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x37, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x37, NULL, 0) ;
			
			goto hfc_probe_error;
		}
	}	/* FCLNX-0651 */

	HFC_DBGPRT("instance_num = %d ap->instance = %d\n",instance, ap->instance);
	hfc_conf_setup(ap);
	if (hfc_manage_info.hfcldd_mp_mod) {		/* FCLNX-0651*/
		hfc_manage_info.npubp->hfc_set_retry_cnt(ap);                                   /* FCLNX-0534 */
	}	/* FCLNX-0651*/

//	if((hfc_manage_info.hfcplus_enable)&&(HFC_MMODE_CHECK_BASIC(ap))){	/* FCLNX-0506 */
	if((hfc_manage_info.hfcldd_mp_mod)&&(HFC_MMODE_CHECK_BASIC(ap))){	/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */
		hfc_manage_info.npubp->hfc_isolconf_setup(ap);  /* FCLNX-0506 */
		if(hfc_manage_info.npubp->hfc_allocate_errcnt_info(ap)){
			goto hfc_attach_error;
		}
	}
	else{
		ap->login_retry=HFC_LOGIN_RETRY;
		ap->els_retry=HFC_ELS_RETRY;
//		ap->to_reset_retry=HFC_TO_RESET_RETRY;	/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */
		ap->scsi_to_retry=HFC_SCSI_TO_RETRY;
	}

	if (hsd_host) {								/* FCLNX-0429 */
		hfc_manage_info.npubp->hfc_mp_set_hsd_parm(hsd_host, ap, dma_mask); /* FCLNX-0521 */
	}

	if( hfc_attach(ap) ){
		HFC_ERRPRT("hfcldd :  Failed to allocate adapter resource\n");
		goto hfc_attach_error;
	}

	/* Initialize package type and fw_init table */
	HFC_DBGPRT(" hfcldd : hfcl_probe_one - initialize package type \n");

	if ( hfc_start_adapter(ap) ) {
		goto hfc_attach_error;
	}

	set_bit(HFC_ENABLE, (ulong *)&ap->status);							/* hfcends */
	ap->open_status = 0;												/* hfcends */

	if (ap->automap == 0) {
		hfc_get_target_bindings(ap);
	}

   /* Register special file */
	HFC_DBGPRT(" hfcldd : hfc_detect - set device special file\n");
	if(hfc_manage_info.instance == 0) {
		hfc_major = register_chrdev(0, "hfcldd", &hfc_fops);
		if (0 > hfc_major) {
			HFC_DBGPRT("%s(): register_chrdev rc=%d\n", 	__func__, hfc_major);
			reg_chrdev = 1;
		}
		HFC_DBGPRT(" hfcldd : hfc_detect - major # = %d\n", hfc_major);
	}
						
	ap->dev_major = hfc_major;										/* FCLNX_001 */
	HFC_DBGPRT(" hfcldd : hfc_detect - minor # = %d\n", ap->dev_minor);
	hfc_manage_info.major = hfc_major;								/* FCLNX_001 */
	hfc_manage_info.adap_info_arg[ap->dev_minor] = ap;

	hfcldd_driver_template.can_queue = ap->can_queue;
	hfcldd_driver_template.sg_tablesize = ap->sg_tblsize;

	HFC_DBGPRT("hfcldd : hfcl_detect hfc_manage_info.instance = %d\n",
		hfc_manage_info.instance);
	hfc_manage_info.instance++;

	pci_set_drvdata(pdev,host);				/* FCLNX-0290 */
	HFC_DBGPRT(" hfcldd : hfc_detect scsi-add-host\n");
	if ( scsi_add_host(host, &pdev->dev) ){
		if( raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC6, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC6, NULL, 0) ;
		goto error_disable_adapter;
	}							/* FCLNX-0290 */
	
	if (hsd_host) {										/* FCLNX-0429 */
		if ( hfc_manage_info.npubp->hfc_mp_scsi_add_host(hsd_host)){
			if( raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC6, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC6, NULL, 0) ;

			goto error_disable_adapter;
		}
	}
	
#ifdef HFC_SYSFS_FCGW																		/* FC-GW */
	fc_host_node_name(host)			= ap->node_name;						/* FC-GW */
	fc_host_port_name(host)			= ap->ww_name;							/* FC-GW */
#endif /* HFC_SYSFS_FCGW */			/* FC-GW */

	/* MSI handler */
	if(ap->pkg.type != HFC_PKTYPE_FPP){
		if(hfc_msi_enable && !ap->defparam) {
			if (pci_enable_msi(pdev) == 0) {
				HFC_DBGPRT("hfc_probe_one() -- pci_enable_msi(pdev) success\n");
				ap->msi_flag=1;
			}
			else {
				HFC_DBGPRT("hfc_probe_one() -- pci_enable_msi(pdev) fail\n");
				goto hfc_msi_error;
			}
		}
	}

	/* register int_handler */	
	HFC_DBGPRT(" hfcldd : hfcl_detect - register int_handler \n");
	if (request_irq(pdev->irq, hfc_intr, SA_INTERRUPT|SA_SHIRQ, "hfcldd", (void *)ap)) {
		if( raslog_install )
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x4F, NULL, 0) ;
		else
			hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x4F, NULL, 0) ;
		goto hfc_irq_error;
	}
	
	mpap = ap->mp_adap_info; /* FCLNX-GPL-284 */
	
	if((hfc_pxe_boot != 0)|| (bind_err == TRUE) ){          /* FCLNX-0634 */
		goto skip_link_init;
	}

	if(hfc_manage_info.hfcplus_enable){							/* FCLNX-0506 */
		if ( ap->isol_force == HFC_PRT_FRC_ISOL ){		/* Force port isolated state *//* - FCLNX-546 - */

			set_bit( HFC_ISOL, (ulong *)&ap->status );
			ap->isol_detail = HFC_ISOLATE_PORT_C;
			goto skip_link_init;
		}
		else if (ap->isol_force == HFC_CHKSTP_FRC_ISOL ){

			set_bit( HFC_ISOL, (ulong *)&ap->status );
			ap->isol_detail = HFC_ISOLATE_CHKSTP;

			HFC_ADAPLOCK_IRQSAVE(flags);
			
			set_bit( HFC_HWISOL, (ulong *)&mpap->status );	
			
			HFC_ADAPUNLOCK_IRQRESTORE(flags);

			goto skip_link_init;
		}
	
	}
	
	if( test_bit(HFC_HWCHKSTOP, (ulong *)&mpap->status) ) /* FCLNX-GPL-284 */
	{	/* This core's state was "check stop". */
		/* So, we chage this port's state to "check stop". */
		HFC_ISSUE_CSTP(ap, FALSE, HFC_ABEND_FCSTP_IML );						/* FCLNX-GPL-527 */
		/* And, we skip "link inititalize". */
		goto skip_link_init;
	}

	HFC_DBGPRT( " ** hfcldd : hfc_detect - start hfc_initialize \n"); 
	if( !test_bit(HFC_ATTACH, (ulong *)&ap->attach_status ) ){					/* FCLNX-0228 */
		HFC_DBGPRT( " ** hfcldd : hfc_detect - skip hfc_initialize \n"); 
	} else {
		if ( HFC_MMODE_CHECK_SHADOW(ap) && HFC_MMODE_CHECK_REBOOT(ap) )
		{	/* Skip link initlialization and open int_a_mask */
			HFC_DBGPRT( " ** hfcldd : hfc_detect - skip hfc_initialize by mlpf reboot\n"); 
			hfc_write_reg( ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )HFC_INTA_MASK_MLPF ); 
		}
		else
			hfc_initialize(ap, 0);				/* FCLNX-0514 */
	}																			/* FCLNX-0228 */

#ifdef HFC_SYSFS_FCGW																			/* FC-GW */
	for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)										/* FC-GW */
	{
		struct target_info *target;
		target = hfc_hash_target_info(ap, lp);
		
		if (target != NULL)
		{
			hfc_rport_add(ap, target);
		}
	}
#endif /* HFC_SYSFS_FCGW */					/* FC-GW */

	HFC_ADAPLOCK_IRQSAVE(flags);						/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */
	if(!hfc_check_hba_isolation(ap))					/* FCLNX-GPL-349 */
		ap->hba_isolation = HFC_ISOL_STOP;
	HFC_ADAPUNLOCK_IRQRESTORE(flags);					/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */


	if(hfc_manage_info.hfcldd_mp_mod) {
		if (hfc_manage_info.lg_target_info) hfc_manage_info.npubp->hfc_make_lgpath();
	}
	
	HFC_DBGPRT(" hfcldd : hfc_scan_host\n");
	
	if (hsd_host != NULL) {												/* FCLNX-0429 */
		hfc_manage_info.npubp->hfc_mp_scsi_scan_host(hsd_host);			/* FCLNX-0429 */
	}
	
	scsi_scan_host(host);
	
	if(hfc_manage_info.hfcldd_mp_mod) {
		if (hfc_manage_info.lg_target_info) hfc_manage_info.npubp->hfc_scsi_host_rescan();
	}


	HFC_EXIT("hfc_detect");

skip_link_init:

	return 0;

hfc_irq_error:
hfc_msi_error:
error_disable_adapter:
	if(reg_chrdev != 0){
		unregister_chrdev(hfc_major, "hfcldd");
	}

hfc_attach_error:
//	if(hfc_manage_info.hfcplus_enable){	/* FCLNX-0506 *//* FCLNX-GPL-394 */
	if ( hfc_manage_info.hfcldd_mp_mod ){	/* FCLNX-GPL-349 *//* FCLNX-GPL-394 */
		hfc_manage_info.npubp->hfc_free_errcnt_info(ap);		/* FCLNX-0506 */
	}
hfc_pktype_error:
	pci_release_regions(pdev);
	iounmap((void *)(ap->mem_base_addr));

hfc_probe_error:
	if (hsd_host != NULL) {												/* FCLNX-0429 */
		hfc_manage_info.npubp->hfc_mp_scsi_host_put(hsd_host);			/* FCLNX-0429 */
	}
	
	scsi_host_put(host);
		
hfc_host_alloc_error:
	pci_disable_device(pdev);

hfc_probe_enable_error:

	return error;
}

/*
 * Function: hfc_remove_one
 *
 * Purpose: Driver release 
 *
 * Arguments:
 *  pdev	 - Pointer to pdev 
 *
 * Returns: -
 *
 * Notes:
 */
void __devexit
hfc_remove_one(struct pci_dev *pdev)
{
	struct Scsi_Host *host = pci_get_drvdata(pdev);
	struct adap_info *ap = (struct adap_info *)host->hostdata;

	HFC_DBGPRT("hfcldd hfcl_remove_one\n");
	
#ifdef HFC_SYSFS_FCGW
	fc_remove_host(host);														/* FC-GW */
#endif /* HFC_SYSFS_FCGW */
	
	scsi_remove_host(host);
	
	if (hfc_manage_info.npubp->hfc_mp_hsd_enable){ 								/* FCLNX-0651 */
		hfc_manage_info.npubp->hfc_mp_scsi_host_remove(host, ap);				/* FCLNX-0429 */
	}
	/* FCLNX-0651 */
	
	hfc_release(host);
	pci_disable_device(pdev);
	scsi_host_put(host);

	pci_set_drvdata(pdev,NULL);
}


int hfc_probe_internal(struct adap_info *ap, int *skip_linkini, ulong *flags)
{
	struct Scsi_Host *hsd_host=NULL;		/* FCLNX-0429 */ /* FCLNX-0671 */
	int rtn=0,i=0;
	int wk_can_queue = 0, wk_sg_tblsize = 0;
	struct mp_adap_info	*mpap=NULL;
	ushort  bind_err = FALSE;			/* FCLNX-0634 */
	int alloc_hsdlhost = 0;				/* FCLNX-0677 */
	
	
	/* loading hfcldd_conf.o */

//	hfclddconf = (char *)hfcldd_cnf();	/* FCLNX-0671 */

	if (!hfclddconf) {
		HFC_DBGPRT("hfcldd%d : hfc_probe_internal - hfclddconf pointer = 0.\n",ap->dev_minor);
	}

	hfc_param_setup();
	/* Update Adapter_bindings();*/
	hfc_get_adapter_bindings();	/* FCLNLX-0675 */

	if(hfc_manage_info.hfcldd_mp_mod) {/* FCLNX-0671 */
		if(hfc_manage_info.hfcplus_enable){ 
			/* Loading HBA isolation Parameter */
			hfc_manage_info.npubp->hfc_isolparam_setup();	/* FCLNX-0506 */
		}
	}/* FCLNX-0671 */

	if ( (ap->pkg.vender_id == (ushort) hfc_read_cnfg (ap, HFC_HOST_VENDER_ID, 0x2))
	  && (ap->pkg.device_id == (ushort) hfc_read_cnfg (ap, HFC_HOST_DEVICE_ID, 0x2)) ) {
		uchar code,reg;
		
		code = ap->pkg.code;
		reg  = (uchar) hfc_read_reg_ext(ap, 0x005, 0x1);

		if (code == 0x86)
			code &= 0xfb;

		if (reg  == 0x86)
			reg  &= 0xfb;

		if (code != reg)
			rtn = EIO;
	}
	else {
		rtn = EIO;
	}

	if (rtn)
		return (rtn);

	hfc_get_adapter_port_no(ap);		/* Read adapter port# */
	hfc_read_hfcbios(ap);				/* Read hbabios data */
	rtn = hfc_search_adapter_number(ap);

	if ( rtn == MAX_ADAP_CNT + 1) { /* Incorrect Instance number is specified */
		goto hfc_attach_error_internal;
	}
	else if ( (rtn == -1) || (( ap->defparam == 1 )&&(ap->automap == 1)) ) { /* instance number is not specified or Force Default Setting */ /* FCLNX-0634 */
		for (i=0; i<MAX_ADAP_CNT; i++) {
			if(hfc_manage_info.adap_info_arg[i] == NULL){
				if((( ap->defparam == 1 )&&( ap->automap == 1 ))
				/* FCLNX-0671 */
				||(hfc_manage_info.adap_bind[i] == -1))
				break;
			}
		}
		if (i == MAX_ADAP_CNT)
			goto hfc_attach_error_internal;
		ap->instance = i;
	}
	else if (hfc_manage_info.adap_info_arg[rtn] != NULL){	/*The instance number is also attached.*/ /* FCLNX-0634 */
		bind_err = TRUE;
		hfc_errlog(NULL,NULL,NULL,HFC_ERRLOG_TYPE_NONE,ERRID_HFCP_EVNT3,0x50,NULL,0) ;/* FCLNX-0671 */
		for (i=0; i<MAX_ADAP_CNT; i++) {
			if(hfc_manage_info.adap_info_arg[i] == NULL){
				break;
			}
		}
		if (i == MAX_ADAP_CNT)
			goto hfc_attach_error_internal;
		ap->instance=i;
	}
										/* FCLNX-0671 */
	else {	 /* instance number is specified */
		ap->instance = rtn;
	}

	if(( ap->defparam ==1 )&&( ap->automap== 1 )){	/* FCLNX-0630 *//* FCLNX-0671 */
		ap->unique_id	= instance; /* FCLNX-0476 *//* FCLNX-0671 */
		ap->dev_minor	= instance; /* FCLNX-0476 *//* FCLNX-0671 */
	}
	else{	/* FCLNX-0630 */ /* FCLNX-0671 */ 
		ap->unique_id   = ap->instance;/* FCLNX-0476 *//* FCLNX-0671 */
		ap->dev_minor   = ap->instance;/* FCLNX-0476 *//* FCLNX-0671 */
	}  	/* FCLNX-0630 */ /* FCLNX-0671 */

	/* FCLNX-0671 */
	if (hfc_manage_info.npubp->hfc_mp_hsd_enable)
	{	
		HFC_ADAPUNLOCK_IRQRESTORE(*flags);
		/* Don't get spin_lock_irq before this routine */
		if ((hsd_host = hfc_manage_info.npubp->hfc_mp_search_hsd_host(ap->hosts, ap)) == NULL){
			hsd_host = hfc_manage_info.npubp->hfc_mp_scsi_host_alloc(ap->hosts, ap);
			alloc_hsdlhost = 1;
		}
		HFC_ADAPLOCK_IRQSAVE(*flags);
			
		if (hsd_host == NULL) {
			if( raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0x37, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0x37, NULL, 0) ;
			
			goto hfc_probe_error_internal;
		}
	}/* FCLNX-0671 */
	
	wk_can_queue = ap->can_queue;	/* FCLNX-0677 */
	wk_sg_tblsize = ap->sg_tblsize;	/* FCLNX-0677 */

	hfc_conf_setup(ap);
	if (hfc_manage_info.hfcldd_mp_mod) {		/* FCLNX-0671 */
		/* Setting Path Retry Count of Same Path for HFC-PCM */
		hfc_manage_info.npubp->hfc_set_retry_cnt(ap);
	}	/* FCLNX-0671 */

	if((hfc_manage_info.hfcplus_enable)&&(HFC_MMODE_CHECK_BASIC(ap))){/* FCLNX-0671 */
		hfc_manage_info.npubp->hfc_isolconf_setup(ap);  /* FCLNX-0506 */
		if(hfc_manage_info.npubp->hfc_allocate_errcnt_info(ap)){
			goto hfc_attach_error_internal;
		}
	}
	
	if((hsd_host)&&(alloc_hsdlhost)){								/* FCLNX-0671 */
		hfc_manage_info.npubp->hfc_mp_set_hsd_parm(hsd_host, ap, ap->dma_mask); 
	}											/* FCLNX-0671 */


	/* Area allocation required (except DMA area ) */
	HFC_DBGPRT( "  hfcldd : hfc_probe_internal - allocate memory area\n"); 
	if ( hfc_allocate_memory(ap, TRUE) ) {
		HFC_ERRPRT("hfcldd%d :  Failed to allocate adapter memory resource.\n",ap->dev_minor);
		return (EIO);
	}

	/* Allocate init_table, xob, xrb, mailbox, seg_info, soft_log (for dma) */

	HFC_DBGPRT( "  hfcldd : hfc_probe_internal - allocate DMA area\n"); 
	if ( hfc_allocate_dma(ap) ) {
		HFC_ERRPRT("hfcldd%d :  Failed to allocate adapter DMA resource.\n",ap->dev_minor);
		hfc_free_memory(ap, TRUE);
		return (EIO);
	}

	if ( hfc_start_adapter(ap) ) {
		HFC_DBGPRT( " ** hfcldd : hfc_probe_internal - Failed hfc_start_adapter.\n"); 
		hfc_free_dma(ap);
		hfc_free_memory(ap, TRUE);
		return (EIO);
	}

	/* Get Persistent Binding Infomation */
	if (ap->automap == 0) {
		hfc_get_target_bindings(ap);
	}
	
	hfc_manage_info.adap_info_arg[ap->dev_minor] = ap;	/* FCLNX-0671 */
	
	ap->can_queue = wk_can_queue;	/* FCLNX-0671 */
	ap->sg_tblsize = wk_sg_tblsize;	/* FCLNX-0671 */
	
	if ((hsd_host)&&(alloc_hsdlhost)) {										/* FCLNX-0429 *//* FCLNX-0671 */
		HFC_ADAPUNLOCK_IRQRESTORE(*flags);
		if ( hfc_manage_info.npubp->hfc_mp_scsi_add_host(hsd_host) ){	/* FCLNX-0683 */
			if( raslog_install )
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xC6, NULL, 0) ;
			else
				hfc_errlog(NULL, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xC6, NULL, 0) ;
			HFC_ADAPLOCK_IRQSAVE(*flags);
			goto hfc_attach_error_internal;
		}
		HFC_ADAPLOCK_IRQSAVE(*flags);
	}

	if(bind_err == TRUE){          /* FCLNX-0634 *//* FCLNX-0671 */
		*skip_linkini = 1;
		return(0);
	}

	if(hfc_manage_info.hfcplus_enable){							/* FCLNX-0506 */
		if ( ap->isol_force == HFC_PRT_FRC_ISOL ){ /* Force port isolated state *//* - FCLNX-546 - *//* FCLNX-0671 */

			set_bit( HFC_ISOL, (ulong *)&ap->status );
			ap->isol_detail = HFC_ISOLATE_PORT_C;
			*skip_linkini = 1;
			return(0);
		}
		else if (ap->isol_force == HFC_CHKSTP_FRC_ISOL ){

			set_bit( HFC_ISOL, (ulong *)&ap->status );
			ap->isol_detail = HFC_ISOLATE_CHKSTP;

			mpap = ap->mp_adap_info;
			set_bit( HFC_HWISOL, (ulong *)&mpap->status );	
			
			*skip_linkini = 1;
			return(0);
		}
	
	}

	return (0);

hfc_attach_error_internal:
	if(hfc_manage_info.hfcplus_enable){	/* FCLNX-0506 */
		hfc_manage_info.npubp->hfc_free_errcnt_info(ap);		/* FCLNX-0506 *//* FCLNX-0671 */
	}

hfc_probe_error_internal:
	if (hsd_host != NULL) {												/* FCLNX-0429 *//* FCLNX-0671 */
		hfc_manage_info.npubp->hfc_mp_scsi_host_put(hsd_host);			/* FCLNX-0429 *//* FCLNX-0671 */
	}
			
	return(ENOMEM);

}



int hfc_remove_internal(struct adap_info *ap, ulong *flags)
{
	int wait=0,lp=0;
	struct target_info *target=NULL;

	/* Don't get spin_lock_irq before this routine. */
	if (hfc_manage_info.npubp->hfc_mp_hsd_enable){ 								/* FCLNX-0671 */
		
		/*  Forced to Offline All Target Path */
		for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)
		{
			target = hfc_hash_target_info(ap, lp);
			if (target != NULL)
			{
				hfc_manage_info.npubp->hfc_forced_offline_e(target, FALSE);
			}
		}
		
		/* Remove own Scsi_host for hsdldd */
		hfc_manage_info.npubp->hfc_mp_remove_unused_hsdlhost(ap->hosts,flags);
		/* Remove other Scsi_host for hsdldd */
	}


	/* Crear Adapter Attach Bit*/
	clear_bit(HFC_ATTACH, (ulong *)&ap->attach_status);	/* FCLNX-0672 */

	do {													/* FCLNX-0459 */
		wait=0;
		
		if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info)
			wait = hfc_manage_info.npubp->hfc_wait_mp_ioend(ap);
		
		if (!wait)
			wait = hfc_release_adp(ap);
		
		/* sleep for a while */
		HFC_ADAPUNLOCK_IRQRESTORE(*flags);
		msleep(1);
		HFC_ADAPLOCK_IRQSAVE(*flags);
	} while (wait);											/* FCLNX-0459 */
	
	if(hfc_manage_info.hfcplus_enable)						/* FCLNX-0506 *//* FCLNX-671 */
		hfc_manage_info.npubp->hfc_free_errcnt_info(ap);	/* FCLNX-0506 *//* FCLNX-671 */
	
	/* Release DMA memory area */
	hfc_free_dma(ap);
	
	if (hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info) {
		hfc_manage_info.npubp->hfc_remove_lgpath(ap);
	}
	
	/* Release adapter memory area */
	hfc_free_memory(ap, TRUE);
	
	hfc_manage_info.adap_info_arg[ap->dev_minor] = NULL;					/* FCLNX-0317 */

	return (0);
}


#ifdef HFC_SYSFS_FCGW
static void hfc_get_rport_dev_loss_tmo(struct fc_rport *);
static void hfc_set_rport_dev_loss_tmo(struct fc_rport *hfc_rport, uint32_t timeout);
static void hfc_get_starget_node_name(struct scsi_target *);
static void hfc_get_starget_port_name(struct scsi_target *);
static void hfc_get_starget_port_id(struct scsi_target *);
static void hfc_get_host_port_id(struct Scsi_Host *);

struct fc_function_template hfcldd_fc_function_template = {	/* FC-GW */

	.show_host_node_name=			1,
	.show_host_port_name=			1,
	.get_host_port_id=				hfc_get_host_port_id,
	.show_host_port_id=				1,

	.dd_fcrport_size=				sizeof(struct target_info),

	.get_starget_node_name=			hfc_get_starget_node_name,
	.show_starget_node_name=		1,
	.get_starget_port_name=			hfc_get_starget_port_name,
	.show_starget_port_name=		1,
	.get_starget_port_id=			hfc_get_starget_port_id,
	.show_starget_port_id=			1,

	.get_rport_dev_loss_tmo=		hfc_get_rport_dev_loss_tmo,
	.set_rport_dev_loss_tmo=		hfc_set_rport_dev_loss_tmo,
	.show_rport_dev_loss_tmo=		1,
};


/*
 * Function:    hfc_get_rport_dev_loss_tmo
 *
 * Purpose:     
 *
 * Arguments:   
 *  hfc_rport  -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_get_rport_dev_loss_tmo(struct fc_rport *hfc_rport)
{
	hfc_rport->dev_loss_tmo = 14;
}


/*
 * Function:    hfc_set_rport_dev_loss_tmo
 *
 * Purpose:     
 *
 * Arguments:   
 *  hfc_rport  -
 *  timeout    -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_set_rport_dev_loss_tmo(struct fc_rport *hfc_rport, uint32_t timeout)
{
	hfc_rport->dev_loss_tmo = timeout;
}


/*
 * Function:    hfc_get_starget_node_name
 *
 * Purpose:     
 *
 * Arguments:   
 *  starget    -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_get_starget_node_name(struct scsi_target *starget)
{
	struct Scsi_Host *shost;
	struct target_info *target;

	shost = dev_to_shost(starget->dev.parent);
	if (shost != NULL) {
		target = hfc_hash_target_info(((struct adap_info *)shost->hostdata), starget->id );
		
		if (target != NULL) {
			fc_starget_node_name(starget) = target->node_name;
		}
	}
}


/*
 * Function:    hfc_get_starget_port_name
 *
 * Purpose:     
 *
 * Arguments:   
 *  starget    -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_get_starget_port_name(struct scsi_target *starget)
{
	struct Scsi_Host *shost;
	struct target_info *target;

	shost = dev_to_shost(starget->dev.parent);
	if (shost != NULL) {
		target = hfc_hash_target_info(((struct adap_info *)shost->hostdata), starget->id );
		
		if (target != NULL) {
			fc_starget_port_name(starget) = target->ww_name;
		}
	}
}


/*
 * Function:    hfc_get_starget_port_id
 *
 * Purpose:     
 *
 * Arguments:   
 *  starget    -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_get_starget_port_id(struct scsi_target *starget)
{
	struct Scsi_Host *shost;
	struct target_info *target;

	shost = dev_to_shost(starget->dev.parent);
	if (shost != NULL) {
		target = hfc_hash_target_info(((struct adap_info *)shost->hostdata), starget->id );
		
		if (target != NULL) {
			fc_starget_port_id(starget) = target->scsi_id;
		}
	}
}


/*
 * Function:    hfc_get_host_port_id
 *
 * Purpose:     
 *
 * Arguments:   
 *  host       -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_get_host_port_id(struct Scsi_Host *host)
{
	fc_host_port_id(host) = ((struct adap_info *)host->hostdata)->scsi_id;
}


/*
 * Function:    hfc_rport_add
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         -
 *  target     -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_rport_add(struct adap_info *ap, struct target_info *target)
{
	struct fc_rport_identifiers rport_ids;
	struct fc_rport *rport=NULL;

	if (target->rport != NULL) {
		fc_remote_port_unblock(target->rport);
		return;
	}

	memset(&rport_ids,0,sizeof(rport_ids));
	rport_ids.node_name = target->node_name;
	rport_ids.port_name = target->ww_name;
	rport_ids.port_id   = target->scsi_id;
	rport_ids.roles     =  FC_RPORT_ROLE_UNKNOWN;
	rport_ids.roles     |= FC_RPORT_ROLE_FCP_TARGET;
	target->rport = rport = fc_remote_port_add(ap->hosts, 0, &rport_ids);

	if (!rport) {
		HFC_ERRPRT("hfcldd%d : Unable to allocate fc remote port.\n",ap->dev_minor);
		return;
	}
	
	if (rport->scsi_target_id != -1 && rport->scsi_target_id < MAX_TARGET_PROBE) {

		if (target->target_id != rport->scsi_target_id)
			HFC_ERRPRT("hfcldd%d : target->target_id != rport->scsi_target_id.\n",ap->dev_minor);
			
		rport->scsi_target_id = target->target_id;
	}
	
	rport->dd_data = target;
}


/*
 * Function:    hfc_rport_delete
 *
 * Purpose:     
 *
 * Arguments:   
 *  target     -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_rport_delete(struct target_info *target)
{
	if ((target->rport != NULL) && 	!test_bit(HFC_WAIT_CLOSE, (ulong *)&target->ap->status)) {
		fc_remote_port_delete(target->rport);
		target->rport = NULL;
	}
}
#endif /* HFC_SYSFS_FCGW */		/* _HFC_FCGW */


void hfc_reset_all_timer (struct adap_info *ap)
{
	int						i;
	struct target_info		*target;
	struct wtimer			*w_timer;
	struct dev_info			*dev;	/* FCLNX-GPL-353 *//* FCLNX-GPL-394 */

	/* Stop timer (adap_info) */
	for(i=0; i< HFC_MAX_TMR ; i++){	/* FCLNX-GPL-394 */
		switch(i){
			case HFC_ELS_TMR :
			case HFC_LINKINIT_TMR :
			case HFC_MB_TMR :
				w_timer = &ap->mb_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
				
			case HFC_REBOOT_DELAY_TMR :
			case HFC_DIAG_DELAY_TMR :
				w_timer = &ap->reboot_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN0_TMR :
				w_timer = &ap->loop_dev_info[0].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN1_TMR :
				w_timer = &ap->loop_dev_info[1].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN2_TMR :
				w_timer = &ap->loop_dev_info[2].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			
			case HFC_LUN3_TMR :
				w_timer = &ap->loop_dev_info[3].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN4_TMR :
				w_timer = &ap->loop_dev_info[4].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN5_TMR :
				w_timer = &ap->loop_dev_info[5].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN6_TMR :
				w_timer = &ap->loop_dev_info[6].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LUN7_TMR :
				w_timer = &ap->loop_dev_info[7].loop_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_LINKUP_TMR     :
			case HFC_LINKUP2_TMR    :								/* FCLNX-241*/
				w_timer = &ap->linkup_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_TARGET_RST_TMR: 
				w_timer = &ap->loop_dev_info[0].loop_wdog;
				break;												/* FCLNX-0274 */

			case HFC_ABORT_TMR: 
				w_timer = &ap->loop_dev_info[0].loop_wdog;
				break;												/* FCLNX-0274 */

			case HFC_SCSI_CMD_TMR:
				w_timer = &ap->loop_dev_info[0].loop_wdog;
				break;

			case HFC_MCK_DELAY_TMR :
				w_timer = &ap->mck_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_MCKINT_TMR :
				w_timer = &ap->mckint_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			
			case HFC_LOGIN_DELAY_TMR :								/* FCLNX-0270 */
				w_timer = &ap->lgdelay_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;												/* FCLNX-0270 */

			case HFC_MLPF_FMCK_TMR :
				w_timer = &ap->fmck_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_MLPF_MCKEND_TMR :
				w_timer = &ap->mckend_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;

			case HFC_MLPF_FCSTP_TMR :
				w_timer = &ap->fcstp_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			case HFC_LDLERR_TMR :	/*FCLNX-0506*/
				w_timer = &ap->ldlerr_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			case HFC_LDSERR_TMR :	/*FCLNX-0506*/
				w_timer = &ap->ldserr_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			case HFC_IFERR_TMR :	/*FCLNX-0506*/
				w_timer = &ap->iferr_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			case HFC_TOERR_TMR :	/*FCLNX-0506*/
				w_timer = &ap->toerr_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			case HFC_ISOLATE_DELAY_TMR :
				w_timer = &ap->fwisol_wdog;
				if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, NULL, NULL, 0, i, 0,1);
				break;
			default:
				break;


		}
	}

	for (i=0;i<ap->max_target;i++){
		target = ap -> target_arg[i];

		/* Cancel scn linkup timer */	
		w_timer = &target->scnlinkup_wdog;
		if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
		hfc_watchdog_enter( ap, target, NULL, 0, HFC_SCN_LINKUP_TMR, 0,1);

		/* Cancel delay timer */
		w_timer = &target->delay_wdog;
		if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
		hfc_watchdog_enter( ap, target, NULL, 0, HFC_DELAY_TMR, 0,1);

		/* cancel wexec tmer */
		w_timer = &target->wexec_wdog;
		if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, target, NULL, 0, HFC_WEXEC_TMR, 0,1);
		
		/* Cancel isolate timer for Target*/	/*FCLNX-GPL-327*//*FCLNX-GPL-394 */
		w_timer = &target->tgt_ldlerr_wdog;
		if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
			hfc_watchdog_enter( ap, target, NULL, 0, HFC_TGT_LDLERR_TMR, 0,1);

		w_timer = &target->tgt_ldserr_wdog;
		if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
			hfc_watchdog_enter( ap, target, NULL, 0, HFC_TGT_LDSERR_TMR, 0,1);
		/*FCLNX-GPL-327*/

		/* cancel restart tmer */													/* FCLNX-GPL-328	*//* FCLNX-GPL-353 */
		w_timer = &target->restart_wdog;
		if( (w_timer != NULL) && (w_timer->timer_flag & HFC_TIMER_VALID) )
					hfc_watchdog_enter( ap, target, NULL, 0, HFC_RESTART_TMR, 0,1);

		dev = target->dev;															/* FCLNX-GPL-038	*//* FCLNX-GPL-353 */
		if(dev != NULL) {
			/* stop LUN Reset Delay Timer */
			hfc_manage_info.npubp->hfc_all_clear_dev_info(ap, dev);
		}																			/* FCLNX-GPL-038	*//* FCLNX-GPL-353 *//*FCLNX-GPL-394 */
	}


}


int hfc_check_hba_isolation(struct adap_info *ap)				/* FCLNX-GPL-349 */
{
	uint logdata[4];
	
	if (ap->pkg.type == HFC_PKTYPE_FPP)
		return(HFC_ISOL_STOP);
	
	if ((ap->pkg.port <= 1)||((ap->pkg.port > 1)&&(ap->fw_support & 0x80))){
//	if ((ap->pkg.port <= 1) || ((ap->pkg.port > 1) && (ap->fw_init_p->func2 & HFC_FWF_MLPF_ISOL))) {
		return(HFC_ISOL_START);
	}
	else {
		if ((ap->ld_err_limit_s)||(ap->if_err_limit)||(ap->to_err_limit)||(ap->rt_err_enable)) {
			logdata[0] = ap->ld_err_limit_s;
			logdata[1] = ap->if_err_limit;
			logdata[2] = ap->to_err_limit;
			logdata[3] = ap->rt_err_enable;
			hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_EVNT3, 0xD6, (uchar *)&logdata[0], 16);
		}
		return(HFC_ISOL_STOP);
	}
	return(HFC_ISOL_STOP);
}																/* FCLNX-GPL-349 */
